//////////////////////////////////////////////////////////////////////////////
// File:        app.cpp
// Purpose:     Editor sample application
// Maintainer:  Otto Wyss
// Created:     2003-01-20
// RCS-ID:      $Id: app.cpp,v 1.2 2005/03/18 19:59:36 wyo Exp $
// Copyright:   (c) 2005 wyoGuide
// Licence:     wxWindows
//////////////////////////////////////////////////////////////////////////////

//----------------------------------------------------------------------------
// headers
//----------------------------------------------------------------------------

// For compilers that support precompilation, includes <wx/wx.h>.
#include <wx/wxprec.h>

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all 'standard' wxWidgets headers)
#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

//! wxWidgets headers
#include <wx/accel.h>    // accelerator support
#include <wx/cmdline.h>  // command line support
#include <wx/config.h>   // configuration support
#include <wx/dnd.h>      // drag&drop support
#include <wx/dynarray.h> // dynamic array support
#include <wx/dynlib.h>   // dynamic library support
#include <wx/filename.h> // filename support
#include <wx/fs_zip.h>   // ZIP filesystem support
#include <wx/html/helpctrl.h> // Html help support
#include <wx/intl.h>     // internationalisation
#include <wx/ipc.h>      // IPC support
#include <wx/log.h>      // log message support
#include <wx/notebook.h> // notebook support
#include <wx/toolbar.h>  // toolbars support
#include <wx/settings.h> // system settings
#include <wx/snglinst.h> // single instance checker
#include <wx/splitter.h> // splitter window
#include <wx/statline.h> // static line
#include <wx/string.h>   // strings support

//! wxCode headers
#if defined(__linux__)
#include "wx/crashprint.h" // CrashPrint backtrace
#endif
#include "wx/hyperlink.h"  // HyperLink control

//! application headers
#include "defsext.h"     // Additional definitions
#include "private.h"     // Platform specific definitions

#include "edit.h"        // Edit module
#include "charpick.h"    // CharPicker dialog
#include "filelist.h"    // FileList control
#include "prefdlg.h"     // Preferences dialog
#include "prefs.h"       // Preferences

// detecting memory leaks on Windows with _CrtSetBreakAlloc (<memory_number>)
// #include <crtdbg.h>

//----------------------------------------------------------------------------
// resources
//----------------------------------------------------------------------------

// the application icon (under Windows and OS/2 it is in resources)
#if defined(__WXGTK__) || defined(__WXMOTIF__) || defined(__WXMAC__) || defined(__WXMGL__) || defined(__WXX11__)
    #include "app.xpm"
#endif


//============================================================================
// declarations
//============================================================================

const wxString APP_NAME = _T("wyoEditor");
const wxString APP_SYNOPSIS = _("\
A nice editor with a well designed and consistent look and feel.\
");
const wxString APP_DESCR = _("\
This is a nice editor with a well designed and consistent look and feel. \n\
It uses the wxScintilla (wxWidgets) which itself is based on the \n\
Scintilla library (see web page for the link).\
");

const wxString APP_MAINT = _T("Otto Wyss");
const wxString APP_VENDOR = _T("wyoGuide");
const wxString APP_COPYRIGTH = _T("(C) 2004 Otto Wyss");
const wxString APP_LICENCE = _T("wxWindows");

const wxString APP_VERSION = _T("2.3.0");
const wxString APP_BUILD = _T(__DATE__);

const wxString APP_WEBSITE = _T("http://wyoguide.sourceforge.net");
const wxString APP_MAIL = _T("mailto:wyoguide-users@lists.sourceforge.net");

const wxString APP_INFOS = _("\
As a small reward I'd like to get some feedback about this app. Best if \n\
you first use it a little bit (i.e. a month) and then click the web link \n\
below. Afterwards set the \"Feedback has been given\" flag (preferences).\
");

const wxString IPC_START = _T("StartOther");

const wxString INSTALLATION = _T("Installation");
const wxString INSTALL_ABOUT = _T("About");
const wxString INSTALL_DATE = _T("Date");
const wxString INSTALL_VERSION = _T("Version");

const wxString LOCATION = _T("Location");
const wxString LOCATION_X = _T("Xpos");
const wxString LOCATION_Y = _T("Ypos");
const wxString LOCATION_W = _T("Width");
const wxString LOCATION_H = _T("Height");
const wxString LOCATION_BROWSE = _T("BrowseWidth");

const wxString HELPFILE = _T("Help/Helpfile");

const wxString RECENTFILES = _T("Recent Files/file");

const wxString NONAME = _("<Untitled>");
const wxString RESULT = _("<Result>");

class AppBook;


//----------------------------------------------------------------------------
//! global application name
wxString g_appname;

#if defined(__linux__)
//! crashprint
wxCrashPrint g_crashprint;
#endif

//! global help provider
wxHtmlHelpController *g_help = NULL;
wxString g_helppath;
wxArrayString g_helpbooks;

//! global print data, to remember settings during the session
wxPrintData *g_printData = (wxPrintData*) NULL;
wxPageSetupData *g_pageSetupData = (wxPageSetupData*) NULL;

//! global status text
wxString g_statustext;


//----------------------------------------------------------------------------
//! application
class App: public wxApp {
    friend class AppAbout;
    friend class AppFrame;
    friend class AppIPCConnection;

public:
    //! the main function called durning application start
    virtual bool OnInit ();

    //! application exit function
    virtual int OnExit ();

#if defined(__linux__)
    //! fatal exeption handling
    void OnFatalException();
#endif

private:
    //! object used to check if another program instance is running
    wxSingleInstanceChecker *m_singleInstance;

    //! the wxIPC server
    wxServerBase *m_serverIPC;

    //! variable for internationalization
    wxLocale m_locale;

    //! commandline
    wxArrayString *m_fnames;
    bool ProcessCmdLine (wxChar** argv, int argc = 0);

    //! frame window
#if !wxCHECK_VERSION(2, 5, 0)
    WX_DEFINE_ARRAY (AppFrame*, AppFrames);
#else
    WX_DEFINE_ARRAY_PTR (AppFrame*, AppFrames);
#endif
    AppFrames m_frames;
    void CreateFrame (wxArrayString *fnames);
    void RemoveFrame (AppFrame *frame);
    bool ProcessRemote (wxChar** argv, int argc = 0);

    //! help file path
    bool LocateHelp (bool force = false);

    //! recent file list
    wxFileList *m_recents;

};

// created dynamically by wxWidgets
DECLARE_APP (App);

//----------------------------------------------------------------------------
//! about box
class AppAbout: public wxDialog {

public:
    //! constructor
    AppAbout (wxWindow *parent,
              int milliseconds = 0);

    // event handlers
    void OnClose (wxCloseEvent &event);
    void OnCancel (wxCommandEvent &event);
    void OnTimerEvent (wxTimerEvent &event);

private:
    // timer
    wxTimer *m_timer;

    wxCheckBox *m_noSplash;

    DECLARE_EVENT_TABLE()
};

//----------------------------------------------------------------------------
//! IPC connection
class AppIPCConnection : public wxConnection {

public:
    //! application IPC connection
    AppIPCConnection(): wxConnection (m_buffer, WXSIZEOF(m_buffer)) {
    }

    //! execute handler
    virtual bool OnExecute (const wxString& WXUNUSED(topic),
                            wxChar *data,
                            int size,
                            wxIPCFormat WXUNUSED(format)) {
        wxChar** argv;
        int argc = 0;
        int i;
        for (i=0; i<size; i++) {
            if ((i > 0) && (data[i] == '\0') && (data[i-1] == '\0')) break;
            if (data[i] == '\0') argc++;
        }
        argv = new wxChar*[argc];
        int p = 0;
        wxChar* temp = data;
        for (i=0; i<argc; i++) {
            argv[i] = new wxChar [wxStrlen (temp) + 1];
            wxStrcpy (argv[i], temp);
            p = wxStrlen (temp) + 1;
            temp += p;
        }
        bool ok = wxGetApp().ProcessRemote (argv, argc);
        for (i=0; i<argc; i++) {
            delete [] argv[i];
        }
        delete [] argv;
        return ok;
    }

private:
   // character buffer
   wxChar m_buffer[4096];

};

//----------------------------------------------------------------------------
//! IPC server
class AppIPCServer : public wxServer {

public:
    //! accept conncetion handler
    virtual wxConnectionBase *OnAcceptConnection (const wxString& topic) {
        if (topic != IPC_START) return NULL;
        return new AppIPCConnection;
    }
};

//----------------------------------------------------------------------------
//! frame
class AppFrame: public wxFrame {
    friend class App;
    friend class AppBook;
    friend class AppAbout;
    friend class AllWindows;
    friend class DropTargets;

public:
    //! constructor
    AppFrame (const wxString &title,
              const wxArrayString &fnames,
              int frameNr);

    //! destructor
    ~AppFrame ();

    //! event handlers
    //! common event handlers
    void OnAbout (wxCommandEvent &event);
    void OnActivate (wxActivateEvent &event);
    void OnActivateApp (wxActivateEvent &event);
    void OnClose (wxCloseEvent &event);
    void OnEraseBackground (wxEraseEvent &WXUNUSED(event)) { /* avoid flickering */ }
    void OnExit (wxCommandEvent &event);
    void OnHelp (wxCommandEvent &event);
    void OnHelpLoad (wxCommandEvent &event);
    void OnIdle (wxIdleEvent &event);
    void OnTimerEvent (wxTimerEvent &event);
    //! file event handlers
    void OnFileNew (wxCommandEvent &event);
    void OnFileOpen (wxCommandEvent &event);
    void OnFileSave (wxCommandEvent &event);
    void OnFileSaveUI (wxUpdateUIEvent &event);
    void OnFileSaveAsUI (wxUpdateUIEvent &event);
    void OnFileSaveAs (wxCommandEvent &event);
    void OnFileClose (wxCommandEvent &event);
    void OnFileCloseUI (wxUpdateUIEvent &event);
    void OnFileRecents (wxCommandEvent &event);
    void OnProperties (wxCommandEvent &event);
    void OnPropertiesUI (wxUpdateUIEvent &event);
    void OnPrintSetup (wxCommandEvent &event);
    void OnPrintPreview (wxCommandEvent &event);
    void OnPrintPreviewUI (wxUpdateUIEvent &event);
    void OnPrint (wxCommandEvent &event);
    void OnPrintUI (wxUpdateUIEvent &event);
    //! edit event handlers
    void OnEditEvent (wxCommandEvent &event);
    void OnEditEventUI (wxUpdateUIEvent &event);
    void OnFindInFiles (wxCommandEvent &event);
    void OnFindInFilesUI (wxUpdateUIEvent &event);
    void OnPreferences (wxCommandEvent &event);
    void OnPreferencesUI (wxUpdateUIEvent &event);
    //! view event handlers
    void OnBrowseInfo (wxCommandEvent &event);
    void OnBrowseInfoUI (wxUpdateUIEvent &event);
    void OnToolbars (wxCommandEvent &event);
    void OnToolbarsUI (wxUpdateUIEvent &event);
    void OnStatusbar (wxCommandEvent &event);
    void OnStatusbarUI (wxUpdateUIEvent &event);
    void OnCharTable (wxCommandEvent &event);
    void OnCharTableUI (wxUpdateUIEvent &event);
    //! window event handlers
    void OnPagePrev (wxCommandEvent &event);
    void OnPagePrevUI (wxUpdateUIEvent &event);
    void OnPageNext (wxCommandEvent &event);
    void OnPageNextUI (wxUpdateUIEvent &event);
    void OnWindows (wxCommandEvent &event);
    void OnAllWindows (wxCommandEvent &event);
    void OnFrameLayout (wxCommandEvent &event);
    //! popup menu event handlers
    void OnPageActive (wxCommandEvent &event);
    void OnPageClose (wxCommandEvent &event);
    //! notebook event handlers
    void OnPageChange (wxNotebookEvent &event);
    //! others  event handlers
    void OnBrowseSelect (wxCommandEvent &event);
    void OnPageSelect (wxCommandEvent &event);
    //! custom  event handlers
    void OnResultFound (wxCommandEvent &event);
    void OnOpenAndGoto (wxCommandEvent &event);

    //! help controller
    wxHtmlHelpController& GetHelpController() {return *g_help; }

private:
    // activate state
    bool m_activateInProgress;

    // edit
    EditText *m_edit;
    int m_editNr;
    void FileOpen (wxArrayString fnames);
    void FileClose (int pageNr = -1, bool force = false);
    bool m_SetSelection;

    // browse
    EditBrowse *m_browse;
    int m_browseWidth;

    // result
    EditText *m_result;
    void ResultOpen ();
    void ResultClose ();

    EditFindThread *m_thread;
    EditFindThread *StartThread (EditText *result);

    //! main window position and size
    int m_frameNr;
    wxRect DetermineFrameSize ();
    void StoreFrameSize (wxRect rect);

    //! creates the application menu bar
    void CreateMenu ();

    //! creates a tool bar
    wxToolBar *m_toolbar;
    void CreateToolbar ();
    void DeleteToolbar ();
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
    wxComboBox *m_browseSelect;
    wxComboBox *m_pageSelect;
#endif

    //! creates status bar
    wxStatusBar *m_statusbar;
    void CreateStatusbar ();
    void DeleteStatusbar ();

    // print preview position and size
    wxRect DeterminePrintSize ();

    // update functions
    wxTimer *m_updateTimer;
    void UpdateStatustext ();
    void UpdateTitle ();

    //! notebook page
    AppBook *m_book;
    int m_pageNr;
    int m_hitNr;
    bool CreatePage (const wxString &fname);
    void PageHasChanged (int pageNr = -1);
    int GetEditCount ();
    int GetEditPageNr (const wxString &fname);
    int GetResultPageNr ();

    //! recent file list
    wxFileList *m_recents;
    wxMenu *m_recentsMenu;

    //! open file list
    wxFileList *m_filelist;
    wxMenu *m_filelistMenu;

    DECLARE_EVENT_TABLE()
};

//----------------------------------------------------------------------------
//! notebook
class AppBook: public wxNotebook {

public:
    //! constructor
    AppBook (wxWindow *parent,
             wxWindowID id = -1,
             const wxPoint& pos = wxDefaultPosition,
             const wxSize& size = wxDefaultSize,
             long style = 0);

    //! destructor
    ~AppBook ();

    // event handlers
    void OnTabActivate (wxMouseEvent &event);

private:
    AppFrame *m_frame;

    // context menu of the tab region
    wxMenu *m_bookMenu1;
    wxMenu *m_bookMenu2;
    void CreateBookMenus ();

    DECLARE_EVENT_TABLE()
};

//----------------------------------------------------------------------------
//! AllWindows
class AllWindows: public wxDialog {

public:
    //! constructor
    AllWindows (AppFrame *frame);

    // event handlers
    void OnSetActive (wxCommandEvent& event);
    void OnCancel (wxCommandEvent& event);
    void OnOkay (wxCommandEvent& event);

private:
    AppFrame *m_frame;
    wxListBox *m_windows;

    DECLARE_EVENT_TABLE()
};

//----------------------------------------------------------------------------
//! drop targets
class DropTargets: public wxDropTarget {

public:

    //! constructor
    DropTargets (AppFrame *frame);

    virtual wxDragResult OnData (wxCoord x, wxCoord y, wxDragResult def);
    bool OnDataFiles (wxCoord x, wxCoord y, const wxArrayString& filenames);
    bool OnDataText (wxCoord x, wxCoord y, const wxString& data);

    wxDragResult OnDragOver (wxCoord x, wxCoord y, wxDragResult def);
    wxDragResult OnEnter (wxCoord x, wxCoord y, wxDragResult def);
    void OnLeave();

private:

    AppFrame *m_frame;

    wxFileDataObject *m_file;
    wxTextDataObject *m_text;

};

//----------------------------------------------------------------------------
//! drop targets composite
class DropTargetsComposite: public wxDataObjectComposite {

public:

    //! constructor
    DropTargetsComposite () { m_dataObjectLast = NULL; };

    bool SetData (const wxDataFormat& format, size_t len, const void *buf) {
        m_dataObjectLast = GetObject (format);
        wxCHECK_MSG ( m_dataObjectLast, FALSE, wxT("unsupported format in wxDataObjectComposite"));
        return m_dataObjectLast->SetData (len, buf);
    }

    wxDataObjectSimple *GetLastDataObject() { return m_dataObjectLast; }

private:
    wxDataObjectSimple *m_dataObjectLast;

};


//============================================================================
// implementation
//============================================================================

IMPLEMENT_APP (App)

//----------------------------------------------------------------------------
// App
//----------------------------------------------------------------------------

bool App::OnInit () {

#if defined(__linux__)
    // fatal exceptions handling
    wxHandleFatalExceptions (true);
#endif

#if defined(__WINDOWS__)
    // detecting memory leaks on Windows with _CrtSetBreakAlloc (<memory_number>)
    //_CrtSetBreakAlloc (<memory_number>);
#endif

    // set application and vendor name
    SetAppName (APP_NAME);
    SetVendorName (APP_VENDOR);
    g_appname.Append (APP_NAME);

    // do essential initialization
    m_singleInstance = NULL;
    m_serverIPC = NULL;
    m_fnames = NULL;
    m_recents = NULL;

    // Set and check for single instance running
    wxString name = g_appname + wxString::Format (_T("%s"), wxGetUserId().c_str());
    m_singleInstance = new wxSingleInstanceChecker (name);
    if (m_singleInstance->IsAnotherRunning()) {
        wxClient client;
        wxConnectionBase *conn = client.MakeConnection (wxEmptyString, name + _T(".ipc"),
                                                        IPC_START);
        if (conn) {
            wxString dataStr;
            int i;
            for (i = 0; i < argc; ++i) {
                wxFileName w(argv[i]);
                w.Normalize (wxPATH_NORM_ALL & ~wxPATH_NORM_CASE);
                dataStr.Append (w.GetFullPath());
                dataStr.Append (_T(" "));
            }
            wxChar data[4096];
            wxStrcpy (data, dataStr.c_str());
            int size = 0;
            for (i = 0; i < argc; ++i) {
                wxFileName w(argv[i]);
                w.Normalize (wxPATH_NORM_ALL & ~wxPATH_NORM_CASE);
                size += wxStrlen (w.GetFullPath());
                data[size] = '\0';
                size += 1;
            }
            data[size] = '\0';
            size += 1;
            if (conn->Execute (data, size)) return false;
        }
        delete conn;
    }

    // IPC server
    m_serverIPC = new AppIPCServer ();
    if (!m_serverIPC->Create (name + _T(".ipc"))) {
        delete m_serverIPC;
        m_serverIPC = NULL;
    }

    // initialize localisazion
    m_locale.Init();
    wxString appPath = wxFileName(argv[0]).GetPath (wxPATH_GET_VOLUME);
    if (appPath.IsEmpty()) appPath = wxGetCwd();
    wxString appName = wxFileName(argv[0]).GetName();
#ifdef __WXMAC__
    appPath += appName + _T(".app/Contents/Resources"));
#endif
    m_locale.AddCatalog (appPath + wxFileName::GetPathSeparator() + appName);

    // initializes help (Html help needs the Zip filestytem!?!)
    wxFileSystem::AddHandler (new wxZipFSHandler);
    g_help = new wxHtmlHelpController;
    LocateHelp ();

    // initialize print data and setup
    g_printData = new wxPrintData;
    g_pageSetupData = new wxPageSetupDialogData;

    // load preferences
    Prefs::LoadValuesPageCommon ();
    Prefs::LoadValuesPageLanguages ();
    Prefs::LoadValuesPageStyles ();
    Prefs::LoadValuesPageAnalyse ();

    // get and process command line
    m_fnames = new wxArrayString();
    ProcessCmdLine (argv, argc);

    // recent file list
    m_recents = new wxFileList (4,
                                myFLIST_STARTSEPARATOR |
                                myFLIST_CTRL_SHIFT_KEYS |
                                myFLIST_LRUORDER);
    m_recents->Load (RECENTFILES);

    // create application frame
    int nr = m_frames.GetCount();
    m_frames.Add (new AppFrame (g_appname, *m_fnames, nr));

    // open application frame
    m_frames[0]->Layout ();
    m_frames[0]->Show (true);
    SetTopWindow (m_frames[0]);

    // about box shown for a few seconds
    CheckInstallation (APP_VERSION);
    if (!g_CommonPrefs.noSplash) new AppAbout (NULL, 3000);

    return true;
}

int App::OnExit () {
    // delete single instance checker
    if (m_singleInstance) delete m_singleInstance;

    // delete IPC server
    if (m_serverIPC) delete m_serverIPC;

    // delete help
    if (g_help) delete g_help;

    // delete global print data and setup
    if (g_printData) delete g_printData;
    if (g_pageSetupData) delete g_pageSetupData;

    // delete preferences
    Prefs::DeleteValuesPageCommon ();
    Prefs::DeleteValuesPageLanguages ();
    Prefs::DeleteValuesPageStyles ();
    Prefs::DeleteValuesPageAnalyse ();

    // delete commandline fnames
    if (m_fnames) delete m_fnames;

    // save and delete recent file list
    if (m_recents) {
        m_recents->Save (RECENTFILES);
        delete m_recents;
    }

    return 0;
}

#if defined(__linux__)
void App::OnFatalException () {
    g_crashprint.Report();
}
#endif

bool App::ProcessCmdLine (wxChar** argv, int argc) {

    // get and process command line
    static const wxCmdLineEntryDesc cmdLineDesc[] = {
        {wxCMD_LINE_PARAM,  NULL, NULL, _T("input files"),
         wxCMD_LINE_VAL_STRING,
         wxCMD_LINE_PARAM_OPTIONAL | wxCMD_LINE_PARAM_MULTIPLE},
        {wxCMD_LINE_NONE}
    };
    wxCmdLineParser parser (cmdLineDesc, argc, argv);

    // get filenames from the commandline
    m_fnames->Clear();
    if (parser.Parse() == 0) {
        for (size_t paramNr=0; paramNr < parser.GetParamCount(); paramNr++) {
            m_fnames->Add (parser.GetParam (paramNr));
        }
    }

    return true;
}

bool App::ProcessRemote (wxChar** argv, int argc) {

    // process remote request
    ProcessCmdLine (argv, argc);
    AppFrame* frame = (AppFrame*)wxTheApp->GetTopWindow();
    frame->FileOpen (*m_fnames);
    frame->Maximize (false);

    return true;
}

void App::CreateFrame (wxArrayString *fnames) {
    int nr = m_frames.GetCount();
    m_frames.Add (new AppFrame (g_appname, *fnames, nr));
    m_frames[nr]->Layout ();
    m_frames[nr]->Show (true);
    SetTopWindow (m_frames[nr]);
}

void App::RemoveFrame (AppFrame *frame) {
    m_frames.Remove (frame);
}

bool App::LocateHelp (bool force) {
    wxString appPath = wxFileName(argv[0]).GetPath (wxPATH_GET_VOLUME);
    if (appPath.IsEmpty()) appPath = wxGetCwd();
    wxString appName = wxFileName(argv[0]).GetName();
#ifdef __WXMAC__
    appPath += appName + _T(".app/Contents/Resources"));
#endif
    wxString helpfile = appPath + wxFileName::GetPathSeparator() + appName + _T(".htb");
    wxConfig* cfg = new wxConfig (g_appname);
    helpfile = cfg->Read (HELPFILE, helpfile);
    delete cfg;
    while (force || !helpfile.IsEmpty() && !g_help->AddBook (helpfile)) {
        wxFileDialog dlg (NULL, _("Locate help"), g_helppath, wxEmptyString, _T("*.htb"),
                          wxOPEN | wxFILE_MUST_EXIST);
        if (dlg.ShowModal() == wxID_OK) {
            helpfile = dlg.GetPath ();
        }else{
            if (wxMessageBox (_("No help file was chosen, would you like to be \nasked again after next application start?"),
                              _("Locate help"),
                              wxYES_NO | wxICON_QUESTION) == wxYES) {
                wxConfig* cfg = new wxConfig (g_appname);
                if (cfg->Exists (HELPFILE)) cfg->DeleteEntry (HELPFILE);
                delete cfg;
                return false;
            }
            helpfile = wxEmptyString;
        }
        wxConfig* cfg = new wxConfig (g_appname);
        cfg->Write (HELPFILE, helpfile);
        delete cfg;
        force = false;
    }
    g_helppath = wxFileName(helpfile).GetPath (wxPATH_GET_VOLUME);
    return true;
}


//----------------------------------------------------------------------------
// AppAbout
//----------------------------------------------------------------------------

BEGIN_EVENT_TABLE (AppAbout, wxDialog)
    EVT_CLOSE (                 AppAbout::OnClose)
    EVT_BUTTON (wxID_CANCEL,    AppAbout::OnCancel)
    EVT_TIMER (myID_ABOUTTIMER, AppAbout::OnTimerEvent)
END_EVENT_TABLE ()

AppAbout::AppAbout (wxWindow *parent,
                    int milliseconds)
        : wxDialog (parent, -1, _("Preferences"),
                    wxDefaultPosition, wxDefaultSize,
                    wxDEFAULT_DIALOG_STYLE | wxSTAY_ON_TOP) {

    // set title
    SetTitle (_("About..."));

    // set timer if any
    m_timer = NULL;
    if (milliseconds > 0) {
        m_timer = new wxTimer (this, myID_ABOUTTIMER);
        m_timer->Start (milliseconds, wxTIMER_ONE_SHOT);
    }

    // about info
    wxFlexGridSizer *aboutinfo = new wxFlexGridSizer (2, 0, 2);
    aboutinfo->Add (new wxStaticText(this, -1, _("Version: ")));
    aboutinfo->Add (new wxStaticText(this, -1, APP_VERSION));
    aboutinfo->Add (new wxStaticText(this, -1, _("Written by: ")));
    aboutinfo->Add (new wxStaticText(this, -1, APP_MAINT));
    aboutinfo->Add (new wxStaticText(this, -1, _("Licence type: ")));
    aboutinfo->Add (new wxStaticText(this, -1, APP_LICENCE));
    aboutinfo->Add (new wxStaticText(this, -1, _("wxScintilla: ")));
    aboutinfo->Add (new wxStaticText(this, -1, wxSCINTILLA_VERSION));
    aboutinfo->Add (new wxStaticText(this, -1, _("wxWidgets: ")));
    aboutinfo->Add (new wxStaticText(this, -1, wxVERSION_STRING));
    aboutinfo->Add (new wxStaticText(this, -1, _("Copyright: ")));
    aboutinfo->Add (new wxStaticText(this, -1, APP_COPYRIGTH));

    // about icontitle//info
    wxBoxSizer *aboutpane = new wxBoxSizer (wxHORIZONTAL);
    wxBitmap bitmap = wxBitmap(wxICON (app));
    aboutpane->Add (new wxStaticBitmap (this, -1, bitmap),
                    0, wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 40);
    aboutpane->Add (aboutinfo, 1, wxEXPAND);
    aboutpane->Add (60, 0);

    // about complete
    wxBoxSizer *totalpane = new wxBoxSizer (wxVERTICAL);
    totalpane->Add (0, 20);
    wxStaticText *appname = new wxStaticText(this, -1, g_appname);
    appname->SetFont (wxFont (24, wxDEFAULT, wxNORMAL, wxBOLD));
    totalpane->Add (appname, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 40);
    totalpane->Add (0, 10);
    totalpane->Add (aboutpane, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT | wxBOTTOM, 10);
    totalpane->Add (new wxStaticText(this, -1, APP_SYNOPSIS),
                    0, wxALIGN_LEFT | wxLEFT | wxRIGHT | wxBOTTOM, 10);
    totalpane->Add (new wxStaticText(this, -1, APP_DESCR),
                    0, wxALIGN_LEFT | wxLEFT | wxRIGHT | wxBOTTOM, 10);
    wxHyperLink *website = new wxHyperLink (this, -1, APP_WEBSITE);
    wxString url1 = APP_WEBSITE;
    url1.Append (_T("/index.php?page=edit.html"));
    website->SetURL (url1);
    totalpane->Add (website, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT | wxBOTTOM, 10);
    totalpane->Add (new wxStaticText(this, -1, APP_INFOS),
                    0, wxALIGN_LEFT | wxLEFT | wxRIGHT | wxBOTTOM, 10);
    wxHyperLink *feedback = new wxHyperLink (this, -1, APP_WEBSITE + _T("/feedback"));
    wxString url2 = APP_WEBSITE;
    url2.Append (_T("/feedback.php?component=") + g_appname);
    feedback->SetURL (url2);
    totalpane->Add (feedback, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT | wxBOTTOM, 10);
    m_noSplash = new wxCheckBox (this, -1, _("Disable splash on startup"));
    m_noSplash->SetValue(g_CommonPrefs.noSplash);
    totalpane->Add (m_noSplash, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT | wxBOTTOM, 10);
    wxButton *okButton = new wxButton (this, wxID_CANCEL, _("OK"));
    okButton->SetDefault();
    totalpane->Add (okButton, 0, wxALIGN_CENTER | wxALL, 10);

    SetSizerAndFit (totalpane);

    CentreOnParent();
    Show();
}

//----------------------------------------------------------------------------
// event handlers
void AppAbout::OnClose (wxCloseEvent& WXUNUSED(event)) {
    if (m_timer)  {
        delete m_timer;
        m_timer = NULL;
    }
    this->Destroy();
}

void AppAbout::OnCancel (wxCommandEvent &WXUNUSED(event)) {
    if (m_noSplash->GetValue()) g_CommonPrefs.noSplash = true;
    Prefs::SaveValuesPageCommon ();
    Close ();
}

void AppAbout::OnTimerEvent (wxTimerEvent &WXUNUSED(event)) {
    Close ();
}


//----------------------------------------------------------------------------
// AppFrame
//----------------------------------------------------------------------------

BEGIN_EVENT_TABLE (AppFrame, wxFrame)
    // common events
    EVT_ACTIVATE (                   AppFrame::OnActivate)
    EVT_ACTIVATE_APP (               AppFrame::OnActivateApp)
    EVT_CLOSE (                      AppFrame::OnClose)
    EVT_ERASE_BACKGROUND (           AppFrame::OnEraseBackground)
    EVT_IDLE (                       AppFrame::OnIdle)
    EVT_TIMER (myID_UPDATETIMER,     AppFrame::OnTimerEvent)
    // file events
    EVT_MENU (wxID_NEW,              AppFrame::OnFileNew)
    EVT_MENU (myID_NEWOTHER,         AppFrame::OnFileNew)
    EVT_MENU (wxID_OPEN,             AppFrame::OnFileOpen)
    EVT_MENU (myID_OPENOTHER,        AppFrame::OnFileOpen)
    EVT_MENU (wxID_SAVE,             AppFrame::OnFileSave)
    EVT_UPDATE_UI (wxID_SAVE,        AppFrame::OnFileSaveUI)
    EVT_MENU (wxID_SAVEAS,           AppFrame::OnFileSaveAs)
    EVT_UPDATE_UI (wxID_SAVEAS,      AppFrame::OnFileSaveAsUI)
    EVT_MENU (wxID_CLOSE,            AppFrame::OnFileClose)
    EVT_UPDATE_UI (wxID_CLOSE,       AppFrame::OnFileCloseUI)
    EVT_MENU_RANGE (wxID_FILE1,
                    wxID_FILE9,      AppFrame::OnFileRecents)
    EVT_MENU (myID_PROPERTIES,       AppFrame::OnProperties)
    EVT_UPDATE_UI (myID_PROPERTIES,  AppFrame::OnPropertiesUI)
    EVT_MENU (wxID_PRINT_SETUP,      AppFrame::OnPrintSetup)
    EVT_MENU (wxID_PREVIEW,          AppFrame::OnPrintPreview)
    EVT_UPDATE_UI (wxID_PREVIEW,     AppFrame::OnPrintPreviewUI)
    EVT_MENU (wxID_PRINT,            AppFrame::OnPrint)
    EVT_UPDATE_UI (wxID_PRINT,       AppFrame::OnPrintUI)
    EVT_MENU (wxID_EXIT,             AppFrame::OnExit)
    // edit events
    EVT_MENU (wxID_CLEAR,            AppFrame::OnEditEvent)
    EVT_UPDATE_UI (wxID_CLEAR,       AppFrame::OnEditEventUI)
    EVT_MENU (wxID_CUT,              AppFrame::OnEditEvent)
    EVT_UPDATE_UI (wxID_CUT,         AppFrame::OnEditEventUI)
    EVT_MENU (wxID_COPY,             AppFrame::OnEditEvent)
    EVT_UPDATE_UI (wxID_COPY,        AppFrame::OnEditEventUI)
    EVT_MENU (wxID_PASTE,            AppFrame::OnEditEvent)
    EVT_UPDATE_UI (wxID_PASTE,       AppFrame::OnEditEventUI)
    EVT_MENU (myID_INDENTINC,        AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_INDENTINC,   AppFrame::OnEditEventUI)
    EVT_MENU (myID_INDENTRED,        AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_INDENTRED,   AppFrame::OnEditEventUI)
    EVT_MENU (wxID_SELECTALL,        AppFrame::OnEditEvent)
    EVT_UPDATE_UI (wxID_SELECTALL,   AppFrame::OnEditEventUI)
    EVT_MENU (myID_SELECTLINE,       AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_SELECTLINE,  AppFrame::OnEditEventUI)
    EVT_MENU (wxID_REDO,             AppFrame::OnEditEvent)
    EVT_UPDATE_UI (wxID_REDO,        AppFrame::OnEditEventUI)
    EVT_MENU (wxID_UNDO,             AppFrame::OnEditEvent)
    EVT_UPDATE_UI (wxID_UNDO,        AppFrame::OnEditEventUI)
    EVT_MENU (wxID_FIND,             AppFrame::OnEditEvent)
    EVT_UPDATE_UI (wxID_FIND,        AppFrame::OnEditEventUI)
    EVT_MENU (myID_FINDNEXT,         AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_FINDNEXT,    AppFrame::OnEditEventUI)
    EVT_MENU (myID_FINDINFILES,      AppFrame::OnFindInFiles)
    EVT_UPDATE_UI (myID_FINDINFILES, AppFrame::OnFindInFilesUI)
    EVT_MENU (myID_REPLACE,          AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_REPLACE,     AppFrame::OnEditEventUI)
    EVT_MENU (myID_REPLACENEXT,      AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_REPLACENEXT, AppFrame::OnEditEventUI)
    EVT_MENU (myID_BRACEMATCH,       AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_BRACEMATCH,  AppFrame::OnEditEventUI)
    EVT_MENU (myID_GOTO,             AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_GOTO,        AppFrame::OnEditEventUI)
    EVT_MENU (myID_BOOKMARKTOGGLE,   AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_BOOKMARKTOGGLE, AppFrame::OnEditEventUI)
    EVT_MENU (myID_BOOKMARKNEXT,     AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_BOOKMARKNEXT, AppFrame::OnEditEventUI)
    EVT_MENU (myID_BOOKMARKPREV,     AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_BOOKMARKPREV, AppFrame::OnEditEventUI)
    EVT_MENU (myID_BOOKMARKCLEAR,    AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_BOOKMARKCLEAR, AppFrame::OnEditEventUI)
    EVT_MENU (myID_PREFS,            AppFrame::OnPreferences)
    EVT_UPDATE_UI (myID_PREFS,       AppFrame::OnPreferencesUI)
    // view events
    EVT_MENU (myID_BROWSEINFO,       AppFrame::OnBrowseInfo)
    EVT_UPDATE_UI (myID_BROWSEINFO,  AppFrame::OnBrowseInfoUI)
    EVT_MENU (myID_TOOLBARS,         AppFrame::OnToolbars)
    EVT_UPDATE_UI (myID_TOOLBARS,    AppFrame::OnToolbarsUI)
    EVT_MENU (myID_STATUSBAR,        AppFrame::OnStatusbar)
    EVT_UPDATE_UI (myID_STATUSBAR,   AppFrame::OnStatusbarUI)
//?    EVT_MENU (myID_TWOWINDOWS,       AppFrame::OnEditEvent)
    EVT_MENU_RANGE (myID_HILIGHTFIRST, myID_HILIGHTLAST,
                                     AppFrame::OnEditEvent)
    EVT_UPDATE_UI_RANGE (myID_HILIGHTFIRST, myID_HILIGHTLAST,
                                     AppFrame::OnEditEventUI)
    EVT_MENU (myID_DISPLAYEOL,       AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_DISPLAYEOL,  AppFrame::OnEditEventUI)
    EVT_MENU (myID_DISPLAYFOLD,      AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_DISPLAYFOLD, AppFrame::OnEditEventUI)
    EVT_MENU (myID_FOLDTOGGLE,       AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_FOLDTOGGLE,  AppFrame::OnEditEventUI)
    EVT_MENU (myID_INDENTGUIDE,      AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_INDENTGUIDE, AppFrame::OnEditEventUI)
    EVT_MENU (myID_LINENUMBER,       AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_LINENUMBER,  AppFrame::OnEditEventUI)
    EVT_MENU (myID_LONGLINEON,       AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_LONGLINEON,  AppFrame::OnEditEventUI)
    EVT_MENU (myID_OVERTYPE,         AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_OVERTYPE,    AppFrame::OnEditEventUI)
    EVT_MENU (myID_READONLY,         AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_READONLY,    AppFrame::OnEditEventUI)
    EVT_MENU (myID_WHITESPACE,       AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_WHITESPACE,  AppFrame::OnEditEventUI)
    EVT_MENU (myID_WRAPMODEON,       AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_WRAPMODEON,  AppFrame::OnEditEventUI)
    EVT_UPDATE_UI (myID_USECHARSET,  AppFrame::OnEditEventUI)
    EVT_MENU_RANGE (myID_CHARSET_ANSI, myID_CHARSET_VIETNAMESE,
                                     AppFrame::OnEditEvent)
    EVT_UPDATE_UI_RANGE (myID_CHARSET_ANSI, myID_CHARSET_VIETNAMESE,
                                     AppFrame::OnEditEventUI)
    // extra events
    EVT_UPDATE_UI (myID_CHANGECASE,  AppFrame::OnEditEventUI)
    EVT_MENU (myID_CHANGELOWER,      AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_CHANGELOWER, AppFrame::OnEditEventUI)
    EVT_MENU (myID_CHANGEUPPER,      AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_CHANGEUPPER, AppFrame::OnEditEventUI)
    EVT_UPDATE_UI (myID_WHITESPACETO,AppFrame::OnEditEventUI)
    EVT_MENU (myID_WHITETOSPACE,     AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_WHITETOSPACE,AppFrame::OnEditEventUI)
    EVT_MENU (myID_WHITETOTAB,       AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_WHITETOTAB,  AppFrame::OnEditEventUI)
    EVT_UPDATE_UI (myID_CONVERTEOL,  AppFrame::OnEditEventUI)
    EVT_MENU (myID_CONVERTCR,        AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_CONVERTCR,   AppFrame::OnEditEventUI)
    EVT_UPDATE_UI (myID_CONVERTCRLF, AppFrame::OnEditEventUI)
    EVT_MENU (myID_CONVERTCRLF,      AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_CONVERTLF,   AppFrame::OnEditEventUI)
    EVT_MENU (myID_CONVERTLF,        AppFrame::OnEditEvent)
    EVT_MENU (myID_TRAILINGSPACE,    AppFrame::OnEditEvent)
    EVT_UPDATE_UI (myID_TRAILINGSPACE, AppFrame::OnEditEventUI)
    EVT_MENU (myID_CHARTABLE,        AppFrame::OnCharTable)
    EVT_UPDATE_UI (myID_CHARTABLE,     AppFrame::OnCharTableUI)
    // window events
    EVT_MENU (myID_PAGEPREV,         AppFrame::OnPagePrev)
    EVT_UPDATE_UI (myID_PAGEPREV,    AppFrame::OnPagePrevUI)
    EVT_MENU (myID_PAGENEXT,         AppFrame::OnPageNext)
    EVT_UPDATE_UI (myID_PAGENEXT,    AppFrame::OnPageNextUI)
    EVT_MENU_RANGE (myID_WINDOW1, myID_WINDOW9,
                                     AppFrame::OnWindows)
    EVT_MENU (myID_WINDOWS,          AppFrame::OnAllWindows)
    EVT_MENU (myID_FRAMELAYOUT,      AppFrame::OnFrameLayout)
    // help events
    EVT_MENU (wxID_ABOUT,            AppFrame::OnAbout)
    EVT_MENU (wxID_HELP,             AppFrame::OnHelp)
    EVT_MENU (wxID_HELP_CONTENTS,    AppFrame::OnHelp)
    EVT_MENU (myID_HELP_LOAD,        AppFrame::OnHelpLoad)
    // popup menu  events
    EVT_MENU (myID_PAGEACTIVE,       AppFrame::OnPageActive)
    EVT_MENU (myID_PAGECLOSE,        AppFrame::OnPageClose)
    // notebook events
    EVT_NOTEBOOK_PAGE_CHANGED (-1,   AppFrame::OnPageChange)
    // other events
    EVT_COMBOBOX (myID_BROWSESELECT, AppFrame::OnBrowseSelect)
    EVT_COMBOBOX (myID_PAGESELECT,   AppFrame::OnPageSelect)
    // custom events
    myEVT_RESULT_FOUND (             AppFrame::OnResultFound)
    myEVT_RESULT_SELECTED (          AppFrame::OnOpenAndGoto)
END_EVENT_TABLE ()

// protection for threads
static wxCriticalSection gs_critsect;

AppFrame::AppFrame (const wxString &title,
                    const wxArrayString &fnames,
                    int frameNr)
        : wxFrame ((wxFrame *)NULL, -1, title, wxDefaultPosition, wxDefaultSize,
                    wxDEFAULT_FRAME_STYLE) {

    // intitialize important variables
    m_activateInProgress = false;
    m_edit = NULL;
    m_editNr = 0;
    m_browse = NULL;
    m_pageNr = -1;
    m_SetSelection = false;
    m_updateTimer = NULL;
    m_thread = NULL;
    m_result = NULL;

    // set icon
    SetIcon (wxICON (app));

    // set frame size
    m_frameNr = frameNr;
    SetSize (DetermineFrameSize ());

    // recent file list
    m_recents = ((App*)wxTheApp)->m_recents;

    // open file list
    m_filelist = new wxFileList (-1, myFLIST_STARTSEPARATOR|myFLIST_CTRL_KEYS);

    // create menu
    CreateMenu ();

    // initialize toolbars
    m_toolbar = NULL;
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
    m_browseSelect = NULL;
    m_pageSelect = NULL;
#endif
    if (!m_toolbar && g_CommonPrefs.showToolbar) {
        CreateToolbar ();
    }

    // initialize statusbar
    m_statusbar = NULL;
    if (!m_statusbar && g_CommonPrefs.showStatusbar) {
        CreateStatusbar ();
    }
    m_updateTimer = new wxTimer (this, myID_UPDATETIMER);

    // initialize drop targets
    SetDropTarget (new DropTargets (this));

    // create frame panel
    wxPanel *panel = new wxPanel (this, -1, wxDefaultPosition, wxDefaultSize,
                                 wxTAB_TRAVERSAL|wxCLIP_CHILDREN|wxNO_BORDER);

    // initialize notebook
    m_book = new AppBook (panel, -1, wxDefaultPosition, wxDefaultSize,
                          wxNB_FIXEDWIDTH|wxCLIP_CHILDREN);
    m_book->SetDropTarget (new DropTargets (this));
    m_hitNr = -1;

    // layout frame panel
    wxBoxSizer *panelsizer = new wxBoxSizer (wxVERTICAL);
    panelsizer->Add (new wxStaticLine (panel, -1), 0, wxEXPAND);
    panelsizer->Add (0, 2);
    panelsizer->Add (m_book, 1, wxEXPAND);
    panel->SetSizerAndFit (panelsizer);

    // open first page
    m_editNr += 1;
    m_book->Freeze();
    CreatePage (NONAME + wxString::Format (_T(".%d"), m_editNr));
    PageHasChanged (m_pageNr);
    m_book->Thaw();

    // open any startup files
    FileOpen (fnames);

    // update information
    g_statustext = _("Welcome to the ") + g_appname + _(" application.");
}

AppFrame::~AppFrame () {
    if (m_updateTimer) {
        delete m_updateTimer;
        m_updateTimer = NULL;
    }
    m_filelist->RemoveMenu (m_filelistMenu);
    delete m_filelist;
    delete m_book;
}

// common event handlers
void AppFrame::OnActivate (wxActivateEvent &event) {
    if (!event.GetActive() || m_activateInProgress) return;
    wxWindow *win = wxGetActiveWindow();
    if (win) ((App*)wxTheApp)->SetTopWindow (win);
    m_activateInProgress = true;
    m_book->Freeze();
    int nr = 0;
    EditText *edit;
    wxLogNull nolog; // suppress log messages
    while (nr < (int)m_book->GetPageCount()) {
        edit = (EditText*)((wxSplitterWindow*)m_book->GetPage (nr))->GetWindow2();
        if (!edit || !edit->IsEdit()) {
            nr += 1;
            continue;
        }
        wxDateTime filetime = edit->GetModificationTime();
        if (filetime > wxDateTime((time_t)0)) {
            wxFileName fname (edit->GetFilename());
            if (edit->GetFilename().IsEmpty()) {
                // do nothing if no file name
            }else if (!fname.FileExists()) {
                wxMessageBox (edit->GetFilename() + _(" does not exist!"),
                              _("File modification check"),
                              wxOK | wxICON_INFORMATION);
            }else if (fname.GetModificationTime() > filetime) {
                if (wxMessageBox (edit->GetFilename() +
                                  _(" has been modified externally. \nWould you like to reload the file?"),
                                  _("File modification check"),
                                  wxYES_NO | wxICON_QUESTION) == wxYES) {
                    int visibleLine = edit->GetFirstVisibleLine() +
                                      edit->LinesOnScreen();
                    int currentPos = edit->GetCurrentPos();
                    edit->LoadFile (edit->GetFilename());
                    edit->SetModificationTime (fname.GetModificationTime());
                    edit->GotoLine (visibleLine);
                    int line = edit->GetCurrentLine() -
                               (edit->GetFirstVisibleLine()+2);
                    if (line > 0) edit->LineScroll (0, line);
                    edit->GotoPos (currentPos);
                }
            }
        }
        nr += 1;
    }
    m_activateInProgress = false;
    PageHasChanged();
    m_book->Thaw();
    event.Skip();
}

void AppFrame::OnActivateApp (wxActivateEvent &event) {
    OnActivate (event);
}

void AppFrame::OnClose (wxCloseEvent &event) {
    m_book->Freeze();
    int i;
    int cnt = GetEditCount();
    for (i = 0; i < cnt; i++) {
        FileClose (-1, true);
    }
    if (m_edit && m_edit->Modified()) {
        if (event.CanVeto()) event.Veto (true);
        m_book->Thaw();
        return;
    }
    if (myIsKeyDown (WXK_CONTROL) && !myIsKeyDown ('Q')) {
        StoreFrameSize (GetRect ());
    }
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
    if (m_pageSelect) m_filelist->RemoveCbox (m_pageSelect);
#endif
    m_recents->RemoveMenu (m_recentsMenu);
    ((App*)wxTheApp)->RemoveFrame (this);
    m_book->Thaw();
    Destroy();
}

void AppFrame::OnAbout (wxCommandEvent &WXUNUSED(event)) {
    new AppAbout (this);
}

void AppFrame::OnExit (wxCommandEvent &WXUNUSED(event)) {
    Close (true);
}

void AppFrame::OnHelp (wxCommandEvent &WXUNUSED(event)) {
    if (m_edit && (m_edit->GetSelectionEnd()-m_edit->GetSelectionStart() > 0)) {
// wxHtmlHelpController::KeywordIndex might not be supported
#if HELP_INDEX_KEYWORD
        g_help->KeywordSearch (m_edit->GetSelectedText(), wxHELP_SEARCH_INDEX);
#else
        g_help->KeywordSearch (m_edit->GetSelectedText());
#endif
        return;
    }
    wxWindow *active = wxGetActiveWindow();
    wxString helptext;
    while (active && helptext.IsEmpty()) {
        helptext = active->GetHelpText();
        active = GetParent();
    }
    g_help->DisplayContents();
}

void AppFrame::OnHelpLoad (wxCommandEvent &WXUNUSED(event)) {
    ((App*)wxTheApp)->LocateHelp (true);
}

void AppFrame::OnIdle (wxIdleEvent &event) {
    if (m_updateTimer && !m_updateTimer->IsRunning ()) {;
        m_updateTimer->Start (100, wxTIMER_ONE_SHOT);
        UpdateStatustext();
        UpdateTitle();
    }
    event.Skip();
}

void AppFrame::OnTimerEvent (wxTimerEvent &event) {
    if (m_updateTimer) m_updateTimer->Stop ();
    if (m_browse) m_browse->OnTimerEvent (event);
}

// file event handlers
void AppFrame::OnFileNew (wxCommandEvent &event) {
    if (((g_CommonPrefs.usePages) && (event.GetId() == wxID_NEW)) ||
        ((!g_CommonPrefs.usePages) && (event.GetId() == myID_NEWOTHER))) {
        m_book->Freeze();
        m_editNr += 1;
        CreatePage (NONAME + wxString::Format (_T(".%d"), m_editNr));
        PageHasChanged (m_pageNr);
        m_book->Thaw();
    }else{
        wxArrayString fnames;
        ((App*)wxTheApp)->CreateFrame (&fnames);
    }
}

void AppFrame::OnFileOpen (wxCommandEvent &event) {
    if (((g_CommonPrefs.usePages) && (event.GetId() == wxID_OPEN)) ||
        ((!g_CommonPrefs.usePages) && (event.GetId() == myID_OPENOTHER))) {
        if (!m_edit && !m_edit->IsEdit()) return;
    }
    wxString fname = m_edit->GetFilename();
    wxArrayString fnames;
    wxFileDialog dlg (this, _("Open files"), wxEmptyString, wxEmptyString, _("Any file (*)|*"),
                      wxOPEN | wxFILE_MUST_EXIST | wxMULTIPLE | wxCHANGE_DIR);
    if (!fname.IsEmpty()) dlg.SetPath (wxFileName (fname).GetFullPath());
    if (dlg.ShowModal() != wxID_OK) return;
    dlg.GetPaths (fnames);
    if (((g_CommonPrefs.usePages) && (event.GetId() == wxID_OPEN)) ||
        ((!g_CommonPrefs.usePages) && (event.GetId() == myID_OPENOTHER))) {
        FileOpen (fnames);
    }else{
        ((App*)wxTheApp)->CreateFrame (&fnames);
    }
}

void AppFrame::OnFileSave (wxCommandEvent &event) {
    if (!m_edit) return;
    if (!m_edit->Modified()) {
        wxMessageBox (_("There is nothing to save!"), _("Save file"),
                      wxOK | wxICON_INFORMATION);
        return;
    }
    if (m_book->GetPageText (m_pageNr).Contains (NONAME)) {
        OnFileSaveAs (event);
    }else{
        m_edit->SaveFile ();
    }
}

void AppFrame::OnFileSaveUI (wxUpdateUIEvent &event) {
    event.Enable (m_edit && m_edit->Modified());
}

void AppFrame::OnFileSaveAs (wxCommandEvent &WXUNUSED(event)) {
    if (!m_edit) return;
    wxFileDialog dlg (this, _("Save file"), wxEmptyString, wxEmptyString, _("Any file (*)|*"),
                      wxSAVE | wxOVERWRITE_PROMPT);
    if (dlg.ShowModal() != wxID_OK) return;
    wxString fname = dlg.GetPath();
    if (m_edit->SaveFile (fname)) {
        wxString filename = wxFileName (fname).GetFullName();
        m_book->SetPageText (m_pageNr, filename);
        m_filelist->Replace (fname, m_pageNr);
        m_recents->Add (fname);
    }
}

void AppFrame::OnFileSaveAsUI (wxUpdateUIEvent &event) {
    event.Enable (m_edit && m_edit->IsEdit());
}

void AppFrame::OnFileClose (wxCommandEvent &WXUNUSED(event)) {
    FileClose ();
}

void AppFrame::OnFileCloseUI (wxUpdateUIEvent &event) {
    event.Enable (m_book && (m_book->GetPageCount() > 0));
}

void AppFrame::OnFileRecents (wxCommandEvent &event) {
    wxArrayString fnames;
    fnames.Clear();
    fnames.Add (m_recents->Item (event.GetId() - wxID_FILE1));
    FileOpen (fnames);
}

void AppFrame::OnProperties (wxCommandEvent &WXUNUSED(event)) {
    if (!m_edit) return;
    EditProperties (m_edit, 0);
}

void AppFrame::OnPropertiesUI (wxUpdateUIEvent &event) {
    event.Enable (m_edit && m_edit->IsEdit());
}

void AppFrame::OnPrintSetup (wxCommandEvent &WXUNUSED(event)) {
    (*g_pageSetupData) = * g_printData;
    wxPageSetupDialog pageSetupDialog(this, g_pageSetupData);
    pageSetupDialog.ShowModal();
    (*g_printData) = pageSetupDialog.GetPageSetupData().GetPrintData();
    (*g_pageSetupData) = pageSetupDialog.GetPageSetupData();
}

void AppFrame::OnPrintPreview (wxCommandEvent &WXUNUSED(event)) {
    wxPrintDialogData printDialogData( *g_printData);
    wxPrintPreview *preview =
        new wxPrintPreview (new EditPrint (m_edit),
                            new EditPrint (m_edit),
                            &printDialogData);
    if (!preview->Ok()) {
        delete preview;
        wxMessageBox (_("There was a problem with previewing.\nPerhaps your current printer is not correctly?"),
                      _("Previewing"), wxOK | wxICON_INFORMATION);
        return;
    }
    wxRect rect = DeterminePrintSize();
    wxPreviewFrame *frame = new wxPreviewFrame (preview, this, _("Print Preview"));
    frame->SetSize (rect);
    frame->Centre(wxBOTH);
    frame->Initialize();
    frame->Show(true);
}

void AppFrame::OnPrintPreviewUI (wxUpdateUIEvent &event) {
    event.Enable (m_edit && m_edit->IsEdit());
}

void AppFrame::OnPrint (wxCommandEvent &WXUNUSED(event)) {
    wxPrintDialogData printDialogData( *g_printData);
    wxPrinter printer (&printDialogData);
    EditPrint printout (m_edit);
    if (!printer.Print (this, &printout, true)) {
        if (wxPrinter::GetLastError() == wxPRINTER_ERROR) {
            wxMessageBox (_("There was a problem with printing.\nPerhaps your current printer is not correctly?"),
                          _("Previewing"), wxOK | wxICON_INFORMATION);
            return;
        }
    }
    (*g_printData) = printer.GetPrintDialogData().GetPrintData();
}

void AppFrame::OnPrintUI (wxUpdateUIEvent &event) {
    event.Enable (m_edit && m_edit->IsEdit());
}

// edit event handlers
void AppFrame::OnEditEvent (wxCommandEvent &event) {
    if (m_edit) m_edit->ProcessEvent (event);
}

void AppFrame::OnEditEventUI (wxUpdateUIEvent &event) {
    if (m_edit && m_edit->IsEdit()) {
        m_edit->ProcessEvent (event);
    }else{
        event.Enable (false);
    }

}

void AppFrame::OnFindInFiles (wxCommandEvent &WXUNUSED(event)) {
    if (m_thread) {
        if (wxMessageBox (_("Would you like to stop the find?"), _("Find in Files"),
                          wxYES_NO | wxICON_QUESTION) == wxYES) {
            m_thread->Delete();
            m_thread = NULL;
        }
        return;
    }
    if (!m_edit || !m_edit->IsEdit()) return;
    EditText *edit = m_edit;
    ResultOpen();
    if (!m_result || !m_result->IsResult()) return;
    if (!(m_result->FindInFiles (edit))) return;
    m_thread = StartThread (m_result);
    if (!m_thread) return;
}

void AppFrame::OnFindInFilesUI (wxUpdateUIEvent &event) {
    event.Enable (m_edit && m_edit->IsEdit());
}

void AppFrame::OnPreferences (wxCommandEvent &WXUNUSED(event)) {
    PreferenceDlg (this);
    EditText *edit;
    int pageNr;
    for (pageNr = 0; pageNr < (int)m_book->GetPageCount(); pageNr++) {
        edit = (EditText*)((wxSplitterWindow*)m_book->GetPage (pageNr))->GetWindow2();
        if (edit->IsEdit()) {
            edit->InitializeLanguage (edit->DetermineLanguage (edit->GetFilename ()));
            edit->InitializeIdentTable ();
        }
    }
    if (!m_toolbar && g_CommonPrefs.showToolbar) {
        CreateToolbar ();
    }else if (m_toolbar && !g_CommonPrefs.showToolbar) {
        DeleteToolbar ();
    }
    if (!m_statusbar && g_CommonPrefs.showStatusbar) {
        CreateStatusbar ();
    }else if (m_statusbar && !g_CommonPrefs.showStatusbar) {
        DeleteStatusbar ();
    }
    g_statustext = _("Preferences command");
}

void AppFrame::OnPreferencesUI (wxUpdateUIEvent &event) {
    event.Enable (m_edit && m_edit->IsEdit());
}

// view event handlers
void AppFrame::OnBrowseInfo (wxCommandEvent &WXUNUSED(event)) {
    wxSplitterWindow *split = (wxSplitterWindow*)m_book->GetPage (m_pageNr);
    if (split->GetSashPosition() == 1) {
        if (!m_edit->GetAnalyse()) {
            wxMessageBox (_("This language setting doesn't provide browse infos"),
                          _("Show browse info feedback"), wxOK | wxICON_INFORMATION);
            return;
        }
        m_edit->NotifyEditBrowse (myEVT_COMMON_NOTIFY, -1);
        split->SetSashPosition (m_browseWidth);
    }else{
        split->SetSashPosition (1);
    }
}

void AppFrame::OnBrowseInfoUI (wxUpdateUIEvent &event) {
    if (m_book && (m_book->GetPageCount() > 0)) {
        event.Enable (true);
        wxSplitterWindow *split = (wxSplitterWindow*)m_book->GetPage (m_pageNr);
        event.Check (split && split->GetSashPosition() > 1);
    }else{
        event.Enable (false);
    }
}

void AppFrame::OnToolbars (wxCommandEvent &WXUNUSED(event)) {
    if (!m_toolbar) {
        CreateToolbar ();
    }else{
        DeleteToolbar ();
    }
    g_statustext = _("Tool bar command");
}

void AppFrame::OnToolbarsUI (wxUpdateUIEvent &event) {
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
    event.Enable (true);
    event.Check (m_toolbar != NULL);
#else
    event.Enable (false);
    event.Check (false);
#endif
}

void AppFrame::OnStatusbar (wxCommandEvent &WXUNUSED(event)) {
    if (!m_statusbar) {
        CreateStatusbar ();
    }else{
        DeleteStatusbar ();
    }
    g_statustext = _("Status bar command");
}

void AppFrame::OnStatusbarUI (wxUpdateUIEvent &event) {
    event.Enable (true);
    event.Check (m_statusbar != NULL);
}

void AppFrame::OnCharTable (wxCommandEvent &WXUNUSED(event)) {
    wxString chrs;
    myCharPickerDlg (this, &chrs);
    if (!chrs.IsEmpty()) m_edit->AddText (chrs);
}

void AppFrame::OnCharTableUI (wxUpdateUIEvent &event) {
    event.Enable (m_edit && m_edit->IsEdit());
}

// window event handlers
void AppFrame::OnPagePrev (wxCommandEvent &WXUNUSED(event)) {
    m_book->Freeze();
    m_book->AdvanceSelection (false);
    PageHasChanged();
    m_book->Thaw();
}

void AppFrame::OnPagePrevUI (wxUpdateUIEvent &event) {
    event.Enable (m_book && (m_book->GetPageCount() > 1));
}

void AppFrame::OnPageNext (wxCommandEvent &WXUNUSED(event)) {
    m_book->Freeze();
    m_book->AdvanceSelection (true);
    PageHasChanged();
    m_book->Thaw();
}

void AppFrame::OnPageNextUI (wxUpdateUIEvent& event) {
    event.Enable (m_book && (m_book->GetPageCount() > 1));
}

void AppFrame::OnWindows (wxCommandEvent &event) {
    int pageNr = event.GetId() - myID_WINDOW1;
    if (pageNr == m_pageNr) return;
    m_book->Freeze();
    m_SetSelection = true;
    m_book->SetSelection (pageNr);
    m_SetSelection = false;
    PageHasChanged();
    m_book->Thaw();
}

void AppFrame::OnAllWindows (wxCommandEvent &WXUNUSED(event)) {
    AllWindows (this);
}

void AppFrame::OnFrameLayout (wxCommandEvent &WXUNUSED(event)) {
    StoreFrameSize (GetRect ());
    wxMessageBox (_("The position and size of the window are stored."),
                  _("Store window size"), wxOK | wxICON_INFORMATION);
}

// popup menu event handlers
void AppFrame::OnPageActive (wxCommandEvent &WXUNUSED(event)) {
    if (m_hitNr == m_pageNr) return;
    m_book->Freeze();
    m_SetSelection = true;
    m_book->SetSelection (m_hitNr);
    m_SetSelection = false;
    PageHasChanged();
    m_book->Thaw();
}

void AppFrame::OnPageClose (wxCommandEvent &WXUNUSED(event)) {
    if (m_hitNr >= 0) {
        FileClose (m_hitNr);
    }
}

// notebook event handlers
void AppFrame::OnPageChange (wxNotebookEvent &WXUNUSED(event)) {
    m_book->Freeze();
    if (m_SetSelection) return;
    PageHasChanged();
    m_book->Thaw();
}

// other event handlers
void AppFrame::OnBrowseSelect (wxCommandEvent &event) {
    if (m_browse) m_browse->ProcessEvent (event);
}

void AppFrame::OnPageSelect (wxCommandEvent &WXUNUSED(event)) {
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
    if (!m_pageSelect) return;
    int pageNr = m_pageSelect->GetSelection();
    if (pageNr == m_pageNr) return;
    m_book->Freeze();
    m_SetSelection = true;
    m_book->SetSelection (pageNr);
    m_SetSelection = false;
    PageHasChanged();
    m_book->Thaw();
#endif
}

// custom event handlers
void AppFrame::OnResultFound (wxCommandEvent &event) {
    if (m_result == NULL) return;
    m_result->SetReadOnly (false);
    m_result->InsertText (m_result->GetLength(), event.GetString());
    m_result->SetReadOnly (true);
    int exitcode = event.GetInt();
    if (exitcode == 0) {
        g_statustext = _("Find in files ended");
        m_thread = NULL;
    }else if (exitcode < 0) {
        g_statustext = _("Find in files ***canceled***");
        m_thread = NULL;
    }
}

void AppFrame::OnOpenAndGoto (wxCommandEvent &event) {
    wxString fname = (event.GetString ()).BeforeLast (':');
    if (fname.IsEmpty()) return;
    wxString ln = (event.GetString ()).AfterLast (':');
    long line;
    ln.ToLong (&line);
    wxArrayString fnames; fnames.Add (fname);
    FileOpen (fnames);

    if (line > 0) {
        m_edit->GotoLine (line-1);
        line = m_edit->GetCurrentLine() - (m_edit->GetFirstVisibleLine()+2);
        if (line > 0) m_edit->LineScroll (0, line);
    }else{
        m_edit->GotoLine (0);
    }
}

// private functions
void AppFrame::CreateMenu () {

    // File menu
    wxMenu *menuFile = new wxMenu;
    menuFile->Append (wxID_NEW, _("&New\tCtrl+N"));
    if (!g_CommonPrefs.usePages) {
        menuFile->Append (myID_NEWOTHER, _("New page\tCtrl+Shift+N"));
    }else{
        menuFile->Append (myID_NEWOTHER, _("New window\tCtrl+Shift+N"));
    }
    menuFile->Append (wxID_OPEN, _("&Open ...\tCtrl+O"));
    if (!g_CommonPrefs.usePages) {
        menuFile->Append (myID_OPENOTHER, _("Open page ...\tCtrl+Shift+O"));
    }else{
        menuFile->Append (myID_OPENOTHER, _("Open window ...\tCtrl+Shift+O"));
    }
    menuFile->Append (wxID_SAVE, _("&Save\tCtrl+S"));
    menuFile->Append (wxID_SAVEAS, _("Save &as ...\tCtrl+Shift+S"));
    menuFile->Append (wxID_CLOSE, _("&Close\tCtrl+W"));
    m_recents->UseMenu (menuFile, wxID_FILE1, -1, 24,
                        myPARTIAL_BEGIN | myPARTIAL_END);
    m_recentsMenu = menuFile;
    menuFile->AppendSeparator();
    menuFile->Append (myID_PROPERTIES, _("Propert&ies ...\tCtrl+I"));
    menuFile->AppendSeparator();
    menuFile->Append (wxID_PRINT_SETUP, _("Print Set&up ..."));
    menuFile->Append (wxID_PREVIEW, _("Print Pre&view\tCtrl+Shift+P"));
    menuFile->Append (wxID_PRINT, _("&Print ...\tCtrl+P"));
#ifndef __WXMAC__
    menuFile->AppendSeparator();
#else
    wxApp::s_macExitMenuItemId = wxID_EXIT;
#endif
    menuFile->Append (wxID_EXIT, _("&Quit\tCtrl+Q"));

    // Edit menu
    wxMenu *menuEdit = new wxMenu;
    menuEdit->Append (wxID_UNDO, _("&Undo\tCtrl+Z"));
#ifndef __WXMSW__
    menuEdit->Append (wxID_REDO, _("&Redo\tCtrl+Shift+Z"));
#else
    menuEdit->Append (wxID_REDO, _("&Redo\tCtrl+Y"));
#endif
    menuEdit->AppendSeparator();
    menuEdit->Append (wxID_CUT, _("Cu&t\tCtrl+X"));
    menuEdit->Append (wxID_COPY, _("&Copy\tCtrl+C"));
    menuEdit->Append (wxID_PASTE, _("&Paste\tCtrl+V"));
    menuEdit->Append (wxID_CLEAR, _("&Delete\tDel"));
    menuEdit->AppendSeparator();
    menuEdit->Append (wxID_FIND, _("&Find ...\tCtrl+F"));
    menuEdit->Append (myID_FINDNEXT, _("Find &next\tF3"));
    menuEdit->Append (myID_FINDINFILES, _("Find &in files\tCtrl+Alt+F"));
    menuEdit->Append (myID_REPLACE, _("&Replace ...\tCtrl+H"));
    menuEdit->Append (myID_REPLACENEXT, _("Replace &again\tShift+F4"));
    menuEdit->AppendSeparator();
    menuEdit->Append (myID_GOTO, _("&Goto ...\tCtrl+G"));
    menuEdit->Append (myID_BRACEMATCH, _("&Match brace\tCtrl+M"));
    menuEdit->AppendSeparator();
    menuEdit->Append (myID_BOOKMARKTOGGLE, _("Toggle bookmark\tCtrl+D"));
    menuEdit->Append (myID_BOOKMARKNEXT, _("Next &bookmark\tCtrl+B"));
    menuEdit->Append (myID_BOOKMARKPREV, _("Previous bookmark\tCtrl+Shift+B"));
    menuEdit->Append (myID_BOOKMARKCLEAR, _("Clear all bookmarks"));
    menuEdit->AppendSeparator();
    menuEdit->Append (myID_INDENTINC, _("Indent increase\tTab"));
    menuEdit->Append (myID_INDENTRED, _("Indent reduce\tBksp"));
    menuEdit->Enable (myID_INDENTRED, false);
    menuEdit->AppendSeparator();
    menuEdit->Append (wxID_SELECTALL, _("&Select all\tCtrl+A"));
    menuEdit->Append (myID_SELECTLINE, _("Select &line\tCtrl+L"));
#ifndef __WXMAC__
    menuEdit->AppendSeparator();
#else
    wxApp::s_macPreferencesMenuItemId = myID_PREFS;
#endif
    menuEdit->Append (myID_PREFS, _("Pr&eferences ..."));

    // hilight submenu
    wxMenu *menuHilight = new wxMenu;
    int Nr;
    for (Nr = 0; Nr < g_LanguagePrefsSize; Nr++) {
        menuHilight->AppendRadioItem (myID_HILIGHTFIRST + Nr,
                                      g_LanguagePrefs [Nr].name);
    }

    // charset submenu
    wxMenu *menuCharset = new wxMenu;
    menuCharset->Append (myID_CHARSET_ANSI, _("&ANSI (Windows)"));
    menuCharset->Append (myID_CHARSET_EASTEUROPE, _("Eastern europe"));
    menuCharset->Append (myID_CHARSET_GB2312, _("GB2312"));
    menuCharset->Append (myID_CHARSET_HANGUL, _("Hangul"));
    menuCharset->Append (myID_CHARSET_HEBREW, _("Hebrew"));
    menuCharset->Append (myID_CHARSET_SHIFTJIS, _("Shiftjis"));
#if defined __WXMSW__
    menuCharset->Append (myID_CHARSET_ARABIC, _("Arabic"));
    menuCharset->Append (myID_CHARSET_BALTIC, _("Baltic"));
    menuCharset->Append (myID_CHARSET_CHINESEBIG5, _("Chinese (big5)"));
    menuCharset->Append (myID_CHARSET_GREEK, _("Greek"));
    menuCharset->Append (myID_CHARSET_JOHAB, _("Johab"));
    menuCharset->Append (myID_CHARSET_MAC, _("Macintosh"));
    menuCharset->Append (myID_CHARSET_OEM, _("OEM"));
    menuCharset->Append (myID_CHARSET_RUSSIAN, _("Russian"));
    menuCharset->Append (myID_CHARSET_SYMBOL, _("Symbol"));
    menuCharset->Append (myID_CHARSET_THAI, _("Thai"));
    menuCharset->Append (myID_CHARSET_TURKISH, _("Turkish"));
    menuCharset->Append (myID_CHARSET_VIETNAMESE, _("Vietnamese"));
#endif

    // View menu
    wxMenu *menuView = new wxMenu;
    menuView->AppendCheckItem (myID_BROWSEINFO, _("Browse &info"));
    menuView->AppendCheckItem (myID_TOOLBARS, _("Tool &bar"));
    menuView->AppendCheckItem (myID_STATUSBAR, _("Status b&ar"));
    menuView->AppendSeparator();
//?    menuView->Append (myID_TWOWINDOWS, _("&Use two windows"));
//?    menuView->AppendSeparator();
    menuView->Append (myID_HILIGHTLANG, _("&Highlight language"), menuHilight);
    menuView->AppendSeparator();
    menuView->AppendCheckItem (myID_FOLDTOGGLE, _("&Toggle current fold\tCtrl+T"));
    menuView->AppendCheckItem (myID_OVERTYPE, _("&Overwrite mode\tIns"));
    menuView->AppendCheckItem (myID_WRAPMODEON, _("&Wrap mode\tCtrl+U"));
    menuView->AppendSeparator();
    menuView->AppendCheckItem (myID_WHITESPACE, _("Show white&space"));
    menuView->AppendCheckItem (myID_DISPLAYEOL, _("Show &end of lines"));
    menuView->AppendCheckItem (myID_LINENUMBER, _("Show line &numbers"));
    menuView->AppendCheckItem (myID_INDENTGUIDE, _("Show indent &guides"));
    menuView->AppendCheckItem (myID_LONGLINEON, _("Show long line &marker"));
    menuView->AppendCheckItem (myID_DISPLAYFOLD, _("Show fold &level number"));
    menuView->AppendSeparator();
    menuView->Append (myID_USECHARSET, _("Use &code page of"), menuCharset);

    // change case submenu
    wxMenu *menuChangeCase = new wxMenu;
    menuChangeCase->Append (myID_CHANGEUPPER, _("&Upper case"));
    menuChangeCase->Append (myID_CHANGELOWER, _("&Lower case"));

    // change whitespace submenu
    wxMenu *menuWhitespace = new wxMenu;
    menuWhitespace->Append (myID_WHITETOSPACE, _("&Spaces"));
    menuWhitespace->Append (myID_WHITETOTAB, _("&Tabulators"));

    // convert EOL submenu
    wxMenu *menuConvertEOL = new wxMenu;
    menuConvertEOL->Append (myID_CONVERTCRLF, _("CR+LF (&Windows)"));
    menuConvertEOL->Append (myID_CONVERTCR, _("CR (&Macintosh)"));
    menuConvertEOL->Append (myID_CONVERTLF, _("LF (&Unix)"));

    // Extra menu
    wxMenu *menuExtra = new wxMenu;
    menuExtra->AppendCheckItem (myID_READONLY, _("&Readonly mode"));
    menuExtra->AppendSeparator();
    menuExtra->Append (myID_CHANGECASE, _("Change &case to"), menuChangeCase);
    menuExtra->AppendSeparator();
    menuExtra->Append (myID_WHITESPACETO, _("Change &whitespace to"), menuWhitespace);
    menuExtra->Append (myID_CONVERTEOL, _("Convert line &endings to"), menuConvertEOL);
    menuExtra->Append (myID_TRAILINGSPACE, _("Remove &trailing whitespace"));
    menuExtra->AppendSeparator();
    menuExtra->Append (myID_CHARTABLE, _("Show char&set table"));

    // Window menu
    wxMenu *menuWindow = new wxMenu;
    menuWindow->Append (myID_PAGEPREV, _("&Previous\tCtrl+Shift+Tab"));
    menuWindow->Append (myID_PAGENEXT, _("&Next\tCtrl+Tab"));
    m_filelist->UseMenu (menuWindow, myID_WINDOW1, -1, 24,
                         myPARTIAL_BEGIN | myPARTIAL_END);
    m_filelistMenu = menuWindow;
    menuWindow->AppendSeparator();
    menuWindow->Append (myID_WINDOWS, _("&All windows ..."));
    menuWindow->AppendSeparator();
    menuWindow->Append (myID_FRAMELAYOUT, _("Store window size"));

    // Help menu
    wxMenu *menuHelp = new wxMenu;
    menuHelp->Append (wxID_HELP_CONTENTS, _("&Content\tF1"));
    menuHelp->Append (wxID_HELP, _("&Index\tF1"));
    menuHelp->AppendSeparator();
    menuHelp->Append (myID_HELP_LOAD, _("&Load help books"));
#ifndef __WXMAC__
    menuHelp->AppendSeparator();
#else
    wxApp::s_macAboutMenuItemId = wxID_ABOUT;
#endif
    menuHelp->Append (wxID_ABOUT, _("&About ...\tShift+F1"));

    // construct menu
    wxMenuBar *menuBar = new wxMenuBar;
    menuBar->Append (menuFile, _("&File"));
    menuBar->Append (menuEdit, _("&Edit"));
    menuBar->Append (menuView, _("&View"));
    menuBar->Append (menuExtra, _("E&xtra"));
    menuBar->Append (menuWindow, _("&Window"));
    menuBar->Append (menuHelp, _("&Help"));
#ifdef __WXMAC__
    wxApp::s_macHelpMenuTitleName = _("&Help");
#endif
    SetMenuBar (menuBar);

}

void AppFrame::CreateToolbar () {
    if (m_toolbar) return;

    // create toolbar
    m_toolbar = wxFrame::CreateToolBar (wxTB_TEXT|wxTB_FLAT);

    // create session selector box
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
    m_pageSelect = new wxComboBox (m_toolbar, myID_PAGESELECT, wxEmptyString,
                                   wxDefaultPosition, wxSize (32*GetCharWidth(),-1),
                                   0, NULL, wxCB_READONLY);
    m_toolbar->AddControl (m_pageSelect);
    m_filelist->UseCbox (m_pageSelect, 32);
#endif
    if (m_pageNr >= 0) m_filelist->SetSelection (m_pageNr);

    // create class browse box
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
    m_browseSelect = new wxComboBox (m_toolbar, myID_BROWSESELECT, wxEmptyString,
                                     wxDefaultPosition, wxSize (m_browseWidth,-1),
                                     0, NULL, wxCB_READONLY);
    m_browseSelect->Enable (false);
    m_toolbar->AddControl (m_browseSelect);
#endif

    m_toolbar->Realize();
}

void AppFrame::DeleteToolbar () {
    if (!m_toolbar) return;

    // remove session selector box
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
    m_filelist->RemoveCbox (m_pageSelect);
#endif

    // delete toolbar
    SetToolBar (NULL);
    delete m_toolbar;
    m_toolbar = NULL;
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
    m_browseSelect = NULL;
    m_pageSelect = NULL;
#endif
}

void AppFrame::CreateStatusbar () {
    if (m_statusbar) return;
    int ch = GetCharWidth();
#if defined(__WXGTK__)
    ch -= ch/6; // reduce GTK font width by 15%
#endif
    const int widths[] = {-1, 4*ch, 4*ch, 6*ch, 8*ch, 20*ch};
    m_statusbar = CreateStatusBar (WXSIZEOF(widths), wxST_SIZEGRIP, myID_STATUSBAR);
    m_statusbar->SetStatusWidths (WXSIZEOF(widths), widths);
    SendSizeEvent();
}

void AppFrame::DeleteStatusbar () {
    if (!m_statusbar) return;
    SetStatusBar (NULL);
    delete m_statusbar;
    m_statusbar = NULL;
    SendSizeEvent();
}

wxRect AppFrame::DetermineFrameSize () {

    const int minFrameWidth = 80;
    const int minFrameHight = 80;

    wxSize scr = wxGetDisplaySize();

    // determine default frame position/size
    wxRect normal;
    if (scr.x <= 640) {
        normal.x = 0;
        normal.width = scr.x;
    }else if (scr.x <= 800) {
        normal.x = scr.x / 12;
        normal.width = scr.x - normal.x - 10;
    }else if (scr.x <= 1024) {
        normal.x = scr.x / 10;
        normal.width = wxMin (scr.x - normal.x - 20, 840); // about A4
    }else{
        normal.x = 1024 / 10; // same as x=1024
        normal.width = 840; // about A4
    }
    if (scr.y <= 640) {
        normal.y = 0;
        normal.height = scr.y;
    }else if (scr.y <= 800) {
        normal.y = 0;
        normal.height = scr.y - normal.y - 30;
    }else if (scr.y <= 1024) {
        normal.y = 10;
        normal.height = scr.y - normal.y - 30;
    }else{
        normal.y = 20;
        normal.height = 1200; // about A4
    }

    // determine sash position
    m_browseWidth = wxMin (160, normal.width / 6);
    m_browseWidth = wxMax (100, m_browseWidth);

    // load stored size or defaults
    wxRect rect = normal;
    wxConfig* cfg = new wxConfig (g_appname);
    int i;
    for (i = 0; i <= m_frameNr; i++) {
        wxString key = LOCATION + wxString::Format (_T("%d"), m_frameNr - i);
        if (cfg->Exists (key)) {
            rect.x = cfg->Read (key + _T("/") + LOCATION_X, rect.x);
            rect.y = cfg->Read (key + _T("/") + LOCATION_Y, rect.y);
            rect.width = cfg->Read (key + _T("/") + LOCATION_W, rect.width);
            rect.height = cfg->Read (key + _T("/") + LOCATION_H, rect.height);
            m_browseWidth = cfg->Read (key + _T("/") + LOCATION_BROWSE, m_browseWidth);
            break;
        }
    }
    delete cfg;
    if (i > 0) {
        rect.x += 20 * i;
        rect.y += 20 * i;
    }

    // check for reasonable values (within screen)
    rect.x = wxMin (abs (rect.x), (scr.x - minFrameWidth));
    rect.y = wxMin (abs (rect.y), (scr.y - minFrameHight));
    rect.width = wxMax (abs (rect.width), (minFrameWidth));
    rect.width = wxMin (abs (rect.width), (scr.x - rect.x));
    rect.height = wxMax (abs (rect.height), (minFrameHight));
    rect.height = wxMin (abs (rect.height), (scr.y - rect.y));

    return rect;
}

void AppFrame::StoreFrameSize (wxRect rect) {

    // store size
    wxConfig* cfg = new wxConfig (g_appname);
    wxString key = LOCATION + wxString::Format (_T("%d"), m_frameNr);
    cfg->Write (key + _T("/") + LOCATION_X, rect.x);
    cfg->Write (key + _T("/") + LOCATION_Y, rect.y);
    cfg->Write (key + _T("/") + LOCATION_W, rect.width);
    cfg->Write (key + _T("/") + LOCATION_H, rect.height);
    cfg->Write (key + _T("/") + LOCATION_BROWSE, m_browseWidth);
    delete cfg;
}

void AppFrame::FileOpen (wxArrayString fnames) {
    m_book->Freeze();
    int firstPageNr = -1;
    for (size_t n = 0; n < fnames.GetCount(); n++) {
        wxFileName w(fnames[n]);
        w.Normalize (wxPATH_NORM_ALL & ~wxPATH_NORM_CASE);
        fnames[n] = w.GetFullPath();
        wxString filename = wxFileName (fnames[n]).GetFullName();
        int pageNr = GetEditPageNr (fnames[n]);
        if (pageNr >= 0) {
            m_SetSelection = true;
            m_book->SetSelection (pageNr);
            m_SetSelection = false;
            m_pageNr = pageNr;
        }else if (m_edit && m_edit->IsEdit() &&
                  !m_edit->Modified() && m_edit->GetFilename().IsEmpty()) {
            if (m_edit->LoadFile (fnames[n])) {
                m_book->SetPageText (m_pageNr, filename);
                m_filelist->Replace (fnames[n], m_pageNr);
            }
        }else{
            CreatePage (fnames[n]);
        }
        if (firstPageNr < 0) firstPageNr = m_pageNr;
        m_filelist->SetSelection (m_pageNr);
        m_recents->Add (fnames[n]);
    }
    if (firstPageNr >= 0) {
        m_pageNr = firstPageNr;
        m_SetSelection = true;
        m_book->SetSelection (m_pageNr);
        m_SetSelection = false;
        g_statustext = _("Opened file: ") + fnames[0];
    }
    PageHasChanged (m_pageNr);
    m_book->Thaw();
}

void AppFrame::FileClose (int pageNr, bool force) {
    if (pageNr == -1) pageNr = m_book->GetSelection();
    EditText *edit = (EditText*)((wxSplitterWindow*)m_book->GetPage (pageNr))->GetWindow2();
    if (edit->IsEdit()) {
        if (!edit) return;
        if (edit->Modified()) {
            if (wxMessageBox (_("Text is not saved, save before closing?"), _("Close"),
                              wxYES_NO | wxICON_QUESTION) == wxYES) {
                edit->SaveFile();
                if (edit->Modified()) {
                    wxMessageBox (_("Text could not be saved!"), _("Close abort"),
                                  wxOK | wxICON_EXCLAMATION);
                    return;
                }
            }
        }
    }else if (edit->IsResult()) {
        ResultClose();
    }
    if (!force) m_book->Freeze();
    if (GetEditCount() > 0) {
        if ((GetEditCount() > 1) || edit->IsResult() || force) {
            wxString name;
            if (edit->IsEdit()) name = edit->GetFilename();
            if (name.IsEmpty()) name = m_book->GetPageText (pageNr);
            m_filelist->Remove (name);
            m_book->DeletePage (pageNr);
        }else{
            m_editNr = 1;
            wxString noname = NONAME + wxString::Format (_T(".%d"), m_editNr);
            m_book->SetPageText (m_pageNr, noname);
            m_edit->SetFilename (wxEmptyString);
            m_edit->ClearAll();
            m_edit->SetSavePoint();
            m_filelist->Add (noname);
            wxSplitterWindow *split = (wxSplitterWindow*)m_book->GetPage (m_pageNr);
            split->SetSashPosition (1);
        }
    }
    PageHasChanged (m_book->GetSelection());
    if (!force) m_book->Thaw();
}

bool AppFrame::CreatePage (const wxString &fname) {
    wxSplitterWindow *split = new wxSplitterWindow (m_book, -1,
                     wxDefaultPosition, wxDefaultSize, wxSP_NOBORDER);
    split->SetMinimumPaneSize (1);
    EditText *win;
    if (!fname.Contains (RESULT)) {
        win = new EditText (split, -1, edit_EDIT, m_browseWidth);
    }else{
        win = new EditText (split, -1, edit_RESULT, 1);
    }
    win->SetDropTarget (new DropTargets (this));
    EditBrowse *browse = new EditBrowse (split, -1, win,
                         wxDefaultPosition, wxSize (m_browseWidth,-1));
    split->SplitVertically (browse, win, 1);
    if (!(fname.Contains (NONAME) || fname.Contains (RESULT)) && !win->LoadFile (fname)) {
        delete browse;
        delete win;
        delete split;
        return false;
    }
    if (!fname.Contains (RESULT)) {
        m_edit = win;
    }else{
        m_result = win;
    }
    m_pageNr = GetEditCount();
    m_browse = browse;
    m_book->InsertPage (m_pageNr, split, wxFileName (fname).GetFullName(), true);
    m_filelist->Add (fname);
    return true;
}

void AppFrame::PageHasChanged (int pageNr) {
    if (m_book->GetPageCount() == 0) {
        m_pageNr = -1;
        m_browse = NULL;
        m_edit = NULL;
        m_result = NULL;
        m_filelist->SetSelection (0);
        return;
    }
    if (pageNr == -1) {
        pageNr = m_book->GetSelection();
        if (m_edit) m_edit->SetFocus();
        if (m_result) m_result->SetFocus();
        if (m_pageNr == pageNr) return;
    }
    if ((int)m_book->GetPageCount() <= pageNr) pageNr = m_book->GetPageCount()-1;
    if (pageNr >= 0) {
        m_pageNr = pageNr;
        EditBrowse *win1 = (EditBrowse*)((wxSplitterWindow*)m_book->GetPage (pageNr))->GetWindow1();
        EditText *win2 = (EditText*)((wxSplitterWindow*)m_book->GetPage (pageNr))->GetWindow2();
        win2->SetFocus();
        if (win2->IsEdit()) {
            m_browse = win1;
            m_edit = win2;
            m_result = NULL;
        }else{
            m_browse = NULL;
            m_edit = NULL;
            m_result = win2;
        }
        m_filelist->SetSelection (pageNr);
        wxSplitterWindow *split = (wxSplitterWindow*)m_book->GetPage (m_pageNr);
        if (m_edit && m_edit->GetAnalyse()) {
#if !wxCHECK_VERSION(2, 5, 0)
            wxUsleep (100); // wxSplitterWindows has a bug, therefore this delay
#else
            wxMilliSleep (100); // wxSplitterWindows has a bug, therefore this delay
#endif
            if (g_CommonPrefs.showBrowse) {
                split->SetSashPosition (m_browseWidth);
            }else{
                split->SetSashPosition (1);
            }
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
            if (m_browseSelect) {
                m_browse->ComboboxCreate ();
                m_browseSelect->Enable (true);
            }
#endif
        }else{
            split->SetSashPosition (1);
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
            if (m_browseSelect) {
                m_browseSelect->Clear ();
                m_browseSelect->Enable (false);
            }
#endif
        }
    }else{
        m_pageNr = -1;
        m_browse = NULL;
        m_edit = NULL;
        m_result = NULL;
        m_filelist->SetSelection (0);
    }
    if (m_edit) m_edit->SetFocus();
    if (m_result) m_result->SetFocus();
}

int AppFrame::GetEditCount () {
    int pageNr;
    int count = 0;
    for (pageNr = 0; pageNr < (int)m_book->GetPageCount(); pageNr++) {
        EditText *win = (EditText*)((wxSplitterWindow*)m_book->GetPage (pageNr))->GetWindow2();
        if (win->IsEdit()) count += 1;
    }
    return count;
}

int AppFrame::GetEditPageNr (const wxString &fname) {
    int pageNr;
    for (pageNr = 0; pageNr < (int)m_book->GetPageCount(); pageNr++) {
        EditText *edit = (EditText*)((wxSplitterWindow*)m_book->GetPage (pageNr))->GetWindow2();
        if (edit->IsEdit() && (edit->GetFilename() == fname)) {
            return pageNr;
        }
    }
    return -1;
}

int AppFrame::GetResultPageNr () {
    int pageNr;
    for (pageNr = 0; pageNr < (int)m_book->GetPageCount(); pageNr++) {
        EditText *edit = (EditText*)((wxSplitterWindow*)m_book->GetPage (pageNr))->GetWindow2();
        if (edit->IsResult()) {
            return pageNr;
        }
    }
    return -1;
}

wxRect AppFrame::DeterminePrintSize () {

    wxSize scr = wxGetDisplaySize();

    // determine position and size (shifting 16 left and down)
    wxRect rect = GetRect();
    rect.x += 16;
    rect.y += 16;
    rect.width = wxMin (rect.width, (scr.x - rect.x));
    rect.height = wxMin (rect.height, (scr.x - rect.y));

    return rect;
}

void AppFrame::ResultOpen () {
    m_book->Freeze();
    int pageNr = GetResultPageNr();
    if (pageNr < 0) {
        wxString resname = RESULT;
        CreatePage (RESULT);
        m_result->SetWrapMode (wxSCI_WRAP_NONE);
        m_result->SetReadOnly (true);
    }else{
        m_pageNr = pageNr;
        m_result = m_edit;
        m_result->GotoPos (m_result->GetLength());
    }
    m_SetSelection = true;
    m_book->SetSelection (m_pageNr);
    m_SetSelection = false;
    PageHasChanged (m_pageNr);
    m_book->Thaw();
}

void AppFrame::ResultClose () {
    m_result = NULL;
}

EditFindThread *AppFrame::StartThread (EditText *result) {
    // create thread
    EditFindThread *thread = new EditFindThread (result);
    if (thread->Create() != wxTHREAD_NO_ERROR) {
        wxLogError (_T("Can't create thread!"));
        return NULL;
    }

    wxCriticalSectionLocker enter (gs_critsect);
    if (thread->Run() != wxTHREAD_NO_ERROR) {
        wxLogError (_T("Can't start thread!"));
        return NULL;
    }

    return thread;
}

void AppFrame::UpdateStatustext () {
    if (!m_statusbar) return;
    wxStatusBar* sb = GetStatusBar();
    if (g_statustext != sb->GetStatusText (0)) SetStatusText (g_statustext, 0);
    if (m_edit) {
        wxString text;
        text = m_edit->GetReadOnly()? _T("RO"): _T("RW");
        if (text != sb->GetStatusText (1)) SetStatusText (text, 1);
        text = m_edit->GetOvertype()? _T("OVR"): _T("INS");
        if (text != sb->GetStatusText (2)) SetStatusText (text, 2);
        switch (m_edit->GetEOLMode()) {
            case wxSCI_EOL_CR: {text = _T("CR"); break; }
            case wxSCI_EOL_CRLF: {text = _T("CR+LF"); break; }
            case wxSCI_EOL_LF: {text = _T("LF"); break; }
            default: {text = _T("???"); break; }
        }
        if (text != sb->GetStatusText (3)) SetStatusText (text, 3);
        switch (m_edit->GetCodePage()) {
            case wxSCI_CHARSET_ANSI: {text = _T("ANSI"); break;}
            case wxSCI_CHARSET_EASTEUROPE: {text = _("E-EUROPE"); break;}
            case wxSCI_CHARSET_GB2312: {text = _("GB2312"); break;}
            case wxSCI_CHARSET_HANGUL: {text = _("HANGUL"); break;}
            case wxSCI_CHARSET_HEBREW: {text = _("HEBREW");break;}
            case wxSCI_CHARSET_SHIFTJIS: {text = _("SHIFTJIS"); break;}
#if defined __WXMSW__
            case wxSCI_CHARSET_ARABIC: {text = _("ARABIC"); break;}
            case wxSCI_CHARSET_BALTIC: {text = _("BALTIC"); break;}
            case wxSCI_CHARSET_CHINESEBIG5: {text = _("CHINESE"); break;}
            case wxSCI_CHARSET_GREEK: {text = _("GREEK"); break;}
            case wxSCI_CHARSET_JOHAB: {text = _("JOHAB"); break;}
            case wxSCI_CHARSET_MAC: {text = _("MAC"); break;}
            case wxSCI_CHARSET_OEM: {text = _("OEM"); break;}
            case wxSCI_CHARSET_RUSSIAN: {text = _("RUSSIAN"); break;}
            case wxSCI_CHARSET_SYMBOL: {text = _("SYMBOL"); break;}
            case wxSCI_CHARSET_THAI: {text = _("THAI"); break;}
            case wxSCI_CHARSET_TURKISH: {text = _("TURKISH"); break;}
            case wxSCI_CHARSET_VIETNAMESE: {text = _("VIETNAME"); break;}
#endif
            default: {text = _T("DEFAULT"); break;}
        }
        if (text != sb->GetStatusText (4)) SetStatusText (text, 4);
        int curLine = m_edit->GetCurrentLine();
        int curPos = m_edit->GetCurrentPos() -
                     m_edit->PositionFromLine (-curLine);
        text.Printf (_("Line: %d, Col: %d"), curLine+1, curPos+1);
        if (text != sb->GetStatusText (5)) SetStatusText (text, 5);
    }
}

void AppFrame::UpdateTitle () {
    wxString title = g_appname;
    if (m_edit) {
        wxString fname = m_edit->GetFilename();
        wxString mode = m_edit->Modified()? _T(" *"): wxEmptyString;
        if (m_edit->GetReadOnly()) {
           title += _T("   > ") + fname + mode + _T(" <");
        }else{
           title += _T("   [ ") + fname + mode + _T(" ]");
        }
    }
    if (title != GetTitle()) SetTitle (title);
}


//----------------------------------------------------------------------------
// AppBook
//----------------------------------------------------------------------------

BEGIN_EVENT_TABLE (AppBook, wxNotebook)
    EVT_RIGHT_UP (        AppBook::OnTabActivate)
END_EVENT_TABLE()

AppBook::AppBook (wxWindow *parent,
                  wxWindowID id,
                  const wxPoint& pos,
                  const wxSize& size,
                  long style)
       : wxNotebook (parent, id, pos, size, style) {

    // initialize frame pointer
    m_frame = (AppFrame*)wxTheApp->GetTopWindow();

    // create menu
    m_bookMenu1 = new wxMenu;
    m_bookMenu2 = new wxMenu;
// wxNotebook::HitTest might not be supported
#if NOTEBOOK_HITTEST
    CreateBookMenus ();
#endif

}

AppBook::~AppBook () {
    delete m_bookMenu1;
    delete m_bookMenu2;
}

//----------------------------------------------------------------------------
// event handlers
void AppBook::OnTabActivate (wxMouseEvent &event) {
    wxPoint pt;
    pt.x = event.GetX();
    pt.y = event.GetY();
// wxNotebook::HitTest might not be supported
#if NOTEBOOK_HITTEST
    long flags = 0;
    int pageNr = HitTest (pt, &flags);
    if (pageNr < 0) return;
    m_frame->m_hitNr = pageNr;
    if (pageNr == m_frame->m_pageNr) {
        PopupMenu (m_bookMenu1, pt);
    }else{
        PopupMenu (m_bookMenu2, pt);
    }
#endif
}

// private functions
void AppBook::CreateBookMenus () {

    wxMenuBar *menuBar = m_frame->GetMenuBar();
    wxString label;
    int tabPos;

    // create menu 1
    if (menuBar->FindItem (wxID_SAVE)) {
        label = menuBar->GetLabel (wxID_SAVE);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_bookMenu1->Append (wxID_SAVE, label);
    }
    if (menuBar->FindItem (wxID_SAVEAS)) {
        label = menuBar->GetLabel (wxID_SAVEAS);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_bookMenu1->Append (wxID_SAVEAS, label);
    }
    if (menuBar->FindItem (wxID_CLOSE)) {
        label = menuBar->GetLabel (wxID_CLOSE);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_bookMenu1->Append (wxID_CLOSE, label);
    }
    m_bookMenu1->AppendSeparator();
    if (menuBar->FindItem (wxID_PRINT)) {
        label = menuBar->GetLabel (wxID_PRINT);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_bookMenu1->Append (wxID_PRINT, label);
    }

    // create menu 2
    m_bookMenu2->Append (myID_PAGEACTIVE, _("Activate"));
    m_bookMenu2->Append (myID_PAGECLOSE, _("Close"));

}


//----------------------------------------------------------------------------
// AllWindows
//----------------------------------------------------------------------------

BEGIN_EVENT_TABLE (AllWindows, wxDialog)
    EVT_BUTTON (myID_PAGEACTIVE, AllWindows::OnSetActive)
    EVT_BUTTON (wxID_CANCEL,     AllWindows::OnCancel)
    EVT_BUTTON (wxID_OK,         AllWindows::OnOkay)
END_EVENT_TABLE()

AllWindows::AllWindows (AppFrame *frame)
          : wxDialog (frame, -1, wxEmptyString,
                      wxDefaultPosition, wxDefaultSize,
                      wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER) {

    // sets the application title
    SetTitle (_("All windows"));
    m_frame = frame;

    // listbox
    m_windows = new wxListBox (this, -1, wxDefaultPosition, wxSize(320, 240),
                               0, NULL, wxLB_HSCROLL);
    m_windows->Set (*m_frame->m_filelist);

    // buttons
    wxBoxSizer *buttonpane = new wxBoxSizer (wxVERTICAL);
    wxButton *okButton = new wxButton (this, wxID_OK, _("&OK"));
    okButton->SetDefault();
    buttonpane->Add (okButton, 0, wxALIGN_TOP);
    buttonpane->Add (0, 6);
    wxButton *actiateButton = new wxButton (this, myID_PAGEACTIVE, _("&Activate"));
    buttonpane->Add (actiateButton, 0, wxALIGN_TOP);
    buttonpane->Add (0, 6);
    wxButton *cancelButton = new wxButton (this, wxID_CANCEL, _("&Cancel"));
    buttonpane->Add (cancelButton, 0, wxALIGN_BOTTOM);

    wxBoxSizer *selectpane = new wxBoxSizer (wxHORIZONTAL);
    selectpane->Add (m_windows, 1, wxEXPAND);
    selectpane->Add (20, 0);
    selectpane->Add (buttonpane, 0, wxEXPAND);

    wxBoxSizer *totalpane = new wxBoxSizer (wxVERTICAL);
    totalpane->Add (new wxStaticText(this, -1, _("Select window:")),
                    0, wxALIGN_LEFT | wxTOP | wxLEFT | wxRIGHT, 10);
    totalpane->Add (selectpane, 1, wxEXPAND | wxALL, 10);

    SetSizerAndFit (totalpane);

    ShowModal();
}

//----------------------------------------------------------------------------
// event handlers

void AllWindows::OnSetActive (wxCommandEvent &WXUNUSED(event)) {
    int pageNr = m_windows->GetSelection();
    if (pageNr != wxNOT_FOUND) {
        m_frame->m_book->Freeze();
        m_frame->m_SetSelection = true;
        m_frame->m_book->SetSelection (pageNr);
        m_frame->m_SetSelection = false;
        m_frame->PageHasChanged ();
        m_frame->m_book->Thaw();
    }
}

void AllWindows::OnCancel (wxCommandEvent &WXUNUSED(event)) {
    EndModal (wxID_CANCEL);
}

void AllWindows::OnOkay (wxCommandEvent &WXUNUSED(event)) {
    int pageNr = m_windows->GetSelection();
    if (pageNr != wxNOT_FOUND) {
        m_frame->m_book->Freeze();
        m_frame->m_SetSelection = true;
        m_frame->m_book->SetSelection (pageNr);
        m_frame->m_SetSelection = false;
        m_frame->PageHasChanged ();
        m_frame->m_book->Thaw();
    }
    EndModal (wxID_OK);
}


//----------------------------------------------------------------------------
// DropTargets
//----------------------------------------------------------------------------

DropTargets::DropTargets (AppFrame *frame)
           : wxDropTarget () {

    m_frame = frame;

    // create targets
    m_file = new wxFileDataObject();
    m_text = new wxTextDataObject();

    // set composite target
    DropTargetsComposite *data = new DropTargetsComposite();
    data->Add ((wxDataObjectSimple*)m_file);
    data->Add ((wxDataObjectSimple*)m_text, true); // set as preferred
    SetDataObject (data);

}

//----------------------------------------------------------------------------
// drop handlers

wxDragResult DropTargets::OnData (wxCoord x, wxCoord y, wxDragResult def) {

    if (!GetData()) return wxDragNone;

    // get object
    wxDataObjectSimple *obj = ((DropTargetsComposite *)GetDataObject())->GetLastDataObject();
    if (obj == ((wxDataObjectSimple*)m_file)) {
        if (!OnDataFiles (x, y, m_file->GetFilenames())) def = wxDragNone;
    }else if (obj == ((wxDataObjectSimple*)m_text)) {
        if (!OnDataText (x, y, m_text->GetText())) def = wxDragNone;
    }else{
        def = wxDragNone;
    }

    return def;
}

bool DropTargets::OnDataFiles (wxCoord WXUNUSED(x), wxCoord WXUNUSED(y),
                               const wxArrayString& filenames) {
    for (size_t n = 0; n < filenames.Count(); n++) {
        if (wxFileName(filenames[n]).IsDir()) {
            wxMessageBox (_("It is currently not allowed to drop \ndirectories onto the editor!"),
                          _("Error"), wxOK | wxICON_INFORMATION);
            return FALSE;
        }
    }
    m_frame->FileOpen (filenames);
    wxActivateEvent evt (wxEVT_ACTIVATE);
    wxPostEvent (m_frame, evt);
    return TRUE;
}

bool DropTargets::OnDataText (wxCoord x, wxCoord y, const wxString& data) {
    bool ok = m_frame->m_edit->DoDropText (x, y, data);
    if (ok) {
        wxActivateEvent evt (wxEVT_ACTIVATE);
        wxPostEvent (m_frame, evt);
    }
    return ok;
}

wxDragResult DropTargets::OnDragOver (wxCoord x, wxCoord y, wxDragResult def) {
    return m_frame->m_edit->DoDragOver (x, y, def);
}

wxDragResult DropTargets::OnEnter (wxCoord x, wxCoord y, wxDragResult def) {
    return m_frame->m_edit->DoDragEnter (x, y, def);
}

void DropTargets::OnLeave() {
    m_frame->m_edit->DoDragLeave();
}
