//////////////////////////////////////////////////////////////////////////////
// File:        charpick.cpp
// Purpose:     Character picker dialog
// Maintainer:  Otto Wyss
// Created:     2003-08-14
// RCS-ID:      $Id: charpick.cpp,v 1.1 2005/03/17 20:03:41 wyo Exp $
// Copyright:   (c) 2005 wyoGuide
// Licence:     wxWindows
//////////////////////////////////////////////////////////////////////////////

//----------------------------------------------------------------------------
// information
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
// headers
//----------------------------------------------------------------------------

// For compilers that support precompilation, includes <wx/wx.h>.
#include <wx/wxprec.h>

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all 'standard' wxWidgets headers)
#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

// wxWidgets headers
#include <wx/accel.h>    // accelerator support

#include "defsext.h"     // Additional definitions
#include "charpick.h"    // CharPicker dialog
#include "charsets.h"    // Character sets


//----------------------------------------------------------------------------
// resources
//----------------------------------------------------------------------------


//============================================================================
// declarations
//============================================================================
const int COLS = 16;


//============================================================================
// implementation
//============================================================================

//----------------------------------------------------------------------------
// myCharPickerDlg
//----------------------------------------------------------------------------

BEGIN_EVENT_TABLE (myCharPickerDlg, wxDialog)
    EVT_BUTTON (wxID_CANCEL,     myCharPickerDlg::OnCancel)
    EVT_BUTTON (wxID_OK,         myCharPickerDlg::OnOkay)
    EVT_MENU   (wxID_HELP,       myCharPickerDlg::OnHelp)
    EVT_GRID_CELL_LEFT_DCLICK (  myCharPickerDlg::OnCellSelected)
END_EVENT_TABLE()

myCharPickerDlg::myCharPickerDlg (wxWindow *parent,
                                  wxString *chrs)
               : wxDialog (parent, -1, _("CharPicker"),
                           wxDefaultPosition, wxDefaultSize) {

    //accelerators (for help)
    const int nEntries = 1 ;
    wxAcceleratorEntry entries[nEntries];
    entries[0].Set (wxACCEL_NORMAL, WXK_F1, wxID_HELP);
    wxAcceleratorTable accel (nEntries, entries);
    SetAcceleratorTable (accel);

    // charpane
    m_charpane = new wxGrid (this, -1, wxDefaultPosition, wxSize(560, -1));
    m_charpane->CreateGrid (WXSIZEOF(tcharcodelist)/COLS, COLS);
    m_charpane->SetRowLabelSize (0);
    m_charpane->SetColLabelSize (0);
    m_charpane->SetDefaultColSize (45);
    m_charpane->SetEditable (FALSE);
    m_charpane->EnableDragGridSize (false);
    m_charpane->SetDefaultCellAlignment (wxALIGN_CENTRE, wxALIGN_CENTRE);
    m_charpane->EnableGridLines (true);

    int i, j;
    for (i = 0; i < WXSIZEOF(tcharcodelist)/COLS; ++i) {
        for (j = 0; j < COLS; ++j) {
            if (tcharcodelist[(i*COLS)+j].isinsert) {
                m_charpane->SetCellValue (i, j, tcharcodelist[(i*COLS)+j].pschar);
            }else{
                m_charpane->SetCellValue (i, j, wxEmptyString);
                m_charpane->SetReadOnly (i, j);
            }
        }
    }
    m_charpane->AutoSize();

    // buttons
    wxBoxSizer *buttonpane = new wxBoxSizer (wxHORIZONTAL);
    wxButton *pickButton = new wxButton (this, wxID_OK, _("&Pick"));
    pickButton->SetDefault();
    buttonpane->Add (pickButton, 0, wxALIGN_CENTER);
    buttonpane->Add (6, 0);
    wxButton *cancelButton = new wxButton (this, wxID_CANCEL, _("Cancel"));
    buttonpane->Add (cancelButton, 0, wxALIGN_CENTER);

    wxBoxSizer *totalpane = new wxBoxSizer (wxVERTICAL);
    totalpane->Add (m_charpane, 0, wxALIGN_CENTER | wxALL, 10);
    totalpane->Add (buttonpane, 0, wxALIGN_CENTER | wxALL, 10);

    SetSizerAndFit (totalpane);

    ShowModal();
    *chrs = m_chrs;
}

myCharPickerDlg::~myCharPickerDlg () {
    delete m_charpane;
}

//----------------------------------------------------------------------------
// event handlers

void myCharPickerDlg::OnCancel (wxCommandEvent &WXUNUSED(event)) {
    EndModal (wxID_CANCEL);
}

void myCharPickerDlg::OnCellSelected (wxGridEvent& event) {
    int row = event.GetRow();
    int col = event.GetCol();
    if ((row*COLS)+col != 0) { // 0,0 not allowed
        m_chrs = wxChar ((row*COLS)+col);
        EndModal (wxID_OK);
    }
}

void myCharPickerDlg::OnHelp (wxCommandEvent &WXUNUSED(event)) {
    g_help->Display(_T("charpick.html#CharPicker"));
}

void myCharPickerDlg::OnOkay (wxCommandEvent &WXUNUSED(event)) {
    int row = m_charpane->GetCursorRow();
    int col = m_charpane->GetCursorColumn();
    if ((row*COLS)+col != 0) { // 0,0 not allowed
        m_chrs = wxChar ((row*COLS)+col);
    }
    EndModal (wxID_OK);
}
