//////////////////////////////////////////////////////////////////////////////
// File:        charsets.h
// Purpose:     Character set tables
// Maintainer:  Otto Wyss
// Created:     2003-08-20
// RCS-ID:      $Id: charsets.h,v 1.1 2005/03/17 20:03:41 wyo Exp $
// Copyright:   (c) 2005 wyoGuide
// Licence:     wxWindows
//////////////////////////////////////////////////////////////////////////////

//============================================================================
// declarations
//============================================================================

enum usedColor {
  RED,
  BLACK,
  BLUE
};

static struct {
  int    iascii;
  int  isinsert;
  int   icolour;
  wxChar *pschar;
  wxChar *psdescr;
} tcharcodelist[]={
   {  0, 0, RED, _T("NUL"), _T("Null (CTRL-@)")}
  ,{  1, 1, RED, _T("SOH"), _T("Start of heading (CTRL-A)")}
  ,{  2, 1, RED, _T("STX"), _T("Start of text (CTRL-B)")}
  ,{  3, 1, RED, _T("ETX"), _T("End of text (CTRL-C)")}
  ,{  4, 1, RED, _T("EOT"), _T("End of transmission (CTRL-D)")}
  ,{  5, 1, RED, _T("ENQ"), _T("Enquiry (CTRL-E)")}
  ,{  6, 1, RED, _T("ACK"), _T("Acknowledge (CTRL-F)")}
  ,{  7, 1, RED, _T("BEL"), _T("Bell (CTRL-G)")}
  ,{  8, 1, RED, _T("BS"), _T("Backspace (CTRL-H)")}
  ,{  9, 1, RED, _T("HT"), _T("Horizontal tab (CTRL-I)")}
  ,{ 10, 0, RED, _T("LF"), _T("Linefeed (CTRL-J)")}
  ,{ 11, 1, RED, _T("VT"), _T("Vertical tab (CTRL-K)")}
  ,{ 12, 1, RED, _T("FF"), _T("Form feed (CTRL-L)")}
  ,{ 13, 0, RED, _T("CR"), _T("Carriage return (CTRL-M)")}
  ,{ 14, 1, RED, _T("SO"), _T("Shift out (CTRL-N)")}
  ,{ 15, 1, RED, _T("SI"), _T("Shift in (CTRL-O)")}
  ,{ 16, 1, RED, _T("DLE"), _T("Data link escape (CTRL-P)")}
  ,{ 17, 1, RED, _T("DC1"), _T("Device Control, X-ON (CTRL-Q)")}
  ,{ 18, 1, RED, _T("DC2"), _T("Device Control (CTRL-R)")}
  ,{ 19, 1, RED, _T("DC3"), _T("Device Control, X-OFF (CTRL-S)")}
  ,{ 20, 1, RED, _T("DC4"), _T("Device Control (CTRL-T)")}
  ,{ 21, 1, RED, _T("NAK"), _T("Negative Acknowledge (CTRL-U)")}
  ,{ 22, 1, RED, _T("SYN"), _T("Synchronous idle (CTRL-V)")}
  ,{ 23, 1, RED, _T("ETB"), _T("End of transmission block (CTRL-W)")}
  ,{ 24, 1, RED, _T("CAN"), _T("Cancel (CTRL-X)")}
  ,{ 25, 1, RED, _T("EM"), _T("End of medium (CTRL-Y)")}
  ,{ 26, 1, RED, _T("SUB"), _T("Substitute (CTRL-Z)")}
  ,{ 27, 1, RED, _T("ESC"), _T("Escape (CTRL-[)")}
  ,{ 28, 1, RED, _T("FS"), _T("File separator (CTRL- )")}
  ,{ 29, 1, RED, _T("GS"), _T("Group separator (CTRL-])")}
  ,{ 30, 1, RED, _T("RS"), _T("Record separator (CTRL-^)")}
  ,{ 31, 1, RED, _T("US"), _T("Unit separator (CTRL-_)")}
  ,{ 32, 1, RED, _T("SPC"), _T("Space")}
  ,{ 33, 1, BLACK, _T("!"), _T("Exclamation mark")}
  ,{ 34, 1, BLACK, _T("\""), _T("Double quote")}
  ,{ 35, 1, BLACK, _T("#"), _T("Hash")}
  ,{ 36, 1, BLACK, _T("$"), _T("Dollar")}
  ,{ 37, 1, BLACK, _T("%"), _T("Percent")}
  ,{ 38, 1, BLACK, _T("&"), _T("Ampersand")}
  ,{ 39, 1, BLACK, _T("'"), _T("Apostrophe")}
  ,{ 40, 1, BLACK, _T("("), _T("Open Parenthesis")}
  ,{ 41, 1, BLACK, _T(")"), _T("Close Parenthesis")}
  ,{ 42, 1, BLACK, _T("*"), _T("Asterisk")}
  ,{ 43, 1, BLACK, _T("+"), _T("Plus")}
  ,{ 44, 1, BLACK, _T(","), _T("Comma")}
  ,{ 45, 1, BLACK, _T("-"), _T("Minus (hyphen)")}
  ,{ 46, 1, BLACK, _T("."), _T("Dot (period)")}
  ,{ 47, 1, BLACK, _T("/"), _T("Slash (forward-slash)")}
  ,{ 48, 1, BLACK, _T("0"), _T("Digit zero")}
  ,{ 49, 1, BLACK, _T("1"), _T("Digit one")}
  ,{ 50, 1, BLACK, _T("2"), _T("Digit two")}
  ,{ 51, 1, BLACK, _T("3"), _T("Digit three")}
  ,{ 52, 1, BLACK, _T("4"), _T("Digit four")}
  ,{ 53, 1, BLACK, _T("5"), _T("Digit five")}
  ,{ 54, 1, BLACK, _T("6"), _T("Digit six")}
  ,{ 55, 1, BLACK, _T("7"), _T("Digit seven")}
  ,{ 56, 1, BLACK, _T("8"), _T("Digit eight")}
  ,{ 57, 1, BLACK, _T("9"), _T("Digit nine")}
  ,{ 58, 1, BLACK, _T(":"), _T("Colon")}
  ,{ 59, 1, BLACK, _T(";"), _T("Semicolon")}
  ,{ 60, 1, BLACK, _T("<"), _T("Less than")}
  ,{ 61, 1, BLACK, _T("="), _T("Equals")}
  ,{ 62, 1, BLACK, _T(">"), _T("Greater than")}
  ,{ 63, 1, BLACK, _T("?"), _T("Question mark")}
  ,{ 64, 1, BLACK, _T("@"), _T("At")}
  ,{ 65, 1, BLACK, _T("A"), _T("Upper case alphabetic character A")}
  ,{ 66, 1, BLACK, _T("B"), _T("Upper case alphabetic character B")}
  ,{ 67, 1, BLACK, _T("C"), _T("Upper case alphabetic character C")}
  ,{ 68, 1, BLACK, _T("D"), _T("Upper case alphabetic character D")}
  ,{ 69, 1, BLACK, _T("E"), _T("Upper case alphabetic character E")}
  ,{ 70, 1, BLACK, _T("F"), _T("Upper case alphabetic character F")}
  ,{ 71, 1, BLACK, _T("G"), _T("Upper case alphabetic character G")}
  ,{ 72, 1, BLACK, _T("H"), _T("Upper case alphabetic character H")}
  ,{ 73, 1, BLACK, _T("I"), _T("Upper case alphabetic character I")}
  ,{ 74, 1, BLACK, _T("J"), _T("Upper case alphabetic character J")}
  ,{ 75, 1, BLACK, _T("K"), _T("Upper case alphabetic character K")}
  ,{ 76, 1, BLACK, _T("L"), _T("Upper case alphabetic character L")}
  ,{ 77, 1, BLACK, _T("M"), _T("Upper case alphabetic character M")}
  ,{ 78, 1, BLACK, _T("N"), _T("Upper case alphabetic character N")}
  ,{ 79, 1, BLACK, _T("O"), _T("Upper case alphabetic character O")}
  ,{ 80, 1, BLACK, _T("P"), _T("Upper case alphabetic character P")}
  ,{ 81, 1, BLACK, _T("Q"), _T("Upper case alphabetic character Q")}
  ,{ 82, 1, BLACK, _T("R"), _T("Upper case alphabetic character R")}
  ,{ 83, 1, BLACK, _T("S"), _T("Upper case alphabetic character S")}
  ,{ 84, 1, BLACK, _T("T"), _T("Upper case alphabetic character T")}
  ,{ 85, 1, BLACK, _T("U"), _T("Upper case alphabetic character U")}
  ,{ 86, 1, BLACK, _T("V"), _T("Upper case alphabetic character V")}
  ,{ 87, 1, BLACK, _T("W"), _T("Upper case alphabetic character W")}
  ,{ 88, 1, BLACK, _T("X"), _T("Upper case alphabetic character X")}
  ,{ 89, 1, BLACK, _T("Y"), _T("Upper case alphabetic character Y")}
  ,{ 90, 1, BLACK, _T("Z"), _T("Upper case alphabetic character Z")}
  ,{ 91, 1, BLACK, _T("["), _T("Open square bracket")}
  ,{ 92, 1, BLACK, _T("\\"), _T("Backslash")}
  ,{ 93, 1, BLACK, _T("]"), _T("Close square bracket")}
  ,{ 94, 1, BLACK, _T("^"), _T("Caret")}
  ,{ 95, 1, BLACK, _T("_"), _T("Underscore")}
  ,{ 96, 1, BLACK, _T("`"), _T("Back quote")}
  ,{ 97, 1, BLACK, _T("a"), _T("Lower case alphabetic character a")}
  ,{ 98, 1, BLACK, _T("b"), _T("Lower case alphabetic character b")}
  ,{ 99, 1, BLACK, _T("c"), _T("Lower case alphabetic character c")}
  ,{100, 1, BLACK, _T("d"), _T("Lower case alphabetic character d")}
  ,{101, 1, BLACK, _T("e"), _T("Lower case alphabetic character e")}
  ,{102, 1, BLACK, _T("f"), _T("Lower case alphabetic character f")}
  ,{103, 1, BLACK, _T("g"), _T("Lower case alphabetic character g")}
  ,{104, 1, BLACK, _T("h"), _T("Lower case alphabetic character h")}
  ,{105, 1, BLACK, _T("i"), _T("Lower case alphabetic character i")}
  ,{106, 1, BLACK, _T("j"), _T("Lower case alphabetic character j")}
  ,{107, 1, BLACK, _T("k"), _T("Lower case alphabetic character k")}
  ,{108, 1, BLACK, _T("l"), _T("Lower case alphabetic character l")}
  ,{109, 1, BLACK, _T("m"), _T("Lower case alphabetic character m")}
  ,{110, 1, BLACK, _T("n"), _T("Lower case alphabetic character n")}
  ,{111, 1, BLACK, _T("o"), _T("Lower case alphabetic character o")}
  ,{112, 1, BLACK, _T("p"), _T("Lower case alphabetic character p")}
  ,{113, 1, BLACK, _T("q"), _T("Lower case alphabetic character q")}
  ,{114, 1, BLACK, _T("r"), _T("Lower case alphabetic character r")}
  ,{115, 1, BLACK, _T("s"), _T("Lower case alphabetic character s")}
  ,{116, 1, BLACK, _T("t"), _T("Lower case alphabetic character t")}
  ,{117, 1, BLACK, _T("u"), _T("Lower case alphabetic character u")}
  ,{118, 1, BLACK, _T("v"), _T("Lower case alphabetic character v")}
  ,{119, 1, BLACK, _T("w"), _T("Lower case alphabetic character w")}
  ,{120, 1, BLACK, _T("x"), _T("Lower case alphabetic character x")}
  ,{121, 1, BLACK, _T("y"), _T("Lower case alphabetic character y")}
  ,{122, 1, BLACK, _T("z"), _T("Lower case alphabetic character z")}
  ,{123, 1, BLACK, _T("{"), _T("Open brace")}
  ,{124, 1, BLACK, _T("|"), _T("Vertical bar")}
  ,{125, 1, BLACK, _T(")}"), _T("Close brace")}
  ,{126, 1, BLACK, _T("~"), _T("Tilde")}
  ,{127, 1, RED, _T("DEL"), _T("Delete")}
  ,{128, 1, RED, _T("XXX"), _T("<control>")}
  ,{129, 1, RED, _T("XXX"), _T("<control>")}
  ,{130, 1, RED, _T("BPH"), _T("<control> = break permitted here")}
  ,{131, 1, RED, _T("NBH"), _T("<control> = no break here")}
  ,{132, 1, RED, _T("XXX"), _T("<control>")}
  ,{133, 1, RED, _T("NEL"), _T("<control> = next line")}
  ,{134, 1, RED, _T("SSA"), _T("<control> = start of a selected area")}
  ,{135, 1, RED, _T("ESA"), _T("<control> = end of selected area")}
  ,{136, 1, RED, _T("HTS"), _T("<control> = character tabulation set")}
  ,{137, 1, RED, _T("HTJ"), _T("<control> = character tabulation with justification")}
  ,{138, 1, RED, _T("VTS"), _T("<control> = line tabulation set")}
  ,{139, 1, RED, _T("PLD"), _T("<control> = partial line down")}
  ,{140, 1, RED, _T("PLU"), _T("<control> = partial line up")}
  ,{141, 1, RED, _T("RI "), _T("<control> = reverse line feed")}
  ,{142, 1, RED, _T("SS2"), _T("<control> = single shift two")}
  ,{143, 1, RED, _T("SS3"), _T("<control> = single shift three")}
  ,{144, 1, RED, _T("DCS"), _T("<control> = device control string")}
  ,{145, 1, RED, _T("PU1"), _T("<control> = private use one")}
  ,{146, 1, RED, _T("PU2"), _T("<control> = private use two")}
  ,{147, 1, RED, _T("STS"), _T("<control> = set transmit state")}
  ,{148, 1, RED, _T("CCH"), _T("<control> = cancel character")}
  ,{149, 1, RED, _T("MW "), _T("<control> = message waiting")}
  ,{150, 1, RED, _T("SPA"), _T("<control> = start of guarded area")}
  ,{151, 1, RED, _T("EPA"), _T("<control> = end of gaurded area")}
  ,{152, 1, RED, _T("SOS"), _T("<control> = start of string")}
  ,{153, 1, RED, _T("XXX"), _T("<control>")}
  ,{154, 1, RED, _T("SCI"), _T("<control> = single character introducer")}
  ,{155, 1, RED, _T("CSI"), _T("<control> = control sequence intoducer")}
  ,{156, 1, RED, _T("ST "), _T("<control> = string terminator")}
  ,{157, 1, RED, _T("OSC"), _T("<control> = operating system command")}
  ,{158, 1, RED, _T("PM "), _T("<control> = privacy message")}
  ,{159, 1, RED, _T("APC"), _T("<control> = application program command")}
  ,{160, 1, BLUE, _T("NBSP"), _T("No break space")}
  ,{161, 1, BLUE, _T(""), _T("Inverted exclamation mark")}
  ,{162, 1, BLUE, _T(""), _T("Cent sign")}
  ,{163, 1, BLUE, _T(""), _T("Pound sign")}
  ,{164, 1, BLUE, _T(""), _T("Currency sign")}
  ,{165, 1, BLUE, _T(""), _T("Yen sign")}
  ,{166, 1, BLUE, _T(""), _T("Broken bar")}
  ,{167, 1, BLUE, _T(""), _T("Section sign")}
  ,{168, 1, BLUE, _T(""), _T("Diaeresis")}
  ,{169, 1, BLUE, _T(""), _T("Copyright sign")}
  ,{170, 1, BLUE, _T(""), _T("Feminine ordinal indicator")}
  ,{171, 1, BLUE, _T(""), _T("Left-pointing double angle quatation mark")}
  ,{172, 1, BLUE, _T(""), _T("Not sign")}
  ,{173, 1, BLUE, _T(""), _T("Soft hyphen")}
  ,{174, 1, BLUE, _T(""), _T("Registered sign")}
  ,{175, 1, BLUE, _T(""), _T("Macron")}
  ,{176, 1, BLUE, _T(""), _T("Degree sign")}
  ,{177, 1, BLUE, _T(""), _T("Plus-minus  sign")}
  ,{178, 1, BLUE, _T(""), _T("Superscript two")}
  ,{179, 1, BLUE, _T(""), _T("Superscript three")}
  ,{180, 1, BLUE, _T(""), _T("Acute accent")}
  ,{181, 1, BLUE, _T(""), _T("Micro sign")}
  ,{182, 1, BLUE, _T(""), _T("Pilcrow sign")}
  ,{183, 1, BLUE, _T(""), _T("Middle dot")}
  ,{184, 1, BLUE, _T(""), _T("Cedilla")}
  ,{185, 1, BLUE, _T(""), _T("Superscript one")}
  ,{186, 1, BLUE, _T(""), _T("Masculine ordinal indicator")}
  ,{187, 1, BLUE, _T(""), _T("Right-pointing double angle quotation mark")}
  ,{188, 1, BLUE, _T(""), _T("Vulgar fraction quarter")}
  ,{189, 1, BLUE, _T(""), _T("Vulgar fraction one half")}
  ,{190, 1, BLUE, _T(""), _T("Vulgar fraction three quarters")}
  ,{191, 1, BLUE, _T(""), _T("Inverted question mark")}
  ,{192, 1, BLUE, _T(""), _T("Latin capital letter A with grave")}
  ,{193, 1, BLUE, _T(""), _T("Latin capital letter A with acute")}
  ,{194, 1, BLUE, _T(""), _T("Latin capital letter A with circumflex")}
  ,{195, 1, BLUE, _T(""), _T("Latin capitla letter A with tilde")}
  ,{196, 1, BLUE, _T(""), _T("Latin capital letter A with diaeresis")}
  ,{197, 1, BLUE, _T(""), _T("Latin capital letter A with ring above")}
  ,{198, 1, BLUE, _T(""), _T("Latin capital letter AE = latin capital ligature AE")}
  ,{199, 1, BLUE, _T(""), _T("Latin capital letter C with cedilla")}
  ,{200, 1, BLUE, _T(""), _T("Latin capital letter E with grave")}
  ,{201, 1, BLUE, _T(""), _T("Latin capital letter E with acute")}
  ,{202, 1, BLUE, _T(""), _T("Latin capital letter E with circumflex")}
  ,{203, 1, BLUE, _T(""), _T("Latin capital letter E with diaeresis")}
  ,{204, 1, BLUE, _T(""), _T("Latin capital letter I with grave")}
  ,{205, 1, BLUE, _T(""), _T("Latin capital letter I with acute")}
  ,{206, 1, BLUE, _T(""), _T("Latin capital letter I with circumflex")}
  ,{207, 1, BLUE, _T(""), _T("Latin capital letter I with diaeresis")}
  ,{208, 1, BLUE, _T(""), _T("Latin capital letter I with acute")}
  ,{209, 1, BLUE, _T(""), _T("Latin capital letter N with tilde")}
  ,{210, 1, BLUE, _T(""), _T("Latin capital letter O with grave")}
  ,{211, 1, BLUE, _T(""), _T("Latin capital letter O with acute")}
  ,{212, 1, BLUE, _T(""), _T("Latin capital letter O with circumflex")}
  ,{213, 1, BLUE, _T(""), _T("Latin capital letter O with tilde")}
  ,{214, 1, BLUE, _T(""), _T("Latin capital letter O with diaeresis")}
  ,{215, 1, BLUE, _T(""), _T("Multiplication sign")}
  ,{216, 1, BLUE, _T(""), _T("Latin capital letter O with stroke")}
  ,{217, 1, BLUE, _T(""), _T("Latin capital letter U with grave")}
  ,{218, 1, BLUE, _T(""), _T("Latin capital letter U with acute")}
  ,{219, 1, BLUE, _T(""), _T("Latin capital letter U with circumflex")}
  ,{220, 1, BLUE, _T(""), _T("Latin capital letter U with diaeresis")}
  ,{221, 1, BLUE, _T(""), _T("Latin capital letter Y with acute")}
  ,{222, 1, BLUE, _T(""), _T("Latin capital letter thorn")}
  ,{223, 1, BLUE, _T(""), _T("Latin small letter sharp")}
  ,{224, 1, BLUE, _T(""), _T("Latin small letter A with grave")}
  ,{225, 1, BLUE, _T(""), _T("Latin small letter A with acute")}
  ,{226, 1, BLUE, _T(""), _T("Latin small letter A with circumflex")}
  ,{227, 1, BLUE, _T(""), _T("Latin small letter A with a tilde")}
  ,{228, 1, BLUE, _T(""), _T("Latin small letter A diaeresis")}
  ,{229, 1, BLUE, _T(""), _T("Latin small letter A with a ring")}
  ,{230, 1, BLUE, _T(""), _T("Latin small letter AE = latin small ligature AE")}
  ,{231, 1, BLUE, _T(""), _T("Latin small letter C with cedilla")}
  ,{232, 1, BLUE, _T(""), _T("Latin small letter E with grave")}
  ,{233, 1, BLUE, _T(""), _T("Latin small letter E with acute")}
  ,{234, 1, BLUE, _T(""), _T("Latin small letter E with circumflex")}
  ,{235, 1, BLUE, _T(""), _T("Latin small letter E with diaeresis")}
  ,{236, 1, BLUE, _T(""), _T("Latin small letter I with grave")}
  ,{237, 1, BLUE, _T(""), _T("Latin small letter I with acute")}
  ,{238, 1, BLUE, _T(""), _T("Latin small letter I with circumflex")}
  ,{239, 1, BLUE, _T(""), _T("Latin small letter I with diaeresis")}
  ,{240, 1, BLUE, _T(""), _T("Latin small letter ETH")}
  ,{241, 1, BLUE, _T(""), _T("Latin small letter N with tilde")}
  ,{242, 1, BLUE, _T(""), _T("Latin small letter O with grave")}
  ,{243, 1, BLUE, _T(""), _T("Latin smal letter O with acute")}
  ,{244, 1, BLUE, _T(""), _T("Latin small letter O with circumflex")}
  ,{245, 1, BLUE, _T(""), _T("Latin small letter O with tilde")}
  ,{246, 1, BLUE, _T(""), _T("Latin small letter O with diaeresis")}
  ,{247, 1, BLUE, _T(""), _T("Division sign")}
  ,{248, 1, BLUE, _T(""), _T("Latin small letter O with stroke")}
  ,{249, 1, BLUE, _T(""), _T("Latin small letter U with grave")}
  ,{250, 1, BLUE, _T(""), _T("Latin small letter U with acute")}
  ,{251, 1, BLUE, _T(""), _T("Latin small letter U circumflex")}
  ,{252, 1, BLUE, _T(""), _T("Latin small letter U with diaeresis")}
  ,{253, 1, BLUE, _T(""), _T("Latin small letter Y with acute")}
  ,{254, 1, BLUE, _T(""), _T("Latin small letter thorn")}
  ,{255, 1, BLUE, _T(""), _T("Latin small letter Y with diaeresis")}
};

