//////////////////////////////////////////////////////////////////////////////
// File:        edit.cpp
// Purpose:     Edit main module
// Maintainer:  Otto Wyss
// Created:     2003-01-20
// RCS-ID:      $Id: edit.cpp,v 1.1 2005/03/17 20:03:43 wyo Exp $
// Copyright:   (c) 2005 wyoGuide
// Licence:     wxWindows
//////////////////////////////////////////////////////////////////////////////

//----------------------------------------------------------------------------
// information
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
// headers
//----------------------------------------------------------------------------

// For compilers that support precompilation, includes <wx/wx.h>.
#include <wx/wxprec.h>

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all 'standard' wxWidgets headers)
#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

//! wxWidgets headers
#include <wx/dir.h>      // directories support
#include <wx/listimpl.cpp> // list support???
#include <wx/filename.h> // filename support
#include <wx/textfile.h> // text file io support

//! application headers
#include "defsext.h"     // additional definitions

#include "dialogs.h"     // dialogs
#include "edit.h"        // edit module


//----------------------------------------------------------------------------
// resources
//----------------------------------------------------------------------------


//============================================================================
// declarations
//============================================================================

DEFINE_EVENT_TYPE (myEVT_COMMON_NOTIFY)
DEFINE_EVENT_TYPE (myEVT_COMMON_UPDATE)
DEFINE_EVENT_TYPE (myEVT_RESULT_FOUND)
DEFINE_EVENT_TYPE (myEVT_RESULT_SELECTED)

const int MARKER_BOOKMARK = 1;

WX_DEFINE_LIST (IdentTableList);

const int NOTIFY_INSERT = 1;
const int NOTIFY_DELETE = 2;
const int NOTIFY_UPDATE = 3;

const wxString LEVEL2PREFIX = _T(" - ");
const wxString HOMEPOSITION = _T("<homeposition>");


//============================================================================
// implementation
//============================================================================

//----------------------------------------------------------------------------
// EditText
//----------------------------------------------------------------------------

BEGIN_EVENT_TABLE (EditText, wxScintilla)
    // common
    myEVT_COMMON_NOTIFY (              EditText::OnCommonNotify)
    EVT_RIGHT_UP (                     EditText::OnContextMenu)
    EVT_ERASE_BACKGROUND (             EditText::OnEraseBackground)
    // edit
    EVT_MENU (wxID_CLEAR,              EditText::OnEditClear)
    EVT_UPDATE_UI (wxID_CLEAR,         EditText::OnEditClearUI)
    EVT_MENU (wxID_CUT,                EditText::OnEditCut)
    EVT_UPDATE_UI (wxID_CUT,           EditText::OnEditCutUI)
    EVT_MENU (wxID_COPY,               EditText::OnEditCopy)
    EVT_UPDATE_UI (wxID_COPY,          EditText::OnEditCopyUI)
    EVT_MENU (wxID_PASTE,              EditText::OnEditPaste)
    EVT_UPDATE_UI (wxID_PASTE,         EditText::OnEditPasteUI)
    EVT_MENU (myID_INDENTINC,          EditText::OnEditIndentInc)
    EVT_UPDATE_UI (myID_INDENTINC,     EditText::OnEditIndentIncUI)
    EVT_MENU (myID_INDENTRED,          EditText::OnEditIndentRed)
    EVT_UPDATE_UI (myID_INDENTRED,     EditText::OnEditIndentRedUI)
    EVT_MENU (wxID_SELECTALL,          EditText::OnEditSelectAll)
    EVT_UPDATE_UI (wxID_SELECTALL,     EditText::OnEditSelectAllUI)
    EVT_MENU (myID_SELECTLINE,         EditText::OnEditSelectLine)
    EVT_UPDATE_UI (myID_SELECTLINE,    EditText::OnEditSelectLineUI)
    EVT_MENU (wxID_REDO,               EditText::OnEditRedo)
    EVT_UPDATE_UI (wxID_REDO,          EditText::OnEditRedoUI)
    EVT_MENU (wxID_UNDO,               EditText::OnEditUndo)
    EVT_UPDATE_UI (wxID_UNDO,          EditText::OnEditUndoUI)
    // find
    EVT_MENU (wxID_FIND,               EditText::OnFind)
    EVT_UPDATE_UI (wxID_FIND,          EditText::OnFindUI)
    EVT_MENU (myID_FINDNEXT,           EditText::OnFindNext)
    EVT_UPDATE_UI (myID_FINDNEXT,      EditText::OnFindNextUI)
    EVT_MENU (myID_REPLACE,            EditText::OnReplace)
    EVT_UPDATE_UI (myID_REPLACE,       EditText::OnReplaceUI)
    EVT_MENU (myID_REPLACENEXT,        EditText::OnReplaceNext)
    EVT_UPDATE_UI (myID_REPLACENEXT,   EditText::OnReplaceNextUI)
    EVT_MENU (myID_BRACEMATCH,         EditText::OnBraceMatch)
    EVT_UPDATE_UI (myID_BRACEMATCH,    EditText::OnBraceMatchUI)
    EVT_MENU (myID_GOTO,               EditText::OnGoto)
    EVT_UPDATE_UI (myID_GOTO,          EditText::OnGotoUI)
    EVT_MENU (myID_BOOKMARKTOGGLE,     EditText::OnBookmarkToggle)
    EVT_UPDATE_UI (myID_BOOKMARKTOGGLE, EditText::OnBookmarkToggleUI)
    EVT_MENU (myID_BOOKMARKNEXT,       EditText::OnBookmarkNext)
    EVT_UPDATE_UI (myID_BOOKMARKNEXT,  EditText::OnBookmarkNextUI)
    EVT_MENU (myID_BOOKMARKPREV,       EditText::OnBookmarkPrev)
    EVT_UPDATE_UI (myID_BOOKMARKPREV,  EditText::OnBookmarkPrevUI)
    EVT_MENU (myID_BOOKMARKCLEAR,      EditText::OnBookmarkClear)
    EVT_UPDATE_UI (myID_BOOKMARKCLEAR, EditText::OnBookmarkClearUI)
    // view
    EVT_MENU_RANGE (myID_HILIGHTFIRST, myID_HILIGHTLAST,
                                       EditText::OnHilightLang)
    EVT_UPDATE_UI_RANGE (myID_HILIGHTFIRST, myID_HILIGHTLAST,
                                       EditText::OnHilightLangUI)
    EVT_MENU (myID_DISPLAYEOL,         EditText::OnDisplayEOL)
    EVT_UPDATE_UI (myID_DISPLAYEOL,    EditText::OnDisplayEOLUI)
    EVT_MENU (myID_DISPLAYFOLD,        EditText::OnFoldLevel)
    EVT_UPDATE_UI (myID_DISPLAYFOLD,   EditText::OnFoldLevelUI)
    EVT_MENU (myID_INDENTGUIDE,        EditText::OnIndentGuide)
    EVT_UPDATE_UI (myID_INDENTGUIDE,   EditText::OnIndentGuideUI)
    EVT_MENU (myID_LINENUMBER,         EditText::OnLineNumber)
    EVT_UPDATE_UI (myID_LINENUMBER,    EditText::OnLineNumberUI)
    EVT_MENU (myID_LONGLINEON,         EditText::OnLongLineOn)
    EVT_UPDATE_UI (myID_LONGLINEON,    EditText::OnLongLineOnUI)
    EVT_MENU (myID_WHITESPACE,         EditText::OnWhiteSpace)
    EVT_UPDATE_UI (myID_WHITESPACE,    EditText::OnWhiteSpaceUI)
    EVT_MENU (myID_FOLDTOGGLE,         EditText::OnFoldToggle)
    EVT_UPDATE_UI (myID_FOLDTOGGLE,    EditText::OnFoldToggleUI)
    EVT_MENU (myID_OVERTYPE,           EditText::OnSetOverType)
    EVT_UPDATE_UI (myID_OVERTYPE,      EditText::OnSetOverTypeUI)
    EVT_MENU (myID_READONLY,           EditText::OnSetReadOnly)
    EVT_UPDATE_UI (myID_READONLY,      EditText::OnSetReadOnlyUI)
    EVT_MENU (myID_WRAPMODEON,         EditText::OnWrapmodeOn)
    EVT_UPDATE_UI (myID_WRAPMODEON,    EditText::OnWrapmodeOnUI)
    EVT_UPDATE_UI (myID_USECHARSET,    EditText::OnUseCharsetUI)
    EVT_MENU_RANGE (myID_CHARSET_ANSI, myID_CHARSET_VIETNAMESE,
                                       EditText::OnUseCharset)
    EVT_UPDATE_UI_RANGE (myID_CHARSET_ANSI, myID_CHARSET_VIETNAMESE,
                                       EditText::OnUseCharsetUI)
    // extra
    EVT_UPDATE_UI (myID_CHANGECASE,    EditText::OnChangeCaseUI)
    EVT_MENU (myID_CHANGELOWER,        EditText::OnChangeCase)
    EVT_UPDATE_UI (myID_CHANGELOWER,   EditText::OnChangeCaseUI)
    EVT_MENU (myID_CHANGEUPPER,        EditText::OnChangeCase)
    EVT_UPDATE_UI (myID_CHANGEUPPER,   EditText::OnChangeCaseUI)
    EVT_UPDATE_UI (myID_WHITESPACETO,  EditText::OnWhitespaceToUI)
    EVT_MENU (myID_WHITETOSPACE,       EditText::OnTabsToSpaces)
    EVT_UPDATE_UI (myID_WHITETOSPACE,  EditText::OnWhitespaceToUI)
    EVT_MENU (myID_WHITETOTAB,         EditText::OnSpacesToTabs)
    EVT_UPDATE_UI (myID_WHITETOTAB,    EditText::OnWhitespaceToUI)
    EVT_UPDATE_UI (myID_CONVERTEOL,    EditText::OnConvertEOLUI)
    EVT_MENU (myID_CONVERTCR,          EditText::OnConvertEOL)
    EVT_UPDATE_UI (myID_CONVERTCR,     EditText::OnConvertEOLUI)
    EVT_MENU (myID_CONVERTCRLF,        EditText::OnConvertEOL)
    EVT_UPDATE_UI (myID_CONVERTCRLF,   EditText::OnConvertEOLUI)
    EVT_MENU (myID_CONVERTLF,          EditText::OnConvertEOL)
    EVT_UPDATE_UI (myID_CONVERTLF,     EditText::OnConvertEOLUI)
    EVT_MENU (myID_TRAILINGSPACE,      EditText::OnTrailingSpace)
    EVT_UPDATE_UI (myID_TRAILINGSPACE, EditText::OnTrailingSpaceUI)
    // scintilla
    EVT_SCI_DOUBLECLICK (-1,           EditText::OnResultSelected)
    EVT_SCI_MARGINCLICK (-1,           EditText::OnMarginClick)
    EVT_SCI_MODIFIED (-1,              EditText::OnModified)
    EVT_SCI_CHARADDED (-1,             EditText::OnCharAdded)
END_EVENT_TABLE()

EditText::EditText (wxWindow *parent, wxWindowID id,
                    int type,
                    int browseWidth,
                    const wxPoint &pos,
                    const wxSize &size,
                    long style)
        : wxScintilla (parent, id, pos, size, style|wxCLIP_CHILDREN) {

    m_type = type;
    m_browseWidth = browseWidth;
    m_textLoading = false;
    m_filename = _T("");
    m_filetime = wxDateTime((time_t)0);
    m_analyse = NULL;
    m_language = NULL;

    m_LineNrID = 0;
    m_LineNrMargin = TextWidth (wxSCI_STYLE_LINENUMBER, _T("_999999"));
    m_FoldingID = 1;
    m_FoldingMargin = 16;
    m_DividerID = 2;
    m_DividerMargin = 8;
    m_foldLevel = 0;

    // set menus
    m_ContextMenu = new wxMenu;
    m_menuChangeCase = new wxMenu;
    createContextMenu ();

    // create dialogs
    m_FindReplaceDlg = new myFindReplaceDlg (this);
    m_startpos = 0;
    m_replace = false;
    m_GotoDlg = new myGotoDlg (this);

    // default font for all styles
    SetViewEOL (g_CommonPrefs.displayEOLEnable);
    SetIndentationGuides (g_CommonPrefs.indentGuideEnable);
    SetMarginWidth (m_LineNrID,
                    g_CommonPrefs.lineNumberEnable? m_LineNrMargin: 0);
    SetEdgeMode (g_CommonPrefs.longLineOnEnable?
                 wxSCI_EDGE_LINE: wxSCI_EDGE_NONE);
    SetViewWhiteSpace (g_CommonPrefs.whiteSpaceEnable?
                       wxSCI_WS_VISIBLEALWAYS: wxSCI_WS_INVISIBLE);
    SetOvertype (g_CommonPrefs.overTypeInitial);
    SetReadOnly (g_CommonPrefs.readOnlyInitial);
    SetWrapMode (g_CommonPrefs.wrapModeInitial?
                 wxSCI_WRAP_WORD: wxSCI_WRAP_NONE);
    wxFont font (10, wxTELETYPE, wxNORMAL, wxNORMAL);
    StyleSetFont (wxSCI_STYLE_DEFAULT, font);
    StyleSetForeground (wxSCI_STYLE_DEFAULT, wxColour (_T("BLACK")));
    StyleSetBackground (wxSCI_STYLE_DEFAULT, wxColour (_T("WHITE")));
    StyleSetForeground (wxSCI_STYLE_LINENUMBER, wxColour (_T("DARK GREY")));
    StyleSetBackground (wxSCI_STYLE_LINENUMBER, wxColour (_T("WHITE")));
    StyleSetForeground(wxSCI_STYLE_INDENTGUIDE, wxColour (_T("DARK GREY")));

    // set visibility
    SetVisiblePolicy (wxSCI_VISIBLE_STRICT|wxSCI_VISIBLE_SLOP, 1);
    SetXCaretPolicy (wxSCI_CARET_EVEN|wxSCI_VISIBLE_STRICT|wxSCI_CARET_SLOP, 1);
    SetYCaretPolicy (wxSCI_CARET_EVEN|wxSCI_VISIBLE_STRICT|wxSCI_CARET_SLOP, 1);

    // markers
    MarkerDefine (MARKER_BOOKMARK, wxSCI_MARK_CIRCLE);
    MarkerDefine (wxSCI_MARKNUM_FOLDER,wxSCI_MARK_DOTDOTDOT);
    MarkerSetBackground (wxSCI_MARKNUM_FOLDER, wxColour (_T("BLACK")));
    MarkerDefine (wxSCI_MARKNUM_FOLDEROPEN,wxSCI_MARK_ARROWDOWN);
    MarkerSetBackground (wxSCI_MARKNUM_FOLDEROPEN, wxColour (_T("BLACK")));
    MarkerDefine (wxSCI_MARKNUM_FOLDERSUB,wxSCI_MARK_EMPTY);
    MarkerDefine (wxSCI_MARKNUM_FOLDEREND,wxSCI_MARK_DOTDOTDOT);
    MarkerDefine (wxSCI_MARKNUM_FOLDEROPENMID, wxSCI_MARK_ARROWDOWN);
    MarkerDefine (wxSCI_MARKNUM_FOLDERMIDTAIL, wxSCI_MARK_EMPTY);
    MarkerDefine (wxSCI_MARKNUM_FOLDERTAIL, wxSCI_MARK_EMPTY);

    // clear wrong default keys
#if !defined(__WXGTK__)
    CmdKeyClear (wxSCI_KEY_TAB, 0);
    CmdKeyClear (wxSCI_KEY_TAB, wxSCI_SCMOD_SHIFT);
#endif
    CmdKeyClear ('A', wxSCI_SCMOD_CTRL);
#if !defined(__WXGTK__)
    CmdKeyClear ('C', wxSCI_SCMOD_CTRL);
#endif
    CmdKeyClear ('D', wxSCI_SCMOD_CTRL);
    CmdKeyClear ('D', wxSCI_SCMOD_SHIFT | wxSCI_SCMOD_CTRL);
    CmdKeyClear ('F', wxSCI_SCMOD_ALT | wxSCI_SCMOD_CTRL);
    CmdKeyClear ('L', wxSCI_SCMOD_CTRL);
    CmdKeyClear ('L', wxSCI_SCMOD_SHIFT | wxSCI_SCMOD_CTRL);
    CmdKeyClear ('T', wxSCI_SCMOD_CTRL);
    CmdKeyClear ('T', wxSCI_SCMOD_SHIFT | wxSCI_SCMOD_CTRL);
    CmdKeyClear ('U', wxSCI_SCMOD_CTRL);
    CmdKeyClear ('U', wxSCI_SCMOD_SHIFT | wxSCI_SCMOD_CTRL);
#if !defined(__WXGTK__)
    CmdKeyClear ('V', wxSCI_SCMOD_CTRL);
    CmdKeyClear ('X', wxSCI_SCMOD_CTRL);
#endif
    CmdKeyClear ('Y', wxSCI_SCMOD_CTRL);
#if !defined(__WXGTK__)
    CmdKeyClear ('Z', wxSCI_SCMOD_CTRL);
#endif

    // miscelaneous
    UsePopUp (0);
    SetLayoutCache (wxSCI_CACHE_PAGE);
    SetBufferedDraw (1);
    SetModEventMask (wxSCI_MOD_BEFOREDELETE |wxSCI_MOD_DELETETEXT |
                     wxSCI_MOD_INSERTTEXT | wxSCI_PERFORMED_USER |
                     wxSCI_PERFORMED_UNDO | wxSCI_PERFORMED_REDO |
                     wxSCI_MOD_CHANGESTYLE);

    // initialize scrollbar width
    m_maxWidth = 0;
    m_scrollWidthChange = false;

    // set default language
    StyleClearAll();
    m_language = &g_LanguagePrefs[0];
    InitializeLanguage (m_language->name);
    InitializeAnalyse (m_language->name);
    InitializeIdentTable ();

}

EditText::~EditText () {

    // remove created objects
    if (m_GotoDlg) delete m_GotoDlg;
    if (m_FindReplaceDlg) delete m_FindReplaceDlg;
    if (m_ContextMenu) delete m_ContextMenu;
}

//----------------------------------------------------------------------------
// common event handlers
void EditText::OnCommonNotify (wxCommandEvent &event) {
    int position = event.GetInt();
    if (position >= 0) {
        ProcessIdentTable (LineFromPosition (position));
    }else{
        ProcessIdentTable (-1);
    }
    NotifyEditBrowse (myEVT_COMMON_NOTIFY, position);
}

void EditText::OnContextMenu (wxMouseEvent &event) {
    PopupMenu (m_ContextMenu, event.GetX(), event.GetY());
}

// edit event handlers
void EditText::OnEditClear (wxCommandEvent &WXUNUSED(event)) {
    if (GetReadOnly()) return; // delete without selection deletes the next char!
    Clear ();
    g_statustext = _("Cleared selected text: ") + GetSelectedText();
}

void EditText::OnEditClearUI (wxUpdateUIEvent &event) {
    event.Enable (!GetReadOnly()); // delete without selection deletes the next char!
}

void EditText::OnEditCut (wxCommandEvent &WXUNUSED(event)) {
    if (GetReadOnly() || (GetSelectionEnd()-GetSelectionStart() <= 0)) return;
    Cut ();
    g_statustext = _("Cut selected text: ") + GetSelectedText();
}

void EditText::OnEditCutUI (wxUpdateUIEvent &event) {
    event.Enable (!GetReadOnly() && (GetSelectionEnd()-GetSelectionStart() > 0));
}

void EditText::OnEditCopy (wxCommandEvent &WXUNUSED(event)) {
    if (GetSelectionEnd()-GetSelectionStart() <= 0) return;
    Copy ();
    g_statustext = _("Copied selected text: ") + GetSelectedText();
}

void EditText::OnEditCopyUI (wxUpdateUIEvent &event) {
    event.Enable (GetSelectionEnd()-GetSelectionStart() > 0);
}

void EditText::OnEditPaste (wxCommandEvent &WXUNUSED(event)) {
    if (!CanPaste()) return;
    Paste ();
    g_statustext = _("Pasted selected text: ") + GetSelectedText();
}

void EditText::OnEditPasteUI (wxUpdateUIEvent &event) {
    event.Enable (!GetReadOnly() && CanPaste());
}

void EditText::OnEditIndentInc (wxCommandEvent &WXUNUSED(event)) {
    CmdKeyExecute (wxSCI_CMD_TAB);
    g_statustext = _("Increased indent if cursor in indent range, else tabulator");
}

void EditText::OnEditIndentIncUI (wxUpdateUIEvent &event) {
    event.Enable (!GetReadOnly());
}

void EditText::OnEditIndentRed (wxCommandEvent &WXUNUSED(event)) {
    CmdKeyExecute (wxSCI_CMD_DELETEBACK);
    g_statustext = _("Reduced indent if cursor in indent range, else backspace");
}

void EditText::OnEditIndentRedUI (wxUpdateUIEvent &event) {
    event.Enable (!GetReadOnly());
}

void EditText::OnEditSelectAll (wxCommandEvent &WXUNUSED(event)) {
    SetSelection (0, GetLength ());
    g_statustext = _("Selected all text");
}

void EditText::OnEditSelectAllUI (wxUpdateUIEvent &event) {
    event.Enable (GetLength() > 0);
}

void EditText::OnEditSelectLine (wxCommandEvent &WXUNUSED(event)) {
    int lineStart = PositionFromLine (GetCurrentLine());
    int lineEnd = PositionFromLine (GetCurrentLine() + 1);
    SetSelection (lineStart, lineEnd);
    g_statustext = _("Selected line text");
}

void EditText::OnEditSelectLineUI (wxUpdateUIEvent &event) {
    event.Enable (GetLength() > 0);
}

void EditText::OnEditRedo (wxCommandEvent &WXUNUSED(event)) {
    if (!CanRedo()) return;
    Redo ();
    g_statustext = _("Redone the last action");
}

void EditText::OnEditRedoUI (wxUpdateUIEvent &event) {
    event.Enable (!GetReadOnly() && CanRedo());
}

void EditText::OnEditUndo (wxCommandEvent &WXUNUSED(event)) {
    if (!CanUndo()) return;
    Undo ();
    g_statustext = _("Undone the last action");
}

void EditText::OnEditUndoUI (wxUpdateUIEvent &event) {
    event.Enable (!GetReadOnly() && CanUndo());
}

void EditText::OnFind (wxCommandEvent &event) {
    m_startpos = GetCurrentPos();
    SetTargetStart (m_startpos);
    SetTargetEnd (m_startpos);
    m_replace = false;
    if (GetSelectionEnd()-GetSelectionStart() > 0) {
        m_FindReplaceDlg->SetFindString (GetSelectedText());
    }
    if (m_FindReplaceDlg->ShowModal () != wxID_OK) return;
    OnFindNext (event);
}

void EditText::OnFindUI (wxUpdateUIEvent &event) {
    event.Enable (GetLength() > 0);
}

void EditText::OnFindNext (wxCommandEvent &WXUNUSED(event)) {
    wxString findtext = m_FindReplaceDlg->GetFindString();
    if (findtext.IsEmpty()) {
        wxMessageBox (_("Can't find anything with empty string"),
                      _("Find string"), wxOK | wxICON_INFORMATION, this);
        return;
    }
    m_startpos = GetCurrentPos();
    SetTargetStart (m_startpos);
    SetTargetEnd (m_startpos);
    int pos = FindString (findtext, m_FindReplaceDlg->GetFlags());
    if (pos >= 0) {
        EnsureCaretVisible();
        SetSelection (pos, pos + findtext.Length());
        g_statustext = _("Found text: ") + findtext;
    }else{
        wxMessageBox (_("Can't anymore find the string: \"") + findtext + _("\""),
                      _("Find string"), wxOK | wxICON_INFORMATION, this);
        g_statustext = _("No more text found!");
    }
    SetFocus ();
}

void EditText::OnFindNextUI (wxUpdateUIEvent &event) {
    event.Enable (GetLength() > 0);
}

void EditText::OnReplace (wxCommandEvent &event) {
    m_startpos = GetCurrentPos();
    SetTargetStart (m_startpos);
    SetTargetEnd (m_startpos);
    m_replace = true;
    if (GetSelectionEnd()-GetSelectionStart() > 0) {
        m_FindReplaceDlg->SetFindString (GetSelectedText());
    }
    int retCode = m_FindReplaceDlg->ShowModal (myFR_REPLACEDIALOG);
    if (retCode == wxID_OK) {
        OnFindNext (event);
    }else if (retCode == myID_REPLACE) {
        OnReplaceNext (event);
    }else if (retCode == myID_REPLACEALL) {
        OnReplaceAll (event);
    }
}

void EditText::OnReplaceUI (wxUpdateUIEvent &event) {
    event.Enable (!GetReadOnly() && (GetLength() > 0));
}

void EditText::OnReplaceNext (wxCommandEvent &WXUNUSED(event)) {
    if (!m_replace) return;
    wxString findtext = m_FindReplaceDlg->GetFindString();
    if (findtext.IsEmpty()) {
        wxMessageBox (_("Can't find anything with empty string"),
                      _("Find string"), wxOK | wxICON_INFORMATION, this);
        return;
    }
    int pos = FindString (findtext, m_FindReplaceDlg->GetFlags());
    if (pos >= 0) {
        SetSelection (pos, pos + findtext.Length());
        wxString replacetext = m_FindReplaceDlg->GetReplaceString ();
        ReplaceSelection (replacetext);
        EnsureCaretVisible();
        SetSelection (pos, pos + replacetext.Length());
        g_statustext = _("Replaced text: ") + findtext;
    }else{
        wxMessageBox (_("Can't anymore find the string: \"") + findtext + _("\""),
                      _("Replace string"), wxOK | wxICON_INFORMATION, this);
        g_statustext = _("No more text found!");
    }
    SetFocus ();
}

void EditText::OnReplaceNextUI (wxUpdateUIEvent &event) {
    event.Enable (!GetReadOnly() && (GetLength() > 0));
}

void EditText::OnReplaceAll (wxCommandEvent &WXUNUSED(event)) {
    if (!m_replace) return;
    wxString findtext = m_FindReplaceDlg->GetFindString();
    if (findtext.IsEmpty()) {
        wxMessageBox (_("Can't find anything with empty string"),
                      _("Find string"), wxOK | wxICON_INFORMATION, this);
        return;
    }
    BeginUndoAction();
    while (FindString (findtext, m_FindReplaceDlg->GetFlags()) >= 0) {
        wxString replacetext = m_FindReplaceDlg->GetReplaceString ();
        ReplaceTarget (replacetext);
    }
    EndUndoAction();
    wxMessageBox (_("All occurence of text \"") + findtext + _("\" have been replaced"),
                  _("Replace string"), wxOK | wxICON_INFORMATION, this);
    g_statustext = _("All occurence have been replaced");
    m_replace = false;
}

void EditText::OnBraceMatch (wxCommandEvent &WXUNUSED(event)) {
    int min = GetCurrentPos ();
    int max = BraceMatch (min);
    if (max > (min+1)) {
        BraceHighlight (min+1, max);
        SetSelection (min+1, max);
        g_statustext = _("Matched braces: ") + GetSelectedText();
    }else{
        BraceBadLight (min);
        g_statustext = _("No braces matched");
    }
}

void EditText::OnBraceMatchUI (wxUpdateUIEvent &event) {
    event.Enable (GetLength() > 0);
}

void EditText::OnGoto (wxCommandEvent &WXUNUSED(event)) {
    if (m_GotoDlg->ShowModal() != wxID_OK) return;
    int line = m_GotoDlg->GetPosition ();
    if (line > 0) {
        GotoLine (line - 1);
        line = GetCurrentLine() - (GetFirstVisibleLine()+2);
        if (line > 0) LineScroll (0, line);
        g_statustext = _("Went to line: ") + wxString::Format (_T("%d"), line);
    }else{
        GotoLine (0);
        g_statustext = _("Went to first line");
    }
    SetFocus ();
}

void EditText::OnGotoUI (wxUpdateUIEvent &event) {
    event.Enable (GetLineCount() > 1);
}

void EditText::OnBookmarkToggle (wxCommandEvent &WXUNUSED(event)) {
    int line = GetCurrentLine();
    if (MarkerGet (line) & (1 << MARKER_BOOKMARK)) {
        MarkerDelete (line, MARKER_BOOKMARK);
    }else{
        MarkerAdd (line, MARKER_BOOKMARK);
    }
}

void EditText::OnBookmarkToggleUI (wxUpdateUIEvent &event) {
    event.Enable (GetLength() > 0);
}

void EditText::OnBookmarkNext (wxCommandEvent &WXUNUSED(event)) {
    int line = GetCurrentLine();
    int next;
    next = MarkerNext (line + 1, 1 << MARKER_BOOKMARK);
    if (next < 0) next = MarkerNext (0, 1 << MARKER_BOOKMARK);
    if ((next >= 0) && (next != line)) {
        GotoLine (next);
        line = GetCurrentLine() - (GetFirstVisibleLine()+2);
        if (line > 0) LineScroll (0, line);
    }
}

void EditText::OnBookmarkNextUI (wxUpdateUIEvent &event) {
    int line = MarkerNext (0, 1 << MARKER_BOOKMARK);
    event.Enable ((GetLength() > 0) && (line >= 0));
}

void EditText::OnBookmarkPrev (wxCommandEvent &WXUNUSED(event)) {
    int line = GetCurrentLine();
    int prev;
    prev = MarkerPrevious (line - 1, 1 << MARKER_BOOKMARK);
    if (prev < 0) prev = MarkerPrevious (GetLineCount(), 1 << MARKER_BOOKMARK);
    if ((prev >= 0) && (prev != line)) {
        GotoLine (prev);
        line = GetCurrentLine() - (GetFirstVisibleLine()+2);
        if (line > 0) LineScroll (0, line);
    }
}

void EditText::OnBookmarkPrevUI (wxUpdateUIEvent &event) {
    int line = MarkerNext (0, 1 << MARKER_BOOKMARK);
    event.Enable ((GetLength() > 0) && (line >= 0));
}

void EditText::OnBookmarkClear (wxCommandEvent &WXUNUSED(event)) {
    int next = MarkerNext (0, 1 << MARKER_BOOKMARK);
    while (next >= 0) {
        MarkerDelete (next, MARKER_BOOKMARK);
        next = MarkerNext (next, 1 << MARKER_BOOKMARK);
    }
}

void EditText::OnBookmarkClearUI (wxUpdateUIEvent &event) {
    int line = MarkerNext (0, 1 << MARKER_BOOKMARK);
    event.Enable ((GetLength() > 0) && (line >= 0));
}

void EditText::OnHilightLang (wxCommandEvent &event) {
    StyleClearAll();
    m_language = &g_LanguagePrefs[event.GetId() - myID_HILIGHTFIRST];
    InitializeLanguage (m_language->name);
    InitializeAnalyse (m_language->name);
    InitializeIdentTable ();
    if (g_CommonPrefs.showBrowse && GetAnalyse()) {
        ((wxSplitterWindow*)GetParent())->SetSashPosition (m_browseWidth);
    }else{
        ((wxSplitterWindow*)GetParent())->SetSashPosition (1);
    }
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
    wxToolBar *toolbar = ((wxFrame*)wxTheApp->GetTopWindow())->GetToolBar();
    if (toolbar) {
        wxComboBox *browsebox = (wxComboBox *)toolbar->FindControl (myID_BROWSESELECT);
        if (browsebox) browsebox->Enable (GetAnalyse());
    }
#endif
}

void EditText::OnHilightLangUI (wxUpdateUIEvent &event) {
    event.Enable (true);
    int lang = event.GetId() - myID_HILIGHTFIRST;
    if ((m_language) &&(m_language->name == g_LanguagePrefs[lang].name)) {
        event.Check (true);
    }
}

void EditText::OnDisplayEOL (wxCommandEvent &WXUNUSED(event)) {
    SetViewEOL (!GetViewEOL());
}

void EditText::OnDisplayEOLUI (wxUpdateUIEvent &event) {
    event.Enable (true);
    event.Check (GetViewEOL());
}

void EditText::OnFoldLevel (wxCommandEvent &WXUNUSED(event)) {
    m_foldLevel = m_foldLevel? 0: wxSCI_FOLDFLAG_LEVELNUMBERS;
    SetFoldFlags (wxSCI_FOLDFLAG_LINEBEFORE_CONTRACTED |
                  wxSCI_FOLDFLAG_LINEAFTER_CONTRACTED |
                  m_foldLevel);
}

void EditText::OnFoldLevelUI (wxUpdateUIEvent &event) {
    event.Enable (true);
    event.Check (m_foldLevel > 0);
}

void EditText::OnIndentGuide (wxCommandEvent &WXUNUSED(event)) {
    SetIndentationGuides (!GetIndentationGuides());
}

void EditText::OnIndentGuideUI (wxUpdateUIEvent &event) {
    event.Enable (true);
    event.Check (GetIndentationGuides());
}

void EditText::OnLineNumber (wxCommandEvent &WXUNUSED(event)) {
    SetMarginWidth (m_LineNrID,
                    GetMarginWidth (m_LineNrID) == 0? m_LineNrMargin: 0);
}

void EditText::OnLineNumberUI (wxUpdateUIEvent &event) {
    event.Enable (true);
    event.Check (GetMarginWidth (m_LineNrID) > 0);
}

void EditText::OnLongLineOn (wxCommandEvent &WXUNUSED(event)) {
    SetEdgeMode (GetEdgeMode() == 0? wxSCI_EDGE_LINE: wxSCI_EDGE_NONE);
}

void EditText::OnLongLineOnUI (wxUpdateUIEvent &event) {
    event.Enable (true);
    event.Check (GetEdgeMode() != 0);
}

void EditText::OnWhiteSpace (wxCommandEvent &WXUNUSED(event)) {
    SetViewWhiteSpace (GetViewWhiteSpace() == 0?
                       wxSCI_WS_VISIBLEALWAYS: wxSCI_WS_INVISIBLE);
}

void EditText::OnWhiteSpaceUI (wxUpdateUIEvent &event) {
    event.Enable (true);
    event.Check (GetViewWhiteSpace() != 0);
}

void EditText::OnFoldToggle (wxCommandEvent &WXUNUSED(event)) {
    ToggleFold (GetFoldParent (GetCurrentLine() + 1));
}

void EditText::OnFoldToggleUI (wxUpdateUIEvent &event) {
    event.Enable (g_CommonPrefs.foldEnable &&
                  (GetFoldLevel (GetCurrentLine()) > wxSCI_FOLDLEVELBASE));
    event.Check (GetFoldExpanded (GetCurrentLine()));
}

void EditText::OnSetOverType (wxCommandEvent &WXUNUSED(event)) {
    SetOvertype (!GetOvertype());
}

void EditText::OnSetOverTypeUI (wxUpdateUIEvent &event) {
    event.Enable (!GetReadOnly() && (GetLength() > 0));
    event.Check (GetOvertype());
}

void EditText::OnSetReadOnly (wxCommandEvent &WXUNUSED(event)) {
    SetReadOnly (!GetReadOnly());
}

void EditText::OnSetReadOnlyUI (wxUpdateUIEvent &event) {
    event.Enable (GetLength() > 0);
    event.Check (GetReadOnly());
}

void EditText::OnWrapmodeOn (wxCommandEvent &WXUNUSED(event)) {
    SetWrapMode (GetWrapMode() == 0? wxSCI_WRAP_WORD: wxSCI_WRAP_NONE);
}

void EditText::OnWrapmodeOnUI (wxUpdateUIEvent &event) {
    event.Enable (GetLength() > 0);
    event.Check (GetWrapMode() > 0);
}

void EditText::OnUseCharset (wxCommandEvent &event) {
    int Nr;
    wxString text;
    int charset = GetCodePage();
    switch (event.GetId()) {
        case myID_CHARSET_ANSI: {
            charset = wxSCI_CHARSET_ANSI;
            text = _("ANSI (Windows)");
            break;}
        case myID_CHARSET_EASTEUROPE: {
            charset = wxSCI_CHARSET_EASTEUROPE;
            text = _("Eastern europe");
            break;}
        case myID_CHARSET_GB2312: {
            charset = wxSCI_CHARSET_GB2312;
            text = _("GB2312");
            break;}
        case myID_CHARSET_HANGUL: {
            charset = wxSCI_CHARSET_HANGUL;
            text = _("Hangul");
            break;}
        case myID_CHARSET_HEBREW: {
            charset = wxSCI_CHARSET_HEBREW;
            text = _("Hebrew");
            break;}
        case myID_CHARSET_SHIFTJIS: {
            charset = wxSCI_CHARSET_SHIFTJIS;
            text = _("Shiftjis");
            break;}
#if defined __WXMSW__
        case myID_CHARSET_ARABIC: {
            charset = wxSCI_CHARSET_ARABIC;
            text = _("Arabic");
            break;}
        case myID_CHARSET_BALTIC: {
            charset = wxSCI_CHARSET_BALTIC;
            text = _("Baltic");
            break;}
        case myID_CHARSET_CHINESEBIG5: {
            charset = wxSCI_CHARSET_CHINESEBIG5;
            text = _("Chinese (big5)");
            break;}
        case myID_CHARSET_GREEK: {
            charset = wxSCI_CHARSET_GREEK;
            text = _("Greek");
            break;}
        case myID_CHARSET_JOHAB: {
            charset = wxSCI_CHARSET_JOHAB;
            text = _("Johab");
            break;}
        case myID_CHARSET_MAC: {
            charset = wxSCI_CHARSET_MAC;
            text = _("Macintosh");
            break;}
        case myID_CHARSET_OEM: {
            charset = wxSCI_CHARSET_OEM;
            text = _("OEM");
            break;}
        case myID_CHARSET_RUSSIAN: {
            charset = wxSCI_CHARSET_RUSSIAN;
            text = _("Russian");
            break;}
        case myID_CHARSET_SYMBOL: {
            charset = wxSCI_CHARSET_SYMBOL;
            text = _("Symbol");
            break;}
        case myID_CHARSET_THAI: {
            charset = wxSCI_CHARSET_THAI;
            text = _("Thai");
            break;}
        case myID_CHARSET_TURKISH: {
            charset = wxSCI_CHARSET_TURKISH;
            text = _("Turkish");
            break;}
        case myID_CHARSET_VIETNAMESE: {
            charset = wxSCI_CHARSET_VIETNAMESE;
            text = _("Vietnamese");
            break;}
#endif
        default: {
#if defined __WXMSW__
            charset = wxSCI_CHARSET_DEFAULT;
#else
            charset = wxSCI_CHARSET_ANSI;
#endif
            text = _("Default");
            break;}
    }
    for (Nr = 0; Nr < wxSCI_STYLE_LASTPREDEFINED; Nr++) {
        StyleSetCharacterSet (Nr, charset);
    }
    SetCodePage (charset);
    g_statustext = _("Use code page: ") + text;
}

void EditText::OnUseCharsetUI (wxUpdateUIEvent &event) {
    event.Enable (!GetReadOnly());
}

void EditText::OnChangeCase (wxCommandEvent &event) {
    switch (event.GetId()) {
        case myID_CHANGELOWER: {
            CmdKeyExecute (wxSCI_CMD_LOWERCASE);
            g_statustext = _("Changed selected text to lowercase.");
            break;
        }
        case myID_CHANGEUPPER: {
            CmdKeyExecute (wxSCI_CMD_UPPERCASE);
            g_statustext = _("Changed selected text to uppercase.");
            break;
        }
    }
}

void EditText::OnChangeCaseUI (wxUpdateUIEvent &event) {
    event.Enable (!GetReadOnly() && (GetSelectionEnd()-GetSelectionStart() > 0));
}

void EditText::OnConvertEOL (wxCommandEvent &event) {
    int eolMode = GetEOLMode();
    switch (event.GetId()) {
        case myID_CONVERTCR: { eolMode = wxSCI_EOL_CR; break;}
        case myID_CONVERTCRLF: { eolMode = wxSCI_EOL_CRLF; break;}
        case myID_CONVERTLF: { eolMode = wxSCI_EOL_LF; break;}
    }
    ConvertEOLs (eolMode);
    SetEOLMode (eolMode);
    g_statustext = _("Converted line endings to ");
    switch (event.GetId()) {
        case myID_CONVERTCRLF: { g_statustext += _("CR+LF (Windows)."); break;}
        case myID_CONVERTCR: { g_statustext += _("CR (Macintosh)."); break;}
        case myID_CONVERTLF: { g_statustext += _("LF (Unix)."); break;}
    }
}

void EditText::OnConvertEOLUI (wxUpdateUIEvent &event) {
    event.Enable (!GetReadOnly() && (GetLength() > 0));
}

void EditText::OnWhitespaceToUI (wxUpdateUIEvent &event) {
    event.Enable (!GetReadOnly() && (GetLength() > 0));
}

void EditText::OnSpacesToTabs (wxCommandEvent &WXUNUSED(event)) {
    SpacesToTabs();
    g_statustext = _("All occurence have been replaced");
}

void EditText::OnTabsToSpaces (wxCommandEvent &WXUNUSED(event)) {
    TabsToSpaces();
    g_statustext = _("All occurence have been replaced");
}

void EditText::OnTrailingSpace (wxCommandEvent &WXUNUSED(event)) {
    BeginUndoAction();
    int lineNr;
    for (lineNr = 0; lineNr < GetLineCount(); ++lineNr) {
        int lineStart = PositionFromLine (lineNr);
        int lineEnd = GetLineEndPosition (lineNr);
        int pos;
        for (pos = lineEnd; pos > lineStart; pos--) {
            char chr = GetCharAt (pos-1);
            if ((chr != ' ') && (chr != '\t')) break;
        }
        if (pos < lineEnd) {
            SetTargetStart (pos);
            SetTargetEnd (lineEnd);
            ReplaceTarget (wxEmptyString);
        }
    }
    EndUndoAction();
}

void EditText::OnTrailingSpaceUI (wxUpdateUIEvent &event) {
    event.Enable (!GetReadOnly() && (GetLength() > 0));
}

void EditText::OnResultSelected (wxScintillaEvent &event) {
    if (!IsResult()) {
        event.Skip();
        return;
    }
    GotoLine (GetCurrentLine());
    int line = GetCurrentLine() - (GetFirstVisibleLine()+2);
    if (line > 0) LineScroll (0, line);
    wxString lineNr = GetLine (GetCurrentLine());
    wxString fname = lineNr.BeforeFirst ('\t');
    wxString pos = fname.AfterLast ('(');
    fname = fname.BeforeLast ('(');
    fname = fname.Trim ();
    pos = pos.BeforeLast (')');
    wxCommandEvent evt (myEVT_RESULT_SELECTED, myEVT_OPENANDGOTO);
    evt.SetString (fname + _T(":") + pos);
    wxPostEvent ((wxFrame*)wxTheApp->GetTopWindow(), evt);
}

void EditText::OnMarginClick (wxScintillaEvent &event) {
    if (event.GetMargin() == m_FoldingID) {
        int lineClick = LineFromPosition (event.GetPosition());
        int levelClick = GetFoldLevel (lineClick);
        if ((levelClick & wxSCI_FOLDLEVELHEADERFLAG) > 0) {
            ToggleFold (lineClick);
        }
    }
}

void EditText::OnModified (wxScintillaEvent &event) {
    if (m_textLoading) return; // do nothing during loading
    if (event.GetModificationType() & wxSCI_MOD_BEFOREDELETE) {
        m_scrollWidthChange = true;
    }else if (event.GetModificationType() & wxSCI_MOD_DELETETEXT) {
        if (m_scrollWidthChange) SetScrollWidth ();
        IdentTableDelete (event.GetPosition(), event.GetLength());
        NotifyEditBrowse (myEVT_COMMON_NOTIFY, event.GetPosition());
    }else if (event.GetModificationType() & wxSCI_MOD_INSERTTEXT) {
        SetScrollWidth (GetCurrentLine());
        IdentTableInsert (event.GetPosition(), event.GetLength());
    }else if (event.GetModificationType() & wxSCI_MOD_CHANGESTYLE) {
        int l;
        for (l = 0; l < (event.GetLinesAdded()+1); ++l) {
            ProcessIdentTable (LineFromPosition (event.GetPosition()) + l);
            NotifyEditBrowse (myEVT_COMMON_NOTIFY, event.GetPosition());
        }
    }
}

void EditText::OnCharAdded (wxScintillaEvent &event) {
    if (m_textLoading) return; // do nothing during loading
    char chr = event.GetKey();
    int currentLine = GetCurrentLine();
    // Change this if support for mac files with \r is needed
    if (chr == '\n') {
        int lineInd = 0;
        if (currentLine > 0) {
            lineInd = GetLineIndentation(currentLine - 1);
        }
        if (lineInd == 0) return;
        SetLineIndentation (currentLine, lineInd);
        GotoPos(PositionFromLine (currentLine) + lineInd);
    }
}

//----------------------------------------------------------------------------
// private functions
void EditText::createContextMenu () {

    wxMenuBar *menuBar = ((wxFrame*)wxTheApp->GetTopWindow())->GetMenuBar();
    wxString label;
    int tabPos;

    // create menu
    if (menuBar->FindItem (wxID_CUT)) {
        label = menuBar->GetLabel (wxID_CUT);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_ContextMenu->Append (wxID_CUT, label);
    }
    if (menuBar->FindItem (wxID_COPY)) {
        label = menuBar->GetLabel (wxID_COPY);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_ContextMenu->Append (wxID_COPY, label);
    }
    if (menuBar->FindItem (wxID_PASTE)) {
        label = menuBar->GetLabel (wxID_PASTE);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_ContextMenu->Append (wxID_PASTE, label);
    }
    if (menuBar->FindItem (wxID_CLEAR)) {
        label = menuBar->GetLabel (wxID_CLEAR);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_ContextMenu->Append (wxID_CLEAR, label);
    }
    m_ContextMenu->AppendSeparator();
    if (menuBar->FindItem (wxID_FIND)) {
        label = menuBar->GetLabel (wxID_FIND);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_ContextMenu->Append (wxID_FIND, label);
    }
    if (menuBar->FindItem (myID_FINDNEXT)) {
        label = menuBar->GetLabel (myID_FINDNEXT);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_ContextMenu->Append (myID_FINDNEXT, label);
    }
    if (menuBar->FindItem (myID_REPLACE)) {
        label = menuBar->GetLabel (myID_REPLACE);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_ContextMenu->Append (myID_REPLACE, label);
    }
    if (menuBar->FindItem (myID_REPLACENEXT)) {
        label = menuBar->GetLabel (myID_REPLACENEXT);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_ContextMenu->Append (myID_REPLACENEXT, label);
    }
    m_ContextMenu->AppendSeparator();
    if (menuBar->FindItem (myID_BRACEMATCH)) {
        label = menuBar->GetLabel (myID_BRACEMATCH);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_ContextMenu->Append (myID_BRACEMATCH, label);
    }
    if (menuBar->FindItem (myID_GOTO)) {
        label = menuBar->GetLabel (myID_GOTO);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_ContextMenu->Append (myID_GOTO, label);
    }
    m_ContextMenu->AppendSeparator();
    if (menuBar->FindItem (myID_SELECTLINE)) {
        label = menuBar->GetLabel (myID_SELECTLINE);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_ContextMenu->Append (myID_SELECTLINE, label);
    }
    m_ContextMenu->AppendSeparator();
    if (menuBar->FindItem (myID_CHANGEUPPER)) {
        label = menuBar->GetLabel (myID_CHANGEUPPER);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_menuChangeCase->Append (myID_CHANGEUPPER, label);
    }
    if (menuBar->FindItem (myID_CHANGELOWER)) {
        label = menuBar->GetLabel (myID_CHANGELOWER);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_menuChangeCase->Append (myID_CHANGELOWER, label);
    }
    if (menuBar->FindItem (myID_CHANGECASE)) {
        label = menuBar->GetLabel (myID_CHANGECASE);
        label.Replace (_T("_"), wxEmptyString);
        tabPos = label.Find (_T("\t"));
        if (tabPos > 0) label.Truncate (tabPos);
        m_ContextMenu->Append (myID_CHANGECASE, label, m_menuChangeCase);
    }

}

wxString EditText::DetermineLanguage (const wxString &filename) {

    // determine language from filepatterns
    wxString filepattern;
    wxString pattern;
    int languageNr;
    for (languageNr = 1; languageNr < g_LanguagePrefsSize; languageNr++) {
        filepattern = g_LanguagePrefs[languageNr].filepattern;
        filepattern.Lower();
        while (!filepattern.IsEmpty()) {
            pattern = filepattern.BeforeFirst (';');
            if ((pattern == filename) ||
                (pattern == (filename.BeforeLast ('.') + _T(".*"))) ||
                (pattern == (_T("*.") + filename.AfterLast ('.')))) {
                m_language = &g_LanguagePrefs[languageNr];
                return m_language->name;
            }
            filepattern = filepattern.AfterFirst (';');
        }
    }
    m_language = &g_LanguagePrefs[0];
    return m_language->name;

}

bool EditText::InitializeLanguage (const wxString &name) {

    // determine language
    int languageNr;
    LanguageInfo *cur = NULL;
    for (languageNr = 1; languageNr < g_LanguagePrefsSize; languageNr++) {
        if (g_LanguagePrefs [languageNr].name == name) break;
    }
    if (languageNr >= g_LanguagePrefsSize) languageNr = 0;
    cur = &g_LanguagePrefs [languageNr];

    // load help books if any
    wxString helpbooks = cur->helpbooks;
    while (!helpbooks.IsEmpty()) {
        wxString helpbook = helpbooks.BeforeFirst (';');
        if ((g_helpbooks.Index(helpbook) == wxNOT_FOUND) &&
            (g_help->AddBook (g_helppath + wxFileName::GetPathSeparator() + helpbook))) {
                g_helpbooks.Add (helpbook);
        }
        helpbooks = helpbooks.AfterFirst (';');
    }

    // set language and lexer
    SetLexer (m_language->lexer);
    if ((cur->lexer == wxSCI_LEX_ASP) ||
        (cur->lexer == wxSCI_LEX_HTML) ||
        (cur->lexer == wxSCI_LEX_PHP) ||
        (cur->lexer == wxSCI_LEX_XML)) {
        SetStyleBits (7);
    }else{
        SetStyleBits (5);
    }

    // set margin for line numbers
    SetMarginType (m_LineNrID, wxSCI_MARGIN_NUMBER);
    StyleSetForeground (wxSCI_STYLE_LINENUMBER, wxColour (_T("DARK GREY")));
    StyleSetBackground (wxSCI_STYLE_LINENUMBER, wxColour (_T("WHITE")));
    SetMarginWidth (m_LineNrID,
                    g_CommonPrefs.lineNumberEnable? m_LineNrMargin: 0);

    // set common styles
    StyleSetForeground (wxSCI_STYLE_DEFAULT, wxColour (_T("DARK GREY")));
    StyleSetForeground (wxSCI_STYLE_INDENTGUIDE, wxColour (_T("DARK GREY")));

    // initialize settings
    if (g_CommonPrefs.syntaxEnable) {
        int off = cur->typeoffset;
        int keywordnr = 0;
        int nr;
        for (nr = TOKEN_DEFAULT; nr < STYLE_TYPES_COUNT; ++nr) {
            if (nr+off > wxSCI_STYLE_MAX) break;
            if (cur->styles[nr].type == -1) continue;
            const StyleInfo &curType = g_StylePrefs[cur->styles[nr].type];
            wxFont font (curType.fontsize, wxTELETYPE, wxNORMAL, wxNORMAL, false,
                         curType.fontname);
            StyleSetFont (nr+off, font);
            if (curType.foreground) {
                StyleSetForeground (nr+off, wxColour (curType.foreground));
            }
            if (curType.background) {
                StyleSetBackground (nr+off, wxColour (curType.background));
            }
            StyleSetBold (nr+off, (curType.fontstyle & TOKEN_STYLE_BOLD) > 0);
            StyleSetItalic (nr+off, (curType.fontstyle & TOKEN_STYLE_ITALIC) > 0);
            StyleSetUnderline (nr+off, (curType.fontstyle & TOKEN_STYLE_UNDERL) > 0);
            StyleSetVisible (nr+off, (curType.fontstyle & TOKEN_STYLE_HIDDEN) == 0);
            StyleSetCase (nr+off, curType.lettercase);
            const wxChar *pwords = cur->styles[nr].words;
            if (pwords) {
                SetKeyWords (keywordnr, pwords);
                keywordnr += 1;
            }
        }
    }

    // set margin as unused
    SetMarginType (m_DividerID, wxSCI_MARGIN_SYMBOL);
    SetMarginWidth (m_DividerID, m_DividerMargin);
    SetMarginSensitive (m_DividerID, false);

    // folding
    SetMarginType (m_FoldingID, wxSCI_MARGIN_SYMBOL);
    SetMarginMask (m_FoldingID, wxSCI_MASK_FOLDERS | (1 << MARKER_BOOKMARK));
    StyleSetBackground (m_FoldingID, wxColour (_T("WHITE")));
    SetMarginWidth (m_FoldingID, 0);
    SetMarginSensitive (m_FoldingID, false);
    if (g_CommonPrefs.foldEnable) {
        SetMarginWidth (m_FoldingID, cur->folds != 0? m_FoldingMargin: 0);
        SetMarginSensitive (m_FoldingID, cur->folds != 0);
        SetProperty (_T("fold"), cur->folds != 0? _T("1"): _T("0"));
        SetProperty (_T("fold.comment"),
                     (cur->folds & FOLD_TYPE_COMMENT) > 0? _T("1"): _T("0"));
        SetProperty (_T("fold.compact"),
                     (cur->folds & FOLD_TYPE_COMPACT) > 0? _T("1"): _T("0"));
        SetProperty (_T("fold.preprocessor"),
                     (cur->folds & FOLD_TYPE_PREPROC) > 0? _T("1"): _T("0"));
        SetProperty (_T("fold.html"),
                     (cur->folds & FOLD_TYPE_HTML) > 0? _T("1"): _T("0"));
        SetProperty (_T("fold.html.preprocessor"),
                     (cur->folds & FOLD_TYPE_HTMLPREP) > 0? _T("1"): _T("0"));
        SetProperty (_T("fold.comment.python"),
                     (cur->folds & FOLD_TYPE_COMMENTPY) > 0? _T("1"): _T("0"));
        SetProperty (_T("fold.quotes.python"),
                     (cur->folds & FOLD_TYPE_QUOTESPY) > 0? _T("1"): _T("0"));
    }
    SetFoldFlags (wxSCI_FOLDFLAG_LINEBEFORE_CONTRACTED |
                  wxSCI_FOLDFLAG_LINEAFTER_CONTRACTED |
                  m_foldLevel);

    // set spaces and indention
    SetIndent (g_CommonPrefs.indentEnable? cur->indent: 0);
    SetBackSpaceUnIndents (true);
    SetTabIndents (true);
    SetTabWidth ((cur->indent > 0)? cur->indent: 4);
    SetUseTabs (!(cur->flags & EDIT_FLAG_USESPACES));

    // others
    SetViewEOL (g_CommonPrefs.displayEOLEnable);
    SetIndentationGuides (g_CommonPrefs.indentGuideEnable);
    SetEdgeColumn (cur->longln);
    SetEdgeMode (g_CommonPrefs.longLineOnEnable? wxSCI_EDGE_LINE: wxSCI_EDGE_NONE);
    SetViewWhiteSpace (g_CommonPrefs.whiteSpaceEnable?
                       wxSCI_WS_VISIBLEALWAYS: wxSCI_WS_INVISIBLE);
    SetOvertype (g_CommonPrefs.overTypeInitial);
    SetReadOnly (g_CommonPrefs.readOnlyInitial);
    SetWrapMode ((g_CommonPrefs.wrapModeInitial || (cur->flags & EDIT_FLAG_WRAPMODE))?
                 wxSCI_WRAP_WORD: wxSCI_WRAP_NONE);

    // check and initialize base language
    wxString basename = cur->basename;
    if (!basename.IsEmpty()) {
        InitializeLanguage (basename);
    }

    return true;
}

bool EditText::GetAnalyse () {
    wxString name = m_analyse->name;
    return (name != DEFAULT_LANGUAGE);
};

void EditText::InitializeAnalyse (const wxString &name) {
    // determine analyse info
    int nr;
    AnalyseInfo *cur = NULL;
    for (nr = 1; nr < g_AnalysePrefsSize; ++nr) {
        cur = &g_AnalysePrefs[nr];
        if (cur->name == name) break;
    }
    if (cur->name != name) nr = 0;
    m_analyse = &g_AnalysePrefs[nr];

    return;
}

void EditText::InitializeIdentTable () {
    m_identTable.DeleteContents (TRUE);
    m_colouriseLine = -1;
    Colourise (0, -1);
    ProcessIdentTable (-1);
    NotifyEditBrowse (myEVT_COMMON_NOTIFY, -1);
}

bool EditText::LoadFile () {

    // get filname
    if (!m_filename) {
        wxFileDialog dlg (this, _T("Open file"), _T(""), _T(""),
                          _T("Any file (*)|*"), wxOPEN | wxFILE_MUST_EXIST | wxCHANGE_DIR);
        if (dlg.ShowModal() != wxID_OK) return false;
        m_filename = dlg.GetPath();
    }

    // load file
    return LoadFile (m_filename);
}

bool EditText::LoadFile (const wxString &filename) {

    wxString buf;

    // load file in edit and clear undo
    if (!filename.IsEmpty()) m_filename = filename;
    m_textLoading = true;
#if !wxCHECK_VERSION(2, 5, 0)
    wxFile file (m_filename, wxFile::read);
    if (!file.IsOpened()) return false;
    long lng = file.Length ();
    if (lng > 0) {
        wxChar *pBuf = buf.GetWriteBuf (lng);
        file.Read (pBuf, lng);
        buf.UngetWriteBuf ();
        SetText (buf);
        EmptyUndoBuffer();
        SetSavePoint();
    }
    file.Close();
#else
    if (!wxScintilla::LoadFile (m_filename)) return false;
#endif
    m_filetime = wxFileName (m_filename).GetModificationTime();
    g_statustext = _("File loaded: ") + m_filename;

    // determine and set EOL mode
    int eolMode = -1;
    bool eolMix = false;
    wxString eolName;
    if (buf.Contains(_T("\r\n"))) {
        eolMode = wxSCI_EOL_CRLF;
        eolName = _("CR+LF (Windows)");
    }else if (buf.Contains(_T("\r"))) {
        if (eolMode != -1) {
            eolMix = true;
        }else{
            eolMode = wxSCI_EOL_CR;
            eolName = _("CR (Macintosh)");
        }
    }else if (buf.Contains(_T("\n"))) {
        if (eolMode != -1) {
            eolMix = true;
        }else{
            eolMode = wxSCI_EOL_LF;
            eolName = _("LF (Unix)");
        }
    }
    if (eolMode != -1) {
        if (eolMix &&
            wxMessageBox (_("Convert all line endings to ") + eolName + _("?"),
                          _("Line endings"), wxYES_NO | wxICON_QUESTION) == wxYES) {
            ConvertEOLs (eolMode);
            g_statustext = _("Converted line endings to ") + eolName;
        }
        SetEOLMode (eolMode);
    }

    // determine scrollbar width
    SetScrollWidth ();

    // determine lexer language
    wxFileName fname (m_filename);
    StyleClearAll();
    wxString language = DetermineLanguage (fname.GetFullName());
    InitializeLanguage (language);
    InitializeAnalyse (language);
    InitializeIdentTable ();
    if (g_CommonPrefs.showBrowse && GetAnalyse()) {
        ((wxSplitterWindow*)GetParent())->SetSashPosition (m_browseWidth);
    }else{
        ((wxSplitterWindow*)GetParent())->SetSashPosition (1);
    }
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
    wxToolBar *toolbar = ((wxFrame*)wxTheApp->GetTopWindow())->GetToolBar();
    if (toolbar) {
        wxComboBox *browsebox = (wxComboBox *)toolbar->FindControl (myID_BROWSESELECT);
        if (browsebox) browsebox->Enable (GetAnalyse());
    }
#endif
    m_textLoading = false;
    return true;
}

bool EditText::SaveFile () {

    // return if no change
    if (!Modified()) return true;

    // get filname
    if (!m_filename) {
        wxFileDialog dlg (this, _T("Save file"), _T(""), _T(""), _T("Any file (*)|*"),
                          wxSAVE | wxOVERWRITE_PROMPT);
        if (dlg.ShowModal() != wxID_OK) return false;
        m_filename = dlg.GetPath();
    }

    // save file
    return SaveFile (m_filename);
}

bool EditText::SaveFile (const wxString &filename) {

    if (g_CommonPrefs.convertToSpaces) TabsToSpaces();
    if (g_CommonPrefs.convertToTabs) SpacesToTabs();

    // return if no change
    if (!Modified() && filename.IsEmpty()) return true;

    // save edit in file and clear undo
    if (!filename.IsEmpty()) m_filename = filename;
    wxFile file (m_filename, wxFile::write);
    if (!file.IsOpened()) return false;
    wxString buf = GetText();
    bool okay = file.Write (buf);
    file.Close();
    if (!okay) return false;
    SetSavePoint();
//?    if (!filename.IsEmpty()) m_filename = filename;
//?    if (!wxScintilla::SaveFile (filename)) {
//?        wxMessageBox (_("Could not save file!"),
//?                      _("Save file"), wxOK | wxICON_INFORMATION, this);
//?        return false;
//?    }
    m_filetime = wxFileName (m_filename).GetModificationTime();
    g_statustext = _("File saved: ") + m_filename;

    return true;
}

bool EditText::Modified () {

    // return modified state
    return (GetModify() && !GetReadOnly());
}

void EditText::SpacesToTabs () {
    m_startpos = 0;
    SetTargetStart (m_startpos);
    SetTargetEnd (m_startpos);
    wxString findtext;
    int i;
    for (i = 0; i < GetTabWidth(); i++) findtext += _T(" ");
    BeginUndoAction();
    while (FindString (findtext, 0) >= 0) {
        ReplaceTarget (_T("\t"));
    }
    EndUndoAction();
}

void EditText::TabsToSpaces () {
    m_startpos = 0;
    SetTargetStart (m_startpos);
    SetTargetEnd (m_startpos);
    wxString replacetext;
    int i;
    for (i = 0; i < GetTabWidth(); i++) replacetext += _T(" ");
    BeginUndoAction();
    while (FindString (_T("\t"), 0) >= 0) {
        ReplaceTarget (replacetext);
    }
    EndUndoAction();
}

int EditText::FindString (const wxString &str, int flags) {

    // initialize direction
    bool down = (flags & myFR_DOWN) > 0;
    bool first;
    if (down) {
        SetTargetStart (GetTargetEnd());
        first = m_startpos <= GetTargetEnd();
        if (first) {
            SetTargetEnd (GetLength() - 1);
        }else{
            SetTargetEnd (m_startpos);
        }
    }else{
        SetTargetStart (GetTargetStart());
        first = m_startpos >= GetTargetStart();
        if (first) {
            SetTargetEnd (0);
        }else{
            SetTargetEnd (m_startpos);
        }
    }

    // set flags
    int findFlags = 0;
    if ((flags & myFR_MATCHCASE) > 0) findFlags |= wxSCI_FIND_MATCHCASE;
    if ((flags & myFR_WHOLEWORD) > 0) findFlags |= wxSCI_FIND_WHOLEWORD;
    if ((flags & myFR_FINDREGEX) > 0) findFlags |= wxSCI_FIND_REGEXP;
    SetSearchFlags (findFlags);

    // search string
    int pos = SearchInTarget (str);
    if (pos >= 0) return pos;

    // search the rest if any
    if (down && first) {
        SetTargetStart (0);
        SetTargetEnd (m_startpos);
    }else if (!down && first) {
        SetTargetStart (GetLength() - 1);
        SetTargetEnd (m_startpos);
    }else{
        return pos;
    }
    return pos = SearchInTarget (str);
}

bool EditText::FindInFiles (EditText *edit) {
    m_replace = false;
    if (edit->GetSelectionEnd()-edit->GetSelectionStart() > 0) {
        m_FindReplaceDlg->SetFindString (edit->GetSelectedText());
    }
    if (m_FindReplaceDlg->GetFindDirectory().IsEmpty()) {
        if (!edit->m_filename.IsEmpty()) {
            m_FindReplaceDlg->SetFindDirectory (wxFileName (edit->m_filename).GetPath (wxPATH_GET_VOLUME));
        }else{
            m_FindReplaceDlg->SetFindDirectory (wxGetCwd());
        }
    }
    m_FindReplaceDlg->SetFlags (m_FindReplaceDlg->GetFlags() | myFR_SUBFOLDER);
    if (m_FindReplaceDlg->GetFindFilespec().IsEmpty()) {
        m_FindReplaceDlg->SetFindFilespec (edit->m_language->filepattern);
    }
    if (m_FindReplaceDlg->ShowModal (myFR_FINDINFILES) != wxID_OK) return false;
    if (m_FindReplaceDlg->GetFindString ().IsEmpty()) {
        wxMessageBox (_("Can't find anything with empty string"),
                      _("Find string"), wxOK | wxICON_INFORMATION, this);
        return false;
    }
    return true;
}

int EditText::FindToken (int position, int length, int &pos, int &len) {
    len = 0;
    for (pos = position; pos < position+length; ++pos) if (GetStyleAt (pos) > TOKEN_DEFAULT) break;
    if (pos == pos+length) return TOKEN_DEFAULT;
    len = position + length - pos;
    int style = GetStyleAt (pos);
    int i;
    for (i = pos; i < pos+len; ++i) if (style != GetStyleAt (i)) break;
    len = i - pos;
    return style;
}

void EditText::SetScrollWidth (int ln) {
    int width = 0;
    int chWidth = GetCharWidth() + 1; // Why is GetCharWidth wrong (+1)???
#if defined(__WXGTK__)
    chWidth -= chWidth/6; // reduce GTK font width by 15%
#endif
    if (ln >= 0) {
        int width = (GetLineEndPosition (ln) - PositionFromLine (ln)) * chWidth;
        m_maxWidth = wxMax (m_maxWidth, width);
    }else{
        m_maxWidth = 0;
        for (ln = 0; ln < GetLineCount(); ++ln) {
            width = (GetLineEndPosition (ln) - PositionFromLine (ln)) * chWidth;
            m_maxWidth = wxMax (m_maxWidth, width);
        }
    }
    width = wxMax (GetClientSize().x -
                  ((g_CommonPrefs.lineNumberEnable? m_LineNrMargin: 0) +
                   (g_CommonPrefs.foldEnable? m_FoldingMargin: 0) +
                   m_DividerMargin),
                  m_maxWidth);
    if (width > 0) wxScintilla::SetScrollWidth (width);
    m_scrollWidthChange = false;
}

void EditText::PostEvent (wxWindow *win, wxCommandEvent& event,
                      int nr, const wxString& str) {
    if (!win) return;
    event.SetInt (nr);
    event.SetString (str);
    wxPostEvent (win, event);
}

IdentTableList::Node *EditText::GetIdentTableItem (wxString name, int level, IdentTableList::Node *ref, int *position) {
    IdentData *data;
    int p = 0;
    IdentTableList::Node *node = m_identTable.GetFirst();
    while (node) {
        data = node->GetData();
        p += data->m_diff;
        bool t = (data->m_name == name);
        t = (data->m_level == level);
        t = (data->m_ref == ref);
        if ((data->m_name == name) && (data->m_level == level) && (data->m_ref == ref)) {
            if (position) *position = p;
            return node;
        }
        node = node->GetNext();
    }
    if (position) *position = p;
    return NULL;
}

IdentTableList::Node *EditText::GetIdentTableItem (int pos, int *position) {
    IdentData *data;
    int p = 0;
    IdentTableList::Node *node = m_identTable.GetFirst();
    while (node) {
        data = node->GetData();
        p += data->m_diff;
        if ((pos  >= (p-data->m_diff)) && (pos < (p+data->m_length))) {
            if (position) *position = p;
            return node;
        }
        node = node->GetNext();
    }
    if (position) *position = p;
    return NULL;
}

IdentTableList::Node *EditText::GetIdentTableItem (IdentTableList::Node *cur, int *position) {
    IdentData *data;
    int p = 0;
    IdentTableList::Node *node = m_identTable.GetFirst();
    while (node) {
        data = node->GetData();
        p += data->m_diff;
        if (node == cur) {
            if (position) *position = p;
            return node;
        }
        node = node->GetNext();
    }
    if (position) *position = p;
    return NULL;
}

void EditText::IdentTableDelete (int pos, int len) {
    int position;
    IdentTableList::Node *cur = GetIdentTableItem (pos, &position);
    if (cur) {
        IdentData *data = cur->GetData();
        if ((pos > position-data->m_diff) && (pos+len <= position)) { // text deletion before node
            data->m_diff -= len;
        }else if ((pos >= position) && (pos+len <= position+data->m_length) &&
                  (len < data->m_length)) { // text deletion within node
        // text deletion before node
            data->m_length -= len;
            data->m_name = GetTextRange (position, position+data->m_length);
            NotifyEditBrowse (myEVT_COMMON_NOTIFY, position);
        }else{
            NotifyEditText (myEVT_COMMON_NOTIFY, -1);
        }
    }
}

void EditText::IdentTableInsert (int pos, int len) {
    if (len == 0) return; // Scintilla sends an empty event
    int position;
    IdentTableList::Node *cur = GetIdentTableItem (pos, &position);
    if (cur) {
        IdentData *data = cur->GetData();
        if ((pos > position-data->m_diff) && (pos+len <= position)) { // text insertion before node
            data->m_diff += len;
        }else if ((pos >= position) && (pos+len <= position+data->m_length) &&
                  (len < data->m_length)) { // text insertion within node
            data->m_length += len;
            data->m_name = GetTextRange (position, position+data->m_length);
            NotifyEditBrowse (myEVT_COMMON_NOTIFY, position);
        }else{
            NotifyEditText (myEVT_COMMON_NOTIFY, -1);
        }
    }
}

IdentTableList::Node *EditText::IdentTableUpdate (int identpos, int identlen, int level, int classpos, int classlen, int *lastpos) {
    int position;
    IdentData *data;
    wxString name = GetTextRange (classpos, classpos+classlen);
    IdentTableList::Node *ref = NULL;
    if (level > 1) {
        ref = GetIdentTableItem (name, 1, NULL);
        if (!ref) {
            data = new IdentData ();
            ref = m_identTable.Insert (m_identTable.GetCount(), data);
            data->m_name = name;
            data->m_length = name.Length();
            data->m_diff = classpos - *lastpos;
            data->m_level = 1;
            data->m_ref = NULL;
            *lastpos = classpos;
        }
    }
    wxString ident = GetTextRange (identpos, identpos+identlen);
    IdentTableList::Node *cur = GetIdentTableItem (identpos, &position);
    if (cur && (identpos == position)) {
        data = cur->GetData();
    }else{
        data = new IdentData ();
        if (cur) {
            cur = m_identTable.Insert (cur, data);
        }else{
            cur = m_identTable.Insert (m_identTable.GetCount(), data);
        }
    }
    data->m_name = ident;
    data->m_length = identlen;
    data->m_diff = identpos - *lastpos;
    data->m_level = level;
    data->m_ref = ref;
    *lastpos = identpos;
    return cur;
}

void EditText::ProcessIdentTable (int line) {
    int startLine;
    int endLine;
    if (line == -1) {
        m_identTable.Clear();
        startLine = 0;
        endLine = GetLineCount();
    }else{
        startLine = line;
        endLine = line + 1;
    }

    // determine info
    int position;
    int length;
    int pos;
    int len;
    int lastpos = 0;
    if (startLine > 0) {
        IdentTableList::Node *n = GetIdentTableItem (PositionFromLine (startLine), &lastpos);
        if (n) lastpos -= n->GetData()->m_diff;
    }
    int style;
    IdentTableList::Node *cur = NULL;
    IdentTableList::Node *ref = NULL;
    int nr;
    for (nr = startLine; nr < endLine; ++nr) {
        int fold = (GetFoldLevel (nr) & wxSCI_FOLDLEVELNUMBERMASK) - wxSCI_FOLDLEVELBASE;
        if (m_analyse->fold && (fold > 0)) continue;
        position = PositionFromLine (nr);
        length = GetLineEndPosition (nr) - position;
        wxString st = GetTextRange (position, position+length);
        pos = position;
        int count = 0;
        int state = 0;
        int classpos = 0;
        int classlen = 0;
        int identpos = 0;
        int identlen = 0;
        while (true) {
            style = FindToken (position, length, pos, len);
            count += 1;
            if (pos == position + length) break;
            if (state == 0) {
                cur = NULL;
                ref = NULL;
            }
            int i;
            for (i = 0; i < ANALYSE_COUNT; ++i) {
                // check correct syntax
                if ((m_analyse->rules[i].state != state) ||
                    (m_analyse->rules[i].style != m_language->styles[style].type)) {
                    continue;
                }
                if (m_analyse->rules[i].first && (count > 1)) {
                    continue;
                }
                wxString keyword = m_analyse->rules[i].keyword;
                if (!keyword.IsEmpty() && (keyword != GetTextRange (pos, pos+len))) {
                    wxString s = GetTextRange (pos, pos+len);
                    continue;
                }
                // save identifier for later use
                if ((m_analyse->rules[i].style == TOKEN_IDENTIFIER) &&
                    (m_analyse->rules[i].level > 0)) {
                    wxString s = GetTextRange (pos, pos+len);
                    if (m_analyse->rules[i].level > 1) {
                        classpos = identpos;
                        classlen = identlen;
                    }
                    identpos = pos;
                    identlen = len;
                }
                // store in ident table if correct
                if ((m_analyse->rules[i].level > 0) &&
                    (m_analyse->rules[i].next == 0)) {
                    wxString s = GetTextRange (classpos, classpos+classlen) +
                                 _T("::") +
                                 GetTextRange (identpos, identpos+identlen);
                    IdentTableUpdate (identpos, identlen, m_analyse->rules[i].level,
                                      classpos, classlen, &lastpos);
                }
                state = m_analyse->rules[i].next;
                break;
            }
            if (i == ANALYSE_COUNT) state = 0;
            length -= pos - position + len;
            position = pos + len;
        }
    }
}

void EditText::NotifyEditBrowse (int evtID, int position) {
    wxCommandEvent evt (evtID, -1);
    PostEvent (((wxSplitterWindow*)GetParent())->GetWindow1(), evt, position);
}

void EditText::NotifyEditText (int evtID, int position) {
    wxCommandEvent evt (evtID, -1);
    PostEvent (this, evt, position);
}


//----------------------------------------------------------------------------
// EditBrowse
//----------------------------------------------------------------------------

BEGIN_EVENT_TABLE (EditBrowse, wxTreeListCtrl)
    // common
    myEVT_COMMON_NOTIFY (              EditBrowse::OnCommonNotify)
    myEVT_COMMON_UPDATE (              EditBrowse::OnCommonUpdate)
    EVT_ERASE_BACKGROUND (             EditBrowse::OnEraseBackground)
    EVT_SIZE (                         EditBrowse::OnSize)
    EVT_TIMER (myID_UPDATETIMER,       EditBrowse::OnTimerEvent)
    // combobox
    EVT_COMBOBOX (myID_BROWSESELECT,   EditBrowse::OnBrowseSelect)
    // tree
    EVT_TREE_ITEM_ACTIVATED (-1,       EditBrowse::OnItemActivated)
END_EVENT_TABLE()

EditBrowse::EditBrowse (wxWindow *parent, wxWindowID id,
                        EditText *edit,
                        const wxPoint &pos,
                        const wxSize &size,
                        long style)
          : wxTreeListCtrl (parent, id, pos, size, style | wxCLIP_CHILDREN |
                                                           wxTR_TWIST_BUTTONS |
                                                           wxTR_HIDE_ROOT) {

    m_edit = edit;
    AddColumn (_T("Class"), GetClientSize().x, wxALIGN_LEFT);
    AddRoot (_("Class browser"));
    SetBackgroundColour (_T("WHITE"));

    m_startLine= 0;
    m_endLine = 0;
}

EditBrowse::~EditBrowse() {
}

// event handlers
void EditBrowse::OnCommonNotify (wxCommandEvent &WXUNUSED(event)) {
    BrowseCreate ();
    ComboboxCreate ();
}

void EditBrowse::OnCommonUpdate (wxCommandEvent &WXUNUSED(event)) {
    BrowseCreate ();
    ComboboxCreate ();
}

void EditBrowse::OnSize (wxSizeEvent &event) {
    SetColumnWidth (GetMainColumn(), GetClientSize().x);
    event.Skip();
}

void EditBrowse::OnTimerEvent (wxTimerEvent &WXUNUSED(event)) {
    int currentLine = m_edit->GetCurrentLine();
    if ((currentLine >= m_startLine) && (currentLine < m_endLine)) return; // do nothing
    BrowseUpdate ();
    ComboboxUpdate ();
}

void EditBrowse::OnBrowseSelect (wxCommandEvent &WXUNUSED(event)) {
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
    wxToolBar *toolbar = ((wxFrame*)wxTheApp->GetTopWindow())->GetToolBar();
    if (!toolbar) return;
    wxComboBox *browsebox = (wxComboBox *)toolbar->FindControl (myID_BROWSESELECT);
    if (!browsebox) return;
    wxTreeItemId item = GetRootItem();
    if (!item.IsOk()) return;
    if (HasFlag(wxTR_HIDE_ROOT)) {
#if !wxCHECK_VERSION(2, 5, 0)
        long cookie = 0;
#else
        wxTreeItemIdValue cookie = 0;
#endif
        item = GetFirstChild (item, cookie);
    }
    if (browsebox->GetSelection() == 0) {
        m_edit->GotoLine (0);
        m_edit->SetFocus();
        return;
    }
    if (!item.IsOk()) return;
    item = GetNext (item);
    int sel;
    for (sel = 1; sel < browsebox->GetSelection(); ++sel) {
        if (!item.IsOk()) return;
        item = GetNext (item);
    }
    IdentTableList::Node *cur = ((EditBrowseData *) GetItemData (item))->GetNode();
    int position;
    m_edit->GetIdentTableItem (cur, &position);
    if (cur) {
        m_edit->GotoLine (m_edit->LineFromPosition (position));
        int line = m_edit->GetCurrentLine() - (m_edit->GetFirstVisibleLine()+2);
        if (line > 0) m_edit->LineScroll (0, line);
        m_edit->SetFocus();
    }
#endif
}

void EditBrowse::OnItemActivated (wxTreeEvent& event) {
    int position;
    if (GetItemText(event.GetItem()) == HOMEPOSITION) {
        m_edit->GotoLine (0);
        m_edit->SetFocus();
        return;
    }
    IdentTableList::Node *cur = ((EditBrowseData *) GetItemData (event.GetItem()))->GetNode();
    m_edit->GetIdentTableItem (cur, &position);
    if (cur) {
        m_edit->GotoLine (m_edit->LineFromPosition (position));
        int line = m_edit->GetCurrentLine() - (m_edit->GetFirstVisibleLine()+2);
        if (line > 0) m_edit->LineScroll (0, line);
        m_edit->SetFocus();
    }

}

// functions
void EditBrowse::BrowseCreate () {
    Freeze();
    IdentData *data;
    IdentData *ref;
    int position = 0;
    int currentLine = m_edit->GetCurrentLine();
    wxTreeItemId root = GetRootItem();
    DeleteChildren (root);
    wxTreeItemId prevIdent = AppendItem (root, HOMEPOSITION);
    wxTreeItemId selectIdent = (wxTreeItemId*)NULL;
    wxTreeItemId classIdent;
    wxTreeItemId functIdent;
#if !wxCHECK_VERSION(2, 5, 0)
    long classCookie = 0;
    long functCookie = 0;
#else
    wxTreeItemIdValue classCookie = 0;
    wxTreeItemIdValue functCookie = 0;
#endif
    IdentTableList::Node *node = m_edit->m_identTable.GetFirst();
    while (node) {
        data = node->GetData();
        int startLine = m_edit->LineFromPosition (position);
        int endLine = m_edit->LineFromPosition (position+data->m_diff);
        if ((currentLine >= startLine) && (currentLine < endLine)) {
            m_startLine = startLine;
            m_endLine = endLine;
            selectIdent = prevIdent;
        }
        position += data->m_diff;
        if (data->m_level == 1) {
            classIdent = GetFirstChild (root, classCookie);
            while (classIdent.IsOk() && (GetItemText (classIdent) != data->m_name)) {
                classIdent = GetNextChild (root, classCookie);
            }
            if (!classIdent.IsOk()) {
                classIdent = AppendItem (root, data->m_name, -1, -1, new EditBrowseData (node));
#if !defined(__WXGTK__)
                SetItemBold (classIdent);
#endif
            }
            prevIdent = classIdent;
        }else if (data->m_level == 2) {
            ref = data->m_ref->GetData();
            classIdent = GetFirstChild (root, classCookie);
            while (classIdent.IsOk() && (GetItemText (classIdent) != ref->m_name)) {
                classIdent = GetNextChild (root, classCookie);
            }
            if (classIdent.IsOk()) {
                functIdent = GetFirstChild (classIdent, functCookie);
                while (functIdent.IsOk() && (GetItemText (functIdent) != data->m_name)) {
                    functIdent = GetNextChild (classIdent, functCookie);
                }
                if (!functIdent.IsOk()) {
                    functIdent = AppendItem (classIdent, data->m_name, -1, -1, new EditBrowseData (node));
                }else{
                    ((EditBrowseData *) GetItemData (functIdent))->SetNode (node);
                }
            }
            prevIdent = functIdent;
        }
        node = node->GetNext();
    }
    ExpandAll (root);
    if (selectIdent) SelectItem (selectIdent);
    if (selectIdent) EnsureVisible (selectIdent);
    Thaw();
}

void EditBrowse::BrowseUpdate () {
    IdentData *data;
    IdentData *ref;
    int position = 0;
    int currentLine = m_edit->GetCurrentLine();
    wxTreeItemId root = GetRootItem();
    wxTreeItemId prevIdent = root;
    wxTreeItemId classIdent;
    wxTreeItemId functIdent;
#if !wxCHECK_VERSION(2, 5, 0)
    long classCookie = 0;
    long functCookie = 0;
#else
    wxTreeItemIdValue classCookie = 0;
    wxTreeItemIdValue functCookie = 0;
#endif
    IdentTableList::Node *node = m_edit->m_identTable.GetFirst();
    while (node) {
        data = node->GetData();
        int startLine = m_edit->LineFromPosition (position);
        int endLine = m_edit->LineFromPosition (position+data->m_diff);
        if ((currentLine >= startLine) && (currentLine < endLine)) {
            m_startLine = startLine;
            m_endLine = endLine;
            SelectItem (prevIdent);
            EnsureVisible (prevIdent);
            break;
        }
        position += data->m_diff;
        if (data->m_level == 1) {
            classIdent = GetFirstChild (root, classCookie);
            while (classIdent.IsOk() && (GetItemText (classIdent) != data->m_name)) {
                classIdent = GetNextChild (root, classCookie);
            }
            prevIdent = classIdent;
        }else if (data->m_level == 2) {
            ref = data->m_ref->GetData();
            classIdent = GetFirstChild (root, classCookie);
            while (classIdent.IsOk() && (GetItemText (classIdent) != ref->m_name)) {
                classIdent = GetNextChild (root, classCookie);
            }
            if (classIdent.IsOk()) {
                functIdent = GetFirstChild (classIdent, functCookie);
                while (functIdent.IsOk() && (GetItemText (functIdent) != data->m_name)) {
                    functIdent = GetNextChild (classIdent, functCookie);
                }
            }
            prevIdent = functIdent;
        }
        node = node->GetNext();
    }
}

void EditBrowse::ComboboxCreate () {
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
    wxToolBar *toolbar = ((wxFrame*)wxTheApp->GetTopWindow())->GetToolBar();
    if (!toolbar) return;
    wxComboBox *browsebox = (wxComboBox *)toolbar->FindControl (myID_BROWSESELECT);
    if (!browsebox) return;
    browsebox->Clear();
    wxTreeItemId item = GetRootItem();
    if (!item.IsOk()) return;
    if (HasFlag(wxTR_HIDE_ROOT)) {
#if !wxCHECK_VERSION(2, 5, 0)
        long cookie = 0;
#else
        wxTreeItemIdValue cookie = 0;
#endif
        item = GetFirstChild (item, cookie);
    }
    IdentData *data;
    int selection = 0;
    int countItem = 0;
    if (item.IsOk()) {
        browsebox->Append (HOMEPOSITION);
        countItem += 1;
        item = GetNext (item);
    }
    while (item.IsOk()) {
        data = ((EditBrowseData *) GetItemData (item))->GetNode()->GetData();
        if (IsSelected (item)) {
            selection = countItem;
        }
        if (data->m_level == 1) {
            browsebox->Append (data->m_name);
        }else if (data->m_level == 2) {
            browsebox->Append (LEVEL2PREFIX + data->m_name);
        }
        countItem += 1;
        item = GetNext (item);
    }
    browsebox->SetSelection (selection);
#endif
}

void EditBrowse::ComboboxUpdate () {
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
    wxToolBar *toolbar = ((wxFrame*)wxTheApp->GetTopWindow())->GetToolBar();
    if (!toolbar) return;
    wxComboBox *browsebox = (wxComboBox *)toolbar->FindControl (myID_BROWSESELECT);
    if (!browsebox) return;
    wxTreeItemId item = GetRootItem();
    if (!item.IsOk()) return;
    if (HasFlag(wxTR_HIDE_ROOT)) {
#if !wxCHECK_VERSION(2, 5, 0)
        long cookie = 0;
#else
        wxTreeItemIdValue cookie = 0;
#endif
        item = GetFirstChild (item, cookie);
    }
    IdentData *data;
    int selection = 0;
    int countItem = 0;
    if (item.IsOk()) {
        countItem += 1;
        item = GetNext (item);
    }
    while (item.IsOk()) {
        data = ((EditBrowseData *) GetItemData (item))->GetNode()->GetData();
        if (IsSelected (item)) {
            selection = countItem;
        }
        countItem += 1;
        item = GetNext (item);
    }
    browsebox->SetSelection (selection);
#endif
}

bool EditBrowse::ExistNode (IdentTableList::Node *cur) {
    IdentTableList::Node *node = m_edit->m_identTable.GetFirst();
    while (node) {
        if (node == cur) return true;
        node = node->GetNext();
    }
    return false;
}


//----------------------------------------------------------------------------
// EditFindThread
//----------------------------------------------------------------------------

EditFindThread::EditFindThread (EditText *result)
              : wxThread () {

    // init parameter
    m_pending = 0;
    m_result = result;
    m_findstr = m_result->m_FindReplaceDlg->GetFindString();
    m_finddir = m_result->m_FindReplaceDlg->GetFindDirectory ();
    m_filespec = m_result->m_FindReplaceDlg->GetFindFilespec ();
    m_findflg = m_result->m_FindReplaceDlg->GetFlags ();

    m_filespec.Pad (1, '^', false);
    m_filespec.Pad (1, '$', true);
    m_filespec.Replace (_T(","), _T("|"));
    m_filespec.Replace (_T(";"), _T("|"));
    m_filespec.Replace (_T("|"), _T("$|^"));
    m_filespec.Replace (_T("."), _T("\\."));
    m_filespec.Replace (_T("*"), _T(".*"));
    m_filespec.Replace (_T("?"), _T("."));
    m_fstr.Compile (m_findstr, ((m_findflg & myFR_MATCHCASE)? 0: wxRE_ICASE) |
                               ((m_findflg & myFR_FINDREGEX)? 0: wxRE_BASIC));
#if defined (__WXMSW__)
    m_fspec.Compile (m_filespec, (wxRE_ICASE));
#else
    m_fspec.Compile (m_filespec);
#endif

}

void *EditFindThread::Entry() {
    m_pending = 1;
    WriteText (_T("Find all occurence of \"") + m_findstr +
               _T("\" in \"") + m_finddir + _T("\"\n"));
    int count = FindInFiles (m_finddir);
    if (m_pending < 0) {
        WriteText (_T("Find all occurence ***canceled***!\n\n"));
    }else{
        WriteText (wxString::Format (_T("Found: %d"), count) +
                   _T(" occurences\n\n"));
    }
    return NULL;
}

void EditFindThread::OnExit() {
}

int EditFindThread::FindInFiles (const wxString &finddir) {
    wxDir dir (finddir);
    if (!dir.IsOpened()) return 0;
    wxString filename;
    wxString line;
    wxString str;
    int ln;
    int count = 0;
    bool more = dir.GetFirst (&filename);
    while (more) {
        wxString fname = finddir + wxFileName::GetPathSeparator() + filename;
        if (!m_filespec.IsEmpty() && !m_fspec.Matches (filename) &&
            !wxFileName::DirExists (fname)) {
            more = dir.GetNext (&filename);
            continue;
        }
        if (wxFileName::DirExists (fname)) {
            if ((m_findflg & myFR_SUBFOLDER) > 0) count += FindInFiles (fname);
            more = dir.GetNext (&filename);
            continue;
        }
        if (TestDestroy()) {
            m_pending = -1;
            return count; // end if thread break
        }
        wxTextFile file (fname);
        if (file.Open() && !file.Eof()) {
            for (str = file.GetFirstLine(); !file.Eof(); str = file.GetNextLine()) {
                if (TestDestroy()) { // end if thread break
                    file.Close();
                    m_pending = -1;
                    return count;
                }
                if (m_fstr.Matches (str)) {
                    ln = (int)file.GetCurrentLine() + 1;
                    WriteText (fname +
                               wxString::Format (_T(" (%d):\t"), ln) +
                               str + _T("\n"));
                    count += 1;
                }
            }
        }
        file.Close();
        more = dir.GetNext (&filename);
    }
    m_pending = 0;
    return count;
}

void EditFindThread::WriteText(const wxString& line) {
    // send it to the main thread
    wxCommandEvent evt (myEVT_RESULT_FOUND, myEVT_RESULTFOUND);
    evt.SetInt (m_pending);
    evt.SetString (line);
    wxPostEvent ((wxFrame*)wxTheApp->GetTopWindow(), evt);
    Yield();
}

//----------------------------------------------------------------------------
// EditProperties
//----------------------------------------------------------------------------

EditProperties::EditProperties (EditText *edit,
                                long style)
              : wxDialog (edit, -1, wxEmptyString,
                          wxDefaultPosition, wxDefaultSize,
                          style|wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER) {

    // sets the application title
    SetTitle (_("Properties"));
    wxString text;

    // fullname
    wxBoxSizer *fullname = new wxBoxSizer (wxHORIZONTAL);
    fullname->Add (10, 0);
    fullname->Add (new wxStaticText (this, -1, _("Filename:"),
                                     wxDefaultPosition, wxSize(80, -1)),
                   0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    fullname->Add (new wxStaticText (this, -1, edit->GetFilename()),
                   0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);

    // text info
    wxGridSizer *textinfo = new wxGridSizer (4, 0, 2);
    textinfo->Add (new wxStaticText (this, -1, _("Language:"),
                                     wxDefaultPosition, wxSize(80, -1)),
                   0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxLEFT, 4);
    textinfo->Add (new wxStaticText (this, -1, edit->m_language->name),
                   0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxRIGHT, 4);
    textinfo->Add (new wxStaticText (this, -1, _("Lexer-ID = "),
                                     wxDefaultPosition, wxSize(80, -1)),
                   0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxLEFT, 4);
    text = wxString::Format (_T("%d"), edit->GetLexer());
    textinfo->Add (new wxStaticText (this, -1, text),
                   0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxRIGHT, 4);
    wxString EOLtype = _T("");
    switch (edit->GetEOLMode()) {
        case wxSCI_EOL_CRLF: {EOLtype = _T("CRLF (Windows)"); break; }
        case wxSCI_EOL_CR: {EOLtype = _T("CR (Macintosh)"); break; }
        case wxSCI_EOL_LF: {EOLtype = _T("LF (Unix)"); break; }
    }
    textinfo->Add (new wxStaticText (this, -1, _("Line endings:"),
                                     wxDefaultPosition, wxSize(80, -1)),
                   0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxLEFT, 4);
    textinfo->Add (new wxStaticText (this, -1, EOLtype),
                   0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxRIGHT, 4);

    // text info box
    wxStaticBoxSizer *textinfos = new wxStaticBoxSizer (
                     new wxStaticBox (this, -1, _("Information")),
                     wxVERTICAL);
    textinfos->Add (textinfo, 0, wxEXPAND);
    textinfos->Add (0, 6);

    // statistic
    wxGridSizer *statistic = new wxGridSizer (4, 0, 2);
    statistic->Add (new wxStaticText (this, -1, _("Total lines:"),
                                     wxDefaultPosition, wxSize(80, -1)),
                    0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxLEFT, 4);
    text = wxString::Format (_T("%d"), edit->GetLineCount());
    statistic->Add (new wxStaticText (this, -1, text),
                    0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxRIGHT, 4);
    statistic->Add (new wxStaticText (this, -1, _("Total chars:"),
                                     wxDefaultPosition, wxSize(80, -1)),
                    0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxLEFT, 4);
    text = wxString::Format (_T("%d"), edit->GetLength());
    statistic->Add (new wxStaticText (this, -1, text),
                    0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxRIGHT, 4);
    statistic->Add (new wxStaticText (this, -1, _("Current line:"),
                                     wxDefaultPosition, wxSize(80, -1)),
                    0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxLEFT, 4);
    text = wxString::Format (_T("%d"), edit->GetCurrentLine());
    statistic->Add (new wxStaticText (this, -1, text),
                    0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxRIGHT, 4);
    statistic->Add (new wxStaticText (this, -1, _("Current pos:"),
                                     wxDefaultPosition, wxSize(80, -1)),
                    0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxLEFT, 4);
    text = wxString::Format (_T("%d"), edit->GetCurrentPos());
    statistic->Add (new wxStaticText (this, -1, text),
                    0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxRIGHT, 4);

    // char/line statistics
    wxStaticBoxSizer *statistics = new wxStaticBoxSizer (
                     new wxStaticBox (this, -1, _("Statistics")),
                     wxVERTICAL);
    statistics->Add (statistic, 0, wxEXPAND);
    statistics->Add (0, 6);

    // total pane
    wxBoxSizer *totalpane = new wxBoxSizer (wxVERTICAL);
    totalpane->Add (fullname, 0, wxEXPAND | wxLEFT | wxRIGHT | wxTOP, 10);
    totalpane->Add (0, 6);
    totalpane->Add (textinfos, 0, wxEXPAND | wxLEFT | wxRIGHT, 10);
    totalpane->Add (0, 10);
    totalpane->Add (statistics, 0, wxEXPAND | wxLEFT | wxRIGHT, 10);
    totalpane->Add (0, 6);
    wxButton *okButton = new wxButton (this, wxID_CANCEL, _("OK"));
    okButton->SetDefault();
    totalpane->Add (okButton, 0, wxALIGN_CENTER | wxALL, 10);

    SetSizerAndFit (totalpane);

    ShowModal();
}

//----------------------------------------------------------------------------
// EditPrint
//----------------------------------------------------------------------------

EditPrint::EditPrint (EditText *edit, wxChar *title)
         : wxPrintout(title) {
    m_edit = edit;
    m_printed = 0;

}

bool EditPrint::OnPrintPage (int page) {

    wxDC *dc = GetDC();
    if (!dc) return false;

    // scale DC
    PrintScaling (dc);

    // print page
    if (page == 1) m_printed = 0;
    m_printed = m_edit->FormatRange (1, m_printed, m_edit->GetLength(),
                                     dc, dc, m_printRect, m_pageRect);

    return true;
}

bool EditPrint::OnBeginDocument (int startPage, int endPage) {

    if (!wxPrintout::OnBeginDocument (startPage, endPage)) {
        return false;
    }

    return true;
}

void EditPrint::GetPageInfo (int *minPage, int *maxPage, int *selPageFrom, int *selPageTo) {

    // initialize values
    *minPage = 0;
    *maxPage = 0;
    *selPageFrom = 0;
    *selPageTo = 0;

    // scale DC if possible
    wxDC *dc = GetDC();
    if (!dc) return;
    PrintScaling (dc);

    // get print page information and convert to printer pixels
    wxSize ppiScr;
    GetPPIScreen (&ppiScr.x, &ppiScr.y);
    wxSize page = g_pageSetupData->GetPaperSize();
    page.x = static_cast<int> (page.x * ppiScr.x / 25.4);
    page.y = static_cast<int> (page.y * ppiScr.y / 25.4);
    m_pageRect = wxRect (0,
                         0,
                         page.x,
                         page.y);

    // get margins information and convert to printer pixels
    int  top = 25; // default 25
    int  bottom = 25; // default 25
    int  left = 20; // default 20
    int  right = 20; // default 20
    wxPoint (top, left) = g_pageSetupData->GetMarginTopLeft();
    wxPoint (bottom, right) = g_pageSetupData->GetMarginBottomRight();
    top = static_cast<int> (top * ppiScr.y / 25.4);
    bottom = static_cast<int> (bottom * ppiScr.y / 25.4);
    left = static_cast<int> (left * ppiScr.x / 25.4);
    right = static_cast<int> (right * ppiScr.x / 25.4);
    m_printRect = wxRect (left,
                          top,
                          page.x - (left + right),
                          page.y - (top + bottom));

    // count pages
    while (HasPage (*maxPage)) {
        m_printed = m_edit->FormatRange (0, m_printed, m_edit->GetLength(),
                                         dc, dc, m_printRect, m_pageRect);
        *maxPage += 1;
    }
    if (*maxPage > 0) *minPage = 1;
    *selPageFrom = *minPage;
    *selPageTo = *maxPage;
    m_printed = 0;
}

bool EditPrint::HasPage (int WXUNUSED(page)) {

    return (m_printed < m_edit->GetLength());
}

bool EditPrint::PrintScaling (wxDC *dc){

    // check for dc, return if none
    if (!dc) return false;

    // get printer and screen sizing values
    wxSize ppiScr;
    GetPPIScreen (&ppiScr.x, &ppiScr.y);
    if (ppiScr.x == 0) { // most possible guess 96 dpi
        ppiScr.x = 96;
        ppiScr.y = 96;
    }
    wxSize ppiPrt;
    GetPPIPrinter (&ppiPrt.x, &ppiPrt.y);
    if (ppiPrt.x == 0) { // scaling factor to 1
        ppiPrt.x = ppiScr.x;
        ppiPrt.y = ppiScr.y;
    }
    wxSize dcSize = dc->GetSize();
    wxSize pageSize;
    GetPageSizePixels (&pageSize.x, &pageSize.y);

    // set user scale
    float scale_x = (float)(ppiPrt.x * dcSize.x) /
                    (float)(ppiScr.x * pageSize.x);
    float scale_y = (float)(ppiPrt.y * dcSize.y) /
                    (float)(ppiScr.y * pageSize.y);
    dc->SetUserScale (scale_x, scale_y);

    return true;
}

