//////////////////////////////////////////////////////////////////////////////
// File:        edit.h
// Purpose:     Edit main module
// Maintainer:  Otto Wyss
// Created:     2003-01-20
// RCS-ID:      $Id: edit.h,v 1.1 2005/03/17 20:03:43 wyo Exp $
// Copyright:   (c) 2005 wyoGuide
// Licence:     wxWindows
//////////////////////////////////////////////////////////////////////////////

#ifndef _EDIT_H_
#define _EDIT_H_

//----------------------------------------------------------------------------
// information
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
// headers
//----------------------------------------------------------------------------

//! wxWidgets headers
#include <wx/list.h>     // list support
#include <wx/regex.h>    // regex support
#include <wx/treectrl.h> // tree support

//! wxCode headers
#include <wx/wxscintilla.h>  // styled text control
#include "wx/treelistctrl.h" // tree support

//! application headers
#include "dialogs.h"     // dialogs
#include "prefs.h"       // preferences


//============================================================================
// declarations
//============================================================================

class EditPrint;
class EditProperties;

BEGIN_DECLARE_EVENT_TYPES()
    DECLARE_EVENT_TYPE(myEVT_COMMON_NOTIFY,   wxEVT_USER_FIRST+1)
    DECLARE_EVENT_TYPE(myEVT_COMMON_UPDATE,   wxEVT_USER_FIRST+2)
    DECLARE_EVENT_TYPE(myEVT_RESULT_FOUND,    wxEVT_USER_FIRST+3)
    DECLARE_EVENT_TYPE(myEVT_RESULT_SELECTED, wxEVT_USER_FIRST+4)
END_DECLARE_EVENT_TYPES()

#define myEVT_COMMON_NOTIFY(fn) DECLARE_EVENT_TABLE_ENTRY (myEVT_COMMON_NOTIFY, -1, -1, \
    (wxObjectEventFunction) (wxEventFunction) (wxCommandEventFunction) & fn, (wxObject *) NULL),
#define myEVT_COMMON_UPDATE(fn) DECLARE_EVENT_TABLE_ENTRY (myEVT_COMMON_UPDATE, -1, -1, \
    (wxObjectEventFunction) (wxEventFunction) (wxCommandEventFunction) & fn, (wxObject *) NULL),
#define myEVT_RESULT_FOUND(fn) DECLARE_EVENT_TABLE_ENTRY (myEVT_RESULT_FOUND, -1, -1, \
    (wxObjectEventFunction) (wxEventFunction) (wxCommandEventFunction) & fn, (wxObject *) NULL),
#define myEVT_RESULT_SELECTED(fn) DECLARE_EVENT_TABLE_ENTRY (myEVT_RESULT_SELECTED, -1, -1, \
    (wxObjectEventFunction) (wxEventFunction) (wxCommandEventFunction) & fn, (wxObject *) NULL),

//----------------------------------------------------------------------------
//! IdentTableList
class IdentData;
WX_DECLARE_LIST (IdentData, IdentTableList);


//----------------------------------------------------------------------------
//! EditText
class EditText: public wxScintilla {
    friend class EditBrowse;
    friend class EditFindThread;
    friend class EditProperties;
    friend class EditPrint;

public:
    //! constructor
    EditText (wxWindow *parent, wxWindowID id = -1,
              int type = edit_EDIT,
              int browseWidth = 1,
              const wxPoint &pos = wxDefaultPosition,
              const wxSize &size = wxDefaultSize,
              long style = 0);

    //! destructor
    ~EditText ();

    // event handlers
    // common
    void OnCommonNotify (wxCommandEvent &event);
    void OnContextMenu (wxMouseEvent &event);
    void OnEraseBackground (wxEraseEvent &WXUNUSED(event)) { /* avoid flickering */ }
    // edit
    void OnEditClear (wxCommandEvent &event);
    void OnEditClearUI (wxUpdateUIEvent &event);
    void OnEditCut (wxCommandEvent &event);
    void OnEditCutUI (wxUpdateUIEvent &event);
    void OnEditCopy (wxCommandEvent &event);
    void OnEditCopyUI (wxUpdateUIEvent &event);
    void OnEditPaste (wxCommandEvent &event);
    void OnEditPasteUI (wxUpdateUIEvent &event);
    void OnEditIndentInc (wxCommandEvent &event);
    void OnEditIndentIncUI (wxUpdateUIEvent &event);
    void OnEditIndentRed (wxCommandEvent &event);
    void OnEditIndentRedUI (wxUpdateUIEvent &event);
    void OnEditSelectAll (wxCommandEvent &event);
    void OnEditSelectAllUI (wxUpdateUIEvent &event);
    void OnEditSelectLine (wxCommandEvent &event);
    void OnEditSelectLineUI (wxUpdateUIEvent &event);
    void OnEditRedo (wxCommandEvent &event);
    void OnEditRedoUI (wxUpdateUIEvent &event);
    void OnEditUndo (wxCommandEvent &event);
    void OnEditUndoUI (wxUpdateUIEvent &event);
    // find
    void OnFind (wxCommandEvent &event);
    void OnFindUI (wxUpdateUIEvent &event);
    void OnFindNext (wxCommandEvent &event);
    void OnFindNextUI (wxUpdateUIEvent &event);
    void OnReplace (wxCommandEvent &event);
    void OnReplaceUI (wxUpdateUIEvent &event);
    void OnReplaceNext (wxCommandEvent &event);
    void OnReplaceNextUI (wxUpdateUIEvent &event);
    void OnReplaceAll (wxCommandEvent &event);
    void OnBraceMatch (wxCommandEvent &event);
    void OnBraceMatchUI (wxUpdateUIEvent &event);
    void OnGoto (wxCommandEvent &event);
    void OnGotoUI (wxUpdateUIEvent &event);
    void OnBookmarkToggle (wxCommandEvent &event);
    void OnBookmarkToggleUI (wxUpdateUIEvent &event);
    void OnBookmarkNext (wxCommandEvent &event);
    void OnBookmarkNextUI (wxUpdateUIEvent &event);
    void OnBookmarkPrev (wxCommandEvent &event);
    void OnBookmarkPrevUI (wxUpdateUIEvent &event);
    void OnBookmarkClear (wxCommandEvent &event);
    void OnBookmarkClearUI (wxUpdateUIEvent &event);
    //! view
    void OnHilightLang (wxCommandEvent &event);
    void OnHilightLangUI (wxUpdateUIEvent &event);
    void OnDisplayEOL (wxCommandEvent &event);
    void OnDisplayEOLUI (wxUpdateUIEvent &event);
    void OnFoldLevel (wxCommandEvent &event);
    void OnFoldLevelUI (wxUpdateUIEvent &event);
    void OnIndentGuide (wxCommandEvent &event);
    void OnIndentGuideUI (wxUpdateUIEvent &event);
    void OnLineNumber (wxCommandEvent &event);
    void OnLineNumberUI (wxUpdateUIEvent &event);
    void OnLongLineOn (wxCommandEvent &event);
    void OnLongLineOnUI (wxUpdateUIEvent &event);
    void OnWhiteSpace (wxCommandEvent &event);
    void OnWhiteSpaceUI (wxUpdateUIEvent &event);
    void OnFoldToggle (wxCommandEvent &event);
    void OnFoldToggleUI (wxUpdateUIEvent &event);
    void OnSetOverType (wxCommandEvent &event);
    void OnSetOverTypeUI (wxUpdateUIEvent &event);
    void OnSetReadOnly (wxCommandEvent &event);
    void OnSetReadOnlyUI (wxUpdateUIEvent &event);
    void OnWrapmodeOn (wxCommandEvent &event);
    void OnWrapmodeOnUI (wxUpdateUIEvent &event);
    void OnUseCharset (wxCommandEvent &event);
    void OnUseCharsetUI (wxUpdateUIEvent &event);
    //! extra
    void OnChangeCase (wxCommandEvent &event);
    void OnChangeCaseUI (wxUpdateUIEvent &event);
    void OnConvertEOL (wxCommandEvent &event);
    void OnConvertEOLUI (wxUpdateUIEvent &event);
    void OnWhitespaceToUI (wxUpdateUIEvent &event);
    void OnSpacesToTabs (wxCommandEvent &event);
    void OnTabsToSpaces (wxCommandEvent &event);
    void OnTrailingSpace (wxCommandEvent &event);
    void OnTrailingSpaceUI (wxUpdateUIEvent &event);
    // scintilla
    void OnResultSelected (wxScintillaEvent &event);
    void OnMarginClick (wxScintillaEvent &event);
    void OnModified  (wxScintillaEvent &event);
    void OnCharAdded  (wxScintillaEvent &event);


    //! language/lexer
    wxString DetermineLanguage (const wxString &filename);
    bool InitializeLanguage (const wxString &filename);
    bool UserSettings (const wxString &filename);
    LanguageInfo* GetLanguageInfo () {return m_language;};

    // analyse info
    bool GetAnalyse ();
    void InitializeAnalyse (const wxString &filename);
    void InitializeIdentTable ();

    //! load/save file
    bool LoadFile ();
    bool LoadFile (const wxString &filename);
    bool SaveFile ();
    bool SaveFile (const wxString &filename);
    bool Modified ();
    wxString GetFilename () {return m_filename;};
    void SetFilename (const wxString &filename) {m_filename = filename;};
    wxDateTime GetModificationTime () {return m_filetime;};
    void SetModificationTime (wxDateTime filetime) {m_filetime = filetime;};

    void SpacesToTabs ();
    void TabsToSpaces ();

    bool FindInFiles (EditText *edit);

    bool IsEdit () {return (m_type == edit_EDIT);};
    bool IsResult () {return (m_type == edit_RESULT);};

    void PostEvent (wxWindow *win, wxCommandEvent& event,
                    int nr  = -1, const wxString& str = wxEmptyString);

    // identifer information update trigger
    void NotifyEditBrowse (int evtID, int position);
    void NotifyEditText (int evtID, int position);

private:
    int m_type;

    // file
    bool m_textLoading;
    wxString m_filename;
    wxDateTime m_filetime;

    // common variables
    wxMenu *m_ContextMenu;
    wxMenu *m_menuChangeCase;

    // create context menu
    void createContextMenu ();

    // find variables
    myFindReplaceDlg *m_FindReplaceDlg;
    int FindString (const wxString &str, int flags);
    int FindToken (int position, int length, int &pos, int &len);
    int m_startpos;
    bool m_replace;
    myGotoDlg *m_GotoDlg;

    // lanugage properties
    LanguageInfo* m_language;

    // analyse info
    AnalyseInfo* m_analyse;

    // margin variables
    int m_LineNrID;
    int m_LineNrMargin;
    int m_FoldingID;
    int m_FoldingMargin;
    int m_DividerID;
    int m_DividerMargin;

    // fold level display
    int m_foldLevel;

    // horizontal scrollbar width
    int m_maxWidth;
    int m_scrollWidthChange;
    void SetScrollWidth (int ln = -1);

    // identifer information
    int m_colouriseLine;
    IdentTableList m_identTable;
    IdentTableList::Node *GetIdentTableItem (wxString name, int level, IdentTableList::Node *ref, int *position = NULL);
    IdentTableList::Node *GetIdentTableItem (IdentTableList::Node *cur, int *position = NULL);
    IdentTableList::Node *GetIdentTableItem (int pos, int *position = NULL);
    void IdentTableDelete (int pos, int len);
    void IdentTableInsert (int pos, int len);
    IdentTableList::Node *IdentTableUpdate (int identpos, int identlen, int level, int classpos, int classlen, int *lastpos);
    void ProcessIdentTable (int line);

    // width of browse window
    int m_browseWidth;

    DECLARE_EVENT_TABLE()
};

//----------------------------------------------------------------------------
//! IdentData

class IdentData : public wxObject {

public:
    //! constructor
    IdentData () {
        m_name = wxEmptyString;
        m_length = 0;
        m_diff = -1;
        m_level = -1;
        m_ref = NULL;
    }

    // data
    wxString m_name;
    int m_length; // length of the name
    int m_diff; // difference between last position and current position
    int m_level;  // name level: 1=class, 2=function
    IdentTableList::Node *m_ref;
};


//----------------------------------------------------------------------------
//! EditBrowse
class EditBrowse: public wxTreeListCtrl {

public:

    //! constructor
    EditBrowse (wxWindow *parent, wxWindowID id,
                EditText *edit,
                const wxPoint &pos = wxDefaultPosition,
                const wxSize &size = wxDefaultSize,
                long style = 0);

    //! destructor
    ~EditBrowse ();

    // event handlers
    // common
    void OnCommonNotify (wxCommandEvent &event);
    void OnCommonUpdate (wxCommandEvent &event);
    void OnEraseBackground (wxEraseEvent &WXUNUSED(event)) { /* avoid flickering */ }
    void OnSize (wxSizeEvent& event);
    void OnTimerEvent (wxTimerEvent &event);
    // combobox
    void OnBrowseSelect (wxCommandEvent &event);
    // tree
    void OnItemActivated (wxTreeEvent& event);

    // functions
    void ComboboxCreate ();
    void ComboboxUpdate ();

private:
    EditText *m_edit;
    void BrowseCreate ();
    void BrowseUpdate ();
    bool ExistNode (IdentTableList::Node *cur);

    int m_startLine;
    int m_endLine;

    DECLARE_EVENT_TABLE()
};

//----------------------------------------------------------------------------
//! EditBrowseData

class EditBrowseData : public wxTreeItemData {

public:
    //! constructor
    EditBrowseData () { }
    EditBrowseData (IdentTableList::Node *node) : m_node (node) { }

    // Get/Set functions
    IdentTableList::Node *GetNode () {return m_node; }
    void SetNode (IdentTableList::Node *node) {m_node = node; }

private:

    // data
    IdentTableList::Node *m_node;
};

//----------------------------------------------------------------------------
//! EditFindThread
class EditFindThread: public wxThread {

public:

    //! constructor
    EditFindThread (EditText *result);

    // thread execution starts here
    virtual void *Entry();

    // called when the thread exits - whether it terminates normally or is
    // stopped with Delete() (but not when it is Kill()ed!)
    virtual void OnExit();

    int FindInFiles (const wxString& finddir);
    void WriteText (const wxString& line);

private:
    int m_pending; // 1==pending, 0=done, -1=error
    EditText *m_result;
    wxString m_findstr;
    wxRegEx m_fstr;
    wxString m_finddir;
    wxString m_filespec;
    wxRegEx m_fspec;
    int m_findflg;

};

//----------------------------------------------------------------------------
//! EditProperties
class EditProperties: public wxDialog {

public:

    //! constructor
    EditProperties (EditText *edit, long style = 0);

private:

};

//----------------------------------------------------------------------------
//! EditPrint
class EditPrint: public wxPrintout {

public:

    //! constructor
    EditPrint (EditText *edit, wxChar *title = _T(""));

    //! event handlers
    bool OnPrintPage (int page);
    bool OnBeginDocument (int startPage, int endPage);

    //! print functions
    bool HasPage (int page);
    void GetPageInfo (int *minPage, int *maxPage, int *selPageFrom, int *selPageTo);

private:
    EditText *m_edit;
    int m_printed;
    wxRect m_pageRect;
    wxRect m_printRect;

    bool PrintScaling (wxDC *dc);
};

#endif // _EDIT_H_

