//////////////////////////////////////////////////////////////////////////////
// File:        filelist.cpp
// Purpose:     wxFileList control
// Maintainer:  Otto Wyss
// Created:     2003-05-19
// RCS-ID:      $Id: filelist.cpp,v 1.1 2005/03/17 20:03:43 wyo Exp $
// Copyright:   (c) 2005 wyoGuide
// Licence:     wxWindows
//////////////////////////////////////////////////////////////////////////////

//----------------------------------------------------------------------------
// information
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
// headers
//----------------------------------------------------------------------------

// For compilers that support precompilation, includes <wx/wx.h>.
#include <wx/wxprec.h>

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all 'standard' wxWidgets headers)
#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

// wxWidgets headers
#include <wx/filename.h> // filename support

// filelist headers
#include "defsext.h"     // Additional definitions
#include "filelist.h"    // wxFileList control


//----------------------------------------------------------------------------
// resources
//----------------------------------------------------------------------------


//============================================================================
// declarations
//============================================================================

const wxString maxEntriesText = _T("Max entries is outside of allowed range!");


//============================================================================
// implementation
//============================================================================


//----------------------------------------------------------------------------
// wxFileList
//----------------------------------------------------------------------------

wxFileList::wxFileList (int maxEntries,
                        long style)
             : wxArrayString () {

    // initialize variables
    m_maxEntries = maxEntries;
    if (maxEntries < 0) m_maxEntries = myFLIST_MAXENTRIES;
    m_curEntry = 0;
    m_style = style;
    Clear ();

}

//----------------------------------------------------------------------------
// event handlers


//----------------------------------------------------------------------------
// public functions

void wxFileList::Load (const wxString& group) {

    // check if allowed
    wxCHECK_RET ((m_maxEntries > 0) && (m_maxEntries <= myFLIST_MAXENTRIES),
                 maxEntriesText);

    // get config
    wxConfig* cfg = new wxConfig (g_appname);
    wxString str;
    int curNr;
    for (curNr = 0; curNr < m_maxEntries; ++curNr) {
        if (cfg->Read (group + wxString::Format (_T("%d"), curNr), &str)) {
            while (Index (str) != wxNOT_FOUND) wxArrayString::Remove (str);
            wxArrayString::Add (str);
        }
    }
    m_curEntry = 0;
    delete cfg;

    // update combobox, menu
    UpdateCboxes ();
    UpdateMenus ();

}

void wxFileList::Save (const wxString& group) {

    // check if allowed
    wxCHECK_RET ((m_maxEntries > 0) && (m_maxEntries <= myFLIST_MAXENTRIES),
                 maxEntriesText);

    // Write config
    wxConfig* cfg = new wxConfig (g_appname);
    int curNr;
    for (curNr = 0; curNr < m_maxEntries; ++curNr) {
        if (curNr < (int)GetCount()) {
            cfg->Write (group + wxString::Format (_T("%d"), curNr), Item (curNr));
        }
    }
    delete cfg;
}

size_t wxFileList::Add (const wxString& str, size_t WXUNUSED(copies)) {

    // insert string
    int i = Index (str);
    if (m_style & myFLIST_LRUORDER) {
#if !wxCHECK_VERSION(2, 5, 0)
        if (i != wxNOT_FOUND) wxArrayString::Remove (i);
#else
        if (i != wxNOT_FOUND) wxArrayString::RemoveAt (i);
#endif
        wxArrayString::Insert (str, 0);
        m_curEntry = 0;
    }else{
        if (i == wxNOT_FOUND) {
            wxArrayString::Add (str);
            m_curEntry = (int)GetCount() - 1;
        }else{
            m_curEntry = i;
        }
    }

    // update combobox, menu
    UpdateCboxes ();
    UpdateMenus ();

    return (int)GetCount();
}

void wxFileList::Insert (const wxString& str, size_t nIndex,
                         size_t WXUNUSED(copies)) {

    // insert string
    int i = Index (str);
    if (m_style & myFLIST_LRUORDER) {
#if !wxCHECK_VERSION(2, 5, 0)
        if (i != wxNOT_FOUND) wxArrayString::Remove (i);
#else
        if (i != wxNOT_FOUND) wxArrayString::RemoveAt (i);
#endif
        wxArrayString::Insert (str, 0);
        m_curEntry = 0;
    }else{
        if (i == wxNOT_FOUND) {
            if (nIndex > (int)GetCount()) nIndex = (int)GetCount();
            wxArrayString::Insert (str, nIndex);
            m_curEntry = nIndex;
        }else{
            m_curEntry = i;
        }
    }

    // update combobox, menu
    UpdateCboxes ();
    UpdateMenus ();
}

void wxFileList::Remove (const wxChar* str) {

    // remove string
    int i = Index (str);
#if !wxCHECK_VERSION(2, 5, 0)
    if (i != wxNOT_FOUND) wxArrayString::Remove (i);
#else
    if (i != wxNOT_FOUND) wxArrayString::RemoveAt (i);
#endif
    int flCount = (int)GetCount();
    if (m_curEntry >= flCount) m_curEntry = flCount - 1;

    // update combobox, menu
    UpdateCboxes ();
    UpdateMenus ();
}

void wxFileList::Remove (size_t nIndex, size_t count) {

    // remove string
#if !wxCHECK_VERSION(2, 5, 0)
    if ((nIndex >= 0) && (nIndex < GetCount())) wxArrayString::Remove (nIndex, count);
#else
    if ((nIndex >= 0) && (nIndex < GetCount())) wxArrayString::RemoveAt (nIndex, count);
#endif
    int flCount = (int)GetCount();
    if (m_curEntry >= flCount) m_curEntry = flCount - 1;

    // update combobox, menu
    UpdateCboxes ();
    UpdateMenus ();
}

void wxFileList::Replace (const wxString& str, size_t nIndex) {

    // replace string
    if ((nIndex >= 0) && (nIndex < GetCount())) wxArrayString::Item (nIndex) = str;

    // update combobox, menu
    UpdateCboxes ();
    UpdateMenus ();
}

bool wxFileList::SetSelection (int nIndex) {

    // set current
    int flCount = (int)GetCount();
    if (!((nIndex >= 0) && (nIndex < flCount))) return false;
    if (m_style & myFLIST_LRUORDER) {
        wxString str = Item (nIndex);
#if !wxCHECK_VERSION(2, 5, 0)
        wxArrayString::Remove (nIndex);
#else
        wxArrayString::RemoveAt (nIndex);
#endif
        wxArrayString::Insert (str, 0);
        m_curEntry = 0;
    }else{
        m_curEntry = nIndex;
    }

    // update combobox, menu
    UpdateCboxes ();
    UpdateMenus ();

    return true;
}

// combobox functions
void wxFileList::RemoveCbox (wxComboBox* cbox) {

    // get combobox entry
    int nr;
    for (nr = 0; nr < (int)m_cboxes.GetCount(); ++nr) {
        if (m_cboxes[nr]->cbox == cbox) break;
    }

    delete m_cboxes[nr];
    m_cboxes.RemoveAt (nr);
}

void wxFileList::UseCbox (wxComboBox* cbox, int length, int flag) {

    // create combobox
    int nr = m_cboxes.GetCount();
    m_cboxes.Add (new flCbox);
    flCbox* cur = m_cboxes[nr];
    cur->cbox = cbox;
    cur->length = length;
    cur->flag = flag;

    // update combobox
    cbox->SetSize (length * cbox->GetCharWidth(), -1);
    UpdateCboxes ();
}

void wxFileList::UpdateCboxes () {

    int flCount = (int)GetCount();

    // update all entries
    int nr;
    for (nr = 0; nr < (int)m_cboxes.GetCount(); ++nr) {

        flCbox* cur = m_cboxes[nr];
        int curCount = cur->cbox->GetCount();
        int maxEntries = wxMax (flCount, curCount);

        wxString name;
        int curNr;
        for (curNr = 0; curNr < maxEntries; ++curNr) {
            if (curNr < flCount) {
                if (cur->length > 0) {
                    name = wxPartialName (Item (curNr), cur->length, cur->flag);
                }else{
                    name = Item (curNr);
                }
                if (curNr < curCount) {
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__) // wxComboBox::SetString not implemented in wxGTK!
                    cur->cbox->SetString (curNr, name);
#else
                    cur->cbox->Delete (0);
                    cur->cbox->Append (name);
#endif
                }else{
                    cur->cbox->Append (name);
                }
            }else{
                cur->cbox->Delete (flCount);
            }
        }

        // set selection
        if (cur->cbox->GetCount() > 0) cur->cbox->SetSelection (m_curEntry);

    }
}

// menu functions
void wxFileList::RemoveMenu (wxMenu* menu) {

    wxMenuItem *item = NULL;
    int flCount = (int)GetCount();

    // get menu entry
    int nr;
    for (nr = 0; nr < (int)m_menus.GetCount(); ++nr) {
        if (m_menus[nr]->menu == menu) break;
    }
    flMenu* cur = m_menus[nr];

    // remove menu entries
    if (m_style & myFLIST_ENDSEPARATOR) {
        if (cur->endSep >= 0) {
            item = cur->menu->FindItemByPosition (cur->endSep);
            if (item) cur->menu->Destroy (item);
            cur->endSep = -1;
        }
    }
    int curNr;
    for (curNr = 0; curNr < flCount; ++curNr) {
        if (cur->first <= cur->last) {
            item = cur->menu->FindItemByPosition (cur->first);
            if (item) cur->menu->Destroy (item);
            cur->last -= 1;
        }
    }
    if (m_style & myFLIST_STARTSEPARATOR) {
        if (cur->startSep >= 0) {
            item = cur->menu->FindItemByPosition (cur->startSep);
            if (item) cur->menu->Destroy (item);
            cur->first -= 1;
            cur->last -= 1;
        }
    }

    delete m_menus[nr];
    m_menus.RemoveAt (nr);
}

void wxFileList::UseMenu (wxMenu* menu, int menuID, int pos, int len, int flag) {

    // create combobox
    int nr = m_menus.GetCount();
    m_menus.Add (new flMenu);
    flMenu* cur = m_menus[nr];
    cur->menu = menu;
    cur->ID = menuID;
    if (pos >= 0) {
       cur->first = pos;
    }else{
       cur->first = cur->menu->GetMenuItemCount();
    }
    cur->last = m_menus[nr]->first - 1;
    cur->len = len;
    cur->flag = flag;
    cur->startSep = -1;
    cur->endSep = -1;

    // update menu
    UpdateMenus ();

}

void wxFileList::UpdateMenus () {

    wxMenuItem *item = NULL;
    int flCount = (int)GetCount();

    // update all entries
    int nr;
    for (nr = 0; nr < (int)m_menus.GetCount(); ++nr) {

        flMenu* cur = m_menus[nr];

        // insert/remove start separator if requested
        if (m_style & myFLIST_STARTSEPARATOR) {
            if ((cur->startSep < 0) && (flCount > 0)) {
                cur->startSep = cur->first;
                cur->menu->InsertSeparator (cur->startSep);
                cur->first += 1;
                cur->last += 1;
            }else if ((cur->startSep >= 0) && (flCount == 0)) {
                item = cur->menu->FindItemByPosition (cur->startSep);
                if (item) cur->menu->Destroy (item);
                cur->startSep = -1;
                cur->first -= 1;
                cur->last -= 1;
            }
        }

        // update entries
        wxString name;
        int curNr;
        for (curNr = 0; curNr < m_maxEntries; ++curNr) {
            if (curNr < flCount) {
                if (cur->len > 0) {
                   name = wxPartialName (Item (curNr), cur->len, cur->flag);
                }else{
                    name = Item (curNr);
                }
                name.Printf (_T("&%d ") + name, curNr+1);
                if (m_style & myFLIST_CTRL_KEYS) {
                    name.Printf (name + _T("\tCtrl+%d"), curNr+1);
                }else if (m_style & myFLIST_CTRL_SHIFT_KEYS) {
                    name.Printf (name + _T("\tCtrl+Shift+%d"), curNr+1);
                }
                if (cur->menu->FindItem (cur->ID + curNr)) {
                    cur->menu->SetLabel (cur->ID + curNr, name);
                }else{
                    cur->last = cur->first + curNr;
                    cur->menu->Insert (cur->last, cur->ID + curNr, name);
                }
            }else{
                if (cur->first + curNr <= cur->last) {
                    item = cur->menu->FindItemByPosition (cur->first + curNr);
                    if (item) cur->menu->Destroy (item);
                    cur->last -= 1;
                }
            }
        }

        // insert/remove end separator if requested
        if (m_style & myFLIST_ENDSEPARATOR) {
            if ((cur->endSep < 0) && (flCount > 0)) {
                cur->endSep = cur->last + 1;
                cur->menu->InsertSeparator (cur->endSep);
            }else if ((cur->endSep >= 0) && (flCount == 0)) {
                item = cur->menu->FindItemByPosition (cur->endSep);
                if (item) cur->menu->Destroy (item);
                cur->endSep = -1;
            }
        }

    }
}

// ----------------------------------------------------------------------------
// wxPartialName
// ----------------------------------------------------------------------------
// Cuts the name to length chars, replacing the cut part with 3 dots
wxString wxPartialName (const wxString &name,
                        int partial,
                        int flag,
                        wxChar separator) {
    if (partial <= 3) return wxEmptyString;
    int length = name.Length();
    if (partial >= length) return name;

    int cnt = partial;
    if (flag & myPARTIAL_BEGIN) cnt = 0;
    if (flag & myPARTIAL_END) cnt = partial - 3;
    if (flag & (myPARTIAL_BEGIN | myPARTIAL_END)) cnt = (partial - 3) / 2;

    wxString temp = name;
    if (separator == '\0') separator = wxFileName::GetPathSeparator();
    int len = 0, start = 0, end = 0;
    while (cnt > 0) {
        len = temp.AfterLast (separator).Length();
        end += len;
        temp.Remove (length - end);
        if (end >= cnt)  break;
        if (temp [(unsigned int)(length - end - 1)] != separator) break;
        end += 1;
        temp.Remove (length - end);
        if (end >= cnt)  break;
    }
    while (true) {
        len = temp.BeforeFirst (separator).Length();
        if ((start + len + end) >= (partial - 3))  break;
        start += len;
        temp.Remove (0, len);
        if (temp [(unsigned int)0] != separator) break;
        start += 1;
        temp.Remove (0, 1);
    }
    temp = wxEmptyString;
    temp.Append (name.Mid (0, start));
    temp.Append (_T("..."));
    if (end > length) end = length;
    temp.Append (name.Mid (length - end, end));
    return temp;
}

