//////////////////////////////////////////////////////////////////////////////
// File:        filelist.h
// Purpose:     wxFileList control
// Maintainer:  Otto Wyss
// Created:     2003-05-19
// RCS-ID:      $Id: filelist.h,v 1.1 2005/03/17 20:03:43 wyo Exp $
// Copyright:   (c) 2005 wyoGuide
// Licence:     wxWindows
//////////////////////////////////////////////////////////////////////////////

#ifndef _MY_FILELIST_H_
#define _MY_FILELIST_H_

#ifdef __GNUG__
    #pragma implementation "filelist.h"
#endif

//----------------------------------------------------------------------------
// information
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
// headers
//----------------------------------------------------------------------------
//! wxWidgets headers
#include <wx/string.h>   // strings support


//============================================================================
// declarations
//============================================================================

//----------------------------------------------------------------------------
//!
static int myFLIST_MAXENTRIES = 9;

enum wxFileListStyles {
    myFLIST_STARTSEPARATOR = 1,
    myFLIST_ENDSEPARATOR = 2,
    myFLIST_LRUORDER = 4,
    myFLIST_CTRL_KEYS = 16,
    myFLIST_CTRL_SHIFT_KEYS = 32
};

enum wxPartialNameFlags {
    myPARTIAL_BEGIN = 1, // keep begin of name
    myPARTIAL_END = 2    // keep end of name
};


//----------------------------------------------------------------------------
//! wxFileList
class wxFileList: public wxArrayString {

public:

    //! default constructor
    wxFileList (int maxEntries = myFLIST_MAXENTRIES, long = myFLIST_STARTSEPARATOR);

    // event handlers

    //! load/save functions
    void Load (const wxString& group = wxEmptyString);
    void Save (const wxString& group = wxEmptyString);

    //! string functions
    size_t Add (const wxString& str, size_t copies = 1);
    void Insert(const wxString& str, size_t nIndex, size_t copies = 1);
    void Remove (const wxChar * str);
    void Remove (size_t nIndex, size_t count = 1);
    void Replace(const wxString& str, size_t nIndex);

    // get/set current
    int GetSelection() const {return m_curEntry; }
    bool SetSelection(int nIndex);

    //! combobox functions
    void RemoveCbox (wxComboBox* cbox);
    void UseCbox (wxComboBox* cbox,
                  int length = -1,
                  int flag = myPARTIAL_BEGIN | myPARTIAL_END);

    //! menu functions
    void RemoveMenu (wxMenu* menu);
    void UseMenu (wxMenu* menu,
                  int menuID = wxID_FILE1,
                  int pos = -1,
                  int len = -1,
                  int flag = myPARTIAL_BEGIN | myPARTIAL_END);

private:

    //! variables
    int m_maxEntries;
    int m_curEntry;
    int m_style;

    // combobox variables
    struct flCbox {
        wxComboBox *cbox;
        int length;
        int flag;
    };
#if !wxCHECK_VERSION(2, 5, 0)
    WX_DEFINE_ARRAY (flCbox*,Cboxes);
#else
    WX_DEFINE_ARRAY_PTR (flCbox*,Cboxes);
#endif
    Cboxes m_cboxes;
    void UpdateCboxes ();

    // menu variables
    struct flMenu {
        wxMenu* menu;
        int ID;
        int first;
        int last;
        int len;
        int flag;
        int startSep;
        int endSep;
    };
#if !wxCHECK_VERSION(2, 5, 0)
    WX_DEFINE_ARRAY (flMenu*, Menus);
#else
    WX_DEFINE_ARRAY_PTR (flMenu*, Menus);
#endif
    Menus m_menus;
    void UpdateMenus ();

};

// ----------------------------------------------------------------------------
// wxPartialName
wxString wxPartialName (const wxString &name,
                        int partial,
                        int flag = (myPARTIAL_BEGIN | myPARTIAL_END),
                        wxChar separator = '\0');

#endif // _MY_FILELIST_H_

