//////////////////////////////////////////////////////////////////////////////
// File:        prefdlg.cpp
// Purpose:     Preferences dialog
// Maintainer:  Otto Wyss
// Created:     2003-04-28
// RCS-ID:      $Id: prefdlg.cpp,v 1.1 2005/03/17 20:03:43 wyo Exp $
// Copyright:   (c) 2005 wyoGuide
// Licence:     wxWindows
//////////////////////////////////////////////////////////////////////////////

//----------------------------------------------------------------------------
// information
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
// headers
//----------------------------------------------------------------------------

// For compilers that support precompilation, includes <wx/wx.h>.
#include <wx/wxprec.h>

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all 'standard' wxWidgets headers)
#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

//! wxWidgets headers
#include <wx/accel.h>    // accelerator support
#include <wx/string.h>   // strings support
#include <wx/fontenum.h> // font support

#include "defsext.h"     // Additional definitions

#include "prefdlg.h"     // Preferences dialog


//----------------------------------------------------------------------------
// resources
//----------------------------------------------------------------------------


//============================================================================
// declarations
//============================================================================

//============================================================================
// implementation
//============================================================================

static void colourEnum_GetNames (wxArrayString *colourNames) {
    colourNames->Add (_T("AQUAMARINE"));
    colourNames->Add (_T("BLACK"));
    colourNames->Add (_T("BLUE"));
    colourNames->Add (_T("BLUE VIOLET"));
    colourNames->Add (_T("BROWN"));
    colourNames->Add (_T("CADET BLUE"));
    colourNames->Add (_T("CORAL"));
    colourNames->Add (_T("CORNFLOWER BLUE"));
    colourNames->Add (_T("CYAN"));
    colourNames->Add (_T("DARK GREY"));
    colourNames->Add (_T("DARK GREEN"));
    colourNames->Add (_T("DARK OLIVE GREEN"));
    colourNames->Add (_T("DARK ORCHID"));
    colourNames->Add (_T("DARK SLATE BLUE"));
    colourNames->Add (_T("DARK SLATE GREY"));
    colourNames->Add (_T("DARK TURQUOISE"));
    colourNames->Add (_T("DIM GREY"));
    colourNames->Add (_T("FIREBRICK"));
    colourNames->Add (_T("FOREST GREEN"));
    colourNames->Add (_T("GOLD"));
    colourNames->Add (_T("GOLDENROD"));
    colourNames->Add (_T("GREY"));
    colourNames->Add (_T("GREEN"));
    colourNames->Add (_T("GREEN YELLOW"));
    colourNames->Add (_T("INDIAN RED"));
    colourNames->Add (_T("KHAKI"));
    colourNames->Add (_T("LIGHT BLUE"));
    colourNames->Add (_T("LIGHT GREY"));
    colourNames->Add (_T("LIGHT STEEL BLUE"));
    colourNames->Add (_T("LIME GREEN"));
    colourNames->Add (_T("LIGHT MAGENTA"));
    colourNames->Add (_T("MAGENTA"));
    colourNames->Add (_T("MAROON"));
    colourNames->Add (_T("MEDIUM AQUAMARINE"));
    colourNames->Add (_T("MEDIUM GREY"));
    colourNames->Add (_T("MEDIUM BLUE"));
    colourNames->Add (_T("MEDIUM FOREST GREEN"));
    colourNames->Add (_T("MEDIUM GOLDENROD"));
    colourNames->Add (_T("MEDIUM ORCHID"));
    colourNames->Add (_T("MEDIUM SEA GREEN"));
    colourNames->Add (_T("MEDIUM SLATE BLUE"));
    colourNames->Add (_T("MEDIUM SPRING GREEN"));
    colourNames->Add (_T("MEDIUM TURQUOISE"));
    colourNames->Add (_T("MEDIUM VIOLET RED"));
    colourNames->Add (_T("MIDNIGHT BLUE"));
    colourNames->Add (_T("NAVY"));
    colourNames->Add (_T("ORANGE"));
    colourNames->Add (_T("ORANGE RED"));
    colourNames->Add (_T("ORCHID"));
    colourNames->Add (_T("PALE GREEN"));
    colourNames->Add (_T("PINK"));
    colourNames->Add (_T("PLUM"));
    colourNames->Add (_T("PURPLE"));
    colourNames->Add (_T("RED"));
    colourNames->Add (_T("SALMON"));
    colourNames->Add (_T("SEA GREEN"));
    colourNames->Add (_T("SIENNA"));
    colourNames->Add (_T("SKY BLUE"));
    colourNames->Add (_T("SLATE BLUE"));
    colourNames->Add (_T("SPRING GREEN"));
    colourNames->Add (_T("STEEL BLUE"));
    colourNames->Add (_T("TAN"));
    colourNames->Add (_T("THISTLE"));
    colourNames->Add (_T("TURQUOISE"));
    colourNames->Add (_T("VIOLET"));
    colourNames->Add (_T("VIOLET RED"));
    colourNames->Add (_T("WHEAT"));
    colourNames->Add (_T("WHITE"));
    colourNames->Add (_T("YELLOW"));
    colourNames->Add (_T("YELLOW GREEN"));
    colourNames->Add (_T("MEDIUM GOLDENROD"));
    colourNames->Add (_T("MEDIUM FOREST GREEN"));
    colourNames->Add (_T("LIGHT MAGENTA"));
    colourNames->Add (_T("MEDIUM GREY"));
}

//----------------------------------------------------------------------------
// PreferenceDlg
//----------------------------------------------------------------------------

BEGIN_EVENT_TABLE (PreferenceDlg, wxDialog)
    EVT_BUTTON (wxID_APPLY,                     PreferenceDlg::OnApply)
    EVT_UPDATE_UI (wxID_APPLY,                  PreferenceDlg::OnApplyUI)
    EVT_BUTTON (wxID_CANCEL,                    PreferenceDlg::OnCancel)
    EVT_BUTTON (wxID_OK,                        PreferenceDlg::OnOkay)
    EVT_BUTTON (wxID_RESET,                     PreferenceDlg::OnReset)
    EVT_BUTTON (wxID_HELP,                      PreferenceDlg::OnHelp)
    EVT_MENU   (wxID_HELP,                      PreferenceDlg::OnHelp)
    // common page
    EVT_CHECKBOX (myID_WHITETOSPACE,            PreferenceDlg::OnConvertTo)
    EVT_CHECKBOX (myID_WHITETOTAB,              PreferenceDlg::OnConvertTo)
    // preferences pages
    EVT_COMBOBOX (myID_PREFS_LANGUAGE,          PreferenceDlg::OnLanguageChoose)
    EVT_COMBOBOX (myID_PREFS_STYLETYPE,         PreferenceDlg::OnStyletypeChoose)
    EVT_COMBOBOX (myID_PREFS_ANALYSE,           PreferenceDlg::OnAnalyseChoose)
    EVT_LISTBOX (myID_PREFS_KEYWORDS,           PreferenceDlg::OnKeywordsChange)
    // style changes
    EVT_COMBOBOX (  myID_PREFS_STYLEFOREGROUND, PreferenceDlg::OnStyleChange)
    EVT_TEXT_ENTER (myID_PREFS_STYLEFOREGROUND, PreferenceDlg::OnStyleChange)
    EVT_COMBOBOX (  myID_PREFS_STYLEBACKGROUND, PreferenceDlg::OnStyleChange)
    EVT_TEXT_ENTER (myID_PREFS_STYLEBACKGROUND, PreferenceDlg::OnStyleChange)
    EVT_COMBOBOX (  myID_PREFS_STYLEFONTNAME,   PreferenceDlg::OnStyleChange)
    EVT_TEXT_ENTER (myID_PREFS_STYLEFONTNAME,   PreferenceDlg::OnStyleChange)
    EVT_TEXT (myID_PREFS_STYLEFONTSIZE,         PreferenceDlg::OnStyleChange)
    EVT_CHECKBOX (myID_PREFS_STYLEBOLD,         PreferenceDlg::OnStyleChange)
    EVT_CHECKBOX (myID_PREFS_STYLEITALIC,       PreferenceDlg::OnStyleChange)
    EVT_CHECKBOX (myID_PREFS_STYLEUNDERLINE,    PreferenceDlg::OnStyleChange)
    EVT_CHECKBOX (myID_PREFS_STYLEHIDDEN,       PreferenceDlg::OnStyleChange)
END_EVENT_TABLE()

PreferenceDlg::PreferenceDlg (wxWindow *parent,
                              long style)
             : wxDialog (parent, -1, _T("PreferenceDlg"),
                         wxDefaultPosition, wxDefaultSize,
                         style|wxDEFAULT_DIALOG_STYLE) {

    //accelerators (for help)
    const int nEntries = 1 ;
    wxAcceleratorEntry entries[nEntries];
    entries[0].Set (wxACCEL_NORMAL, WXK_F1, wxID_HELP);
    wxAcceleratorTable accel (nEntries, entries);
    SetAcceleratorTable (accel);

    // create preference panel
    wxPanel *panel = new wxPanel (this, -1, wxDefaultPosition, wxDefaultSize,
                                 wxTAB_TRAVERSAL|wxCLIP_CHILDREN|wxNO_BORDER);

    // initialize preferences
    m_prefsBook = new wxNotebook (panel, -1, wxDefaultPosition, wxDefaultSize,
                                  wxNB_FIXEDWIDTH);
#if !wxCHECK_VERSION(2, 5, 0)
    wxNotebookSizer *prefsSizer = new wxNotebookSizer (m_prefsBook);
#endif
    m_prefsPageCommon = m_prefsBook->GetPageCount();
    m_prefsBook->AddPage (CreatePageCommon (), PAGE_COMMON, true);
    SetValuesPageCommon ();
    m_prefsPageLanguages = m_prefsBook->GetPageCount();
    m_prefsBook->AddPage (CreatePageLanguages (), PAGE_LANGUAGES, true);
    SetValuesPageLanguages ();
    m_prefsPageStyles = m_prefsBook->GetPageCount();
    m_prefsBook->AddPage (CreatePageStyles (), PAGE_STYLE_TYPES, true);
    SetValuesPageStyles ();
    m_prefsPageAnalyse = m_prefsBook->GetPageCount();
    m_prefsBook->AddPage (CreatePageAnalyse (), PAGE_ANALYSE, true);
    SetValuesPageAnalyse ();
    m_prefsBook->SetSelection (m_prefsPageCommon);

    // lauout the buttons
    wxButton *resetButton = new wxButton (panel, wxID_RESET, _("Default"));
    wxButton *okButton = new wxButton (panel, wxID_OK, _("OK"));
    okButton->SetDefault();
    wxButton *applyButton = new wxButton (panel, wxID_APPLY, _("Apply"));
    wxButton *cancelButton = new wxButton (panel, wxID_CANCEL, _("Cancel"));
    wxBoxSizer *buttonpane = new wxBoxSizer (wxHORIZONTAL);
    buttonpane->Add (resetButton, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    buttonpane->Add (16, 0, 1);
    buttonpane->Add (okButton, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
    buttonpane->Add (6, 0);
    buttonpane->Add (applyButton, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
    buttonpane->Add (6, 0);
    buttonpane->Add (cancelButton, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);

    // set sizer
    wxBoxSizer *panelsizer = new wxBoxSizer (wxVERTICAL);
#if !wxCHECK_VERSION(2, 5, 0)
    panelsizer->Add (prefsSizer, 0, wxEXPAND|wxLEFT|wxRIGHT|wxTOP, 10);
#else
    panelsizer->Add (m_prefsBook, 0, wxEXPAND|wxLEFT|wxRIGHT|wxTOP, 10);
#endif
    panelsizer->Add (0, 6);
    panelsizer->Add (buttonpane, 0, wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM, 10);

    panel->SetSizer (panelsizer);
    panelsizer->Fit (this);
    Centre (wxBOTH);
    ShowModal();
}

PreferenceDlg::~PreferenceDlg () {
}

//----------------------------------------------------------------------------
// event handlers

void PreferenceDlg::OnApply (wxCommandEvent &WXUNUSED(event)) {
    GetValuesPageCommon ();
    Prefs::SaveValuesPageCommon ();
    GetValuesPageLanguages ();
    Prefs::SaveValuesPageLanguages ();
    GetValuesPageStyles ();
    Prefs::SaveValuesPageStyles ();
    GetValuesPageAnalyse ();
    Prefs::SaveValuesPageAnalyse ();
}

void PreferenceDlg::OnApplyUI (wxUpdateUIEvent &event) {
    int pageNr = m_prefsBook->GetSelection ();
    if (pageNr == m_prefsPageCommon) {
        event.Enable (CheckValuesPageCommon());
    }else if (pageNr == m_prefsPageLanguages) {
        event.Enable (CheckValuesPageLanguages());
    }else if (pageNr == m_prefsPageStyles) {
        event.Enable (CheckValuesPageStyles());
    }else if (pageNr == m_prefsPageAnalyse) {
        event.Enable (CheckValuesPageAnalyse());
    }
}

void PreferenceDlg::OnCancel (wxCommandEvent &WXUNUSED(event)) {
    EndModal (wxID_CANCEL);
}

void PreferenceDlg::OnHelp (wxCommandEvent &WXUNUSED(event)) {
    g_help->Display(_T("preferences.html"));
}

void PreferenceDlg::OnOkay (wxCommandEvent &WXUNUSED(event)) {
    GetValuesPageCommon ();
    Prefs::SaveValuesPageCommon ();
    GetValuesPageLanguages ();
    Prefs::SaveValuesPageLanguages ();
    GetValuesPageStyles ();
    Prefs::SaveValuesPageStyles ();
    GetValuesPageAnalyse ();
    Prefs::SaveValuesPageAnalyse ();

    EndModal (wxID_OK);
}

void PreferenceDlg::OnReset (wxCommandEvent &WXUNUSED(event)) {
    int pageNr = m_prefsBook->GetSelection ();
    if (pageNr == m_prefsPageCommon) {
        Prefs::LoadValuesPageCommon (true);
        SetValuesPageCommon ();
    }else if (pageNr == m_prefsPageLanguages) {
        Prefs::LoadValuesPageLanguages (true);
        SetValuesPageLanguages ();
    }else if (pageNr == m_prefsPageStyles) {
        Prefs::LoadValuesPageStyles (true);
        SetValuesPageStyles ();
    }else if (pageNr == m_prefsPageAnalyse) {
        Prefs::LoadValuesPageAnalyse (true);
        SetValuesPageAnalyse ();
    }
}

void PreferenceDlg::OnConvertTo (wxCommandEvent &event) {
    switch (event.GetId()) {
        case myID_WHITETOSPACE: {
            if (m_convertToSpaces->GetValue()) m_convertToTabs->SetValue (false);
            break;}
        case myID_WHITETOTAB: {
            if (m_convertToTabs->GetValue()) m_convertToSpaces->SetValue (false);
            break;}
    }
}

void PreferenceDlg::OnLanguageChoose (wxCommandEvent &WXUNUSED(event)) {
    GetValuesPageLanguages ();
    m_langNr = m_language->GetSelection();
    m_curLanguage = &g_LanguagePrefs [m_langNr];
    SetValuesPageLanguages ();
}

void PreferenceDlg::OnStyletypeChoose (wxCommandEvent &WXUNUSED(event)) {
    GetValuesPageStyles ();
    m_styleNr = m_styletype->GetSelection();
    m_curStyle = &g_StylePrefs [m_styleNr];
    SetValuesPageStyles ();
}

void PreferenceDlg::OnAnalyseChoose (wxCommandEvent &WXUNUSED(event)) {
    GetValuesPageAnalyse ();
    m_curAnalyse = &g_AnalysePrefs [m_analyse->GetSelection()];
    SetValuesPageAnalyse ();
}

void PreferenceDlg::OnKeywordsChange (wxCommandEvent &WXUNUSED(event)) {
    if (m_curLanguage->styles[m_styles->GetSelection()].words) {
        m_words->SetValue (m_curLanguage->styles[m_styles->GetSelection()].words);
    }else{
        m_words->SetValue (wxEmptyString);
    }
}

void PreferenceDlg::OnStyleChange (wxCommandEvent &event) {
    wxFont font (10, wxTELETYPE, wxNORMAL, wxNORMAL);
    switch (event.GetId()) {
        case myID_PREFS_STYLEFOREGROUND: {
            m_sampletext->SetForegroundColour(event.GetString());
            break;}
        case myID_PREFS_STYLEBACKGROUND: {
            m_sampletext->SetBackgroundColour(event.GetString());
            break;}
        case myID_PREFS_STYLEFONTNAME: {
            font = m_sampletext->GetFont ();
            font.SetFaceName (event.GetString());
            m_sampletext->SetFont (font);
            break;}
        case myID_PREFS_STYLEFONTSIZE: {
            font = m_sampletext->GetFont ();
            long fontsize = 0;
            event.GetString().ToLong (&fontsize);
            font.SetPointSize (fontsize);
            m_sampletext->SetFont (font);
            break;}
        case myID_PREFS_STYLEBOLD: {
            font = m_sampletext->GetFont ();
            font.SetWeight (event.GetInt() != 0? wxBOLD: wxNORMAL);
            m_sampletext->SetFont (font);
            break;}
        case myID_PREFS_STYLEITALIC: {
            font = m_sampletext->GetFont ();
            font.SetStyle (event.GetInt() != 0? wxITALIC: wxNORMAL);
            m_sampletext->SetFont (font);
            break;}
        case myID_PREFS_STYLEUNDERLINE: {
            font = m_sampletext->GetFont ();
            font.SetUnderlined (event.GetInt() != 0? true: false);
            m_sampletext->SetFont (font);
            break;}
        case myID_PREFS_STYLEHIDDEN: {
            m_sampletext->Show (event.GetInt() != 0? false: true);
            break;}
    }
    m_sampletext->Refresh();
}


//----------------------------------------------------------------------------
// private functions

wxPanel *PreferenceDlg::CreatePageCommon () {
    wxPanel *panel = new wxPanel (m_prefsBook);
    panel->SetHelpText (_T("preferences.html#common"));

    int ch = GetCharWidth();
#if defined(__WXGTK__)
    ch -= ch/6; // reduce GTK font width by 15%
#endif
    const int sz1 = 20*ch; // width of a text lable

    // splash screen prefs
    m_noSplash = new wxCheckBox (panel, -1, wxEmptyString);
    wxBoxSizer *noSplash = new wxBoxSizer (wxHORIZONTAL);
    noSplash->Add (4, 0);
    noSplash->Add (new wxStaticText (panel, -1, _("No splash on startup"),
                                     wxDefaultPosition, wxSize(sz1, -1)),
                   0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    noSplash->Add (6, 0);
    noSplash->Add (m_noSplash, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // browse info prefs
    m_showBrowse = new wxCheckBox (panel, -1, wxEmptyString);
    wxBoxSizer *showBrowse = new wxBoxSizer (wxHORIZONTAL);
    showBrowse->Add (4, 0);
    showBrowse->Add (new wxStaticText (panel, -1, _("Show class info"),
                                       wxDefaultPosition, wxSize(sz1, -1)),
                     0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    showBrowse->Add (6, 0);
    showBrowse->Add (m_showBrowse, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // toolbar prefs
    m_showToolbar = new wxCheckBox (panel, -1, wxEmptyString);
#if !wxCHECK_VERSION(2, 5, 0) && defined(__WXGTK__)
    m_showToolbar->Enable (false);
#endif
    wxBoxSizer *showToolbar = new wxBoxSizer (wxHORIZONTAL);
    showToolbar->Add (4, 0);
    showToolbar->Add (new wxStaticText (panel, -1, _("Show toolbar"),
                                        wxDefaultPosition, wxSize(sz1, -1)),
                      0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    showToolbar->Add (6, 0);
    showToolbar->Add (m_showToolbar, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);
#if !wxCHECK_VERSION(2, 5, 0) && defined(__WXGTK__)
    showToolbar->Show (m_showToolbar, false);
#endif

    // statusbar prefs
    m_showStatusbar = new wxCheckBox (panel, -1, wxEmptyString);
    wxBoxSizer *showStatusbar = new wxBoxSizer (wxHORIZONTAL);
    showStatusbar->Add (4, 0);
    showStatusbar->Add (new wxStaticText (panel, -1, _("Show statusbar"),
                                        wxDefaultPosition, wxSize(sz1, -1)),
                      0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    showStatusbar->Add (6, 0);
    showStatusbar->Add (m_showStatusbar, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // frames/pages prefs
    m_usePages = new wxCheckBox (panel, -1, wxEmptyString);
    wxBoxSizer *usePages = new wxBoxSizer (wxHORIZONTAL);
    usePages->Add (4, 0);
    usePages->Add (new wxStaticText (panel, -1, _("Use tabbed pages"),
                                     wxDefaultPosition, wxSize(sz1, -1)),
                   0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    usePages->Add (6, 0);
    usePages->Add (m_usePages, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // application prefs
    wxStaticBoxSizer *commonbox = new wxStaticBoxSizer (
                     new wxStaticBox (panel, -1, _("Application preferences")),
                     wxVERTICAL);
    wxGridSizer *commongrid = new wxGridSizer (2, 2, 20);
    commonbox->Add (commongrid, 0, wxEXPAND|wxBOTTOM, 2);
    commongrid->Add (noSplash, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);
    commongrid->Add (usePages, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);
    commongrid->Add (showToolbar, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);
    commongrid->Add (showStatusbar, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);
    commongrid->Add (showBrowse, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // overtype mode prefs
    m_overTypeInitial = new wxCheckBox (panel, -1, wxEmptyString);
    wxBoxSizer *overType = new wxBoxSizer (wxHORIZONTAL);
    overType->Add (4, 0);
    overType->Add (new wxStaticText (panel, -1, _("Overwrite mode"),
                                     wxDefaultPosition, wxSize(sz1, -1)),
                   0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    overType->Add (6, 0);
    overType->Add (m_overTypeInitial, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // wrap mode prefs
    m_wrapModeInitial = new wxCheckBox (panel, -1, wxEmptyString);
    wxBoxSizer *wrapModeOn = new wxBoxSizer (wxHORIZONTAL);
    wrapModeOn->Add (4, 0);
    wrapModeOn->Add (new wxStaticText (panel, -1, _("Wrap mode"),
                                       wxDefaultPosition, wxSize(sz1, -1)),
                     0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    wrapModeOn->Add (6, 0);
    wrapModeOn->Add (m_wrapModeInitial, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // readonly mode prefs
    m_readOnlyInitial = new wxCheckBox (panel, -1, wxEmptyString);
    wxBoxSizer *readOnly = new wxBoxSizer (wxHORIZONTAL);
    readOnly->Add (4, 0);
    readOnly->Add (new wxStaticText (panel, -1, _("Readonly mode"),
                                     wxDefaultPosition, wxSize(sz1, -1)),
                   0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    readOnly->Add (6, 0);
    readOnly->Add (m_readOnlyInitial, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // layout the default prefs
    wxStaticBoxSizer *defaultbox = new wxStaticBoxSizer (
                     new wxStaticBox (panel, -1, _("Initial mode preferences")),
                     wxVERTICAL);
    wxGridSizer *defaultgrid = new wxGridSizer (2, 2, 20);
    defaultbox->Add (defaultgrid, 0, wxEXPAND|wxBOTTOM, 2);
    defaultgrid->Add (overType, 0, wxEXPAND);
    defaultgrid->Add (readOnly, 0, wxEXPAND);
    defaultgrid->Add (wrapModeOn, 0, wxEXPAND);

    // syntax coloring prefs
    m_syntaxEnable = new wxCheckBox (panel, -1, wxEmptyString);
    wxBoxSizer *syntax = new wxBoxSizer (wxHORIZONTAL);
    syntax->Add (4, 0);
    syntax->Add (new wxStaticText (panel, -1, _("Enable syntax colouring"),
                                   wxDefaultPosition, wxSize(sz1, -1)),
                 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    syntax->Add (6, 0);
    syntax->Add (m_syntaxEnable, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // folding prefs
    m_foldEnable = new wxCheckBox (panel, -1, wxEmptyString);
    wxBoxSizer *folding = new wxBoxSizer (wxHORIZONTAL);
    folding->Add (4, 0);
    folding->Add (new wxStaticText (panel, -1, _("Enable folding"),
                                    wxDefaultPosition, wxSize(sz1, -1)),
                  0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    folding->Add (6, 0);
    folding->Add (m_foldEnable, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // indenting prefs
    m_indentEnable = new wxCheckBox (panel, -1, wxEmptyString);
    wxBoxSizer *indenting = new wxBoxSizer (wxHORIZONTAL);
    indenting->Add (4, 0);
    indenting->Add (new wxStaticText (panel, -1, _("Enable autoindent"),
                                      wxDefaultPosition, wxSize(sz1, -1)),
                    0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    indenting->Add (6, 0);
    indenting->Add (m_indentEnable, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // display EOL prefs
    m_displayEOLEnable = new wxCheckBox (panel, -1, wxEmptyString);
    wxBoxSizer *displayEOL = new wxBoxSizer (wxHORIZONTAL);
    displayEOL->Add (4, 0);
    displayEOL->Add (new wxStaticText (panel, -1, _("Show line endings"),
                                       wxDefaultPosition, wxSize(sz1, -1)),
                     0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    displayEOL->Add (6, 0);
    displayEOL->Add (m_displayEOLEnable, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // indent guides prefs
    m_indentGuideEnable = new wxCheckBox (panel, -1, wxEmptyString);
    wxBoxSizer *indentGuide = new wxBoxSizer (wxHORIZONTAL);
    indentGuide->Add (4, 0);
    indentGuide->Add (new wxStaticText (panel, -1, _("Show indent guides"),
                                        wxDefaultPosition, wxSize(sz1, -1)),
                      0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    indentGuide->Add (6, 0);
    indentGuide->Add (m_indentGuideEnable, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // line number prefs
    m_lineNumberEnable = new wxCheckBox (panel, -1, wxEmptyString);
    wxBoxSizer *lineNumber = new wxBoxSizer (wxHORIZONTAL);
    lineNumber->Add (4, 0);
    lineNumber->Add (new wxStaticText (panel, -1, _("Show line numbers"),
                                       wxDefaultPosition, wxSize(sz1, -1)),
                     0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    lineNumber->Add (6, 0);
    lineNumber->Add (m_lineNumberEnable, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // long line prefs
    m_longLineOnEnable = new wxCheckBox (panel, -1, wxEmptyString);
    wxBoxSizer *longLineOn = new wxBoxSizer (wxHORIZONTAL);
    longLineOn->Add (4, 0);
    longLineOn->Add (new wxStaticText (panel, -1, _("Show long line marker"),
                                       wxDefaultPosition, wxSize(sz1, -1)),
                     0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    longLineOn->Add (6, 0);
    longLineOn->Add (m_longLineOnEnable, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // white space prefs
    m_whiteSpaceEnable = new wxCheckBox (panel, -1, wxEmptyString);
    wxBoxSizer *whiteSpace = new wxBoxSizer (wxHORIZONTAL);
    whiteSpace->Add (4, 0);
    whiteSpace->Add (new wxStaticText (panel, -1, _("Show white space"),
                                       wxDefaultPosition, wxSize(sz1, -1)),
                     0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    whiteSpace->Add (6, 0);
    whiteSpace->Add (m_whiteSpaceEnable, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // layout the editor  prefs
    wxStaticBoxSizer *editorbox = new wxStaticBoxSizer (
                     new wxStaticBox (panel, -1, _("Initial show preferences")),
                     wxVERTICAL);
    wxGridSizer *editorgrid = new wxGridSizer (2, 2, 20);
    editorbox->Add (editorgrid, 0, wxEXPAND|wxBOTTOM, 2);
    editorgrid->Add (syntax, 0, wxEXPAND);
    editorgrid->Add (folding, 0, wxEXPAND);
    editorgrid->Add (indenting, 0, wxEXPAND);
    editorgrid->Add (displayEOL, 0, wxEXPAND);
    editorgrid->Add (indentGuide, 0, wxEXPAND);
    editorgrid->Add (lineNumber, 0, wxEXPAND);
    editorgrid->Add (longLineOn, 0, wxEXPAND);
    editorgrid->Add (whiteSpace, 0, wxEXPAND);

    // convert to spaces prefs
    m_convertToSpaces = new wxCheckBox (panel, myID_WHITETOSPACE, wxEmptyString);
    wxBoxSizer *convertToSpaces = new wxBoxSizer (wxHORIZONTAL);
    convertToSpaces->Add (4, 0);
    convertToSpaces->Add (new wxStaticText (panel, -1, _("Convert to spaces"),
                                            wxDefaultPosition, wxSize(sz1, -1)),
                          0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    convertToSpaces->Add (6, 0);
    convertToSpaces->Add (m_convertToSpaces, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // convert to tabs prefs
    m_convertToTabs = new wxCheckBox (panel, myID_WHITETOTAB, wxEmptyString);
    wxBoxSizer *convertToTabs = new wxBoxSizer (wxHORIZONTAL);
    convertToTabs->Add (4, 0);
    convertToTabs->Add (new wxStaticText (panel, -1, _("Convert to tabulators"),
                                          wxDefaultPosition, wxSize(sz1, -1)),
                        0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    convertToTabs->Add (6, 0);
    convertToTabs->Add (m_convertToTabs, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);

    // layout the editor  prefs
    wxStaticBoxSizer *modifybox = new wxStaticBoxSizer (
                     new wxStaticBox (panel, -1, _("Modification preferences")),
                     wxVERTICAL);
    wxGridSizer *modifygrid = new wxGridSizer (2, 2, 20);
    modifybox->Add (modifygrid, 0, wxEXPAND|wxBOTTOM, 2);
    modifygrid->Add (convertToSpaces, 0, wxEXPAND);
    modifygrid->Add (convertToTabs, 0, wxEXPAND);

    // layout panel
    wxBoxSizer *panelpane = new wxBoxSizer (wxVERTICAL);
    panelpane->Add (commonbox, 0, wxEXPAND|wxALL, 10);
    panelpane->Add (defaultbox, 0, wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM, 10);
    panelpane->Add (editorbox, 0, wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM, 10);
    panelpane->Add (modifybox, 0, wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM, 10);
    m_feedbackGiven = new wxCheckBox (panel, -1, _("Feedback has been given"));
    panelpane->Add (m_feedbackGiven, 0, wxALIGN_CENTER|wxBOTTOM, 10);

    // set and return panel
    panel->SetSizer (panelpane);
    return panel;
}

//----------------------------------------------------------------------------
bool PreferenceDlg::CheckValuesPageCommon () {
    return
        (g_CommonPrefs.noSplash != m_noSplash->GetValue()) ||
        (g_CommonPrefs.showBrowse != m_showBrowse->GetValue()) ||
        (g_CommonPrefs.showStatusbar != m_showStatusbar->GetValue()) ||
        (g_CommonPrefs.showToolbar != m_showToolbar->GetValue()) ||
        (g_CommonPrefs.usePages != m_usePages->GetValue()) ||
        (g_CommonPrefs.syntaxEnable != m_syntaxEnable->GetValue()) ||
        (g_CommonPrefs.foldEnable != m_foldEnable->GetValue()) ||
        (g_CommonPrefs.indentEnable != m_indentEnable->GetValue()) ||
        (g_CommonPrefs.displayEOLEnable != m_displayEOLEnable->GetValue()) ||
        (g_CommonPrefs.indentGuideEnable != m_indentGuideEnable->GetValue()) ||
        (g_CommonPrefs.lineNumberEnable != m_lineNumberEnable->GetValue()) ||
        (g_CommonPrefs.longLineOnEnable != m_longLineOnEnable->GetValue()) ||
        (g_CommonPrefs.whiteSpaceEnable != m_whiteSpaceEnable->GetValue()) ||
        (g_CommonPrefs.overTypeInitial != m_overTypeInitial->GetValue()) ||
        (g_CommonPrefs.readOnlyInitial != m_readOnlyInitial->GetValue()) ||
        (g_CommonPrefs.wrapModeInitial != m_wrapModeInitial->GetValue()) ||
        (g_CommonPrefs.convertToSpaces != m_convertToSpaces->GetValue()) ||
        (g_CommonPrefs.convertToTabs != m_convertToTabs->GetValue()) ||
        (g_feedback != m_feedbackGiven->GetValue());
}

//----------------------------------------------------------------------------
void PreferenceDlg::GetValuesPageCommon () {
    g_CommonPrefs.noSplash = m_noSplash->GetValue();
    g_CommonPrefs.showBrowse = m_showBrowse->GetValue();
    g_CommonPrefs.showStatusbar = m_showStatusbar->GetValue();
    g_CommonPrefs.showToolbar = m_showToolbar->GetValue();
    g_CommonPrefs.usePages = m_usePages->GetValue();
    g_CommonPrefs.syntaxEnable = m_syntaxEnable->GetValue();
    g_CommonPrefs.foldEnable = m_foldEnable->GetValue();
    g_CommonPrefs.indentEnable = m_indentEnable->GetValue();
    g_CommonPrefs.displayEOLEnable = m_displayEOLEnable->GetValue();
    g_CommonPrefs.indentGuideEnable = m_indentGuideEnable->GetValue();
    g_CommonPrefs.lineNumberEnable = m_lineNumberEnable->GetValue();
    g_CommonPrefs.longLineOnEnable = m_longLineOnEnable->GetValue();
    g_CommonPrefs.whiteSpaceEnable = m_whiteSpaceEnable->GetValue();
    g_CommonPrefs.overTypeInitial = m_overTypeInitial->GetValue();
    g_CommonPrefs.readOnlyInitial = m_readOnlyInitial->GetValue();
    g_CommonPrefs.wrapModeInitial = m_wrapModeInitial->GetValue();
    g_CommonPrefs.convertToSpaces = m_convertToSpaces->GetValue();
    g_CommonPrefs.convertToTabs = m_convertToTabs->GetValue();
    g_feedback = m_feedbackGiven->GetValue();
}

//----------------------------------------------------------------------------
void PreferenceDlg::SetValuesPageCommon () {
    m_noSplash->SetValue (g_CommonPrefs.noSplash);
    m_showBrowse->SetValue (g_CommonPrefs.showBrowse);
    m_showStatusbar->SetValue (g_CommonPrefs.showStatusbar);
    m_showToolbar->SetValue (g_CommonPrefs.showToolbar);
    m_usePages->SetValue (g_CommonPrefs.usePages);
    m_syntaxEnable->SetValue (g_CommonPrefs.syntaxEnable);
    m_foldEnable->SetValue (g_CommonPrefs.foldEnable);
    m_indentEnable->SetValue (g_CommonPrefs.indentEnable);
    m_displayEOLEnable->SetValue (g_CommonPrefs.displayEOLEnable);
    m_indentGuideEnable->SetValue (g_CommonPrefs.indentGuideEnable);
    m_lineNumberEnable->SetValue (g_CommonPrefs.lineNumberEnable);
    m_longLineOnEnable->SetValue (g_CommonPrefs.longLineOnEnable);
    m_whiteSpaceEnable->SetValue (g_CommonPrefs.whiteSpaceEnable);
    m_overTypeInitial->SetValue (g_CommonPrefs.overTypeInitial);
    m_readOnlyInitial->SetValue (g_CommonPrefs.readOnlyInitial);
    m_wrapModeInitial->SetValue (g_CommonPrefs.wrapModeInitial);
    m_convertToSpaces->SetValue (g_CommonPrefs.convertToSpaces);
    m_convertToTabs->SetValue (g_CommonPrefs.convertToTabs);
    m_feedbackGiven->SetValue (g_feedback);
}

//----------------------------------------------------------------------------
wxPanel *PreferenceDlg::CreatePageLanguages () {
    wxPanel *panel = new wxPanel (m_prefsBook);

    int ch = GetCharWidth();
#if defined(__WXGTK__)
    ch -= ch/6; // reduce GTK font width by 15%
#endif
    const int sz1 = 20*ch; // width of a text lable
    const int sz2 = 40*ch; // width of a full sized text field
    const int sz3 = 15*ch; // width of a checkbox text lable

    // initialize language panel
    m_language = new wxComboBox (panel, myID_PREFS_LANGUAGE, wxEmptyString,
                                 wxDefaultPosition, wxSize(sz1,-1));
    for (m_langNr = 0; m_langNr < g_LanguagePrefsSize; ++m_langNr) {
        m_language->Append (g_LanguagePrefs [m_langNr].name);
    }
    m_langNr = 0;
    m_curLanguage = &g_LanguagePrefs [m_langNr];
    m_language->SetSelection (m_langNr);

    // layout the language panel
    m_lexer = new wxStaticText (panel, -1, _("Lexer-ID = 00"));
    wxBoxSizer *choose = new wxBoxSizer (wxHORIZONTAL);
    choose->Add (10, 0);
    choose->Add (new wxStaticText (panel, -1, _("Choose language:"),
                                   wxDefaultPosition, wxSize(sz1, -1)),
                 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    choose->Add (6, 0);
    choose->Add (m_language, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    choose->Add (6, 0, 1);
    choose->Add (m_lexer, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
    choose->Add (12, 0);

    // layout filepattern
    m_filepattern = new wxTextCtrl (panel, -1, wxEmptyString,
                                    wxDefaultPosition, wxSize(sz2, -1));
    wxBoxSizer *filepattern = new wxBoxSizer (wxHORIZONTAL);
    filepattern->Add (4, 0);
    filepattern->Add (new wxStaticText (panel, -1, _("Filepatterns:"),
                                        wxDefaultPosition, wxSize(sz1, -1)),
                      0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    filepattern->Add (6, 0);
    filepattern->Add (m_filepattern, 1, wxEXPAND|wxALIGN_CENTER_VERTICAL);
    filepattern->Add (4, 0);

    // layout helpbooks
    m_helpbooks = new wxTextCtrl (panel, -1, wxEmptyString,
                                  wxDefaultPosition, wxSize(sz2, -1));
    wxBoxSizer *helpbooks = new wxBoxSizer (wxHORIZONTAL);
    helpbooks->Add (4, 0);
    helpbooks->Add (new wxStaticText (panel, -1, _("Help books:"),
                                      wxDefaultPosition, wxSize(sz1, -1)),
                    0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    helpbooks->Add (6, 0);
    helpbooks->Add (m_helpbooks, 1, wxEXPAND|wxALIGN_CENTER_VERTICAL);
    helpbooks->Add (4, 0);

    // indent prefs
    m_indentiation = new wxTextCtrl (panel, -1, wxEmptyString,
                                     wxDefaultPosition, wxSize(3*ch,-1));
    wxBoxSizer *indentiation = new wxBoxSizer (wxHORIZONTAL);
    indentiation->Add (4, 0);
    indentiation->Add (new wxStaticText (panel, -1, _("Indent value:"),
                                         wxDefaultPosition, wxSize(sz1, -1)),
                       0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    indentiation->Add (6, 0);
    indentiation->Add (m_indentiation, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    indentiation->Add (4, 0);

    // long line prefs
    m_longlinemark = new wxTextCtrl (panel, -1, wxEmptyString,
                                 wxDefaultPosition, wxSize(3*ch,-1));
    wxBoxSizer *longline = new wxBoxSizer (wxHORIZONTAL);
    longline->Add (4, 0);
    longline->Add (new wxStaticText (panel, -1, _("Long line marker (pos):"),
                                     wxDefaultPosition, wxSize(sz1, -1)),
                   0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    longline->Add (6, 0);
    longline->Add (m_longlinemark, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    longline->Add (4, 0);

    // flags prefs
    m_flagsUseSpaces = new wxCheckBox (panel, -1, _("Use spaces"),
                                     wxDefaultPosition, wxSize(sz3, -1));
    m_flagsWrapMode = new wxCheckBox (panel, -1, _("Wrap mode"),
                                      wxDefaultPosition, wxSize(sz3, -1));
    wxBoxSizer *flags = new wxBoxSizer (wxHORIZONTAL);
    flags->Add (4, 0);
    flags->Add (new wxStaticText (panel, -1, _("Flags:"),
                                    wxDefaultPosition, wxSize(sz1, -1)),
                  0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    flags->Add (6, 0);
    flags->Add (m_flagsUseSpaces, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    flags->Add (m_flagsWrapMode, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);

    // layout the language changable prefs
    wxStaticBoxSizer *changepane = new wxStaticBoxSizer (
                     new wxStaticBox (panel, -1, _T("Language settings (changable)")),
                     wxVERTICAL);
    changepane->Add (filepattern, 0, wxEXPAND);
    changepane->Add (0, 6);
    changepane->Add (helpbooks, 0, wxEXPAND);
    changepane->Add (0, 6);
    changepane->Add (indentiation, 0, wxEXPAND);
    changepane->Add (0, 6);
    changepane->Add (longline, 0, wxEXPAND);
    changepane->Add (0, 6);
    changepane->Add (flags, 0, wxEXPAND);
    changepane->Add (0, 4);

    // styles prefs
    m_styles = new wxListBox (panel, myID_PREFS_KEYWORDS);
    m_words = new wxTextCtrl (panel, -1, wxEmptyString,
                              wxDefaultPosition, wxSize(sz1, sz1),
                              wxTE_MULTILINE|wxTE_WORDWRAP);
    m_words->Enable (false);
    wxBoxSizer *stylelist = new wxBoxSizer (wxVERTICAL);
    stylelist->Add (new wxStaticText (panel, -1, _("Styles:"),
                                      wxDefaultPosition, wxSize(sz1, -1)),
                      0, wxALIGN_LEFT);
    stylelist->Add (0, 6);
    stylelist->Add (m_styles, 1, wxEXPAND|wxALIGN_LEFT);
    wxBoxSizer *wordlist = new wxBoxSizer (wxVERTICAL);
    wordlist->Add (new wxStaticText (panel, -1, _("Keywords:"),
                                     wxDefaultPosition, wxSize(sz1, -1)),
                   0, wxALIGN_LEFT);
    wordlist->Add (0, 6);
    wordlist->Add (m_words, 1, wxEXPAND|wxALIGN_LEFT);
    wxBoxSizer *stylepane = new wxBoxSizer (wxHORIZONTAL);
    stylepane->Add (4, 0);
    stylepane->Add (stylelist, 0, wxEXPAND|wxALIGN_LEFT);
    stylepane->Add (6, 0);
    stylepane->Add (wordlist, 1, wxEXPAND|wxALIGN_LEFT);
    stylepane->Add (4, 0);

    // folding prefs
    m_foldComment = new wxCheckBox (panel, -1, _("Comment"),
                                    wxDefaultPosition, wxSize(sz3, -1));
    m_foldComment->Enable (false);
    m_foldCompact = new wxCheckBox (panel, -1, _("Compact"),
                                    wxDefaultPosition, wxSize(sz3, -1));
    m_foldCompact->Enable (false);
    m_foldPreproc = new wxCheckBox (panel, -1, _("Preprocessor"),
                                    wxDefaultPosition, wxSize(sz3, -1));
    m_foldPreproc->Enable (false);
    m_foldHtml = new wxCheckBox (panel, -1, _("Html"),
                                 wxDefaultPosition, wxSize(sz3, -1));
    m_foldHtml->Enable (false);
    m_foldHtmlPrep = new wxCheckBox (panel, -1, _("Html Preproc."),
                                     wxDefaultPosition, wxSize(sz3, -1));
    m_foldHtmlPrep->Enable (false);
    m_foldCommentPy = new wxCheckBox (panel, -1, _("Comment python"),
                                      wxDefaultPosition, wxSize(sz3, -1));
    m_foldCommentPy->Enable (false);
    m_foldQuotesPy = new wxCheckBox (panel, -1, _("Quotes python"),
                                     wxDefaultPosition, wxSize(sz3, -1));
    m_foldQuotesPy->Enable (false);
    m_folding = new wxBoxSizer (wxHORIZONTAL);
    m_folding->Add (4, 0);
    m_folding->Add (new wxStaticText (panel, -1, _("Fold points:"),
                                      wxDefaultPosition, wxSize(sz1, -1)),
                    0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    m_folding->Add (6, 0);
    m_folding->Add (m_foldComment, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    m_folding->Add (m_foldCompact, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    m_folding->Add (m_foldPreproc, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    m_folding->Add (m_foldHtml, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    m_folding->Add (m_foldHtmlPrep, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    m_folding->Add (m_foldCommentPy, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    m_folding->Add (m_foldQuotesPy, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    m_folding->Add (4, 0);

    // layout the language readonly prefs
    wxStaticBoxSizer *prefspane = new wxStaticBoxSizer (
                     new wxStaticBox (panel, -1, _T("Language settings (read only)")),
                     wxVERTICAL);
    prefspane->Add (stylepane, 1, wxEXPAND);
    prefspane->Add (0, 6);
    prefspane->Add (m_folding, 0, wxEXPAND);
    prefspane->Add (0, 4);

    // layout panel
    wxBoxSizer *panelpane = new wxBoxSizer (wxVERTICAL);
    panelpane->Add (choose, 0, wxEXPAND|wxALL, 10);
    panelpane->Add (changepane, 0, wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM, 10);
    panelpane->Add (prefspane, 0, wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM, 10);

    // set and return panel
    panel->SetSizer (panelpane);
    return panel;
}

//----------------------------------------------------------------------------
bool PreferenceDlg::CheckValuesPageLanguages () {
    long indentiation = 0; m_indentiation->GetValue().ToLong (&indentiation);
    long longlinemark = 0; m_longlinemark->GetValue().ToLong (&longlinemark);
    return
        (m_curLanguage->filepattern != m_filepattern->GetValue()) ||
        (m_curLanguage->helpbooks != m_helpbooks->GetValue()) ||
        (m_curLanguage->indent != indentiation) ||
        (m_curLanguage->longln != longlinemark) ||
        ((m_curLanguage->flags & EDIT_FLAG_USESPACES) !=
         (m_flagsUseSpaces->GetValue()? EDIT_FLAG_USESPACES: 0)) ||
        ((m_curLanguage->flags & EDIT_FLAG_WRAPMODE) !=
         (m_flagsWrapMode->GetValue()? EDIT_FLAG_WRAPMODE: 0));
}

//----------------------------------------------------------------------------
void PreferenceDlg::GetValuesPageLanguages () {
    if (m_curLanguage->filepattern != m_filepattern->GetValue()) {
        if (g_LanguageTable [m_langNr].filepattern != m_curLanguage->filepattern) {
            delete m_curLanguage->filepattern;
        }
        m_curLanguage->filepattern = wxStrdup (m_filepattern->GetValue());
    }
    if (m_curLanguage->helpbooks != m_helpbooks->GetValue()) {
        if (g_LanguageTable [m_langNr].helpbooks != m_curLanguage->helpbooks) {
            delete m_curLanguage->helpbooks;
        }
        m_curLanguage->helpbooks = wxStrdup (m_helpbooks->GetValue());
    }
    long indent = 0;
    m_indentiation->GetValue().ToLong (&indent);
    m_curLanguage->indent = indent;
    long longln = 0;
    m_longlinemark->GetValue().ToLong (&longln);
    m_curLanguage->longln = longln;
    int flags = 0;
    flags += m_flagsUseSpaces->GetValue()? EDIT_FLAG_USESPACES: 0;
    flags += m_flagsWrapMode->GetValue()? EDIT_FLAG_WRAPMODE: 0;
    m_curLanguage->flags = flags;
}

//----------------------------------------------------------------------------
void PreferenceDlg::SetValuesPageLanguages () {
    m_lexer->SetLabel (wxString::Format (_("Lexer-ID = %d"), m_curLanguage->lexer));
    wxString filepattern = m_curLanguage->filepattern;
    m_filepattern->SetValue (filepattern);
    wxString helpbooks = m_curLanguage->helpbooks;
    m_helpbooks->SetValue (helpbooks);

    m_styles->Clear();
    for (m_lgStyleNr = 0; m_lgStyleNr < STYLE_TYPES_COUNT; ++m_lgStyleNr) {
        if (m_curLanguage->styles[m_lgStyleNr].type >= 0) {
            m_styles->Append (g_StylePrefs [m_curLanguage->styles[m_lgStyleNr].type].name);
        }
    }
    m_lgStyleNr = 0;
    m_styles->SetSelection (m_lgStyleNr);

    if (m_curLanguage->styles[m_lgStyleNr].words) {
        m_words->SetValue (m_curLanguage->styles[m_lgStyleNr].words);
    }else{
        m_words->SetValue (wxEmptyString);
    }

    bool others = ((m_curLanguage->lexer != wxSCI_LEX_HTML) &&
                   (m_curLanguage->lexer != wxSCI_LEX_PYTHON));
    m_foldComment->SetValue ((m_curLanguage->folds & FOLD_TYPE_COMMENT) > 0);
    m_folding->Show (m_foldComment, others);
    m_foldCompact->SetValue ((m_curLanguage->folds & FOLD_TYPE_COMPACT) > 0);
    m_folding->Show (m_foldCompact, others);
    m_foldPreproc->SetValue ((m_curLanguage->folds & FOLD_TYPE_PREPROC) > 0);
    m_folding->Show (m_foldPreproc, others);
    m_foldHtml->SetValue ((m_curLanguage->folds & FOLD_TYPE_HTML) > 0);
    m_folding->Show (m_foldHtml, m_curLanguage->lexer == wxSCI_LEX_HTML);
    m_foldHtmlPrep->SetValue ((m_curLanguage->folds & FOLD_TYPE_HTMLPREP) > 0);
    m_folding->Show (m_foldHtmlPrep, m_curLanguage->lexer == wxSCI_LEX_HTML);
    m_foldCommentPy->SetValue ((m_curLanguage->folds & FOLD_TYPE_COMMENTPY) > 0);
    m_folding->Show (m_foldCommentPy, m_curLanguage->lexer == wxSCI_LEX_PYTHON);
    m_foldQuotesPy->SetValue ((m_curLanguage->folds & FOLD_TYPE_QUOTESPY) > 0);
    m_folding->Show (m_foldQuotesPy, m_curLanguage->lexer == wxSCI_LEX_PYTHON);

    m_indentiation->SetLabel (wxString::Format (_T("%d"), m_curLanguage->indent));
    m_longlinemark->SetLabel (wxString::Format (_T("%d"), m_curLanguage->longln));

    m_flagsUseSpaces->SetValue ((m_curLanguage->flags & EDIT_FLAG_USESPACES) > 0);
    m_flagsWrapMode->SetValue ((m_curLanguage->flags & EDIT_FLAG_WRAPMODE) > 0);
}

//----------------------------------------------------------------------------
wxPanel *PreferenceDlg::CreatePageStyles () {
    wxPanel *panel = new wxPanel (m_prefsBook);

    int ch = GetCharWidth();
#if defined(__WXGTK__)
    ch -= ch/6; // reduce GTK font width by 15%
#endif
    const int sz1 = 20*ch; // width of a text lable

    // initialize styletypes panel
    m_styletype = new wxComboBox (panel, myID_PREFS_STYLETYPE, wxEmptyString,
                                  wxDefaultPosition, wxSize(sz1,-1));
    for (m_styleNr = 0; m_styleNr < g_StylePrefsSize; ++m_styleNr) {
        m_styletype->Append (g_StylePrefs [m_styleNr].name);
    }
    m_styleNr = 0;
    m_curStyle = &g_StylePrefs [m_styleNr];
    m_styletype->SetSelection (m_styleNr);

    // layout the styletypes panel
    wxBoxSizer *choose = new wxBoxSizer (wxHORIZONTAL);
    choose->Add (10, 0);
    choose->Add (new wxStaticText (panel, -1, _("Choose style:"),
                                   wxDefaultPosition, wxSize(sz1, -1)),
                 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    choose->Add (6, 0);
    choose->Add (m_styletype, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    choose->Add (12, 0);

    // foreground prefs
    wxArrayString *colourNames = new wxArrayString();
    colourEnum_GetNames (colourNames);
    m_foreground = new wxComboBox (panel, myID_PREFS_STYLEFOREGROUND, wxEmptyString,
                                   wxDefaultPosition, wxSize(sz1, -1),
                                   0);
    int f;
    for (f = 0; f < (int)colourNames->GetCount(); ++f) {
        m_foreground->Append (colourNames->Item (f));
    }
    wxBoxSizer *foreground = new wxBoxSizer (wxHORIZONTAL);
    foreground->Add (4, 0);
    foreground->Add (new wxStaticText (panel, -1, _("Foreground:"),
                                       wxDefaultPosition, wxSize(sz1, -1)),
                     0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    foreground->Add (6, 0);
    foreground->Add (m_foreground, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    foreground->Add (4, 0);

    // background prefs
    m_background = new wxComboBox (panel, myID_PREFS_STYLEBACKGROUND, wxEmptyString,
                                   wxDefaultPosition, wxSize(sz1, -1),
                                   0);
    int b;
    for (b = 0; b < (int)colourNames->GetCount(); ++b) {
        m_background->Append (colourNames->Item (b));
    }
    delete colourNames;
    wxBoxSizer *background = new wxBoxSizer (wxHORIZONTAL);
    background->Add (4, 0);
    background->Add (new wxStaticText (panel, -1, _("Background:"),
                                       wxDefaultPosition, wxSize(sz1, -1)),
                     0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    background->Add (6, 0);
    background->Add (m_background, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    background->Add (4, 0);

    // font case prefs
    m_unchanged = new wxRadioButton (panel, -1, _T("Normal"),
                                     wxDefaultPosition, wxSize(12*ch, -1), wxRB_GROUP);
    m_uppercase = new wxRadioButton (panel, -1, _T("Uppercase"),
                                     wxDefaultPosition, wxSize(12*ch, -1));
    m_lowercase = new wxRadioButton (panel, -1, _T("Lowercase"),
                                     wxDefaultPosition, wxSize(12*ch, -1));
    wxBoxSizer *lettercase = new wxBoxSizer (wxHORIZONTAL);
    lettercase->Add (4, 0);
    lettercase->Add (new wxStaticText (panel, -1, _("Letter case:"),
                                       wxDefaultPosition, wxSize(sz1, -1)),
                     0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    lettercase->Add (6, 0);
    lettercase->Add (m_unchanged, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);
    lettercase->Add (m_uppercase, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);
    lettercase->Add (m_lowercase, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);
    lettercase->Add (4, 0);

    // style settings
    wxStaticBoxSizer *stylepane = new wxStaticBoxSizer (
                     new wxStaticBox (panel, -1, _("Style settings")),
                     wxVERTICAL);
    stylepane->Add (foreground, 0, wxEXPAND);
    stylepane->Add (0, 6);
    stylepane->Add (background, 0, wxEXPAND);
    stylepane->Add (0, 6);
    stylepane->Add (lettercase, 0, wxEXPAND);
    stylepane->Add (0, 4);

    // font name prefs
    wxFontEnumerator fontEnum;
    fontEnum.EnumerateFacenames(wxFONTENCODING_SYSTEM, true);
    wxArrayString *fontNames = fontEnum.GetFacenames();
    fontNames->Sort();
    m_fontname = new wxComboBox (panel, myID_PREFS_STYLEFONTNAME, wxEmptyString,
                                 wxDefaultPosition, wxSize(sz1, -1),
                                 0);
    int i;
    for (i = 0; i < (int)fontNames->GetCount(); ++i) {
        m_fontname->Append (fontNames->Item (i));
    }
    wxBoxSizer *fontname = new wxBoxSizer (wxHORIZONTAL);
    fontname->Add (4, 0);
    fontname->Add (new wxStaticText (panel, -1, _("Font name:"),
                                     wxDefaultPosition, wxSize(sz1, -1)),
                   0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    fontname->Add (6, 0);
    fontname->Add (m_fontname, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    fontname->Add (4, 0);

    // font size prefs
    m_fontsize = new wxTextCtrl (panel, myID_PREFS_STYLEFONTSIZE, wxEmptyString,
                                 wxDefaultPosition, wxSize(3*ch,-1));
    wxBoxSizer *fontsize = new wxBoxSizer (wxHORIZONTAL);
    fontsize->Add (4, 0);
    fontsize->Add (new wxStaticText (panel, -1, _("Font size:"),
                                      wxDefaultPosition, wxSize(sz1, -1)),
                    0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    fontsize->Add (6, 0);
    fontsize->Add (m_fontsize, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    fontsize->Add (4, 0);

    // font style prefs
    m_stylebold = new wxCheckBox (panel, myID_PREFS_STYLEBOLD, _("Bold"),
                                  wxDefaultPosition, wxSize(12*ch, -1));
    m_styleitalic = new wxCheckBox (panel, myID_PREFS_STYLEITALIC, _("Italic"),
                                    wxDefaultPosition, wxSize(12*ch, -1));
    m_styleunderl = new wxCheckBox (panel, myID_PREFS_STYLEUNDERLINE, _("Underline"),
                                       wxDefaultPosition, wxSize(12*ch, -1));
    m_stylehidden = new wxCheckBox (panel, myID_PREFS_STYLEHIDDEN, _("Hidden"),
                                    wxDefaultPosition, wxSize(12*ch, -1));
    wxBoxSizer *fontstyle = new wxBoxSizer (wxHORIZONTAL);
    fontstyle->Add (4, 0);
    fontstyle->Add (new wxStaticText (panel, -1, _("Font style:"),
                                      wxDefaultPosition, wxSize(sz1, -1)),
                    0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    fontstyle->Add (6, 0);
    fontstyle->Add (m_stylebold, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);
    fontstyle->Add (m_styleitalic, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);
    fontstyle->Add (m_styleunderl, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);
    fontstyle->Add (m_stylehidden, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL);
    fontstyle->Add (4, 0);

    // sample font
    int h = GetCharHeight();
    m_sampletext = new wxTextCtrl (panel, -1,
                                   _("This text shows the style settings"),
                                   wxDefaultPosition, wxSize(2*sz1,2*h),
                                   wxTE_READONLY);
    wxBoxSizer *sampletext = new wxBoxSizer (wxHORIZONTAL);
    sampletext->Add (4, 0);
    sampletext->Add (new wxStaticText (panel, -1, _("Sample (readonly):"),
                                       wxDefaultPosition, wxSize(sz1, -1)),
                     0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    sampletext->Add (6, 0);
    sampletext->Add (m_sampletext, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    sampletext->Add (4, 0);

    // style settings
    wxStaticBoxSizer *fontspane = new wxStaticBoxSizer (
                     new wxStaticBox (panel, -1, _("Fonts settings")),
                     wxVERTICAL);
    fontspane->Add (fontname, 0, wxEXPAND);
    fontspane->Add (0, 6);
    fontspane->Add (fontsize, 0, wxEXPAND);
    fontspane->Add (0, 6);
    fontspane->Add (fontstyle, 0, wxEXPAND);
    fontspane->Add (0, 6);
    fontspane->Add (sampletext, 0, wxEXPAND);
    fontspane->Add (0, 4);

    // layout panel
    wxBoxSizer *panelpane = new wxBoxSizer (wxVERTICAL);
    panelpane->Add (choose, 0, wxEXPAND|wxALL, 10);
    panelpane->Add (stylepane, 0, wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM, 10);
    panelpane->Add (fontspane, 0, wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM, 10);

    // set and return panel
    panel->SetSizer (panelpane);
    return panel;
}

//----------------------------------------------------------------------------
bool PreferenceDlg::CheckValuesPageStyles () {
    long fontsize = 0; m_fontsize->GetValue().ToLong (&fontsize);
    return
        (m_curStyle->foreground != m_foreground->GetValue()) ||
        (m_curStyle->background != m_background->GetValue()) ||
        (m_curStyle->fontname != m_fontname->GetValue()) ||
        (m_curStyle->fontsize != fontsize) ||
        ((m_curStyle->fontstyle & TOKEN_STYLE_BOLD) !=
         (m_stylebold->GetValue()? TOKEN_STYLE_BOLD: 0)) ||
        ((m_curStyle->fontstyle & TOKEN_STYLE_ITALIC) !=
         (m_styleitalic->GetValue()? TOKEN_STYLE_ITALIC: 0)) ||
        ((m_curStyle->fontstyle & TOKEN_STYLE_UNDERL) !=
         (m_styleunderl->GetValue()? TOKEN_STYLE_UNDERL: 0)) ||
        ((m_curStyle->fontstyle & TOKEN_STYLE_HIDDEN) !=
         (m_stylehidden->GetValue()? TOKEN_STYLE_HIDDEN: 0)) ||
        ((m_curStyle->lettercase & wxSCI_CASE_MIXED) !=
         (m_unchanged->GetValue()? wxSCI_CASE_MIXED: 0)) ||
        ((m_curStyle->lettercase & wxSCI_CASE_UPPER) !=
         (m_uppercase->GetValue()? wxSCI_CASE_UPPER: 0)) ||
        ((m_curStyle->lettercase & wxSCI_CASE_LOWER) !=
         (m_lowercase->GetValue()? wxSCI_CASE_LOWER: 0));
}

//----------------------------------------------------------------------------
void PreferenceDlg::GetValuesPageStyles () {
    if (m_curStyle->foreground != m_foreground->GetValue()) {
        if (g_StyleTable [m_styleNr].foreground != m_curStyle->foreground) {
            delete m_curStyle->foreground;
        }
        m_curStyle->foreground = wxStrdup (m_foreground->GetValue());
    }
    if (m_curStyle->background != m_background->GetValue()) {
        if (g_StyleTable [m_styleNr].background != m_curStyle->background) {
            delete m_curStyle->background;
        }
        m_curStyle->background = wxStrdup (m_background->GetValue());
    }
    if (m_curStyle->fontname != m_fontname->GetValue()) {
        if (g_StyleTable [m_styleNr].fontname != m_curStyle->fontname) {
            delete m_curStyle->fontname;
        }
        m_curStyle->fontname = wxStrdup (m_fontname->GetValue());
    }
    long fontsize = 0;
    m_fontsize->GetValue().ToLong (&fontsize);
    m_curStyle->fontsize = fontsize;
    int fontstyle = 0;
    fontstyle += m_stylebold->GetValue()? TOKEN_STYLE_BOLD: 0;
    fontstyle += m_styleitalic->GetValue()? TOKEN_STYLE_ITALIC: 0;
    fontstyle += m_styleunderl->GetValue()? TOKEN_STYLE_UNDERL: 0;
    fontstyle += m_stylehidden->GetValue()? TOKEN_STYLE_HIDDEN: 0;
    m_curStyle->fontstyle = fontstyle;
    int lettercase = 0;
    lettercase += m_unchanged->GetValue()? wxSCI_CASE_MIXED: 0;
    lettercase += m_uppercase->GetValue()? wxSCI_CASE_UPPER: 0;
    lettercase += m_lowercase->GetValue()? wxSCI_CASE_LOWER: 0;
    m_curStyle->lettercase = lettercase;
}

//----------------------------------------------------------------------------
void PreferenceDlg::SetValuesPageStyles () {
    m_foreground->SetValue (m_curStyle->foreground);
    m_background->SetValue (m_curStyle->background);
    m_fontname->SetValue (m_curStyle->fontname);
    m_fontsize->SetValue (wxString::Format (_T("%d"), m_curStyle->fontsize));
    m_stylebold->SetValue ((m_curStyle->fontstyle & TOKEN_STYLE_BOLD) > 0);
    m_styleitalic->SetValue ((m_curStyle->fontstyle & TOKEN_STYLE_ITALIC) > 0);
    m_styleunderl->SetValue ((m_curStyle->fontstyle & TOKEN_STYLE_UNDERL) > 0);
    m_stylehidden->SetValue ((m_curStyle->fontstyle & TOKEN_STYLE_HIDDEN) > 0);
    m_unchanged->SetValue (m_curStyle->lettercase == wxSCI_CASE_MIXED);
    m_uppercase->SetValue (m_curStyle->lettercase == wxSCI_CASE_UPPER);
    m_lowercase->SetValue (m_curStyle->lettercase == wxSCI_CASE_LOWER);
    // update sample font
    m_sampletext->SetForegroundColour(m_curStyle->foreground);
    m_sampletext->SetBackgroundColour(m_curStyle->background);
    wxFont font (m_curStyle->fontsize, wxTELETYPE,
                 m_curStyle->fontstyle & TOKEN_STYLE_ITALIC? wxITALIC: wxNORMAL,
                 m_curStyle->fontstyle & TOKEN_STYLE_BOLD? wxBOLD: wxNORMAL,
                 m_curStyle->fontstyle & TOKEN_STYLE_UNDERL? true: false,
                 m_curStyle->fontname);
    m_sampletext->SetFont (font);
    m_sampletext->Show (m_curStyle->fontstyle & TOKEN_STYLE_HIDDEN? false: true);
    m_sampletext->Refresh();
}

//----------------------------------------------------------------------------
wxPanel *PreferenceDlg::CreatePageAnalyse () {
    wxPanel *panel = new wxPanel (m_prefsBook);

    int ch = GetCharWidth();
#if defined(__WXGTK__)
    ch -= ch/6; // reduce GTK font width by 15%
#endif
    const int sz1 = 20*ch; // width of a text lable

    // initialize analyse panel
    m_analyse = new wxComboBox (panel, myID_PREFS_ANALYSE, wxEmptyString,
                                 wxDefaultPosition, wxSize(sz1,-1));
    int nr;
    for (nr = 0; nr < g_AnalysePrefsSize; ++nr) {
        m_analyse->Append (g_AnalysePrefs [nr].name);
    }
    m_curAnalyse = &g_AnalysePrefs [0];
    m_analyse->SetSelection (0);

    // choose box
    wxBoxSizer *choose = new wxBoxSizer (wxHORIZONTAL);
    choose->Add (10, 0);
    choose->Add (new wxStaticText (panel, -1, _("Choose language:"),
                                   wxDefaultPosition, wxSize(sz1, -1)),
                 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    choose->Add (6, 0);
    choose->Add (m_analyse, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    choose->Add (12, 0);

    // rules list
    wxBoxSizer *rulelist = new wxBoxSizer (wxHORIZONTAL);
    rulelist->Add (4, 0);
    rulelist->Add (new wxStaticText (panel, -1, _("Rules:"),
                                     wxDefaultPosition, wxSize(sz1, -1)),
                   0, wxALIGN_LEFT);
    m_rules = new wxListCtrl (panel, -1, wxDefaultPosition, wxDefaultSize,
                              wxLC_REPORT|wxLC_HRULES|wxLC_VRULES);
    m_rules->InsertColumn (0, _("State"),   wxLIST_FORMAT_CENTRE, 6*ch);
    m_rules->InsertColumn (1, _("Style"),   wxLIST_FORMAT_LEFT,   wxLIST_AUTOSIZE);
    m_rules->InsertColumn (2, _("Keyword"), wxLIST_FORMAT_LEFT,   12*ch);
    m_rules->InsertColumn (3, _("Level"),   wxLIST_FORMAT_LEFT,   6*ch);
    m_rules->InsertColumn (4, _("Next"),    wxLIST_FORMAT_CENTRE, 6*ch);
    rulelist->Add (6, 0);
    rulelist->Add (m_rules, 1, wxEXPAND|wxALIGN_LEFT);
    rulelist->Add (4, 0);

    // rules pane
    wxStaticBoxSizer *rulespane = new wxStaticBoxSizer (
                     new wxStaticBox (panel, -1, _T("Rules for Identifiere syntax")),
                     wxVERTICAL);
    rulespane->Add (rulelist, 1, wxEXPAND);
    rulespane->Add (0, 4);

    // layout panel
    wxBoxSizer *panelpane = new wxBoxSizer (wxVERTICAL);
    panelpane->Add (choose, 0, wxEXPAND|wxALL, 10);
    panelpane->Add (rulespane, 1, wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM, 10);

    // set and return panel
    panel->SetSizer (panelpane);
    return panel;
}

//----------------------------------------------------------------------------
bool PreferenceDlg::CheckValuesPageAnalyse () {
    return false;
}

//----------------------------------------------------------------------------
void PreferenceDlg::GetValuesPageAnalyse () {
}

//----------------------------------------------------------------------------
void PreferenceDlg::SetValuesPageAnalyse () {
    m_rules->DeleteAllItems();
    wxString text;
    int nr;
    for (nr = 0; nr < ANALYSE_COUNT; ++nr) {
        m_rules->InsertItem (nr, wxEmptyString);
        if (m_curAnalyse->rules[nr].state >= 0) {
            text = wxString::Format(_T("%d"), m_curAnalyse->rules[nr].state);
            m_rules->SetItem (nr, 0, text);
        }
        if (m_curAnalyse->rules[nr].style >= 0) {
            text = g_StylePrefs [m_curAnalyse->rules[nr].style].name;
            m_rules->SetItem (nr, 1, text);
        }
        m_rules->SetItem (nr, 2, m_curAnalyse->rules[nr].keyword);
        if (m_curAnalyse->rules[nr].level >= 0) {
            text = wxString::Format(_T("%d"), m_curAnalyse->rules[nr].level);
            m_rules->SetItem (nr, 3, text);
        }
        if (m_curAnalyse->rules[nr].next >= 0) {
            text = wxString::Format(_T("%d"), m_curAnalyse->rules[nr].next);
            m_rules->SetItem (nr, 4, text);
        }
    }
    m_rules->Layout();
}

