//////////////////////////////////////////////////////////////////////////////
// File:        prefdlg.h
// Purpose:     Preferences dialog
// Maintainer:  Otto Wyss
// Created:     2003-04-28
// RCS-ID:      $Id: prefdlg.h,v 1.1 2005/03/17 20:03:43 wyo Exp $
// Copyright:   (c) 2005 wyoGuide
// Licence:     wxWindows
//////////////////////////////////////////////////////////////////////////////

#ifndef _PREFDLG_H_
#define _PREFDLG_H_

//----------------------------------------------------------------------------
// information
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
// headers
//----------------------------------------------------------------------------

//! wxWidgets headers
#include <wx/listctrl.h> // list ctrl support
#include <wx/notebook.h> // notebook support

//! application headers
#include "defsext.h"     // additional definitions
#include "prefs.h"       // preferences

//============================================================================
// declarations
//============================================================================

//----------------------------------------------------------------------------
//! CheckInstallation
extern void CheckInstallation (wxString appVersion);

//----------------------------------------------------------------------------
//! PreferenceDlg
class PreferenceDlg: public wxDialog {

public:
    //! constructor
    PreferenceDlg (wxWindow *parent,
                   long style = 0);

    //! destructor
    ~PreferenceDlg ();

    //! event handlers
    void OnApply (wxCommandEvent& event);
    void OnApplyUI (wxUpdateUIEvent &event);
    void OnCancel (wxCommandEvent& event);
    void OnOkay (wxCommandEvent& event);
    void OnReset (wxCommandEvent& event);
    void OnHelp (wxCommandEvent &event);

    void OnConvertTo (wxCommandEvent &event);

    void OnLanguageChoose (wxCommandEvent &event);
    void OnStyletypeChoose (wxCommandEvent &event);
    void OnAnalyseChoose (wxCommandEvent &event);
    void OnKeywordsChange (wxCommandEvent &event);
    void OnStyleChange (wxCommandEvent &event);

private:
    Prefs *m_prefs;

    // default fontname
    wxString m_default_fontname;

    //! preferences pages
    wxNotebook *m_prefsBook;
    int m_prefsPageCommon;
    wxPanel *CreatePageCommon ();
    int m_prefsPageLanguages;
    wxPanel *CreatePageLanguages ();
    int m_prefsPageStyles;
    wxPanel *CreatePageStyles ();
    int m_prefsPageAnalyse;
    wxPanel *CreatePageAnalyse ();

    //! check/get/set values
    bool CheckValuesPageCommon ();
    void GetValuesPageCommon ();
    void SetValuesPageCommon ();
    bool CheckValuesPageLanguages ();
    void GetValuesPageLanguages ();
    void SetValuesPageLanguages ();
    bool CheckValuesPageStyles ();
    void GetValuesPageStyles ();
    void SetValuesPageStyles ();
    bool CheckValuesPageAnalyse ();
    void GetValuesPageAnalyse ();
    void SetValuesPageAnalyse ();

    // common variables
    wxCheckBox *m_noSplash;
    wxCheckBox *m_showBrowse;
    wxCheckBox *m_showToolbar;
    wxCheckBox *m_showStatusbar;
    wxCheckBox *m_usePages;
    wxCheckBox *m_syntaxEnable;
    wxCheckBox *m_foldEnable;
    wxCheckBox *m_indentEnable;
    wxCheckBox *m_overTypeInitial;
    wxCheckBox *m_readOnlyInitial;
    wxCheckBox *m_wrapModeInitial;
    wxCheckBox *m_displayEOLEnable;
    wxCheckBox *m_indentGuideEnable;
    wxCheckBox *m_lineNumberEnable;
    wxCheckBox *m_longLineOnEnable;
    wxCheckBox *m_whiteSpaceEnable;
    wxCheckBox *m_convertToSpaces;
    wxCheckBox *m_convertToTabs;

    // language variables
    int m_langNr;
    LanguageInfo *m_curLanguage;
    wxComboBox *m_language;
    wxStaticText *m_lexer;
    wxTextCtrl *m_filepattern;
    wxTextCtrl *m_helpbooks;
    int m_lgStyleNr;
    wxListBox *m_styles;
    wxTextCtrl *m_words;
    wxBoxSizer *m_folding;
    wxCheckBox *m_foldHtml;
    wxCheckBox *m_foldHtmlPrep;
    wxCheckBox *m_foldCommentPy;
    wxCheckBox *m_foldQuotesPy;
    wxCheckBox *m_foldComment;
    wxCheckBox *m_foldCompact;
    wxCheckBox *m_foldPreproc;
    wxTextCtrl *m_indentiation;
    wxTextCtrl *m_longlinemark;
    wxCheckBox *m_flagsUseSpaces;
    wxCheckBox *m_flagsWrapMode;

    // style types variables
    int m_styleNr;
    StyleInfo *m_curStyle;
    wxComboBox *m_styletype;
    wxComboBox *m_foreground;
    wxComboBox *m_background;
    wxComboBox *m_fontname;
    wxTextCtrl *m_fontsize;
    wxCheckBox *m_stylebold;
    wxCheckBox *m_styleitalic;
    wxCheckBox *m_styleunderl;
    wxCheckBox *m_stylehidden;
    wxTextCtrl *m_sampletext;
    wxRadioButton *m_unchanged;
    wxRadioButton *m_uppercase;
    wxRadioButton *m_lowercase;

    // analyse variables
    int m_analyseNr;
    AnalyseInfo *m_curAnalyse;
    wxComboBox *m_analyse;
    wxListCtrl *m_rules;

    // feedback
    wxCheckBox *m_feedbackGiven;

    DECLARE_EVENT_TABLE()
};

#endif // _PREFDLG_H_

