//////////////////////////////////////////////////////////////////////////////
// File:        prefs.cpp
// Purpose:     Preferences initialization
// Maintainer:  Otto Wyss
// Created:     2003-04-04
// RCS-ID:      $Id: prefs.cpp,v 1.1 2005/03/17 20:03:43 wyo Exp $
// Copyright:   (c) 2005 wyoGuide
// Licence:     wxWindows
//////////////////////////////////////////////////////////////////////////////

//----------------------------------------------------------------------------
// headers
//----------------------------------------------------------------------------

// For compilers that support precompilation, includes <wx/wx.h>.
#include <wx/wxprec.h>

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all 'standard' wxWidgets headers)
#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

//! wxWidgets headers

//! wxWidgets/contrib headers

//! application headers
#include "defsext.h"     // Additional definitions

#include "prefs.h"       // Preferences


//============================================================================
// declarations
//============================================================================

const wxString INSTALLATION = _T("Installation");
const wxString INSTALL_ABOUT = _T("About");
const wxString INSTALL_DATE = _T("Date");
const wxString INSTALL_VERSION = _T("Version");

const wxString NOSPLASH = _T("NoSplashScreen");
const wxString SHOWBROWSE = _T("ShowBrowseInfo");
const wxString SHOWSTATUSBAR = _T("ShowStatusbar");
const wxString SHOWTOOLBAR = _T("ShowToolbar");
const wxString USEPAGES = _T("UsePages");

const wxString SYNTAXENABLE = _T("SyntaxColor");
const wxString FOLDENABLE = _T("Folding");
const wxString INDENTENABLE  =_T("Indenting");
const wxString OVERTYPE = _T("OverType");
const wxString READONLY = _T("ReadOnly");
const wxString WRAPMODE = _T("WrapMode");
const wxString DISPLAYEOL = _T("DisplayEOL");
const wxString INDENTGUIDE = _T("IndentGuide");
const wxString LINENUMBER = _T("LineNumbers");
const wxString LONGLINEON = _T("LongLine");
const wxString WHITESPACE = _T("WhiteSpace");

const wxString CONVERTTOSPACES = _T("ConverTospaces");
const wxString CONVERTTOTABS = _T("ConvertToTabs");

const wxString FILEPATTERN = _T("FilePattern");
const wxString HELPBOOKS = _T("HelpBooks");
const wxString INDENTVALUE = _T("IndentValue");
const wxString LONGLINEPOS = _T("LongLinePos");
const wxString FLAGVALUES = _T("FlagValues");

const wxString FOREGROUND = _T("Foreground");
const wxString BACKGROUND = _T("Background");
const wxString FONTNAME = _T("FontName");
const wxString FONTSIZE = _T("FontSize");
const wxString FONTSTYLE = _T("FontStyle");
const wxString LETTERCASE = _T("LetterCase");

//----------------------------------------------------------------------------
//! common
const CommonInfo g_CommonTable = {
    // application prefs
    false, // noSplash
    true,  // showBrowse
    true,  // showStatusbar
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
    true,  // showToolbar
#else
    false, // showToolbar
#endif
    true,  // usePages
    // editor functionality prefs
    true,  // syntaxEnable
    true,  // foldEnable
    true,  // indentEnable
    // display defaults prefs
    false, // overTypeInitial
    false, // readOnlyInitial
    false, // wrapModeInitial
    false, // displayEOLEnable
    false, // IndentGuideEnable
    true,  // lineNumberEnable
    false, // longLineOnEnable
    false, // whiteSpaceEnable
    // modification prefs
    false, // convertToSpaces
    false, // convertToTabs
};
CommonInfo g_CommonPrefs = g_CommonTable;

//----------------------------------------------------------------------------
// keywordlists
// Ada
wxChar* AdaWordlist1 = _T("\
    abort abstract accept access aliased all array at begin body case \
    constant declare delay delta digits do else elsif end entry \
    exception exit for function generic goto if in is limited loop \
    new null of others out package pragma private procedure protected \
    raise range record renames requeue return reverse select separate \
    subtype tagged task terminate then type until use when while with \
    abs and mod not or rem xor \
");

// Apache
wxChar* ApacheWordlist1 = _T("\
    on off standalone inetd force-response-1.0 downgrade-1.0 \
    nokeepalive indexes includes followsymlinks none \
    x-compress x-gzip \
");
wxChar* ApacheWordlist2 = _T("\
    accessconfig accessfilename action addalt addaltbyencoding \
    addaltbytype addcharset adddefaultcharset adddescription \
    addencoding addhandler addicon addiconbyencoding addiconbytype \
    addlanguage addmodule addmoduleinfo addtype agentlog alias \
    aliasmatch allow allowconnect allowoverride anonymous \
    anonymous_authoritative anonymous_logemail \
    anonymous_mustgiveemail anonymous_nouserid anonymous_verifyemail \
    authauthoritative authdbauthoritative authdbgroupfile \
    authdbmauthoritative authdbmgroupfile authdbmgroupfile \
    authdbuserfile authdbmuserfile authdigestfile authgroupfile \
    authname authtype authuserfile bindaddress browsermatch \
    browsermatchnocase bs2000account cachedefaultexpire \
") _T("\
    cachedirlength cachedirlevels cacheforcecompletion \
    cachegcinterval cachelastmodifiedfactor cachemaxexpire \
    cachenegotiateddocs cacheroot cachesize checkspelling \
    clearmodulelist contentdigest cookieexpires cookielog cookielog \
    cookietracking coredumpdirectory customlog defaulticon \
    defaultlanguage defaulttype define deny directory directorymatch \
    directoryindex documentroot errordocument errorlog example \
    expiresactive expiresbytype expiresdefault extendedstatus \
    fancyindexing files filesmatch forcetype group header headername \
    hostnamelookups identitycheck ifdefine ifmodule imapbase \
    imapdefault imapmenu include indexignore indexoptions keepalive \
    keepalivetimeout languagepriority limit limitexcept \
") _T("\
    limitrequestbody limitrequestfields limitrequestfieldsize \
    limitrequestline listen listenbacklog loadfile loadmodule \
    location locationmatch lockfile logformat loglevel maxclients \
    maxkeepaliverequests maxrequestsperchild maxspareservers metadir \
    metafiles metasuffix mimemagicfile minspareservers mmapfile \
    namevirtualhost nocache options order passenv pidfile port \
    proxyblock proxydomain proxypass proxypassreverse \
    proxyreceivebuffersize proxyremote proxyrequests proxyvia qsc \
    readmename redirect redirectmatch redirectpermanent redirecttemp \
    refererignore refererlog removehandler require resourceconfig \
    rewritebase rewritecond rewriteengine rewritelock rewritelog \
    rewriteloglevel rewritemap rewriteoptions rewriterule rlimitcpu \
") _T("\
    rlimitmem rlimitnproc satisfy scoreboardfile script scriptalias \
    scriptaliasmatch scriptinterpretersource scriptlog \
    scriptlogbuffer scriptloglength sendbuffersize serveradmin \
    serveralias servername serverpath serverroot serversignature \
    servertokens servertype setenv setenvif setenvifnocase sethandler \
    singlelisten startservers threadsperchild timeout transferlog \
    typesconfig unsetenv usecanonicalname user userdir virtualhost \
    virtualdocumentroot virtualdocumentrootip virtualscriptalias \
    virtualscriptaliasip xbithack from all \
");

// Assembler
wxChar* AsmWordlist1 = _T("\
    aaa aad aam aas adc add and call cbw clc cld cli cmc cmp cmps \
    cmpsb cmpsw cwd daa das dec div esc hlt idiv imul in inc int into \
    iret ja jae jb jbe jc jcxz je jg jge jl jle jmp jna jnae jnb jnbe \
    jnc jne jng jnge jnl jnle jno jnp jns jnz jo jp jpe jpo js jz \
    lahf lds lea les lods lodsb lodsw loop loope loopew loopne \
    loopnew loopnz loopnzw loopw loopz loopzw mov movs movsb movsw \
    mul neg nop not or out pop popf push pushf rcl rcr ret retf retn \
    rol ror sahf sal sar sbb scas scasb scasw shl shr stc std sti \
    stos stosb stosw sub test wait xchg xlat xlatb xor bound enter \
    ins insb insw leave outs outsb outsw popa pusha pushw arpl lar \
    lsl sgdt sidt sldt smsw str verr verw clts lgdt lidt lldt lmsw \
    ltr bsf bsr bt btc btr bts cdq cmpsd cwde insd iretd iretdf \
") _T("\
    iretf jecxz lfs lgs lodsd loopd looped loopned loopnzd loopzd \
    lss movsd movsx movzx outsd popad popfd pushad pushd pushfd scasd \
    seta setae setb setbe setc sete setg setge setl setle setna \
    setnae setnb setnbe setnc setne setng setnge setnl setnle setno \
    setnp setns setnz seto setp setpe setpo sets setz shld shrd stosd \
    bswap cmpxchg invd  invlpg  wbinvd  xadd lock rep repe repne \
    repnz repz \
");
wxChar* AsmWordlist2 = _T("\
    f2xm1 fabs fadd faddp fbld fbstp fchs fclex fcom fcomp fcompp \
    fdecstp fdisi fdiv fdivp fdivr fdivrp feni ffree fiadd ficom \
    ficomp fidiv fidivr fild fimul fincstp finit fist fistp fisub \
    fisubr fld fld1 fldcw fldenv fldenvw fldl2e fldl2t fldlg2 fldln2 \
    fldpi fldz fmul fmulp fnclex fndisi fneni fninit fnop fnsave \
    fnsavew fnstcw fnstenv fnstenvw fnstsw fpatan fprem fptan frndint \
    frstor frstorw fsave fsavew fscale fsqrt fst fstcw fstenv fstenvw \
    fstp fstsw fsub fsubp fsubr fsubrp ftst fwait fxam fxch fxtract \
    fyl2x fyl2xp1 fsetpm fcos fldenvd fnsaved fnstenvd fprem1 frstord \
    fsaved fsin fsincos fstenvd fucom fucomp fucompp \
");
wxChar* AsmWordlist3 = _T("\
    ah al ax bh bl bp bx ch cl cr0 cr2 cr3 cs cx dh di dl dr0 dr1 dr2 \
    dr3 dr6 dr7 ds dx eax ebp ebx ecx edi edx es esi esp fs gs si sp \
    ss st tr3 tr4 tr5 tr6 tr7 \
");
wxChar* AsmWordlist4 = _T("\
    .186 .286 .286c .286p .287 .386 .386c .386p .387 .486 .486p .8086 \
    .8087 .alpha .break .code .const .continue .cref .data .data?  \
    .dosseg .else .elseif .endif .endw .err .err1 .err2 .errb .errdef \
    .errdif .errdifi .erre .erridn .erridni .errnb .errndef .errnz \
    .exit .fardata .fardata? .if .lall .lfcond .list .listall .listif \
    .listmacro .listmacroall  .model .no87 .nocref .nolist .nolistif \
    .nolistmacro .radix .repeat .sall .seq .sfcond .stack .startup \
    .tfcond .type .until .untilcxz .while .xall .xcref .xlist alias \
    align assume catstr comm comment db dd df dosseg dq dt dup dw \
    echo else elseif elseif1 elseif2 elseifb elseifdef elseifdif \
    elseifdifi elseife elseifidn elseifidni elseifnb elseifndef end \
    endif endm endp ends eq  equ even exitm extern externdef extrn \
");
wxChar* AsmWordlist4_2 = _T("\
    for forc ge goto group gt high highword if if1 if2 ifb ifdef \
    ifdif ifdifi ife  ifidn ifidni ifnb ifndef include includelib \
    instr invoke irp irpc label le length lengthof local low lowword \
    lroffset lt macro mask mod .msfloat name ne offset opattr option \
    org %out page popcontext proc proto ptr public purge pushcontext \
    record repeat rept seg segment short size sizeof sizestr struc \
    struct substr subtitle subttl textequ this title type typedef \
    union while width \
");
wxChar* AsmWordlist5 = _T("\
    $ ? @b @f addr basic byte c carry? dword far far16 fortran fword \
    near near16 overflow? parity? pascal qword real4 real8 real10 \
    sbyte sdword sign? stdcall sword syscall tbyte vararg word zero? \
    flat near32 far32 abs all assumes at casemap common compact cpu \
    dotname emulator epilogue error export expr16 expr32 farstack \
    flat forceframe huge language large listing ljmp loadds m510 \
    medium memory nearstack nodotname noemulator nokeyword noljmp \
    nom510 none nonunique nooldmacros nooldstructs noreadonly \
    noscoped nosignextend nothing notpublic oldmacros oldstructs \
    os_dos para private prologue radix  readonly req scoped setif2 \
    smallstack tiny use16 use32 uses \
");

// Active Server Pages
wxChar* AspWordlist1 = _T("\
    a abbr acronym address applet area b base basefont bdo big \
    blockquote body br button caption center cite code col colgroup \
    dd del dfn dir div dl dt em fieldset font form frame frameset h1 \
    h2 h3 h4 h5 h6 head hr html i iframe img input ins isindex kbd \
    label legend li link map menu meta noframes noscript object ol \
    optgroup option p param pre q s samp script select small span \
    strike strong style sub sup table tbody td textarea tfoot th \
    thead title tr tt u ul var xml xmlns \
");
wxChar* AspWordlist2 = _T("\
    abbr accept-charset accept accesskey action align alink alt \
    archive axis background bgcolor border cellpadding cellspacing \
    char charoff charset checked cite class classid clear codebase \
    codetype color cols colspan compact content coords data datafld \
    dataformatas datapagesize datasrc datetime declare defer dir \
    disabled enctype event face for frame frameborder headers height \
    href hreflang hspace http-equiv id ismap label lang language \
    leftmargin link longdesc marginwidth marginheight maxlength media \
    method multiple name nohref noresize noshade nowrap object onblur \
    onchange onclick ondblclick onfocus onkeydown onkeypress onkeyup \
    onload onmousedown onmousemove onmouseover onmouseout onmouseup \
    onreset onselect onsubmit onunload profile prompt readonly rel \
") _T("\
    rev rows rowspan rules scheme scope selected shape size span src \
    standby start style summary tabindex target text title topmargin \
    type usemap valign value valuetype version vlink vspace width \
    text password checkbox radio submit reset file hidden image \
");

// Avenue
wxChar* AveWordlist1 = _T("\
    nil true false else for if while then elseif end av self in exit \
");

// Baan (11)
wxChar* BaanWordlist1 = _T("\
");

// Bash (12)
wxChar* BashWordlist1 = _T("\
    alias ar asa awk banner basename bash bc bdiff break bunzip2 \
    bzip2 cal calendar case cat cc cd chmod cksum clear cmp col comm \
    compress continue cp cpio crypt csplit ctags cut date dc dd \
    declare deroff dev df diff diff3 dircmp dirname do done du echo \
    ed egrep elif else env esac eval ex exec exit expand export expr \
    false fc fgrep fi file find fmt fold for function functions \
    getconf getopt getopts grep gres hash head help history iconv id \
    if in integer jobs join kill local lc let line ln logname look \
    ls m4 mail mailx make man mkdir more mt mv newgrp nl nm nohup \
    ntps od pack paste patch pathchk pax pcat perl pg pr print printf \
    ps pwd read readonly red return rev rm rmdir sed select set sh \
    shift size sleep sort spell split start stop strings strip stty \
") _T("\
    sum suspend sync tail tar tee test then time times touch tr trap \
    true tsort tty type typeset ulimit umask unalias uname uncompress \
    unexpand uniq unpack unset until uudecode uuencode vi vim vpax \
    wait wc whence which while who wpaste wstart xargs zcat \
");

// Batch (13)
wxChar* BatchWordlist1 = _T("\
    rem set if exist errorlevel for in do break call chcp cd chdir \
    choice cls country ctty date del erase dir echo exit goto loadfix \
    loadhigh mkdir md move path pause prompt rename ren rmdir rd shift \
    time type ver verify vol com con lpt nul \
");

// Bullant (14)
wxChar* BullantWordlist1 = _T("\
    abstract all ancestor and application assert attributes author \
    begin callback class concrete config constants construct continue \
    depends description downcast driver elif else ensures error \
    exception exposure extension false fatal final function generics \
    glyph help hidden host immutable in inherits is kernel label \
    leave library locals mutable none not null obsolete options or \
    other parameters peer private public raise reason restricted \
    retry return returns rollback route security self settings \
    severity step task test transaction true unknown varying warning \
    when method end if until while trap case debug for foreach lock \
    boolean character character$ date date$ datetime datetime$ float \
    hex$ identifier identifier$ integer interval interval$ money \
    money$ raw raw$ string tick tick$ time time$ version version$ \
");

// C++
wxChar* CppWordlist1 = _T("\
    asm auto bool break case catch char class const const_cast \
    continue default delete do double dynamic_cast else enum explicit \
    export extern false float for friend goto if inline int long \
    mutable namespace new operator private protected public register \
    reinterpret_cast return short signed sizeof static static_cast \
    struct switch template this throw true try typedef typeid \
    typename union unsigned using virtual void volatile wchar_t \
    while \
");
wxChar* CppWordlist2 = _T("\
");
wxChar* CppWordlist3 = _T("\
    a addindex addtogroup anchor arg attention author b brief bug c \
    class code date def defgroup deprecated dontinclude e em endcode \
    endhtmlonly endif endlatexonly endlink endverbatim enum example \
    exception f$ f[ f] file fn hideinitializer htmlinclude \
    htmlonly if image include ingroup internal invariant interface \
    latexonly li line link mainpage name namespace nosubgrouping note \
    overload p page par param post pre ref relates remarks return \
    retval sa section see showinitializer since skip skipline struct \
    subsection test throw todo typedef union until var verbatim \
    verbinclude version warning weakgroup $ @ \" & < > # { } \
");
wxChar* CppGlobalClass = _T("\
");

// CS
wxChar* CsWordlist1 = _T("\
    abstract as base bool break byte case catch char checked class \
    const continue decimal default delegate do double else enum event \
    explicit extern false finally fixed float for foreach goto if \
    implicit in int interface internal is lock long namespace new \
    null object operator out override params private protected public \
    readonly ref return sbyte sealed short sizeof stackalloc static \
    string struct switch this throw true try typeof uint ulong \
    unchecked unsafe ushort using virtual void while \
");

// CSS
wxChar* CssWordlist1 = _T("\
    left right top bottom position font-family font-style font-variant \
    font-weight font-size font color background-color background-image \
    background-repeat background-attachment background-position background \
    word-spacing letter-spacing text-decoration vertical-align text-transform \
    text-align text-indent line-height margin-top margin-right margin-bottom \
    margin-left margin padding-top padding-right padding-bottom padding-left \
    padding border-top-width border-right-width border-bottom-width \
    border-left-width border-width border-top border-right border-bottom \
    border-left border border-color border-style width height float clear \
    display white-space list-style-type list-style-image list-style-position \
    list-style \
");
wxChar* CssWordlist2 = _T("\
    first-letter first-line active link visited \
");

// Diff (patch)

// Eiffel
wxChar* EiffelWordlist1 = _T("\
    alias all and any as bit boolean check class character clone \
    create creation current debug deferred div do double else elseif \
    end ensure equal expanded export external false feature forget \
    from frozen general if implies indexing infix inherit inspect \
    integer invariant is language like local loop mod name nochange \
    none not obsolete old once or platform pointer prefix precursor \
    real redefine rename require rescue result retry select separate \
    string strip then true undefine unique until variant void when \
    xor \
");

// Errorlist

// Escript
wxChar* EscriptWordlist1 = _T("\
    basic basicio boats cfgfile file http npc os uo util accessible \
    addmenuitem appendconfigfileelem applyconstraint applydamage \
    applyrawdamage assignrecttoweatherregion append baseskilltorawskill \
    boatfromitem broadcast ban cdbl cint cstr checklineofsight checklosat \
    checkskill consumemana consumereagents consumesubstance createaccount \
    createitematlocation createiteminbackpack createitemininventory \
    createitemincontainer createmenu createmultiatlocation \
    createnpcfromtemplate createrootiteminstoragearea createstoragearea \
    clear_script_profile_counters close damage destroyitem destroymulti \
    destroyrootiteminstoragearea detach disableevents disconnectclient \
    distance disable enableevents enumerateitemsincontainer \
    enumerateonlinecharacters equipfromtemplate equipitem \
") _T("\
    eraseglobalproperty eraseobjproperty enable enabled erase \
    events_waiting exists findconfigelem findobjtypeincontainer \
    findrootiteminstoragearea findstoragearea fclose find fopen fread \
    fseek ftell fwrite gamestat getamount getcommandhelp getconfigint \
    getconfigintkeys getconfigmaxintkey getconfigreal getconfigstring \
    getconfigstringkeys getconfigstringarray getelemproperty \
    getequipmentbylayer getglobalproperty getharvestdifficulty \
    getmapinfo getmenuobjtypes getobjproperty getobjtype getobjtypebyname \
    getproperty getrawskill getregionstring getskill getspelldifficulty \
    getstandingheight getworldheight grantprivilege harvestresource \
    healdamage hex islegalmove insert keys listequippeditems \
    listghostsnearlocation listhostiles listitemsatlocation \
") _T("\
    listitemsnearlocation listitemsnearlocationoftype \
    listmobilesinlineofsight listmobilesnearlocation \
    listmobilesnearlocationex listobjectsinbox loadtusscpfile \
    left len log_profile lower makeboundingbox move moveboat \
    moveboatrelative movecharactertolocation moveitemtocontainer \
    moveitemtolocation move_offline_mobiles openpaperdoll open pack \
    performaction playlightningbolteffect playmovingeffect \
    playmovingeffectxyz playobjectcenteredeffect playsoundeffect \
    playsoundeffectprivate playstationaryeffect printtextabove \
    printtextaboveprivate packages polcore position print queryparam \
    randomdiceroll randomint rawskilltobaseskill readconfigfile readgameclock \
    releaseitem registerforspeechevents registeritemwithboat requestinput \
") _T("\
    reserveitem restartscript resurrect revokeprivilege runawayfrom \
    runawayfromlocation runtoward runtowardlocation reverse \
    run_script_to_completion saveworldstate selectmenuitem2 self \
    sendbuywindow senddialoggump sendevent sendopenspecialcontainer \
    sendpacket sendsellwindow sendskillwindow sendstringastipwindow \
    sendsysmessage sendtextentrygump setanchor setglobalproperty \
    setname setobjproperty setopponent setproperty setrawskill \
    setregionlightlevel setregionweatherlevel setscriptcontroller \
    setwarmode shutdown speakpowerwords splitwords startspelleffect \
    subtractamount systemfindboatbyserial systemfindobjectbyserial \
    say set_critical set_debug set_priority set_priority_divide \
    set_script_option setcmdlevel setdex setint setlightlevel setmaster \
") _T("\
    setname setpassword setstr shrink size sleep sleepms sort spendgold \
    squelch start_script syslog system_rpm target targetcoordinates \
    targetmultiplacement turnawayfrom turnawayfromlocation turnboat \
    turntoward turntowardlocation toggle unloadconfigfile unpack unban \
    unload_scripts upper walkawayfrom walkawayfromlocation walktoward \
    walktowardlocation wander writehtml writehtmlraw wait_for_event \
    movechar_forcelocation moveitem_forcelocation moveitem_normal \
    scriptopt_debug scriptopt_no_interrupt scriptopt_no_runaway \
    te_cancel_disable te_cancel_enable te_style_disable te_style_normal \
    te_style_numerical tgtopt_check_los tgtopt_harmful tgtopt_helpful \
    tgtopt_neutral tgtopt_nocheck_los setprop getprop \
");
wxChar* EscriptWordlist2 = _T("\
    array const dictionary global local var and default in next not \
    or return to include use enum \
");
wxChar* EscriptWordlist3 = _T("\
    while for endfor function program endprogram endfunction foreach \
    case else elseif if endcase endenum endforeach endif endwhile \
");

// Fortran
wxChar* FortranWordlist1 = _T("\
    allocatable allocate assignment backspace block blockdata call \
    case character close common complex contains continue cycle data \
    deallocate default dimension direct do double doubleprecision \
    elemental else elseif elsewhere end endblock endblockdata enddo \
    endfile endforall endfunction endif endinterface endmodule \
    endprogram endselect endsubroutine endtype endwhere entry \
    equivalence err exist exit external forall format formatted \
    function go goto if implicit in inout include inquire integer \
    intent interface intrinsic iolength iostat kind len logical \
    module namelist none null nullify only open operator optional \
    parameter pointer position precision print private procedure \
    program public pure out read readwrite real rec recursive result \
    return rewind save select selectcase sequence sequential stat \
    status stop subroutine target then to type unformatted unit use \
    where while write \
");
wxChar* FortranWordlist2 = _T("\
    abs achar acos acosd adjustl adjustr aimag aimax0 aimin0 aint \
    ajmax0 ajmin0 akmax0 akmin0 all allocated alog alog10 amax0 amax1 \
    amin0 amin1 amod anint any asin asind associated atan atan2 \
    atan2d atand bitest bitl bitlr bitrl bjtest bit_size bktest break \
    btest cabs ccos cdabs cdcos cdexp cdlog cdsin cdsqrt ceiling cexp \
    char clog cmplx conjg cos cosd cosh count cpu_time cshift csin \
    csqrt dabs dacos dacosd dasin dasind datan datan2 datan2d datand \
    date date_and_time dble dcmplx dconjg dcos dcosd dcosh dcotan \
    ddim dexp dfloat dflotk dfloti dflotj digits dim dimag dint dlog \
    dlog10 dmax1 dmin1 dmod dnint dot_product dprod dreal dsign dsin \
    dsind dsinh dsqrt dtan dtand dtanh eoshift epsilon errsns exp \
    exponent float floati floatj floatk floor fraction free huge iabs \
") _T("\
    iachar iand ibclr ibits ibset ichar idate idim idint idnint ieor \
    ifix iiabs iiand iibclr iibits iibset iidim iidint iidnnt iieor \
    iifix iint iior iiqint iiqnnt iishft iishftc iisign ilen imax0 \
    imax1 imin0 imin1 imod index inint inot int int1 int2 int4 int8 \
    iqint iqnint ior ishft ishftc isign isnan izext jiand jibclr \
    jibits jibset jidim jidint jidnnt jieor jifix jint jior jiqint \
    jiqnnt jishft jishftc jisign jmax0 jmax1 jmin0 jmin1 jmod jnint \
    jnot jzext kiabs kiand kibclr kibits kibset kidim kidint kidnnt \
    kieor kifix kind kint kior kishft kishftc kisign kmax0 kmax1 \
    kmin0 kmin1 kmod knint knot kzext lbound leadz len len_trim \
    lenlge lge lgt lle llt log log10 logical lshift malloc matmul \
    max max0 max1 maxexponent maxloc maxval merge min min0 min1 \
") _T("\
    minexponent minloc minval mod modulo mvbits nearest nint not \
    nworkers number_of_processors pack popcnt poppar precision \
    present product radix random random_number random_seed range real \
    repeat reshape rrspacing rshift scale scan secnds \
    selected_int_kind selected_real_kind set_exponent shape sign sin \
    sind sinh size sizeof sngl snglq spacing spread sqrt sum \
    system_clock tan tand tanh tiny transfer transpose trim ubound \
    unpack verify \
");

// HTML
wxChar* HtmlWordlist1 = _T("\
    a abbr acronym address applet area b base basefont bdo big \
    blockquote body br button caption center cite code col colgroup \
    dd del dfn dir div dl dt em fieldset font form frame frameset h1 \
    h2 h3 h4 h5 h6 head hr html i iframe img input ins isindex kbd \
    label legend li link map menu meta noframes noscript object ol \
    optgroup option p param pre q s samp script select small span \
    strike strong style sub sup table tbody td textarea tfoot th \
    thead title tr tt u ul var xml xmlns \
");
wxChar* HtmlWordlist2 = _T("\
    abbr accept-charset accept accesskey action align alink alt \
    archive axis background bgcolor border cellpadding cellspacing \
    char charoff charset checked cite class classid clear codebase \
    codetype color cols colspan compact content coords data datafld \
    dataformatas datapagesize datasrc datetime declare defer dir \
    disabled enctype event face for frame frameborder headers height \
    href hreflang hspace http-equiv id ismap label lang language \
    leftmargin link longdesc marginwidth marginheight maxlength media \
    method multiple name nohref noresize noshade nowrap object onblur \
    onchange onclick ondblclick onfocus onkeydown onkeypress onkeyup \
    onload onmousedown onmousemove onmouseover onmouseout onmouseup \
    onreset onselect onsubmit onunload profile prompt readonly rel \
") _T("\
    rev rows rowspan rules scheme scope selected shape size span src \
    standby start style summary tabindex target text title topmargin \
    type usemap valign value valuetype version vlink vspace width \
    text password checkbox radio submit reset file hidden image \
");

// IDL
wxChar* IdlWordlist1 = _T("\
    aggregatable allocate appobject arrays async async_uuid \
    uto_handle bindable boolean broadcast byte byte_count call_as \
    callback char coclass code comm_status const context_handle \
    context_handle_noserialize context_handle_serialize control \
    cpp_quote custom decode default defaultbind defaultcollelem \
    defaultvalue defaultvtable dispinterface displaybind dllname \
    double dual enable_allocate encode endpoint entry enum \
    error_status_t explicit_handle fault_status first_is float \
    handle_t heap helpcontext helpfile helpstring helpstringcontext \
    helpstringdll hidden hyper id idempotent ignore iid_as \
    iid_is immediatebind implicit_handle import importlib in include \
    in_line int __int64 __int3264 interface last_is lcid length_is \
") _T("\
    library licensed local long max_is maybe message methods \
    midl_pragma midl_user_allocate midl_user_free min_is module \
    ms_union ncacn_at_dsp ncacn_dnet_nsp ncacn_http ncacn_ip_tcp \
    ncacn_nb_ipx ncacn_nb_nb ncacn_nb_tcp ncacn_np ncacn_spx \
    ncacn_vns_spp ncadg_ip_udp ncadg_ipx ncadg_mq ncalrpc nocode \
    nonbrowsable noncreatable nonextensible notify object odl \
    oleautomation optimize optional out out_of_line pipe \
    pointer_default pragma properties propget propput propputref ptr \
    public range readonly ref represent_as requestedit restricted \
    retval shape short signed size_is small source \
    strict_context_handle string struct switch switch_is switch_type \
    transmit_as typedef uidefault union unique unsigned user_marshal \
") _T("\
    usesgetlasterror uuid v1_enum vararg version void wchar_t \
    wire_marshal \
");

// Java
wxChar* JavaWordlist1 = _T("\
    abstract assert boolean break byte case catch char class const \
    continue default do double else extends final finally float for \
    future generic goto if implements import inner instanceof int \
    interface long native new null outer package private protected \
    public rest return short static super switch synchronized this \
    throw throws transient try var void volatile while \
");

// Javascript
wxChar* JavascriptWordlist1 = _T("\
    abstract boolean break byte case catch char class const continue \
    debugger default delete do double else enum export extends final \
    finally float for function goto if implements import in \
    instanceof int interface long native new package private \
    protected public return short static super switch synchronized \
    this throw throws transient try typeof var void volatile while \
    with \
");

// Latex

// Lisp
wxChar* LispWordlist1 = _T("\
    not defun + - * / = < > <= >= princ eval apply funcall quote \
    identity function complement backquote lambda set setq setf defun \
    defmacro gensym make symbol intern symbol name symbol value \
    symbol plist get getf putprop remprop hash make array aref car \
    cdr caar cadr cdar cddr caaar caadr cadar caddr cdaar cdadr cddar \
    cdddr caaaar caaadr caadar caaddr cadaar cadadr caddar cadddr \
    cdaaar cdaadr cdadar cdaddr cddaar cddadr cdddar cddddr cons list \
    append reverse last nth nthcdr member assoc subst sublis nsubst \
    nsublis remove length list length mapc mapcar mapl maplist mapcan \
    mapcon rplaca rplacd nconc delete atom symbolp numberp boundp \
    null listp consp minusp zerop plusp evenp oddp eq eql equal cond \
    case and or let l if prog prog1 prog2 progn go return do dolist \
") _T("\
    dotimes catch throw error cerror break continue errset baktrace \
    evalhook truncate float rem min max abs sin cos tan expt exp sqrt \
    random logand logior logxor lognot bignums logeqv lognand lognor \
    logorc2 logtest logbitp logcount integer length nil \
");

// Lout
wxChar* LoutWordlist1 = _T("\
    @OptGall @Filter @FilterIn @FilterOut @FilterErr @FontDef @Family \
    @Face @Name @Metrics @ExtraMetrics @Mapping @Recode @Common @Rump \
    @Meld @Insert @OneOf @Next @Plus @Minus @Wide @High @HShift @VShift \
    @BeginHeaderComponent @EndHeaderComponent @SetHeaderComponent \
    @ClearHeaderComponent @OneCol @OneRow @HScale @VScale @HCover \
    @VCover @Scale @KernShrink @HContract @VContract @HLimited @VLimited \
    @HExpand @VExpand @StartHVSpan @StartHSpan @StartVSpan @HSpan @VSpan \
    @PAdjust @HAdjust @VAdjust @Rotate @Background @IncludeGraphic \
    @SysIncludeGraphic @Graphic @LinkSource @LinkDest @URLLink \
    @PlainGraphic @Verbatim @RawVerbatim @Case @Yield @BackEnd @Char \
    @Font @Space @YUnit @ZUnit @Break @Underline @SetColour @SetColor \
    @SetTexture @Outline @Language @CurrLang @CurrFamily @CurrFace \
") _T("\
    @CurrYUnit @CurrZUnit @LEnv @LClos @LUse @LEO @Open @Use @NotRevealed \
    @Tagged @Database @SysDatabase @Include @SysInclude \
    @IncludeGraphicRepeated @PrependGraphic @SysIncludeGraphicRepeated \
    @SysPrependGraphic @Target @Null @PageLabel @Galley @ForceGalley \
    @LInput @Split @Tag @Key @Optimize @Merge @Enclose @Begin @End \
    @Moment @Second @Minute @Hour @Day @Month @Year @Century @WeekDay \
    @YearDay @DaylightSaving @@A @@B @@C @@D @@E @@V \
");
wxChar* LoutWordlist2 = _T("\
    &&& && & ^// ^/ ^|| ^| ^& // / || | \
");
wxChar* LoutWordlist3 = _T("\
    def langdef force horizontally into extend import export precedence \
    associativity left right body macro named compulsory following \
    preceding foll_or_prec now \
");

// Lua
wxChar* LuaWordlist1 = _T("\
    and break do else elseif end false for function global if in \
    local nil not or repeat return then true until while \
");
wxChar* LuaWordlist2 = _T("\
    ALERT _ERRORMESSAGE _INPUT _PROMPT _OUTPUT _STDERR _STDIN _STDOUT \
    _VERSION assert call collectgarbage dofile dostring error foreach \
    foreachi gcinfo getn globals loadfile loadstring newtype print \
    rawget rawset require sort tonumber tostring tinsert tremove type \
    unpack \
");
wxChar* LuaWordlist3 = _T("\
    abs acos asin atan atan2 ceil cos deg exp floor format frexp gsub \
    ldexp log log10 max min mod rad random randomseed sin sqrt \
    strbyte strchar strfind strlen strlower strrep strsub strupper \
    tan \
");
wxChar* LuaWordlist4 = _T("\
    openfile closefile readfrom writeto appendto remove rename flush \
    seek tmpfile tmpname read write clock date difftime execute exit \
    getenv setlocale time \
");

// Makefile

// MatLab
wxChar* MatLabWordlist1 = _T("\
    break case catch continue else elseif end for function global if \
    otherwise persistent return switch try while \
");

// MMIX Assembler
wxChar* MmixalWordlist1 = _T("\
    2ADDU 4ADDU 8ADDU 16ADDU ADD ADDU AND ANDNH ANDNL ANDNMH ANDNML \
    BDIF BEV BN BNN BNP BNZ BOD BP BSPEC BYTE BZ CMP CMPU CSEV CSN \
    CSNN CSNP CSNZ CSOD CSP CSWAP CSZ DIV DIVU ESPEC EXPR FADD FCMP \
    FCMPE FDIV FEQL FEQLE FIX FIXU FLOT FLOTU FMUL FREM FSQRT FSUB \
    FUN FUNE GET GETA GO GREG I_BIT INCH INCL INCMH INCML IS JMP LDA \
    LDB LDBU LDHT LDO LDOU LDSF LDT LDTU LDUNC LDVTS LDW LDWU LOC \
    LOCAL MOR MUL MULU MUX MXOR NAND NEG NEGU NNIX NOR NXOR O_BIT \
    OCTA ODIF OR ORH ORL ORMH ORML ORN PBEV PBN PBNN PBNP PBNZ PBOD \
    PBP PBZ POP PREFIX PREGO PRELD PREST PUSHGO PUSHJ PUT RESUME SAVE \
    SET SETH SETL SETMH SETML SFLOT SFLOTU SL SLU SR SRU STB STBU \
    STCO STHT STO STOU STSF STT STTU STUNC STW STWU SUB SUBU SWYM \
    SYNC SYNCD TDIF TETRA TRAP TRIP UNSAVE WDIF WYDEXOR ZSEV ZSN ZSNN \
    ZSNP ZSNZ ZSOD ZSP ZSZ \
");

wxChar* MmixalWordlist2 = _T("\
    rA rB rC rD rE rF rG rH rI rJ rK rL rM rN rO rP rQ rR rS rT rU rV \
    rW rX rY rZ rBB rTT rWW rXX rYY rZZ \
");

wxChar* MmixalWordlist3 = _T("\
    @ Text_Segment Data_Segment Pool_Segment Stack_Segment \
    StdErr StdIn StdOut Fopen Fclose Fread Fwrite Fgets Fputs Fgetws \
    Fputws Ftell Fseek TextRead TextWrite BinaryRead BinaryWrite \
    BinaryReadWrite \
");

// NNCron
wxChar* NNCronWordlist1 = _T("\
    AGAIN ALLOT AND Action BEGIN CASE COMPARE CONSTANT CREATE DO Days \
    ELSE ENDCASE ENDOF EVAL-SUBST EVALUATE FALSE Hours I IF LEAVE \
    LOOP Minutes Months NOT OF OFF ON OR PAD REPEAT Rule SET THEN \
    TRUE Time UNTIL VALUE VARIABLE WHILE WeekDays Years \
");
wxChar* NNCronWordlist2 = _T("\
    </JScript> </SCRIPT> </VBScript> <JScript> <VBScript> ACCESS-DATE \
    BEEP CALL_DIAL CALL_HANGUP CHAR CLIPBOARD CONSOLE CREATION-DATE \
    CUR-DATE DATE- DATE-INTERVAL DELAY DIAL DIR-CREATE DIR-DELETE \
    DIR-EMPTY DIR-SIZE Day@ Domain ERR-MSG EXIST FILE-ACCESS-DATE \
    FILE-APPEND FILE-COPY FILE-CREATE FILE-CREATION-DATE FILE-CROP \
    FILE-DATE FILE-DELETE FILE-EMPTY FILE-EXIST FILE-MOVE FILE-RENAME \
    FILE-SIZE FILE-WRITE FILE-WRITE-DATE FOR-CHILD-WINDOWS FOR-FILES \
    FOR-WINDOWS FREE-SPACE GET-CUR-TIME GET-REG GET-VER GetTickCount \
    HANGUP HIDE-ICON HINT HINT-OFF HINT-POS HINT-SIZE HINTW \
    HOST-EXIST Hour@ IDLE INTERVAL IS-ARCHIVE IS-DIR IS-HIDDEN \
    IS-READONLY IS-SYSTEM KILL LAUNCH LOG LOGGEDON LOGOFF LogonBatch \
    LogonInteractive LogonNetwork MOUSE-LBCLK MOUSE-LBDCLK MOUSE-LBDN \
") _T("\
    MOUSE-LBUP MOUSE-MOVE MOUSE-MOVER MOUSE-MOVEW MOUSE-RBCLK \
    MOUSE-RBDCLK MOUSE-RBDN MOUSE-RBUP MSG Min@ Mon@ \
    MonitorResponseTime NHOST-EXIST No ONLINE PAUSE PLAY-SOUND \
    PLAY-SOUNDW POP3-CHECK POWEROFF PROC-EXIST PROC-TIME PURGE-OLD \
    PURGE-OLDA PURGE-OLDW Password QSTART-APP QSTART-APPW QUERY QUIT \
    RASDomain RASError RASPassword RASPhone RASSecPassword RASUser \
    RE-ALL RE-MATCH REBOOT REG-DELETE-KEY REG-DELETE-VALUE REG-DWORD \
    REG-SZ REMINDER RUN SEND-KEYS SEND-KEYS-DELAY SHOW-ICON SHUTDOWN \
    START-APP START-APPW START-QUIT START-TIME Sec@ SecPassword THINT \
    THINTW TimeSec@ TMSG TQUERY User WDay@ WIN-ACTIVATE WIN-ACTIVE \
    WIN-CLICK WIN-CLOSE WIN-EXIST WIN-HIDE WIN-HWND WIN-MAXIMIZE \
    WIN-MINIMIZE WIN-MOVE WIN-MOVER WIN-RESTORE WIN-SEND-KEYS \
") _T("\
    WIN-SHOW WIN-TERMINATE WIN-TOPMOST WIN-VER WIN-WAIT WINAPI \
    WRITE-DATE WatchClipboard WatchConnect WatchDir WatchDisconnect \
    WatchDriveInsert WatchDriveRemove WatchFile WatchProc \
    WatchProcStop WatchWinActivate WatchWinCreate WatchWinDestroy \
    WatchWindow WinNT Year@ Yes \
");
wxChar* NNCronWordlist3 = _T("\
    ALL AboveNormalPriority AsService BelowNormalPriority FILESONLY \
    HighPriority IdlePriority LoadProfile NoActive NoDel NoLog \
    NoRunAs NormalPriority OnceADay OnceAHour OnceAMonth OnceAWeek \
    RECURSIVE RealtimePriority RunOnce SWHide ShowMaximized \
    ShowMinimized ShowNoActivate ShowNormal StartIn StartPos \
    StartSize TODEPTH WATCH-CHANGE-ATTRIBUTES WATCH-CHANGE-DIR-NAME \
    WATCH-CHANGE-FILE-NAME WATCH-CHANGE-LAST-WRITE \
    WATCH-CHANGE-SECURITY WATCH-CHANGE-SIZE WaitFor WatchSubtree \
    WithoutProfile \
");

// NSIS
wxChar* NSISWordlist1 = _T("\
    What Abort AddSize AllowRootDirInstall AutoCloseWindow BGGradient \
    BrandingText BringToFront CRCCheck Call CallInstDLL Caption \
    ClearErrors CompletedText ComponentText CopyFiles CreateDirectory \
    CreateShortCut Delete DeleteINISec DeleteINIStr DeleteRegKey \
    DeleteRegValue DetailPrint DetailsButtonText DirShow DirText \
    DisabledBitmap EnabledBitmap EnumRegKey EnumRegValue Exch Exec \
    ExecShell ExecWait ExpandEnvStrings File FileClose FileErrorText \
    FileOpen FileRead FileReadByte FileSeek FileWrite FileWriteByte \
    FindClose FindFirst FindNext FindWindow Function FunctionEnd \
    GetCurrentAddress GetDLLVersionLocal GetDllVersion GetFileTime \
    GetFileTimeLocal GetFullPathName GetFunctionAddress GetLabelAddress \
    GetTempFileName Goto HideWindow Icon IfErrors IfFileExists \
") _T("\
    IfRebootFlag InstProgressFlags InstType InstallButtonText \
    InstallColors InstallDir InstallDirRegKey IntCmp IntCmpU IntFmt \
    IntOp IsWindow LicenseData LicenseText MessageBox MiscButtonText \
    Name OutFile Pop Push Quit RMDir ReadEnvStr ReadINIStr ReadRegDword \
    ReadRegStr Reboot RegDLL Rename Return SearchPath Section \
    SectionDivider SectionEnd SectionIn SendMessage SetAutoClose \
    SetCompress SetDatablockOptimize SetDateSave SetDetailsPrint \
    SetDetailsView SetErrors SetFileAttributes SetOutPath SetOverwrite \
    SetRebootFlag ShowInstDetails ShowUninstDetails SilentInstall \
    SilentUnInstall Sleep SpaceTexts StrCmp StrCpy StrLen SubCaption \
    UnRegDLL UninstallButtonText UninstallCaption UninstallEXEName \
    UninstallIcon UninstallSubCaption UninstallText WindowIcon \
") _T("\
    WriteINIStr WriteRegBin WriteRegDword WriteRegExpandStr \
    WriteRegStr WriteUninstaller SectionGetFlags SectionSetFlags \
    SectionSetText SectionGetText LogText LogSet CreateFont \
    SetShellVarContext SetStaticBkColor SetBrandingImage PluginDir \
    SubSectionEnd SubSection CheckBitmap ChangeUI SetFont \
    AddBrandingImage XPStyle LangString !define !undef !ifdef !ifndef \
    !endif !else !macro !echo !warning !error !verbose !macroend \
    !insertmacro !system !include !cd !packhdr \
");
wxChar* NSISWordlist2 = _T("\
    $0 $1 $2 $3 $4 $5 $6 $7 $8 $9 $R0 $R1 $R2 $R3 $R4 $R5 $R6 $R7 $R8 \
    $R9 $CMDLINE $DESKTOP $EXEDIR $HWNDPARENT $INSTDIR $OUTDIR \
    $PROGRAMFILES ${NSISDIR} $\n $\r $QUICKLAUNCH $SMPROGRAMS $SMSTARTUP \
    $STARTMENU $SYSDIR $TEMP $WINDIR \
");
wxChar* NSISWordlist3 = _T("\
    ARCHIVE FILE_ATTRIBUTE_ARCHIVE FILE_ATTRIBUTE_HIDDEN \
    FILE_ATTRIBUTE_NORMAL FILE_ATTRIBUTE_OFFLINE FILE_ATTRIBUTE_READONLY \
    FILE_ATTRIBUTE_SYSTEM FILE_ATTRIBUTE_TEMPORARY HIDDEN HKCC HKCR HKCU \
    HKDD HKEY_CLASSES_ROOT HKEY_CURRENT_CONFIG HKEY_CURRENT_USER \
    HKEY_DYN_DATA HKEY_LOCAL_MACHINE HKEY_PERFORMANCE_DATA HKEY_USERS \
    HKLM HKPD HKU IDABORT IDCANCEL IDIGNORE IDNO IDOK IDRETRY IDYES \
    MB_ABORTRETRYIGNORE MB_DEFBUTTON1 MB_DEFBUTTON2 MB_DEFBUTTON3 \
    MB_DEFBUTTON4 MB_ICONEXCLAMATION MB_ICONINFORMATION MB_ICONQUESTION \
    MB_ICONSTOP MB_OK MB_OKCANCEL MB_RETRYCANCEL MB_RIGHT \
    MB_SETFOREGROUND MB_TOPMOST MB_YESNO MB_YESNOCANCEL NORMAL OFFLINE \
    READONLY SW_SHOWMAXIMIZED SW_SHOWMINIMIZED SW_SHOWNORMAL SYSTEM \
    TEMPORARY auto colored false force hide ifnewer nevershow normal off \
    on show silent silentlog smooth true try \
");
wxChar* NSISWordlist4 = _T("\
    MyFunction MySomethingElse \
");

// Pascal
wxChar* PascalWordlist1 = _T("\
    program const type var begin end array set packed record string \
    if then else while for to downto do with repeat until case of \
    goto exit label procedure function nil file and or not xor div \
    mod unit uses implementation interface external asm inline object \
    constructor destructor virtual far assembler near inherited \
    stdcall cdecl library export exports end. class ansistring raise \
    try except on index name finally resourcestring false true \
    initialization finalization override overload \
");

// Perl
wxChar* PerlWordlist1 = _T("\
    NULL __FILE__ __LINE__ __PACKAGE__ __DATA__ __END__ AUTOLOAD \
    BEGIN CORE DESTROY END EQ GE GT INIT LE LT NE CHECK abs accept \
    alarm and atan2 bind binmode bless caller chdir chmod chomp chop \
    chown chr chroot close closedir cmp connect continue cos crypt \
    dbmclose dbmopen defined delete die do dump each else elsif \
    endgrent endhostent endnetent endprotoent endpwent endservent eof \
    eq eval exec exists exit exp fcntl fileno flock for foreach fork \
    format formline ge getc getgrent getgrgid getgrnam gethostbyaddr \
    gethostbyname gethostent getlogin getnetbyaddr getnetbyname \
    getnetent getpeername getpgrp getppid getpriority getprotobyname \
    getprotobynumber getprotoent getpwent getpwnam getpwuid \
    getservbyname getservbyport getservent getsockname getsockopt \
") _T("\
    glob gmtime goto grep gt hex if index int ioctl join keys kill \
    last lc lcfirst le length link listen local localtime lock log \
    lstat lt m map mkdir msgctl msgget msgrcv msgsnd my ne next no \
    not oct open opendir or ord our pack package pipe pop pos print \
    printf prototype push q qq qr quotemeta qu qw qx rand read \
    readdir readline readlink readpipe recv redo ref rename require \
    reset return reverse rewinddir rindex rmdir s scalar seek \
    seekdir select semctl semget semop send setgrent sethostent \
    setnetent setpgrp setpriority setprotoent setpwent setservent \
    setsockopt shift shmctl shmget shmread shmwrite shutdown sin \
    sleep socket socketpair sort splice split sprintf sqrt srand \
    stat study sub substr symlink syscall sysopen sysread sysseek \
") _T("\
    system syswrite tell telldir tie tied time times tr truncate uc \
    ucfirst umask undef unless unlink unpack unshift untie until use \
    utime values vec wait waitpid wantarray warn while write x xor \
    y \
");

// PHP
wxChar* PhpWordlist1 = _T("\
    a abbr acronym address applet area b base basefont bdo big \
    blockquote body br button caption center cite code col colgroup \
    dd del dfn dir div dl dt em fieldset font form frame frameset h1 \
    h2 h3 h4 h5 h6 head hr html i iframe img input ins isindex kbd \
    label legend li link map menu meta noframes noscript object ol \
    optgroup option p param pre q s samp script select small span \
    strike strong style sub sup table tbody td textarea tfoot th \
    thead title tr tt u ul var xml xmlns \
");
wxChar* PhpWordlist2 = _T("\
    abbr accept-charset accept accesskey action align alink alt \
    archive axis background bgcolor border cellpadding cellspacing \
    char charoff charset checked cite class classid clear codebase \
    codetype color cols colspan compact content coords data datafld \
    dataformatas datapagesize datasrc datetime declare defer dir \
    disabled enctype event face for frame frameborder headers height \
    href hreflang hspace http-equiv id ismap label lang language \
    leftmargin link longdesc marginwidth marginheight maxlength media \
    method multiple name nohref noresize noshade nowrap object onblur \
    onchange onclick ondblclick onfocus onkeydown onkeypress onkeyup \
    onload onmousedown onmousemove onmouseover onmouseout onmouseup \
    onreset onselect onsubmit onunload profile prompt readonly rel \
") _T("\
    rev rows rowspan rules scheme scope selected shape size span src \
    standby start style summary tabindex target text title topmargin \
    type usemap valign value valuetype version vlink vspace width \
    text password checkbox radio submit reset file hidden image \
");

// PL-SQL
wxChar* PLSqlWordlist1 = _T("\
    all alter and any array as asc at authid avg begin between binary_integer \
    body boolean bulk by char char_base check close cluster collect comment \
    commit compress connect constant create current currval cursor date day \
    declare decimal default delete desc distinct do drop else elsif end \
    exception exclusive execute exists exit extends false fetch float for \
    forall from function goto group having heap hour if immediate in index \
    indicator insert integer interface intersect interval into is isolation \
    java level like limited lock long loop max min minus minute mlslabel mod \
    mode month natural naturaln new nextval nocopy not nowait null number \
    number_base ocirowid of on opaque open operator option or order \
    organization others out package partition pctfree pls_integer positive \
    positiven pragma prior private procedure public raise range raw real record \
") _T("\
    ref release return reverse rollback row rowid rownum rowtype savepoint \
    second select separate set share smallint space sql sqlcode sqlerrm start \
    stddev subtype successful sum synonym sysdate table then time timestamp to \
    trigger true type uid union unique update use user validate values varchar \
    varchar2 variance view when whenever where while with work write year zone \
");

// POV-Ray
wxChar* PovrayWordlist1 = _T("\
    declare local include undef fopen fclose read write default version \
    case range break debug error warning if ifdef ifndef switch while \
    macro else end \
");
wxChar* PovrayWordlist2 = _T("\
    camera light_source light_group object blob sphere cylinder box cone \
    sor height_field julia_fractal lathe prism sphere_sweep superellipsoid \
    text torus bicubic_patch disc mesh mesh2 polygon triangle \
    smooth_triangle plane poly cubic quartic quadric isosurface parametric \
    union intersection difference merge function array spline \
    vertex_vectors normal_vectors uv_vectors face_indices normal_indices \
    uv_indices texture texture_list interior_texture texture_map \
    material_map image_map color_map colour_map pigment_map normal_map \
    slope_map bump_map density_map pigment normal material interior finish \
    reflection irid slope pigment_pattern image_pattern warp media \
    scattering density background fog sky_sphere rainbow global_settings \
    radiosity photons pattern transform looks_like projected_through \
    contained_by clipped_by bounded_by \
");
wxChar* PovrayWordlist3 = _T("\
    linear_spline quadratic_spline cubic_spline natural_spline \
    bezier_spline b_spline read write append inverse open perspective \
    orthographic fisheye ultra_wide_angle omnimax panoramic spherical \
    spotlight jitter circular orient media_attenuation media_interaction \
    shadowless parallel refraction collect pass_through global_lights \
    hierarchy sturm smooth gif tga iff pot png pgm ppm jpeg tiff sys \
    ttf quaternion hypercomplex linear_sweep conic_sweep type \
    all_intersections split_union cutaway_textures no_shadow no_image \
    no_reflection double_illuminate hollow uv_mapping all use_index \
    use_color use_colour no_bump_scale conserve_energy fresnel average \
    agate boxed bozo bumps cells crackle cylindrical density_file dents \
    facets granite leopard marble onion planar quilted radial ripples \
") _T("\
    spotted waves wood wrinkles solid use_alpha interpolate magnet \
    noise_generator toroidal ramp_wave triangle_wave sine_wave \
    scallop_wave cubic_wave poly_wave once map_type method fog_type \
    hf_gray_16 charset ascii utf8 rotate scale translate matrix location \
    right up direction sky angle look_at aperture blur_samples \
    focal_point confidence variance radius falloff tightness point_at \
    area_light adaptive fade_distance fade_power threshold strength \
    water_level tolerance max_iteration precision slice u_steps v_steps \
    flatness inside_vector accuracy max_gradient evaluate max_trace \
    precompute target ior dispersion dispersion_samples caustics color \
    colour rgb rgbf rgbt rgbft red green blue filter transmit gray hf \
    fade_color fade_colour quick_color quick_colour brick checker hexagon \
") _T("\
    brick_size mortar bump_size ambient diffuse brilliance crand phong \
    phong_size metallic specular roughness reflection_exponent exponent \
    thickness gradient spiral1 spiral2 agate_turb form metric offset df3 \
    coords size mandel exterior julia control0 control1 altitude \
    turbulence octaves omega lambda repeat flip black-hole orientation \
    dist_exp major_radius frequency phase intervals samples ratio \
    absorption emission aa_threshold aa_level eccentricity extinction \
    distance turb_depth fog_offset fog_alt width arc_angle falloff_angle \
    adc_bailout ambient_light assumed_gamma irid_wavelength \
    number_of_waves always_sample brigthness count error_bound \
    gray_threshold load_file low_error_factor max_sample minimum_reuse \
    nearest_count pretrace_end pretrace_start recursion_limit save_file \
") _T("\
    spacing gather max_trace_level autostop expand_thresholds \
");
wxChar* PovrayWordlist4 = _T("\
    x y z t u v yes no true false on off clock clock_delta clock_on \
    final_clock final_frame frame_number image_height image_width \
    initial_clock initial_frame pi version \
");
wxChar* PovrayWordlist5 = _T("\
    abs acos acosh asc asin asinh atan atanh atan2 ceil cos cosh defined \
    degrees dimensions dimension_size div exp file_exists floor inside \
    int ln log max min mod pow prod radians rand seed select sin sinh \
    sqrt strcmp strlen sum tan tanh val vdot vlength min_extent \
    max_extent trace vaxis_rotate vcross vrotate vnormalize vturbulence \
    chr concat str strlwr strupr substr vstr sqr cube reciprocal pwr \
");

// Properties

// PostScript
wxChar* PSWordlist1 = _T("\
    $error = == FontDirectory StandardEncoding UserObjects abs add \
    aload anchorsearch and arc arcn arcto array ashow astore atan \
    awidthshow begin bind bitshift bytesavailable cachestatus ceiling \
    charpath clear cleardictstack cleartomark clip clippath closefile \
    closepath concat concatmatrix copy copypage cos count countdictstack \
    countexecstack counttomark currentcmykcolor currentcolorspace \
    currentdash currentdict currentfile currentflat currentfont \
    currentgray currenthsbcolor currentlinecap currentlinejoin \
    currentlinewidth currentmatrix currentmiterlimit currentpagedevice \
    currentpoint currentrgbcolor currentscreen currenttransfer cvi cvlit \
    cvn cvr cvrs cvs cvx def defaultmatrix definefont dict dictstack div \
    dtransform dup echo end eoclip eofill eq erasepage errordict exch \
") _T("\
    exec execstack executeonly executive exit exp false file fill \
    findfont flattenpath floor flush flushfile for forall ge get \
    getinterval grestore grestoreall gsave gt idetmatrix idiv \
    idtransform if ifelse image imagemask index initclip initgraphics \
    initmatrix inustroke invertmatrix itransform known kshow le length \
    lineto ln load log loop lt makefont mark matrix maxlength mod moveto \
    mul ne neg newpath noaccess nor not null nulldevice or pathbbox \
    pathforall pop print prompt pstack put putinterval quit rand rcheck \
    rcurveto read readhexstring readline readonly readstring rectstroke \
    repeat resetfile restore reversepath rlineto rmoveto roll rotate \
    round rrand run save scale scalefont search setblackgeneration \
    setcachedevice setcachelimit setcharwidth setcolorscreen \
") _T("\
    setcolortransfer setdash setflat setfont setgray sethsbcolor \
    setlinecap setlinejoin setlinewidth setmatrix setmiterlimit \
    setpagedevice setrgbcolor setscreen settransfer setvmthreshold show \
    showpage sin sqrt srand stack start status statusdict stop stopped \
    store string stringwidth stroke strokepath sub systemdict token \
    token transform translate true truncate type ueofill \
    undefineresource userdict usertime version vmstatus wcheck where \
    widthshow write writehexstring writestring xcheck xor \
");
wxChar* PSWordlist2 = _T("\
    GlobalFontDirectory ISOLatin1Encoding SharedFontDirectory UserObject \
    arct colorimage cshow currentblackgeneration currentcacheparams \
    currentcmykcolor currentcolor currentcolorrendering \
    currentcolorscreen currentcolorspace currentcolortransfer \
    currentdevparams currentglobal currentgstate currenthalftone \
    currentobjectformat currentoverprint currentpacking \
    currentpagedevice currentshared currentstrokeadjust \
    currentsystemparams currentundercolorremoval currentuserparams \
    defineresource defineuserobject deletefile execform execuserobject \
    filenameforall fileposition filter findencoding findresource gcheck \
    globaldict glyphshow gstate ineofill infill instroke inueofill \
    inufill inustroke languagelevel makepattern packedarray printobject \
") _T("\
    product realtime rectclip rectfill rectstroke renamefile \
    resourceforall resourcestatus revision rootfont scheck selectfont \
    serialnumber setbbox setblackgeneration setcachedevice2 \
    setcacheparams setcmykcolor setcolor setcolorrendering \
    setcolorscreen setcolorspace setcolortranfer setdevparams \
    setfileposition setglobal setgstate sethalftone setobjectformat \
    setoverprint setpacking setpagedevice setpattern setshared \
    setstrokeadjust setsystemparams setucacheparams setundercolorremoval \
    setuserparams setvmthreshold shareddict startjob uappend ucache \
    ucachestatus ueofill ufill undef undefinefont undefineresource \
    undefineuserobject upath ustroke ustrokepath vmreclaim writeobject \
    xshow xyshow yshow \
");
wxChar* PSWordlist3 = _T("\
    cliprestore clipsave composefont currentsmoothness \
    findcolorrendering setsmoothness shfill \
");
wxChar* PSWordlist4 = _T("\
    .begintransparencygroup .begintransparencymask .bytestring \
    .charboxpath .currentaccuratecurves .currentblendmode \
    .currentcurvejoin .currentdashadapt .currentdotlength \
    .currentfilladjust2 .currentlimitclamp .currentopacityalpha \
    .currentoverprintmode .currentrasterop .currentshapealpha \
    .currentsourcetransparent .currenttextknockout \
    .currenttexturetransparent .dashpath .dicttomark \
    .discardtransparencygroup .discardtransparencymask \
    .endtransparencygroup .endtransparencymask .execn .filename \
    .filename .fileposition .forceput .forceundef .forgetsave \
    .getbitsrect .getdevice .inittransparencymask .knownget .locksafe \
    .makeoperator .namestring .oserrno .oserrorstring .peekstring \
") _T("\
    .rectappend .runandhide .setaccuratecurves .setblendmode \
    .setcurvejoin .setdashadapt .setdebug .setdefaultmatrix \
    .setdotlength .setfilladjust2 .setlimitclamp .setmaxlength \
    .setopacityalpha .setoverprintmode .setrasterop .setsafe \
    .setshapealpha .setsourcetransparent .settextknockout \
    .settexturetransparent .stringbreak .stringmatch .tempfile \
    .type1decrypt .type1encrypt .type1execchar .unread arccos arcsin \
    copydevice copyscanlines currentdevice finddevice findlibfile \
    findprotodevice flushpage getdeviceprops getenv makeimagedevice \
    makewordimagedevice max min putdeviceprops setdevice \
");

// Python
wxChar* PythonWordlist1 = _T("\
    and assert break class continue def del elif else except exec \
    finally for from global if import in is lambda None not or pass \
    print raise return try while yield \
");
wxChar* PythonWordlist2 = _T("\
    ACCELERATORS ALT AUTO3STATE AUTOCHECKBOX AUTORADIOBUTTON BEGIN \
    BITMAP BLOCK BUTTON CAPTION CHARACTERISTICS CHECKBOX CLASS \
    COMBOBOX CONTROL CTEXT CURSOR DEFPUSHBUTTON DIALOG DIALOGEX \
    DISCARDABLE EDITTEXT END EXSTYLE FONT GROUPBOX ICON LANGUAGE \
    LISTBOX LTEXT MENU MENUEX MENUITEM MESSAGETABLE POPUP PUSHBUTTON \
    RADIOBUTTON RCDATA RTEXT SCROLLBAR SEPARATOR SHIFT STATE3 \
    STRINGTABLE STYLE TEXTINCLUDE VALUE VERSION VERSIONINFO VIRTKEY \
");

// RC

// Ruby
wxChar* RubyWordlist1 = _T("\
    __FILE__ and def end in or self unless __LINE__ begin defined? \
    ensure module redo super until BEGIN break do false next rescue \
    then when END case else for nil retry true while alias class \
    elsif if not return undef yield \
");

// Scriptol
wxChar* ScriptolWordlist1 = _T("\
    act action alias always and array as bool boolean break by byte \
    class case catch const constant continue dyn def define dict do \
    double echo else elsif end enum error false file for float forever \
    function globak gtk in if ifdef import include int integer java \
    javax let long match mod nil not natural null number or print \
    protected public real return redo scan script scriptol sol short \
    super static step until using var text then this true try void \
    volatile while when undef zero \
");

// SQL
wxChar* SqlWordlist1 = _T("\
    absolute action add admin after aggregate alias all allocate alter and any \
    are array as asc assertion at authorization before begin binary bit blob \
    boolean both breadth by call cascade cascaded case cast catalog char \
    character check class clob close collate collation column commit completion \
    connect connection constraint constraints constructor continue corresponding \
    create cross cube current current_date current_path current_role \
    current_time current_timestamp current_user cursor cycle data date day \
    deallocate dec decimal declare default deferrable deferred delete depth \
    deref desc describe descriptor destroy destructor deterministic dictionary \
    diagnostics disconnect distinct domain double drop dynamic each else end \
    end-exec equals escape every except exception exec execute external false \
    fetch first float for foreign found from free full function general get \
") _T("\
    global go goto grant group grouping having host hour identity if ignore \
    immediate in indicator initialize initially inner inout input insert int \
    integer intersect interval into is isolation iterate join key language \
    large last lateral leading left less level like limit local localtime \
    localtimestamp locator map match minute modifies modify module month names \
    national natural nchar nclob new next no none not null numeric object of \
    off old on only open operation option or order ordinality out outer output \
    pad parameter parameters partial path postfix precision prefix preorder \
    prepare preserve primary prior privileges procedure public read reads real \
    recursive ref references referencing relative restrict result return returns \
    revoke right role rollback rollup routine row rows savepoint schema scroll \
    scope search second section select sequence session session_user set sets \
") _T("\
    size smallint some| space specific specifictype sql sqlexception sqlstate \
    sqlwarning start state statement static structure system_user table \
    temporary terminate than then time timestamp timezone_hour timezone_minute \
    to trailing transaction translation treat trigger true under union unique \
    unknown unnest update usage user using value values varchar variable varying \
    view when whenever where with without work write year zone \
");

// TCL
wxChar* TclWordlist1 = _T("\
    after append array auto_execok auto_import auto_load \
    auto_load_index auto_qualify beep binary break case catch cd \
    clock close concat continue dde default echo else elseif encoding \
    eof error eval exec exit expr fblocked fconfigure fcopy file \
    fileevent flush for foreach format gets glob global history if \
    incr info interp join lappend lindex linsert list llength load \
    lrange lreplace lsearch lsort namespace open package pid \
    pkg_mkIndex proc puts pwd read regexp regsub rename resource \
    return scan seek set socket source split string subst switch \
    tclLog tclMacPkgSearch tclPkgSetup tclPkgUnknown tell time trace \
    unknown unset update uplevel upvar variable vwait while bell bind \
    bindtags button canvas checkbutton console destroy entry event \
") _T("\
    focus font frame grab grid image label listbox menu menubutton \
    message pack place radiobutton raise scale scrollbar text tk \
    tkwait toplevel winfo wm tkButtonDown tkButtonEnter \
    tkButtonInvoke tkButtonLeave tkButtonUp tkCancelRepeat \
    tkCheckRadioInvoke tkDarken tkEntryAutoScan tkEntryBackspace \
    tkEntryButton1 tkEntryClosestGap tkEntryInsert tkEntryKeySelect \
    tkEntryMouseSelect tkEntryNextWord tkEntryPaste \
    tkEntryPreviousWord tkEntrySeeInsert tkEntrySetCursor \
    tkEntryTranspose tkEventMotifBindings tkFDGetFileTypes \
    tkFirstMenu tkFocusGroup_Destroy tkFocusGroup_In tkFocusGroup_Out \
    tkFocusOK tkListboxAutoScan tkListboxBeginExtend \
    tkListboxBeginSelect tkListboxBeginToggle tkListboxCancel \
") _T("\
    tkListboxDataExtend tkListboxExtendUpDown tkListboxMotion \
    tkListboxSelectAll tkListboxUpDown tkMbButtonUp tkMbEnter \
    tkMbLeave tkMbMotion tkMbPost tkMenuButtonDown tkMenuDownArrow \
    tkMenuDup tkMenuEscape tkMenuFind tkMenuFindName tkMenuFirstEntry \
    tkMenuInvoke tkMenuLeave tkMenuLeftArrow tkMenuMotion \
    tkMenuNextEntry tkMenuNextMenu tkMenuRightArrow tkMenuUnpost \
    tkMenuUpArrow tkMessageBox tkPostOverPoint tkRecolorTree \
    tkRestoreOldGrab tkSaveGrabInfo tkScaleActivate \
    tkScaleButton2Down tkScaleButtonDown tkScaleControlPress \
    tkScaleDrag tkScaleEndDrag tkScaleIncrement tkScreenChanged \
    tkScrollButton2Down tkScrollButtonDown tkScrollButtonUp \
    tkScrollByPages tkScrollByUnits tkScrollDrag tkScrollEndDrag \
") _T("\
    tkScrollSelect tkScrollStartDrag tkScrollToPos tkScrollTopBottom \
    tkTabToWindow tkTearOffMenu tkTextAutoScan tkTextButton1 \
    tkTextClosestGap tkTextInsert tkTextKeyExtend tkTextKeySelect \
    tkTextNextPara tkTextNextPos tkTextNextWord tkTextPaste \
    tkTextPrevPara tkTextPrevPos tkTextResetAnchor tkTextScrollPages \
    tkTextSelectTo tkTextSetCursor tkTextTranspose tkTextUpDownLine \
    tkTraverseToMenu tkTraverseWithinMenu tk_bisque \
    tk_chooseColor tk_dialog tk_focusFollowsMouse tk_focusNext \
    tk_focusPrev tk_getOpenFile tk_getSaveFile tk_messageBox \
    tk_optionMenu tk_popup tk_setPalette tk_textCopy tk_textCut \
    tk_textPaste \
");

// Text

// Visual Basic
wxChar* VbWordlist1 = _T("\
    addhandler addressof andalso alias and ansi as assembly auto \
    boolean byref byte byval call case catch cbool cbyte cchar cdate \
    cdec cdbl char cint class clng cobj const cshort csng cstr ctype \
    date decimal declare default delegate dim do double each else \
    elseif end enum erase error event exit false finally for friend \
    function get gettype goto  handles if implements imports in \
    inherits integer interface is let lib like long loop me mod \
    module mustinherit mustoverride mybase myclass namespace new next \
    not nothing notinheritable notoverridable object on option \
    optional or orelse overloads overridable overrides paramarray \
    preserve private property protected public raiseevent readonly \
    redim rem removehandler resume return select set shadows shared \
    short single static step stop string structure sub synclock then \
    throw to true try typeof unicode until variant when while with \
    withevents writeonly xor \
");

// VB-Script
wxChar* VbsWordlist1 = _T("\
    and begin case call continue do each else elseif end erase error \
    event exit false for function get gosub goto if implement in load \
    loop lset me mid new next not nothing on or property raiseevent \
    rem resume return rset select set stop sub then to true unload \
    until wend while with withevents attribute alias as boolean byref \
    byte byval const compare currency date declare dim double enum \
    explicit friend global integer let lib long module object option \
    optional preserve private property public redim single static \
    string type variant \
");

// XML


//----------------------------------------------------------------------------
//! languages
const LanguageInfo g_LanguageTable [] = {
    // (default)
    {(wxChar *)DEFAULT_LANGUAGE,
     _T("*.*"),
     wxSCI_LEX_PROPERTIES,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     0,
     0,
     0
    },
    // Ada
    {_T("Ada"),
     _T("*.ads;*.adb"),
     wxSCI_LEX_ADA,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_WORD1, AdaWordlist1}, // KEYWORDS
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_DELIMITER, NULL},
      {TOKEN_CHARACTER, NULL},
      {TOKEN_CHARACTER_EOL, NULL},
      {TOKEN_STRING, NULL},
      {TOKEN_STRING_EOL, NULL},
      {TOKEN_LABEL, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_ERROR, NULL}, // ILLEGAL
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T("then is"), _T("end"),
     0,
     4,
     80,
     0
    },
    // Apache conf
    {_T("Apache config"),
     _T("*.conf"),
     wxSCI_LEX_CONF,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_DEFAULT, NULL}, // EXTENSION,
      {TOKEN_WORD1, ApacheWordlist1}, // PARAMETERS
      {TOKEN_STRING, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_UUID, NULL}, // IP,
      {TOKEN_WORD2, ApacheWordlist2}, // DIRECTIVE
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     0,
     0,
     0
    },
    // Assembler
    {_T("Assembler"),
     _T("*.asm"),
     wxSCI_LEX_ASM,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_STRING, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_WORD1, AsmWordlist1}, // CPU_INSTRUCTIONS
      {TOKEN_WORD2, AsmWordlist2}, // MATH_INSTRUCTIONS
      {TOKEN_WORD3, AsmWordlist3}, // REGISTERS
      {TOKEN_WORD4, AsmWordlist4}, // DIRECTIVE
      {TOKEN_WORD5, AsmWordlist5}, // DIRECTIVE_OPERATORS
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     8,
     80,
     0
    },
    // Active Server Pages
    {_T("ActiveServerPages"),
     _T("*.asp"),
     wxSCI_LEX_HTML,
     _T(""),
     80, _T("HTML"),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, AspWordlist1},
      {TOKEN_STRING, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_STRING_EOL, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL}},
     _T(""), _T(""),
     FOLD_TYPE_HTML | FOLD_TYPE_HTMLPREP,
     2,
     80,
     EDIT_FLAG_USESPACES
    },
    // Avenue
    {_T("Avenue"),
     _T("*.ave"),
     wxSCI_LEX_AVE,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, AveWordlist1}, // KEYWORDS
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_STRING, NULL},
      {TOKEN_VALUE, NULL}, // ENUM
      {TOKEN_STRING_EOL, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_DEFAULT, NULL}, // WORD2
      {TOKEN_DEFAULT, NULL}, // WORD3
      {TOKEN_DEFAULT, NULL}, // WORD4
      {TOKEN_DEFAULT, NULL}, // WORD5
      {TOKEN_DEFAULT, NULL}, // WORD6
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     0,
     0,
     0
    },
    // Baan
    {_T("Baan"),
     _T("*.bc;*.cln"),
     wxSCI_LEX_BAAN,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, BaanWordlist1},
      {TOKEN_STRING, NULL},
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_STRING_EOL, NULL},
      {TOKEN_DEFAULT, NULL}, // WORD2
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     FOLD_TYPE_COMMENT,
     0,
     0,
     0
    },
    // Bash
    {_T("Bash"),
     _T("*.sh;*.bsh"),
     wxSCI_LEX_PERL,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_ERROR, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT, NULL}, // POD
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, BashWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_STRING, NULL},
      {TOKEN_PUNCTUATION, NULL},
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_DEFAULT, NULL}, // SCALAR
      {TOKEN_DEFAULT, NULL}, // ARRAY
      {TOKEN_DEFAULT, NULL}, // HASH
      {TOKEN_DEFAULT, NULL}, // SYMBOLTABLE
      {TOKEN_DEFAULT, NULL},
      {TOKEN_REGEX, NULL},
      {TOKEN_DEFAULT, NULL}, // REGSUBST
      {TOKEN_DEFAULT, NULL}, // LONGQUOTE
      {TOKEN_DEFAULT, NULL}, // BACKTICKS
      {TOKEN_DEFAULT, NULL}, // DATASELECTION
      {TOKEN_DEFAULT, NULL}, // HERE_Q
      {TOKEN_DEFAULT, NULL}, // HERE_QQ
      {TOKEN_DEFAULT, NULL}, // HERE_QX
      {TOKEN_DEFAULT, NULL}, // HERE_STRING_Q
      {TOKEN_DEFAULT, NULL}, // HERE_STRING_QQ
      {TOKEN_DEFAULT, NULL}, // HERE_STRING_QX
      {TOKEN_DEFAULT, NULL}, // HERE_STRING_QR
      {TOKEN_DEFAULT, NULL}, // HERE_STRING_QW
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL}},
     _T(""), _T(""),
     FOLD_TYPE_COMMENT,
     4,
     0,
     0
    },
    // Batch
    {_T("Batch"),
     _T("*.bat;*.cmd;*.nt"),
     wxSCI_LEX_BATCH,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_WORD1, BatchWordlist1}, // KEYWORDS
      {TOKEN_LABEL, NULL},
      {TOKEN_DEFAULT, NULL}, // HIDE
      {TOKEN_COMMAND, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_OPERATOR, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     3,
     80,
     EDIT_FLAG_USESPACES
    },
    // Bullant
    {_T("Bullant"),
     _T("*.ant"),
     wxSCI_LEX_BULLANT,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, BullantWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_CHARACTER, NULL},
      {TOKEN_UUID, NULL},
      {TOKEN_PREPROCESSOR, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     0,
     0,
     0
    },
    // C++
    {_T("C++"),
     _T("*.c;*.cc;*.cpp;*.cxx;*.cs;*.h;*.hh;*.hpp;*.hxx;*.sma"),
     wxSCI_LEX_CPP,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, CppWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_CHARACTER, NULL},
      {TOKEN_UUID, NULL},
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_STRING_EOL, NULL},
      {TOKEN_DEFAULT, NULL}, // VERBATIM
      {TOKEN_REGEX, NULL},
      {TOKEN_COMMENT_SPECIAL, NULL}, // DOXY
      {TOKEN_WORD2, CppWordlist2}, // WORDS2
      {TOKEN_WORD3, CppWordlist3}, // DOXY KEYWORDS
      {TOKEN_ERROR, NULL}, // DOXY ERROR
      {TOKEN_GLOBALCLASS, CppGlobalClass}, // GLOBALCLASS
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T("{"), _T("}"),
     FOLD_TYPE_COMMENT /*?| FOLD_TYPE_PREPROC ?*/,
     4,
     80,
     EDIT_FLAG_USESPACES
    },
    // CS
    {_T("CS"),
     _T("*.cs"),
     wxSCI_LEX_CPP,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, CsWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_CHARACTER, NULL},
      {TOKEN_UUID, NULL},
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_STRING_EOL, NULL},
      {TOKEN_DEFAULT, NULL}, // VERBATIM
      {TOKEN_REGEX, NULL},
      {TOKEN_COMMENT_SPECIAL, NULL}, // DOXY
      {TOKEN_DEFAULT, NULL}, // EXTRA WORDS
      {TOKEN_DEFAULT, NULL}, // DOXY KEYWORDS
      {TOKEN_ERROR, NULL}, // KEYWORDS ERROR
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     FOLD_TYPE_COMMENT,
     4,
     80,
     EDIT_FLAG_USESPACES
    },
    // CSS
    {_T("CSS"),
     _T("*.css"),
     wxSCI_LEX_CSS,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_WORD1, CssWordlist1}, // TAG
      {TOKEN_COMMAND, NULL}, // CLASS
      {TOKEN_COMMAND, CssWordlist2}, // PSEUDOCLASS
      {TOKEN_ERROR, NULL}, // PSEUDOCLASS_UNKNOWN
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_IDENTIFIER, NULL}, // IDENTIFIER UNKNOWN
      {TOKEN_VALUE, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_UUID, NULL}, // ID
      {TOKEN_DEFAULT, NULL}, // IMPORTANT
      {TOKEN_DEFAULT, NULL}, // DIRECTIVE
      {TOKEN_STRING, NULL}, // DOUBLE_STRING
      {TOKEN_CHARACTER, NULL}, // SINGLE_STRING
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T("{"), _T("}"),
     FOLD_TYPE_COMMENT,
     4,
     80,
     EDIT_FLAG_USESPACES
    },
    // Diff (patch)
    {_T("Diff"),
     _T("*.diff;*.patch"),
     wxSCI_LEX_DIFF,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_LABEL, NULL}, // KEYWORDS
      {TOKEN_PREPROCESSOR, NULL}, // HEADER
      {TOKEN_NUMBER, NULL}, // POSITION
      {TOKEN_ERROR, NULL}, // DELETED
      {TOKEN_COMMAND, NULL}, // ADDED
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     0,
     0,
     EDIT_FLAG_USESPACES
    },
    // Eiffel
    {_T("Eiffel"),
     _T("*.e"),
     wxSCI_LEX_EIFFELKW,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, EiffelWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_CHARACTER, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_STRING_EOL, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T("check debug deferred do from if inspect once"), _T("end"),
     FOLD_TYPE_COMMENT,
     4,
     80,
     EDIT_FLAG_USESPACES
    },
    // ErrorList
    {_T("ErrorList"),
     _T("*.err"),
     wxSCI_LEX_ERRORLIST,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     0,
     0,
     0
    },
    // Escript
    {_T("Escript"),
     _T("*.src;*.em"),
     wxSCI_LEX_ESCRIPT,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, EscriptWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_BRACE, NULL},
      {TOKEN_WORD2, EscriptWordlist2}, // KEYWORDS
      {TOKEN_WORD3, EscriptWordlist3}, // KEYWORDS
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T("check debug deferred do from if inspect once"), _T("end"),
     FOLD_TYPE_COMMENT,
     4,
     80,
     0
    },
    // Fortran
    {_T("Fortran"),
     _T("*.f90;*.f95;*.f;*.for"),
     wxSCI_LEX_FORTRAN,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_STRING, NULL}, // STRING1
      {TOKEN_STRING, NULL}, // STRING2
      {TOKEN_STRING_EOL, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_WORD1, FortranWordlist1}, // KEYWORDS
      {TOKEN_WORD2, FortranWordlist2}, // KEYWORDS2
      {TOKEN_WORD3, NULL}, // KEYWORDS3
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_OPERATOR, NULL}, // OPERATOR2
      {TOKEN_LABEL, NULL},
      {TOKEN_DEFAULT, NULL}, // CONTINUATION
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     FOLD_TYPE_COMMENT,
     0,
     80,
     0
    },
    // HTML
    {_T("HTML"),
     _T("*.html;*.htm;*.shtml;*.htd;*.htt;*.cfm;*.tpl;*.dtd;*.hta"),
     wxSCI_LEX_HTML,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_WORD1, HtmlWordlist1}, // TAG
      {TOKEN_PARAMETER, NULL}, // TAG UNKNOWN
      {TOKEN_WORD2, HtmlWordlist2}, // ATTRIBUTE
      {TOKEN_IDENTIFIER, NULL}, // ATTRIBUTE UNKNOWN
      {TOKEN_NUMBER, NULL},
      {TOKEN_STRING, NULL}, // DOUBLE_STRING
      {TOKEN_CHARACTER, NULL}, // SINGLE_STRING
      {TOKEN_VALUE, NULL}, // OTHER
      {TOKEN_COMMENT, NULL},
      {TOKEN_LABEL, NULL}, // ENTITY
      {TOKEN_DEFAULT, NULL}, // TAG_END
      {TOKEN_IDENTIFIER, NULL}, // XML_START
      {TOKEN_IDENTIFIER, NULL}, // XML_END
      {TOKEN_SCRIPT, NULL},
      {TOKEN_SCRIPT, NULL}, // ASP
      {TOKEN_SCRIPT, NULL}, // ASP_AT
      {TOKEN_PARAMETER, NULL}, // CDATA
      {TOKEN_UUID, NULL}, // QUESTION
      {TOKEN_VALUE, NULL},
      {TOKEN_DEFAULT, NULL}, // XC_COMMENT
      {TOKEN_DEFAULT, NULL}, // SGLM_DEFAULT
      {TOKEN_DEFAULT, NULL}, // SGLM_COMMAND
      {TOKEN_DEFAULT, NULL}, // SGLM_1ST_PARAM
      {TOKEN_DEFAULT, NULL}, // SGLM_DOUBLE_STRING
      {TOKEN_DEFAULT, NULL}, // SGLM_SINGLE_STRING
      {TOKEN_DEFAULT, NULL}, // SGLM_ERROR
      {TOKEN_DEFAULT, NULL}, // SGLM_SPECIAL
      {TOKEN_DEFAULT, NULL}, // SGLM_ENTITY
      {TOKEN_DEFAULT, NULL}, // SGLM_COMMENT
      {TOKEN_DEFAULT, NULL}, // SGLM_PARAM_COMMENT
      {TOKEN_DEFAULT, NULL}}, // SGLM_BLOCK_DEFAULT
     _T(""), _T(""),
     FOLD_TYPE_HTML | FOLD_TYPE_HTMLPREP,
     2,
     80,
     EDIT_FLAG_USESPACES
    },
    // IDL
    {_T("IDL"),
     _T("*.idl;*.odl"),
     wxSCI_LEX_CPP,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, IdlWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_CHARACTER, NULL},
      {TOKEN_UUID, NULL},
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_STRING_EOL, NULL},
      {TOKEN_DEFAULT, NULL}, // VERBATIM
      {TOKEN_REGEX, NULL},
      {TOKEN_COMMENT_SPECIAL, NULL}, // DOXY
      {TOKEN_WORD2, NULL}, // EXTRA WORDS
      {TOKEN_WORD3, NULL}, // DOXY KEYWORDS
      {TOKEN_ERROR, NULL}, // KEYWORDS ERROR
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     FOLD_TYPE_COMMENT /*?| FOLD_TYPE_PREPROC ?*/,
     2,
     80,
     EDIT_FLAG_USESPACES
    },
    // Java
    {_T("Java"),
     _T("*.java"),
     wxSCI_LEX_CPP,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, JavaWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_CHARACTER, NULL},
      {TOKEN_UUID, NULL},
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_STRING_EOL, NULL},
      {TOKEN_DEFAULT, NULL}, // VERBATIM
      {TOKEN_REGEX, NULL},
      {TOKEN_COMMENT_SPECIAL, NULL}, // DOXY
      {TOKEN_WORD2, NULL}, // EXTRA WORDS
      {TOKEN_WORD3, NULL}, // DOXY KEYWORDS
      {TOKEN_ERROR, NULL}, // KEYWORDS ERROR
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T("{"), _T("}"),
     FOLD_TYPE_COMMENT /*?| FOLD_TYPE_PREPROC ?*/,
     4,
     80,
     EDIT_FLAG_USESPACES
    },
    // JavaScript
    {_T("JavaScript"),
     _T("*.js"),
     wxSCI_LEX_CPP,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, JavascriptWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_CHARACTER, NULL},
      {TOKEN_UUID, NULL},
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_STRING_EOL, NULL},
      {TOKEN_DEFAULT, NULL}, // VERBATIM
      {TOKEN_REGEX, NULL},
      {TOKEN_COMMENT_SPECIAL, NULL}, // DOXY
      {TOKEN_WORD2, NULL}, // EXTRA WORDS
      {TOKEN_WORD3, NULL}, // DOXY KEYWORDS
      {TOKEN_ERROR, NULL}, // KEYWORDS ERROR
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T("{"), _T("}"),
     FOLD_TYPE_COMMENT /*?| FOLD_TYPE_PREPROC ?*/,
     4,
     80,
     EDIT_FLAG_USESPACES
    },
    // Latex
    {_T("Latex"),
     _T("*.tex;*.sty"),
     wxSCI_LEX_LATEX,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMAND, NULL},
      {TOKEN_DEFAULT, NULL}, // TAG
      {TOKEN_DEFAULT, NULL}, // MATH
      {TOKEN_COMMENT, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     0,
     0,
     EDIT_FLAG_USESPACES
    },
    // Lisp
    {_T("Lisp"),
     _T("*.lsp;*.lisp"),
     wxSCI_LEX_LISP,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, LispWordlist1}, // KEYWORDS
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_STRING, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_STRING_EOL, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_OPERATOR, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     4,
     80,
     EDIT_FLAG_USESPACES
    },
    // Lout
    {_T("Lout"),
     _T("*.lt"),
     wxSCI_LEX_LOUT,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, LoutWordlist1}, // KEYWORDS
      {TOKEN_WORD2, LoutWordlist2}, // KEYWORDS
      {TOKEN_WORD3, LoutWordlist3}, // KEYWORDS
      {TOKEN_DEFAULT, NULL},
      {TOKEN_STRING, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_STRING_EOL, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     4,
     80,
     0
    },
    // Lua
    {_T("Lua"),
     _T("*.lua"),
     wxSCI_LEX_LUA,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, LuaWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_CHARACTER, NULL},
      {TOKEN_DEFAULT, NULL}, // LITERALSTRING
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_STRING_EOL, NULL},
      {TOKEN_WORD2, LuaWordlist2}, // EXTRA WORDS
      {TOKEN_WORD3, LuaWordlist3},
      {TOKEN_WORD4, LuaWordlist4},
      {TOKEN_DEFAULT, NULL}, // WORD5
      {TOKEN_DEFAULT, NULL}, // WORD6
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     4,
     80,
     0
    },
    // Make
    {_T("Make"),
     _T("Makefile;makefile.*,configure.*;*.mak"),
     wxSCI_LEX_MAKEFILE,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_STRING, NULL}, // TARGET
      {TOKEN_STRING_EOL, NULL}, // ID_EOL
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     4,
     80,
     0
    },
    // MatLab
    {_T("MatLab"),
     _T("*.m"),
     wxSCI_LEX_MATLAB,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMAND, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, MatLabWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     0,
     0,
     0
    },
    // MMIX Assembler
    {_T("MMIX Assembler"),
     _T("*.mms"),
     wxSCI_LEX_MMIXAL,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_LABEL, NULL},
      {TOKEN_DEFAULT, NULL}, // OPCODES (not validated)
      {TOKEN_OPERATOR, NULL}, // DIVISION BETWEEN LABELS AND OPCODES
      {TOKEN_WORD1, MmixalWordlist1}, // OPCODES
      {TOKEN_ERROR, NULL},
      {TOKEN_DEFAULT, NULL}, // DIVISION BETWEEN OPCODES AND OPERANDS
      {TOKEN_DEFAULT, NULL}, // DIVISION OF OPERANDS
      {TOKEN_NUMBER, NULL},
      {TOKEN_LABEL, NULL}, // REFERENCE TO LABEL
      {TOKEN_CHARACTER, NULL},
      {TOKEN_STRING, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_NUMBER, NULL}, // HEXNUMBER
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL}, //SYMBOLS
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     16,
     80,
     0
    },
    // NNCron
    {_T("NNCron"),
     _T("*.tab;*.spf"),
     wxSCI_LEX_NNCRONTAB,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_DEFAULT, NULL}, // TASK
      {TOKEN_WORD1, NNCronWordlist1}, // KEYWORDS
      {TOKEN_WORD2, NNCronWordlist2}, // KEYWORDS2
      {TOKEN_WORD3, NNCronWordlist3}, // MODIFIER
      {TOKEN_DEFAULT, NULL}, // ASTERISK
      {TOKEN_NUMBER, NULL},
      {TOKEN_STRING, NULL},
      {TOKEN_DEFAULT, NULL}, // ENVIRONMENT
      {TOKEN_IDENTIFIER, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     0,
     0,
     0
    },
    // NSIS
    {_T("NSIS"),
     _T("*.nsi;*.nsh"),
     wxSCI_LEX_NSIS,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_STRING, NULL}, // STRING DOUBLEQUOTE
      {TOKEN_STRING, NULL}, // STRING LEFT QUOTE
      {TOKEN_STRING, NULL}, // STRING RIGHT QUOTE
      {TOKEN_COMMAND, NSISWordlist1}, // FUNCTION
      {TOKEN_VALUE, NULL}, // VARIABLE
      {TOKEN_LABEL, NSISWordlist3},
      {TOKEN_DEFAULT, NSISWordlist4}, // USER DEFINED
      {TOKEN_DEFAULT, NULL}, // SECTION
      {TOKEN_DEFAULT, NULL}, // SUBSECTION
      {TOKEN_DEFAULT, NULL}, // IF DEF
      {TOKEN_PREPROCESSOR, NULL}, // MACRO DEF
      {TOKEN_VALUE, NSISWordlist2}, // VARIABLE
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     0,
     0,
     0
    },
    // Pascal
    {_T("Pascal"),
     _T("*.pas;*.inc;*.pp"),
     wxSCI_LEX_PASCAL,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, PascalWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_CHARACTER, NULL},
      {TOKEN_UUID, NULL},
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_OPERATOR, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {TOKEN_COMMAND, NULL}, // ASM
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {TOKEN_IDENTIFIER, NULL}},
     _T("begin"), _T("end"),
     FOLD_TYPE_COMMENT /*?| FOLD_TYPE_PREPROC ?*/,
     4,
     80,
     0
    },
    // Perl
    {_T("Perl"),
     _T("*.pl;*.pm;*.cgi;*.pod"),
     wxSCI_LEX_PERL,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_ERROR, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT, NULL}, // POD
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, PerlWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_CHARACTER, NULL},
      {TOKEN_PUNCTUATION, NULL},
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_DEFAULT, NULL}, // SCALAR
      {TOKEN_DEFAULT, NULL}, // ARRAY
      {TOKEN_DEFAULT, NULL}, // HASH
      {TOKEN_DEFAULT, NULL}, // SYMBOLTABLE
      {TOKEN_DEFAULT, NULL},
      {TOKEN_REGEX, NULL},
      {TOKEN_DEFAULT, NULL}, // REGSUBST
      {TOKEN_DEFAULT, NULL}, // LONGQUOTE
      {TOKEN_DEFAULT, NULL}, // BACKTICKS
      {TOKEN_DEFAULT, NULL}, // DATASELECTION
      {TOKEN_DEFAULT, NULL}, // HERE_Q
      {TOKEN_DEFAULT, NULL}, // HERE_QQ
      {TOKEN_DEFAULT, NULL}, // HERE_QX
      {TOKEN_DEFAULT, NULL}, // HERE_STRING_Q
      {TOKEN_DEFAULT, NULL}, // HERE_STRING_QQ
      {TOKEN_DEFAULT, NULL}, // HERE_STRING_QX
      {TOKEN_DEFAULT, NULL}, // HERE_STRING_QR
      {TOKEN_DEFAULT, NULL}, // HERE_STRING_QW
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL}},
     _T("{"), _T("}"),
     FOLD_TYPE_COMMENT,
     4,
     80,
     EDIT_FLAG_USESPACES
    },
    // PHP
    {_T("PHP"),
     _T("*.php3;*.phtml;*.php"),
     wxSCI_LEX_HTML,
     _T(""),
     118, _T("HTML"),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_STRING, NULL}, // DOUBLE_STRING
      {TOKEN_CHARACTER, NULL}, // SINGLE_STRING
      {TOKEN_WORD1, PhpWordlist1},
      {TOKEN_NUMBER, NULL},
      {TOKEN_VALUE, NULL}, // VARIABLE
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL}},
     _T("{"), _T("}"),
     FOLD_TYPE_HTML | FOLD_TYPE_HTMLPREP,
     2,
     80,
     EDIT_FLAG_USESPACES
    },
    // PLSQL
    {_T("PL-SQL"),
     _T("*.spec;*.body;*.sps;*.spb;*.sf;*.sp"),
     wxSCI_LEX_SQL,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, PLSqlWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_CHARACTER, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_STRING_EOL, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     4,
     80,
     0
    },
    // POVray
    {_T("POV-Ray"),
     _T("*.pov"),
     wxSCI_LEX_POV,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_OPERATOR, NULL},
      {-1, NULL},
      {TOKEN_STRING, NULL},
      {TOKEN_STRING_EOL, NULL},
      {TOKEN_WORD1, PovrayWordlist1}, // KEYWORDS
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_WORD2, PovrayWordlist2}, // KEYWORDS
      {TOKEN_WORD3, PovrayWordlist3}, // KEYWORDS
      {TOKEN_WORD4, PovrayWordlist4}, // KEYWORDS
      {TOKEN_WORD5, PovrayWordlist5}, // KEYWORDS
      {-1, NULL}, // KEYWORDS
      {-1, NULL}, // KEYWORDS
      {-1, NULL}, // KEYWORDS
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T("{"), _T("}"),
     FOLD_TYPE_COMMENT,
     0,
     0,
     0
    },
    // Properties
    {_T("Properties"),
     _T("*.properties;*.ini;*.inf;*.reg;*.url;*.cfg;*.cnf;*.aut"),
     wxSCI_LEX_PROPERTIES,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMAND, NULL}, // SECTION
      {TOKEN_OPERATOR, NULL}, // ASSIGNMENT
      {TOKEN_DEFAULT, NULL}, // DEFVAL
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     0,
     0,
     EDIT_FLAG_USESPACES
    },
    // PostScript
    {_T("PostScript"),
     _T("*.ps"),
     wxSCI_LEX_PS,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_DEFAULT, NULL}, // NAME
      {TOKEN_WORD1, PSWordlist1}, // KEYWORDS
      {TOKEN_LABEL, NULL}, // LITERAL
      {TOKEN_PREPROCESSOR, NULL}, // LITERAL IMMEDIATELY EVALUATED
      {TOKEN_WORD1, NULL}, // ARRAY PARENTHESIS
      {TOKEN_BRACE, NULL}, // DICTIONARY PARENTHESIS
      {TOKEN_OPERATOR, NULL}, // ARRAY PARENTHESIS
      {TOKEN_STRING, NULL},
      {TOKEN_STRING, NULL}, // HEXSTRING
      {TOKEN_STRING, NULL},
      {TOKEN_ERROR, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     0,
     0,
     0
    },
    // Python
    {_T("Python"),
     _T("*.py;*.pyw"),
     wxSCI_LEX_PYTHON,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_STRING, NULL},
      {TOKEN_CHARACTER, NULL},
      {TOKEN_WORD1, PythonWordlist1}, // KEYWORDS
      {TOKEN_DEFAULT, NULL}, // TRIPLE
      {TOKEN_DEFAULT, NULL}, // TRIPLEDOUBLE
      {TOKEN_DEFAULT, NULL}, // CLASSNAME
      {TOKEN_DEFAULT, PythonWordlist2}, // DEFNAME
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_DEFAULT, NULL}, // COMMENT_BLOCK
      {TOKEN_STRING_EOL, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     FOLD_TYPE_COMMENTPY | FOLD_TYPE_QUOTESPY,
     4,
     80,
     EDIT_FLAG_USESPACES
    },
    // RC
    {_T("RC"),
     _T("*.rc;*.rc2;*.dlg"),
     wxSCI_LEX_CPP,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_DEFAULT, NULL}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_CHARACTER, NULL},
      {TOKEN_UUID, NULL},
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_STRING_EOL, NULL},
      {TOKEN_DEFAULT, NULL}, // VERBATIM
      {TOKEN_REGEX, NULL},
      {TOKEN_COMMENT_SPECIAL, NULL}, // DOXY
      {TOKEN_WORD2, NULL}, // EXTRA WORDS
      {TOKEN_WORD3, NULL}, // DOXY KEYWORDS
      {TOKEN_ERROR, NULL}, // KEYWORDS ERROR
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     FOLD_TYPE_COMMENT /*?| FOLD_TYPE_PREPROC ?*/,
     4,
     80,
     EDIT_FLAG_USESPACES
    },
    // Ruby
    {_T("Ruby"),
     _T("*.rb"),
     wxSCI_LEX_RUBY,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_STRING, NULL},
      {TOKEN_CHARACTER, NULL},
      {TOKEN_WORD1, RubyWordlist1}, // KEYWORDS
      {TOKEN_DEFAULT, NULL}, // TRIPLE
      {TOKEN_DEFAULT, NULL}, // TRIPLEDOUBLE
      {TOKEN_DEFAULT, NULL}, // CLASSNAME
      {TOKEN_DEFAULT, NULL}, // DEFNAME
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_COMMENT_DOC, NULL}, // COMMENT_BLOCK
      {TOKEN_STRING_EOL, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     4,
     80,
     EDIT_FLAG_USESPACES
    },
    // Scriptol
    {_T("Scriptol"),
     _T("*.sol"),
     wxSCI_LEX_SCRIPTOL,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_STRING, NULL},
      {TOKEN_STRING_EOL, NULL},
      {TOKEN_WORD1, ScriptolWordlist1}, // KEYWORDS
      {TOKEN_DEFAULT, NULL}, // TRIPLE QUOTES
      {TOKEN_DEFAULT, NULL}, // TRIPLE DOUBLEQUOTES
      {TOKEN_DEFAULT, NULL}, // CLASSNAME
      {TOKEN_DEFAULT, NULL}, // DEFNAME
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_COMMENT_DOC, NULL}, // COMMENT_BLOCK
      {TOKEN_STRING_EOL, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     4,
     80,
     0
    },
    // SQL
    {_T("SQL"),
     _T("*.sql"),
     wxSCI_LEX_SQL,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, SqlWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL}, // STRING DOUBLE QUOTED
      {TOKEN_CHARACTER, NULL}, // STRING SINGLE QUOTED
      {TOKEN_PUNCTUATION, NULL}, // SYMBOLS
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_STRING_EOL, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     4,
     80,
     0
    },
    // TCL
    {_T("TCL"),
     _T("*.tcl"),
     wxSCI_LEX_TCL,
     _T(""),
     0, _T(""),
     {{TOKEN_PARAMETER, NULL}, // DEFAULT,
      {TOKEN_COMMENT, NULL},
      {TOKEN_COMMENT_LINE, NULL},
      {TOKEN_COMMENT_DOC, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, TclWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_CHARACTER, NULL},
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_STRING_EOL, NULL},
      {TOKEN_PARAMETER, NULL}, // VERBATIM
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T("{"), _T("}"),
     FOLD_TYPE_COMMENT /*?| FOLD_TYPE_PREPROC ?*/,
     4,
     80,
     0
    },
    // Text
    {_T("Text"),
     _T("*.txt;*.log;*.lst;*.doc;*.diz;*.nfo"),
     wxSCI_LEX_PROPERTIES,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     0,
     0,
     EDIT_FLAG_WRAPMODE
    },
    // VisualBasic
    {_T("VisualBasic"),
     _T("*.vb;*.bas;*.frm;*.cls;*.ctl;*.pag;*.dsr;*.dob"),
     wxSCI_LEX_VB,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, VbWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_DEFAULT, NULL}, // DATE
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     3,
     80,
     EDIT_FLAG_USESPACES
    },
    // WScript
    {_T("VB-Script"),
     _T("*.vbs;*.dsm"),
     wxSCI_LEX_VBSCRIPT,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_COMMENT, NULL},
      {TOKEN_NUMBER, NULL},
      {TOKEN_WORD1, VbsWordlist1}, // KEYWORDS
      {TOKEN_STRING, NULL},
      {TOKEN_PREPROCESSOR, NULL},
      {TOKEN_OPERATOR, NULL},
      {TOKEN_IDENTIFIER, NULL},
      {TOKEN_DEFAULT, NULL}, // DATE
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL},
      {-1, NULL}},
     _T(""), _T(""),
     0,
     3,
     80,
     EDIT_FLAG_USESPACES
    },
    // XML
    {_T("XML"),
     _T("*.xml;*.xsl;*.svg;*.xul;*.xsd;*.dtd;*.xslt;*.axl;*.xrc"),
     wxSCI_LEX_HTML,
     _T(""),
     0, _T(""),
     {{TOKEN_DEFAULT, NULL},
      {TOKEN_DEFAULT, NULL}, // TAG
      {TOKEN_ERROR, NULL}, // TAG_UNKNOWN
      {TOKEN_DEFAULT, NULL}, // ATTRIBUTE
      {TOKEN_ERROR, NULL}, // ATTRIBUTE_UNKNOWN
      {TOKEN_NUMBER, NULL},
      {TOKEN_STRING, NULL}, // DOUBLE_STRING
      {TOKEN_CHARACTER, NULL}, // SINGLE_STRING
      {TOKEN_DEFAULT, NULL}, // OTHER
      {TOKEN_COMMENT, NULL},
      {TOKEN_LABEL, NULL}, // ENTITY
      {TOKEN_DEFAULT, NULL}, // TAG_END
      {TOKEN_IDENTIFIER, NULL}, // XML_START
      {TOKEN_IDENTIFIER, NULL}, // XML_END
      {TOKEN_SCRIPT, NULL}, // SCRIPT
      {TOKEN_SCRIPT, NULL}, // ASP
      {TOKEN_SCRIPT, NULL}, // ASP_AT
      {TOKEN_PARAMETER, NULL}, // CDATA
      {TOKEN_UUID, NULL}, // QUESTION
      {TOKEN_VALUE, NULL},
      {TOKEN_DEFAULT, NULL}, // XC_COMMENT
      {TOKEN_DEFAULT, NULL}, // SGLM_DEFAULT
      {TOKEN_DEFAULT, NULL}, // SGLM_COMMAND
      {TOKEN_DEFAULT, NULL}, // SGLM_1ST_PARAM
      {TOKEN_DEFAULT, NULL}, // SGLM_DOUBLE_STRING
      {TOKEN_DEFAULT, NULL}, // SGLM_SINGLE_STRING
      {TOKEN_DEFAULT, NULL}, // SGLM_ERROR
      {TOKEN_DEFAULT, NULL}, // SGLM_SPECIAL
      {TOKEN_DEFAULT, NULL}, // SGLM_ENTITY
      {TOKEN_DEFAULT, NULL}, // SGLM_COMMENT
      {TOKEN_DEFAULT, NULL}, // SGLM_PARAM_COMMENT
      {TOKEN_DEFAULT, NULL}}, // SGLM_BLOCK_DEFAULT
     _T(""), _T(""),
     FOLD_TYPE_HTML | FOLD_TYPE_HTMLPREP,
     2,
     80,
     EDIT_FLAG_USESPACES
    }
};

const int g_LanguagePrefsSize = WXSIZEOF(g_LanguageTable);
LanguageInfo g_LanguagePrefs [g_LanguagePrefsSize];

//----------------------------------------------------------------------------
//! style types
const StyleInfo g_StyleTable [] = {
    // TOKEN_DEFAULT
    {_T("Default"),
     _T("BLACK"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_WORD1
    {_T("Keyword1"),
     _T("BLUE"), _T("WHITE"),
     _T(""), 10, TOKEN_STYLE_BOLD, 0
    },
    // TOKEN_WORD2
    {_T("Keyword2"),
     _T("DARK BLUE"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_WORD3
    {_T("Keyword3"),
     _T("CORNFLOWER BLUE"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_WORD4
    {_T("Keyword4"),
     _T("CYAN"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_WORD5
    {_T("Keyword5"),
     _T("DARK GREY"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_WORD6
    {_T("Keyword6"),
     _T("GREY"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_COMMENT
    {_T("Comment"),
     _T("FOREST GREEN"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_COMMENT_DOC
    {_T("Comment (Doc)"),
     _T("FOREST GREEN"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_COMMENT_LINE
    {_T("Comment line"),
     _T("FOREST GREEN"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_COMMENT_SPECIAL
    {_T("Special comment"),
     _T("FOREST GREEN"), _T("WHITE"),
     _T(""), 10, TOKEN_STYLE_ITALIC, 0
    },
    // TOKEN_CHARACTER
    {_T("Character"),
     _T("KHAKI"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_CHARACTER_EOL
    {_T("Character (EOL)"),
     _T("KHAKI"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_STRING
    {_T("String"),
     _T("BROWN"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_STRING_EOL
    {_T("String (EOL)"),
     _T("BROWN"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_DELIMITER
    {_T("Delimiter"),
     _T("ORANGE"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_PUNCTUATION
    {_T("Punctuation"),
     _T("ORANGE"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_OPERATOR
    {_T("Operator"),
     _T("BLACK"), _T("WHITE"),
     _T(""), 10, TOKEN_STYLE_BOLD, 0
    },
    // TOKEN_BRACE
    {_T("Label"),
     _T("VIOLET"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_COMMAND
    {_T("Command"),
     _T("BLUE"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_IDENTIFIER
    {_T("Identifier"),
     _T("BLACK"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_LABEL
    {_T("Label"),
     _T("VIOLET"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_NUMBER
    {_T("Number"),
     _T("SIENNA"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_PARAMETER
    {_T("Parameter"),
     _T("VIOLET"), _T("WHITE"),
     _T(""), 10, TOKEN_STYLE_ITALIC, 0
    },
    // TOKEN_REGEX
    {_T("Regular expression"),
     _T("ORCHID"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_UUID
    {_T("UUID"),
     _T("ORCHID"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_VALUE
    {_T("Value"),
     _T("ORCHID"), _T("WHITE"),
     _T(""), 10, TOKEN_STYLE_ITALIC, 0
    },
    // TOKEN_PREPROCESSOR
    {_T("Preprocessor"),
     _T("GREY"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_SCRIPT
    {_T("Script"),
     _T("DARK GREY"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_ERROR
    {_T("Error"),
     _T("RED"), _T("WHITE"),
     _T(""), 10, 0, 0
    },
    // TOKEN_UNDEFINED
    {_T("Undefined"),
     _T("ORANGE"), _T("WHITE"),
     _T(""), 10, 0, 0
    }
};

const int g_StylePrefsSize = WXSIZEOF(g_StyleTable);
StyleInfo g_StylePrefs [g_StylePrefsSize];

//----------------------------------------------------------------------------
//! AnalyseInfo
const AnalyseInfo g_AnalyseTable [] = {
    // (default)
    {(wxChar *)DEFAULT_LANGUAGE,
     false, // fold
     {{-1, -1, false, _T(""), -1, -1},
      {-1, -1, false, _T(""), -1, -1},
      {-1, -1, false, _T(""), -1, -1},
      {-1, -1, false, _T(""), -1, -1},
      {-1, -1, false, _T(""), -1, -1},
      {-1, -1, false, _T(""), -1, -1},
      {-1, -1, false, _T(""), -1, -1},
      {-1, -1, false, _T(""), -1, -1},
      {-1, -1, false, _T(""), -1, -1},
      {-1, -1, false, _T(""), -1, -1}}
    },
    // C++
    {_T("C++"),
     true, // fold
     {{ 0, TOKEN_WORD1,      true,  _T("class"), -1,  1},
      { 1, TOKEN_IDENTIFIER, false, _T(""),       1,  2},
      { 2, TOKEN_OPERATOR,   false, _T(":"),      1,  0},
      { 2, TOKEN_OPERATOR,   false, _T("{"),      1,  0},
      { 0, TOKEN_IDENTIFIER, false, _T(""),       1, 11},
      {11, TOKEN_IDENTIFIER, false, _T(""),       1, 11},
      {11, TOKEN_OPERATOR,   false, _T("::"),     1, 12},
      {12, TOKEN_IDENTIFIER, false, _T(""),       2, 13},
      {13, TOKEN_IDENTIFIER, false, _T(""),       2, 11},
      {13, TOKEN_OPERATOR,   false, _T("("),      2,  0}}
    }
};

const int g_AnalysePrefsSize = WXSIZEOF(g_AnalyseTable);
AnalyseInfo g_AnalysePrefs [g_AnalysePrefsSize];

//----------------------------------------------------------------------------
// feedback
bool g_feedback;

//----------------------------------------------------------------------------
// CheckInstallation
//----------------------------------------------------------------------------

void CheckInstallation (wxString appVersion) {

    wxString about;
    wxString date;
    wxString version;

    // read or create install information
    wxConfig* cfg = new wxConfig (g_appname);
    if (cfg->Exists (INSTALLATION)) {
        about = cfg->Read (INSTALLATION + _T("/") + INSTALL_ABOUT, about);
        date = cfg->Read (INSTALLATION + _T("/") + INSTALL_DATE, date);
        version = cfg->Read (INSTALLATION + _T("/") + INSTALL_VERSION, version);
    }
    if (version.IsEmpty() ||
        (version.BeforeLast ('.') != appVersion.BeforeLast ('.'))) {
        cfg->Write (INSTALLATION + _T("/") + INSTALL_ABOUT, wxEmptyString);
        date = wxDateTime::Today().FormatISODate();
        cfg->Write (INSTALLATION + _T("/") + INSTALL_DATE, date);
        version = appVersion;
        cfg->Write (INSTALLATION + _T("/") + INSTALL_VERSION, version);
    }

    // check installation, enable splash screen and set new time limit
    if ((about != _T("OK")) &&
        (date < (wxDateTime::Today() - wxTimeSpan::Days(31)).FormatISODate())) {
        g_CommonPrefs.noSplash = false;
        Prefs::SaveValuesPageCommon ();
        date = wxDateTime::Today().FormatISODate();
        cfg->Write (INSTALLATION + _T("/") + INSTALL_DATE, date);
    }
    delete cfg;
}

//----------------------------------------------------------------------------
// Prefs
//----------------------------------------------------------------------------

Prefs::Prefs () {

    // load all prefs
    LoadValuesPageCommon ();
    LoadValuesPageLanguages ();
    LoadValuesPageStyles ();
    LoadValuesPageAnalyse ();
}

Prefs::~Prefs () {

    // delete all prefs
    DeleteValuesPageCommon ();
    DeleteValuesPageLanguages ();
    DeleteValuesPageStyles ();
    DeleteValuesPageAnalyse ();

}

//----------------------------------------------------------------------------
void Prefs::LoadValuesPageCommon (bool dflt) {
    g_CommonPrefs = g_CommonTable;
    if (dflt) return;
    wxString key = PAGE_COMMON;
    key.Append (_T("/"));

    // common prefs
    wxConfig *cfg = new wxConfig (g_appname);
    cfg->Read (key + NOSPLASH, &g_CommonPrefs.noSplash);
    cfg->Read (key + SHOWBROWSE, &g_CommonPrefs.showBrowse);
    cfg->Read (key + SHOWSTATUSBAR, &g_CommonPrefs.showStatusbar);
#if wxCHECK_VERSION(2, 5, 0) || !defined(__WXGTK__)
    cfg->Read (key + SHOWTOOLBAR, &g_CommonPrefs.showToolbar);
#else
    g_CommonPrefs.showToolbar = false;
#endif
    cfg->Read (key + USEPAGES, &g_CommonPrefs.usePages);

    // editor prefs
    cfg->Read (key + SYNTAXENABLE, &g_CommonPrefs.syntaxEnable);
    cfg->Read (key + FOLDENABLE, &g_CommonPrefs.foldEnable);
    cfg->Read (key + INDENTENABLE, &g_CommonPrefs.indentEnable);
    cfg->Read (key + OVERTYPE, &g_CommonPrefs.overTypeInitial);
    cfg->Read (key + READONLY, &g_CommonPrefs.readOnlyInitial);
    cfg->Read (key + WRAPMODE, &g_CommonPrefs.wrapModeInitial);
    cfg->Read (key + DISPLAYEOL, &g_CommonPrefs.displayEOLEnable);
    cfg->Read (key + INDENTGUIDE, &g_CommonPrefs.indentGuideEnable);
    cfg->Read (key + LINENUMBER, &g_CommonPrefs.lineNumberEnable);
    cfg->Read (key + LONGLINEON, &g_CommonPrefs.longLineOnEnable);
    cfg->Read (key + WHITESPACE, &g_CommonPrefs.whiteSpaceEnable);

    // modification prefs
    cfg->Read (key + CONVERTTOSPACES, &g_CommonPrefs.convertToSpaces);
    cfg->Read (key + CONVERTTOTABS, &g_CommonPrefs.convertToTabs);

    // feedback
    g_feedback = cfg->Read (INSTALLATION + _T("/") + INSTALL_ABOUT) == _T("OK");
    delete cfg;
}

//----------------------------------------------------------------------------
void Prefs::SaveValuesPageCommon () {
    wxString key = PAGE_COMMON;
    key.Append (_T("/"));

    // common prefs
    wxConfig *cfg = new wxConfig (g_appname);
    cfg->Write (key + NOSPLASH, g_CommonPrefs.noSplash);
    cfg->Write (key + SHOWBROWSE, g_CommonPrefs.showBrowse);
    cfg->Write (key + SHOWSTATUSBAR, g_CommonPrefs.showStatusbar);
    cfg->Write (key + SHOWTOOLBAR, g_CommonPrefs.showToolbar);
    cfg->Write (key + USEPAGES, g_CommonPrefs.usePages);

    // editor prefs
    cfg->Write (key + SYNTAXENABLE, g_CommonPrefs.syntaxEnable);
    cfg->Write (key + FOLDENABLE, g_CommonPrefs.foldEnable);
    cfg->Write (key + INDENTENABLE, g_CommonPrefs.indentEnable);
    cfg->Write (key + DISPLAYEOL, g_CommonPrefs.displayEOLEnable);
    cfg->Write (key + OVERTYPE, g_CommonPrefs.overTypeInitial);
    cfg->Write (key + READONLY, g_CommonPrefs.readOnlyInitial);
    cfg->Write (key + WRAPMODE, g_CommonPrefs.wrapModeInitial);
    cfg->Write (key + INDENTGUIDE, g_CommonPrefs.indentGuideEnable);
    cfg->Write (key + LINENUMBER, g_CommonPrefs.lineNumberEnable);
    cfg->Write (key + LONGLINEON, g_CommonPrefs.longLineOnEnable);
    cfg->Write (key + WHITESPACE, g_CommonPrefs.whiteSpaceEnable);

    // modification prefs
    cfg->Write (key + CONVERTTOSPACES, g_CommonPrefs.convertToSpaces);
    cfg->Write (key + CONVERTTOTABS, g_CommonPrefs.convertToTabs);

    // feedback
    cfg->Write (INSTALLATION + _T("/") + INSTALL_ABOUT, g_feedback? _T("OK"): wxEmptyString);
    delete cfg;
}

//----------------------------------------------------------------------------
void Prefs::DeleteValuesPageCommon () {
    // nothing to delete
}

//----------------------------------------------------------------------------
void Prefs::LoadValuesPageLanguages (bool dflt) {
    LanguageInfo *curPrefs;
    wxString key = PAGE_LANGUAGES;
    key.Append (_T("/"));
    wxString name;

    wxConfig *cfg = new wxConfig (g_appname);
    int languageNr;
    for (languageNr = 0; languageNr < g_LanguagePrefsSize; ++languageNr) {
        g_LanguagePrefs [languageNr] = g_LanguageTable [languageNr];
        curPrefs = &g_LanguagePrefs [languageNr];
        if (dflt) continue;
        name = curPrefs->name;
        name.Append (_T("/"));
        if (cfg->Exists (key + name + FILEPATTERN)) {
            wxString filepattern;
            cfg->Read (key + name + FILEPATTERN, &filepattern);
            curPrefs->filepattern = wxStrdup (filepattern);
        }
        if (cfg->Exists (key + name + HELPBOOKS)) {
            wxString helpbooks;
            cfg->Read (key + name + HELPBOOKS, &helpbooks);
            curPrefs->helpbooks = wxStrdup (helpbooks);
        }
        if (cfg->Exists (key + name + INDENTVALUE)) {
            cfg->Read (key + name + INDENTVALUE, &curPrefs->indent);
        }
        if (cfg->Exists (key + name + LONGLINEPOS)) {
            cfg->Read (key + name + LONGLINEPOS, &curPrefs->longln);
        }
        if (cfg->Exists (key + name + FLAGVALUES)) {
            cfg->Read (key + name + FLAGVALUES, &curPrefs->flags);
        }
    }
    delete cfg;
}

//----------------------------------------------------------------------------
void Prefs::SaveValuesPageLanguages () {
    LanguageInfo *curPrefs;
    wxString key = PAGE_LANGUAGES;
    key.Append (_T("/"));
    wxString name;

    wxConfig *cfg = new wxConfig (g_appname);
    int languageNr;
    for (languageNr = 0; languageNr < g_LanguagePrefsSize; ++languageNr) {
        curPrefs = &g_LanguagePrefs [languageNr];
        name = curPrefs->name;
        name.Append (_T("/"));
        if (g_LanguageTable [languageNr].filepattern !=  curPrefs->filepattern) {
            cfg->Write (key + name + FILEPATTERN, curPrefs->filepattern);
        }else{
            cfg->DeleteEntry (key + name + FILEPATTERN);
        }
        if (g_LanguageTable [languageNr].helpbooks !=  curPrefs->helpbooks) {
            cfg->Write (key + name + HELPBOOKS, curPrefs->helpbooks);
        }else{
            cfg->DeleteEntry (key + name + HELPBOOKS);
        }
        if (g_LanguageTable [languageNr].indent != curPrefs->indent) {
            cfg->Write (key + name + INDENTVALUE, curPrefs->indent);
        }else{
            cfg->DeleteEntry (key + name + INDENTVALUE);
        }
        if (g_LanguageTable [languageNr].longln != curPrefs->longln) {
            cfg->Write (key + name + LONGLINEPOS, curPrefs->longln);
        }else{
            cfg->DeleteEntry (key + name + LONGLINEPOS);
        }
        if (g_LanguageTable [languageNr].flags != curPrefs->flags) {
            cfg->Write (key + name + FLAGVALUES, curPrefs->flags);
        }else{
            cfg->DeleteEntry (key + name + FLAGVALUES);
        }
    }
    delete cfg;
}

//----------------------------------------------------------------------------
void Prefs::DeleteValuesPageLanguages () {
    LanguageInfo *curPrefs;

    wxConfig *cfg = new wxConfig (g_appname);
    int languageNr;
    for (languageNr = 0; languageNr < g_LanguagePrefsSize; ++languageNr) {
        curPrefs = &g_LanguagePrefs [languageNr];
        if (g_LanguageTable [languageNr].filepattern !=  curPrefs->filepattern) {
            delete curPrefs->filepattern;
        }
        if (g_LanguageTable [languageNr].helpbooks !=  curPrefs->helpbooks) {
            delete curPrefs->helpbooks;
        }
    }
    delete cfg;
}

//----------------------------------------------------------------------------
void Prefs::LoadValuesPageStyles (bool dflt) {
    StyleInfo *curPrefs;
    wxString key = PAGE_STYLE_TYPES;
    key.Append (_T("/"));
    wxString name;

    wxConfig *cfg = new wxConfig (g_appname);
    int typeNr;
    for (typeNr = 0; typeNr < g_StylePrefsSize; ++typeNr) {
        g_StylePrefs [typeNr] = g_StyleTable [typeNr];
        curPrefs = &g_StylePrefs [typeNr];
        if (dflt) continue;
        name = curPrefs->name;
        name.Append (_T("/"));
        if (cfg->Exists (key + name + FOREGROUND)) {
            wxString foreground;
            cfg->Read (key + name + FOREGROUND, &foreground);
            curPrefs->foreground = wxStrdup (foreground);
        }
        if (cfg->Exists (key + name + BACKGROUND)) {
            wxString background;
            cfg->Read (key + name + BACKGROUND, &background);
            curPrefs->background = wxStrdup (background);
        }
        if (cfg->Exists (key + name + FONTNAME)) {
            wxString fontname;
            cfg->Read (key + name + FONTNAME, &fontname);
            curPrefs->fontname = wxStrdup (fontname);
        }
        if (cfg->Exists (key + name + FONTSIZE)) {
            cfg->Read (key + name + FONTSIZE, &curPrefs->fontsize);
        }
        if (cfg->Exists (key + name + FONTSTYLE)) {
            cfg->Read (key + name + FONTSTYLE, &curPrefs->fontstyle);
        }
        if (cfg->Exists (key + name + LETTERCASE)) {
            cfg->Read (key + name + LETTERCASE, &curPrefs->lettercase);
        }
    }
    delete cfg;
}

//----------------------------------------------------------------------------
void Prefs::SaveValuesPageStyles () {
    StyleInfo *curPrefs;
    wxString key = PAGE_STYLE_TYPES;
    key.Append (_T("/"));
    wxString name;

    wxConfig *cfg = new wxConfig (g_appname);
    int typeNr;
    for (typeNr = 0; typeNr < g_StylePrefsSize; ++typeNr) {
        curPrefs = &g_StylePrefs [typeNr];
        name = curPrefs->name;
        name.Append (_T("/"));
        if (g_StyleTable [typeNr].foreground != curPrefs->foreground) {
            cfg->Write (key + name + FOREGROUND, curPrefs->foreground);
        }else{
            cfg->DeleteEntry (key + name + FOREGROUND);
        }
        if (g_StyleTable [typeNr].background != curPrefs->background) {
            cfg->Write (key + name + BACKGROUND, curPrefs->background);
        }else{
            cfg->DeleteEntry (key + name + BACKGROUND);
        }
        if (g_StyleTable [typeNr].fontname != curPrefs->fontname) {
            cfg->Write (key + name + FONTNAME, curPrefs->fontname);
        }else{
            cfg->DeleteEntry (key + name + FONTNAME);
        }
        if (g_StyleTable [typeNr].fontsize != curPrefs->fontsize) {
            cfg->Write (key + name + FONTSIZE, curPrefs->fontsize);
        }else{
            cfg->DeleteEntry (key + name + FONTSIZE);
        }
        if (g_StyleTable [typeNr].fontstyle != curPrefs->fontstyle) {
            cfg->Write (key + name + FONTSTYLE, curPrefs->fontstyle);
        }else{
            cfg->DeleteEntry (key + name + FONTSTYLE);
        }
        if (g_StyleTable [typeNr].lettercase != curPrefs->lettercase) {
            cfg->Write (key + name + LETTERCASE, curPrefs->lettercase);
        }else{
            cfg->DeleteEntry (key + name + LETTERCASE);
        }
    }
    delete cfg;
}

//----------------------------------------------------------------------------
void Prefs::DeleteValuesPageStyles () {
    StyleInfo *curPrefs;

    wxConfig *cfg = new wxConfig (g_appname);
    int typeNr;
    for (typeNr = 0; typeNr < g_StylePrefsSize; ++typeNr) {
        curPrefs = &g_StylePrefs [typeNr];
        if (g_StyleTable [typeNr].foreground != curPrefs->foreground) {
            delete curPrefs->foreground;
        }
        if (g_StyleTable [typeNr].background != curPrefs->background) {
            delete curPrefs->background;
        }
        if (g_StyleTable [typeNr].fontname != curPrefs->fontname) {
            delete curPrefs->fontname;
        }
    }
    delete cfg;
}

//----------------------------------------------------------------------------
void Prefs::LoadValuesPageAnalyse (bool dflt) {
    AnalyseInfo *curPrefs;
    wxString key = PAGE_ANALYSE;
    key.Append (_T("/"));
    wxString name;

    wxConfig *cfg = new wxConfig (g_appname);
    int nr;
    for (nr = 0; nr < g_AnalysePrefsSize; ++nr) {
        g_AnalysePrefs [nr] = g_AnalyseTable [nr];
        curPrefs = &g_AnalysePrefs [nr];
        if (dflt) continue;
        name = curPrefs->name;
        name.Append (_T("/"));
    }
    delete cfg;
}

//----------------------------------------------------------------------------
void Prefs::SaveValuesPageAnalyse () {
    AnalyseInfo *curPrefs;
    wxString key = PAGE_ANALYSE;
    key.Append (_T("/"));
    wxString name;

    wxConfig *cfg = new wxConfig (g_appname);
    int nr;
    for (nr = 0; nr < g_AnalysePrefsSize; ++nr) {
        curPrefs = &g_AnalysePrefs [nr];
        name = curPrefs->name;
        name.Append (_T("/"));
    }
    delete cfg;
}

//----------------------------------------------------------------------------
void Prefs::DeleteValuesPageAnalyse () {
    AnalyseInfo *curPrefs;

    wxConfig *cfg = new wxConfig (g_appname);
    int nr;
    for (nr = 0; nr < g_AnalysePrefsSize; ++nr) {
        curPrefs = &g_AnalysePrefs [nr];
    }
    delete cfg;
}

